#!/bin/bash

# cockpit-kylin-scmms-1.0 自动部署脚本
# Liwei F

# 遇错停止
set -u

# ======================================== 权限检测 =======================================
if [ $UID -ne 0 ]; then
    echo "不是root用户，请使用root用户";
    exit -1;
fi

# ======================================== 文件检测 =======================================
file_exist_arr=(
    "/usr/share/cockpit/manager.database/httpd.json"  
	"/usr/share/cockpit/manager.database/mysql-createUser.sql" 
	"/usr/share/cockpit/manager/lib/tables.py" 
	"/usr/share/cockpit/manager/lib/tables.sql" 
	"/etc/httpd/conf/httpd.conf" 
)
file_need_del=(
	"/etc/httpd/conf.d/welcome.conf" 
)
folder_need_make=(
	"/var/www/html/repos" 
	"/tmp1"
)
for file in ${file_exist_arr[*]}; 
do
    if [ ! -e "$file" ]; then
        echo $file" 不存在，请修正后再试";
        exit -1
    fi
done

# 修改端口 需要关闭SE和firewalld
# 修改 SELINUX=enforcing => SELINUX=disabled
systemctl stop firewalld
systemctl disable firewalld
setenforce 0
sed -i 's/SELINUX=enforcing/SELINUX=disabled/' /etc/selinux/config
sed -i 's/Listen 80/Listen 9093/' /etc/httpd/conf/httpd.conf

for file in ${file_need_del[*]};
do
    if [ -e "$file" ]; then
        rm -f $file;
    fi
done

for folder in ${folder_need_make[*]}; do
    if [ ! -d "$folder" ]; then
    	mkdir $folder;
	    chmod 755 -R $folder;
    fi
done

echo "文件检测正常"
# ===================================== 主机ip获取/选择/绑定 ====================================
echo "==================开始检测ip==============="

count=0

for IP in $(ip a | grep inet | grep -v 127.0.0.1 | grep -v inet6 | awk '{print $2}' | cut -d "/" -f1);
do
ip_arr[count]=${IP};
echo "$count：${IP}";
let count++;
done
#count=1

if (( $count == 1 )); 
then 
    ipSelection=${ip_arr[0]};
    # Only 1 ip, then no prompt
else
    ipSelection="unknown";
fi
#echo $count
while [[ "$ipSelection" == "unknown" ]]
do
    echo "请选择ip，输入对应数字选择ip 不输入则默认选择第一个ip （回车下一步）";
    read -p "选项:" ip_input;
    if [ -z "$ip_input" ]; then
        ipSelection=${ip_arr[0]};
        break;
    elif [[ "$ip_input" =~ [^0-9] ]]; then
        echo "非法字符，请重新输入";
    elif [ "$ip_input" -lt 0 -o "$ip_input" -ge $count ]; then
        echo "没有此选项，请重新输入";
    elif [ "$ip_input" -ge 0 -a "$ip_input" -lt $count ]; then
        #echo "debugger t3"
        ipSelection=${ip_arr[ip_input]};
        #echo $ipSelection;
        break;
    else
        ip_input="unknown";
        echo "未知错误";
        exit -1;
    fi
done
echo "已选择："$ipSelection
echo "正在生成ssh密匙..."


if [ ! -f "/root/.ssh/id_rsa" ] || [ ! -f "/root/.ssh/id_rsa.pub" ]; then
    ssh-keygen -t rsa -P "" -f /root/.ssh/id_rsa;
    localkey=$(cat /root/.ssh/id_rsa.pub | grep root@localhost);
    echo $localkey >> "/root/.ssh/authorized_keys";
    #cp id_rsa.pub authorized_keys
    ssh-copy-id -o StrictHostKeyChecking=no root@localhost;
    localhost=$(cat /root/.ssh/known_hosts | grep localhost);
    sed -i "s/localhost/$ipSelection/" /root/.ssh/known_hosts;
    echo $localhost >> /root/.ssh/known_hosts;
else
	localauth=$(cat /root/.ssh/id_rsa.pub | grep root@localhost | head -n 1);
    if [ ! -f "/root/.ssh/authorized_keys" ]; then
		echo $localauth >> "/root/.ssh/authorized_keys";
	else
		islocalauth=0;
		while read line;
		do 
			if [[ "$line" == "$localauth" ]]; then
				islocalauth=1;
			fi
		done < /root/.ssh/authorized_keys
		if [[ "$islocalauth" != "1" ]]; then
			echo $localauth >> "/root/.ssh/authorized_keys";
		fi
	fi
	localhost=$(cat /root/.ssh/known_hosts | grep localhost);
    iphost=$(cat /root/.ssh/known_hosts | grep $ipSelection);
    if [ -z "$localhost" ] && [ -z "$iphost" ]; then
        ssh-copy-id -o StrictHostKeyChecking=no root@localhost;
        sed -i "s/localhost/$ipSelection/" /root/.ssh/known_hosts;
        echo $localhost >> /root/.ssh/known_hosts;
    elif [ ! -z "$localhost" ] && [ -z "$iphost" ]; then
        sed -i "s/localhost/$ipSelection/" /root/.ssh/known_hosts;
        echo $localhost >> /root/.ssh/known_hosts;
    elif [ ! -z "$iphost" ] && [ -z "$localhost" ]; then
        sed -i "s/$ipSelection/localhost/" /root/.ssh/known_hosts;
        echo $iphost >> /root/.ssh/known_hosts;
    else
        echo "已有密匙,已绑定localhost和$ipSelection";
    fi
fi


echo "密匙文件正常"
# =============================== 数据库导入 ==================================

# using mariaDB, mysql
# creating a test .sql file for testing password
# Detect password for mariadb, if none, ask for creating
#

# flg=0 => import prequisites check not yet done
# flg=1 => import prequisites checked done
dbflg=0
dpassword="kylin.2020"
setdbp(){
    # helper function
    # using for updating root's password
    # *** none password only ***
	echo "use mysql;
UPDATE user SET password=password('${1}') WHERE user='root';
flush privileges;
exit" > temp1.sql
    mysql -uroot < temp1.sql
    password=$1
    rm -f temp1.sql

}


dbpassword(){
    # called by dbprompt
    # helper function
    # asking for create a DB password
	while true; do
		echo "请输入密码：";
		read -s dbp;
		echo "请再次输入确认：";
		read -s dbp2;
		if [[ "$dbp" = "$dbp2" ]]; then
			echo "设置密码为:" $dbp;
            setdbp "$dbp";
			break;
		else
			echo "两次密码设置不一致，请重新设置";
			echo;
		fi
	done
}

dbprompt(){
    # called by test_db
    # helper function
    while true; do
		echo "检测到mariaDB的root用户无密码，是否设置密码？";
        read -p "(yes: 自定义密码，no: 终止安装，不输入设置默认密码: kylin.2020)" x;
        if [ -z $x ]; then
			echo "已设置root默认密码 : kylin.2020";
            setdbp "$dpassword";
			break;
		elif [[ "$x" = "yes" ]] || [[ "$x" = "y" ]] || [[ "$x" = "YES" ]] || [[ "$x" = "Y" ]]; then
			dbpassword;
			break;
		elif [[ "$x" = "no" ]] || [[ "$x" = "n" ]] || [[ "$x" = "NO" ]] || [[ "$x" = "N" ]] || [[ "$x" = "x" ]]; then
			echo "无mariaDB密码，用户终止";
            if [ -e "./temp.sql" ]; then
                rm -f temp.sql;
            fi
			exit -1;
		else
			echo "未知指令$x，请重新输入";
		fi 
	done
}


test_db(){
    # test database password
    touch temp.sql
    if [ "$#" = 0 ]; then
        mysql -uroot < temp.sql 2>/dev/null;
    else
        mysql -uroot -p$1 < temp.sql 2>/dev/null;
    fi
    pwe=$?

}

systemctl restart mariadb

test_db
echo "================数据库导入中===============";
if [ "$pwe" = "0" ]; then
# 测试文档无返回值则root用户无密码，询问是否创建密码
    dbprompt;
    dbflg=1;
else
# 测试文档有返回值则root用户有密码，要求用户输入密码使用。
# fixme: rm -f temp.sql
    echo "请输入数据库密码：";
    while true; do
        read -s password;
        test_db "$password";

        if [ "$pwe" = "0" ]; then
	        echo "密码正确，数据导入中，请耐心等待";
            dpassword=$password
            dbflg=1;
            break;
        else
	        echo "密码错误，无法导入数据，请重新输入";
        fi
    done
    
fi
if [ -e "./temp.sql" ]; then
    rm -f temp.sql;
fi

# 改写文件**FIXME** 如果后期修改过manager.database 文件需要重新修改此部分

echo "{\"httpserverip\": \"$ipSelection\", \"httpserverport\": \"9093\"}" > /usr/share/cockpit/manager.database/httpd.json

if [ $dbflg = "0" ]; then
    echo "发生未知错误，无法创建数据库";
else
    mysql -uroot -p"$password" < /usr/share/cockpit/manager.database/mysql-createUser.sql;
    # mysql -uroot -p"$password" -e 'use Syslog' 2>/dev/null;
    # if [ "$?" = "0" ]; then
    #     echo "Syslog DB已存在，此脚本不会重新导入数据表";
    # else
    mysql -uSyslog -pSyslogUploadPass < /usr/share/cockpit/manager.database/mysql-createDB.sql;
    #     mysql -uroot -p"$password" -e "flush privileges";
    #     echo "已成功导入数据表"
    # fi
    mysql -uroot -p"$password" < /usr/share/cockpit/manager/lib/tables.sql;
    echo "已成功导入数据表"

fi


# =============================== 服务重启 =====================================

systemctl restart mariadb
systemctl enable mariadb

python3 /usr/share/cockpit/manager/lib/tables.py

systemctl restart rsyslog
systemctl restart scmms-server
systemctl restart scmms-daemon

echo "====================完成==================="
echo "全部部署已完成，请登录$ipSelection:9090查看"
