#!/usr/bin/python3
import os
import sys
import json
from configparser import ConfigParser
import traceback
import uuid
from pymongo import MongoClient
import datetime

def deleteGroup():
    conn = MongoClient('localhost')
    for val in sys.argv:
        if val.startswith("uuid="):
            grpuuid = val[len("uuid="):]

    try:
        conn["cockpit"]["groups"].remove({"uuid": grpuuid})
        print(json.dumps({"succeed": True, "error": ""}))
    except:
        sys.stderr.write("deleteGroup failed")
        sys.stderr.write(traceback.format_exc())
        sys.exit(1)

def editGroup():
    conn = MongoClient('localhost')
    for val in sys.argv:
        if val.startswith("uuid="):
            grpuuid = val[len("uuid="):]
        if val.startswith("name="):
            name = val[len("name="):]
        if val.startswith("desc="):
            desc = val[len("desc="):]

    try:
        queryres = conn["cockpit"]["groups"].count({"uuid": grpuuid})
        if not queryres == 1:
            sys.stderr.write("指定的分组id不存在")
            sys.exit(1)

        queryres = conn["cockpit"]["groups"].count({"uuid": {"$ne": grpuuid}, "name": name})
        if queryres > 0:
            sys.stderr.write("分组名称已经被占用")
            sys.exit(1)

        conn["cockpit"]["groups"].update({"uuid": grpuuid}, 
            {"$set": {"name": name, "desc": desc}}, upsert=True)
        print(json.dumps({"succeed": True, "error": ""}))
    except:
        sys.stderr.write("addGroup failed")
        sys.stderr.write(traceback.format_exc())
        sys.exit(1)

def addGroup():
    conn = MongoClient('localhost')
    for val in sys.argv:
        if val.startswith("name="):
            name = val[len("name="):]
        if val.startswith("desc="):
            desc = val[len("desc="):]

    try:
        queryres = conn["cockpit"]["groups"].count({"name": name})
        if queryres > 0:
            sys.stderr.write("该分组名称已经存在：" + name)
            sys.exit(1)

        grpuuid = str(uuid.uuid4())
        conn["cockpit"]["groups"].insert({"uuid": grpuuid, "name": name, "desc": desc})
        print(json.dumps({"succeed": True, "error": ""}))
    except:
        sys.stderr.write("addGroup failed")
        sys.stderr.write(traceback.format_exc())
        sys.exit(1)

def getGroups():
    conn = MongoClient('localhost')
    try:
        queryres = conn["cockpit"]["groups"].find({}, {"_id": 0})
        result = []
        for x in queryres:
            result.append(x)

        print(json.dumps(result))
    except:
        sys.stderr.write("getGroups failed")
        sys.stderr.write(traceback.format_exc())
        sys.exit(1)

def setAssert():
    conn = MongoClient('localhost')
    for val in sys.argv:
        if val.startswith("uuid="):
            uuid = val[len("uuid="):]
        if val.startswith("desc="):
            desc = val[len("desc="):]
    try:
        conn["cockpit"]["client_upload"].update({"uuid": uuid}, {"$set": {"desc": desc}}, upsert=True)
        print(json.dumps({
            "succeed": True,
            "error": ""
            }))
    except:
        sys.stderr.write("setAssert failed")
        sys.stderr.write(traceback.format_exc())
        sys.exit(1)

def getAsserts():
    conn = MongoClient('localhost')
    try:
        now = datetime.datetime.now()
        queryres = conn["cockpit"]["client_upload"].find({}, {"_id": 0})
        result = []
        for x in queryres:
            if (now - x["time"]).seconds > 15:
                state = "离线"
            else:
                state = "在线"
            x["state"] = state
            del x["time"]
            del x["pass"]
            result.append(x)

        print(json.dumps(result))
    except:
        sys.stderr.write("getAsserts failed")
        sys.stderr.write(traceback.format_exc())
        sys.exit(1)

def setClientInfo():
    for val in sys.argv:
        if val.startswith("ifclient="):
            if val.find("true") > 0:
                ifclient = 1
            else:
                ifclient = 0
        if val.startswith("serverip="):
            serverip = val[len("serverip="):]
        if val.startswith("serverport="):
            serverport = val[len("serverport="):]
        if val.startswith("localip="):
            localip = val[len("localip="):]
        if val.startswith("localuser="):
            localuser = val[len("localuser="):]
        if val.startswith("localpass="):
            localpass = val[len("localpass="):]

    config = ConfigParser()

    # import pdb;pdb.set_trace()
    # test if uuid exist
    cliuuid = None
    try:
        tmpcfg = ConfigParser()
        tmpcfg.read("/etc/cockpit-client.conf")
        cliuuid = tmpcfg.get("config", "uuid")
    except:
        pass

    if not cliuuid:
        cliuuid = uuid.uuid4()

    config.add_section("config")
    config.set("config", "uuid", str(cliuuid))
    config.set("config", "ifclient", str(ifclient))
    config.set("config", "serverip", serverip)
    config.set("config", "serverport", serverport)
    config.set("config", "localip", localip)
    config.set("config", "localuser", localuser)
    config.set("config", "localpass", localpass)
    config.write(open("/etc/cockpit-client.conf", "w"))
    print(json.dumps({
        "succeed": True, "error": ""
        }))

def getClientInfo():
    config = ConfigParser()
    ifclient = 0
    serverip = ""
    serverport = ""
    localip = ""
    localuser = ""
    localpass = ""

    f = os.popen("ip a | grep ' inet ' | awk '{print $2}' | grep -v 127.0.0.1", "r")
    ips = f.read().split("\n")

    try:
        config.read("/etc/cockpit-client.conf")
        ifclient = config.getint("config", "ifclient")
        serverip = config.get("config", "serverip")
        serverport = config.get("config", "serverport")
        localip = config.get("config", "localip")
        localuser = config.get("config", "localuser")
        localpass = config.get("config", "localpass")
    except:
        sys.stderr.write(traceback.format_exc())

    result = {"succeed": True, "error": "",
            "ifclient": ifclient,
            "serverip": serverip, "serverport": serverport,
            "localip": localip, "localuser": localuser, 
            "localpass": localpass, "ips": ips
            }
    print(json.dumps(result))

def usage():
    sys.stderr.write("usage: python3 manager-cmd.py $para\n")

def main():
    if len(sys.argv) < 2:
        usage()
        sys.exit(1)

    if sys.argv[1] == "getClientInfo":
        getClientInfo()
    elif sys.argv[1] == "setClientInfo":
        setClientInfo()
    elif sys.argv[1] == "getAsserts":
        getAsserts()
    elif sys.argv[1] == "setAssert":
        setAssert()
    elif sys.argv[1] == "getGroups":
        getGroups()
    elif sys.argv[1] == "addGroup":
        addGroup()
    elif sys.argv[1] == "editGroup":
        editGroup()
    elif sys.argv[1] == "deleteGroup":
        deleteGroup()
    else:
        sys.exit(1)

if __name__ == '__main__':
    try:
        main()
    except:
        sys.stderr.write(traceback.format_exc())
        sys.exit(1)
