#!/usr/bin/python3
import os
import sys
import json
import subprocess
import traceback
import uuid
import datetime
import logging
import math
import paramiko
import base64
import time
import pymysql

from configparser import ConfigParser
from logging.handlers import RotatingFileHandler

from flask import Flask
from flask import jsonify, request, Flask, render_template, redirect, url_for
from flask_sqlalchemy import SQLAlchemy


curdirname, curfilename = os.path.split(os.path.realpath(__file__))
LIB_DIR = "../manager/lib"
if not LIB_DIR in sys.path:
    sys.path.append(LIB_DIR)

from tables import Assert, Config, Asgroup, Batchuser, Rules,SoftRepo,RepoStrategy,UpdateStrategy,\
    AssertUserPass,ScmmsUser, AuditLog
from tables import app
from sqlalchemy import and_, or_ #, create_engine

import kyfilelock
import kyencrypt

pymysql.install_as_MySQLdb()

gMonitorLog = None
logging.basicConfig()
Rthandler = RotatingFileHandler('/var/log/flaskserver.log' ,
        maxBytes=2 * 1024 * 1024, backupCount=2)
formatter = logging.Formatter('%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s')
Rthandler.setFormatter(formatter)

gMonitorLog = logging.getLogger('gMonitorLog')
gMonitorLog.setLevel(logging.INFO)
gMonitorLog.addHandler(Rthandler)

gServerPort = 9091

#httpserverip = "10.20.2.196"
#httpserverport = "9093"

httpfile = open("/usr/share/cockpit/manager.database/httpd.json")
httpjson = json.load(httpfile)
httpfile.close()
httpserverip = httpjson["httpserverip"]
httpserverport = httpjson["httpserverport"]
def wrapInfo(msg):
    dt_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S: ")
    gMonitorLog.info(dt_prefix + msg)

def wrapError(msg):
    dt_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S: ")
    gMonitorLog.error(dt_prefix + msg)

#设置日志服务器默认配置,保存在json文件中 (已弃用)
#syslog_serverip = "127.0.0.1"
#syslog_serverport = "3306"
#syslog_user = "Syslog"
#syslog_password = "SyslogUploadPass"
#syslog_database = "Syslog"
db = SQLAlchemy(app)
# 检查数据库有无syslog配置信息，有则导出并使用，无则创建默认
# 该默认设置于/manager.database/mysql-createUser.sql
def get_ip():
    ip = ""
    userpath = "/usr/share/cockpit/manager.database/httpd.json"
    while True:
        if os.access(userpath, os.R_OK):
            break
        else:
            time.sleep(0.01)
            continue
    hostfile = open(userpath,"rb")
    hostlines = json.load(hostfile)
    hostfile.close()
    ip = hostlines["httpserverip"]
    return ip
def checkdb():
    checking = db.session.query(Config).filter(Config.configtype=="syslog").count()
    ip = get_ip()
    if len(ip) == 0:
        wrapError("无法获取ip")
        return
    content = {}
    if checking == 0:
        # 检查Config表中有无'syslog'配置,若没有则创建默认配置
        configtype = 'syslog' 
        editCount = 1
        applyCount = {}
        content = {
            "ip": ip,
            "port": "3306",
            "database": "Syslog",
            "user": "Syslog",
            "password": "SyslogUploadPass",
            "keeptime": "3",
            "editCount": editCount,
            "applyCount": applyCount
        }
        content["applyCount"] = applyCount
        record = Config(configtype=configtype, content=json.dumps(content))
        db.session.add(record)
        db.session.commit()
    else:
        sysconfig = db.session.query(Config).filter(Config.configtype=="syslog").first()
        jsconfig = sysconfig.to_json()
        content = json.loads(jsconfig["content"])
    return content
syslogcontent = checkdb()
# print(syslogcontent)
syslog_serverip = syslogcontent["ip"]
syslog_serverport = syslogcontent["port"]
syslog_user = syslogcontent["user"]
syslog_password = syslogcontent["password"]
syslog_database = syslogcontent["database"]

# syslogfile = open("/usr/share/cockpit/manager.database/syslog.json")
# syslogjson = json.load(syslogfile)
# syslogfile.close()

# 日志模块用到了不同的数据库，需要更新下数据库的配置，
# 另外创建一个 engine 对象，用于在操作数据库的时候指定不同的数据库 
# *使用get_engine()来绑定不同数据库以达到直接execute (bind=db.get_engine(app, "syslog"))

def setappconfig(dbconfig):
    app.config['SQLALCHEMY_BINDS'] = {'syslog': ("mysql://" + dbconfig["user"] + ":" + dbconfig["password"] + "@" + dbconfig["ip"] + ":"+str(dbconfig["port"])+"/" + dbconfig["database"])}
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    app.config['SOLALCHEMY_ECHO'] = False
    app.config['SQLALCHEMY_POOL_RECYCLE'] = 3600
# engine = create_engine(app.config['SQLALCHEMY_BINDS']['syslog'],pool_recycle = 3600,pool_pre_ping = True)

def is_jsonable(x):
    try:
        json.dumps(x)
        return True
    except (TypeError, OverflowError):
        return False

@app.route("/")
def index():
    return "hello world..."


# 获取用户数据接口
@app.route("/getUsersManager")
def getUsersManager():
    #print("getUsersManager")
    try:
        queryres = db.session.query(ScmmsUser).filter().all()
        #print(queryres)
        #print(type(queryres))
       
        #result = [i.to_json() for i in queryres]
        result = []
        for i in queryres:
            tmp = i.to_json()
            print(tmp["password"])
            tmp["password"] = kyencrypt.decrypt(11, tmp["password"])
            result.append(tmp)

        return json.dumps({"succeed": True, "error":" ", "data":result})
    except:
        wrapError("getUsersManager failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed":False, "error": "数据库操作失败"})

#添加用户数据接口
@app.route("/addUsersManager", methods=["POST"])    
def setUsersManager():
    print("addUsersManager")
    args = json.loads(request.get_data())

    name = args.get("name")
    password = args.get("password")
    role = args.get("role")
    createtime = datetime.datetime.now()
    enabled = 1  

    try:
        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})
        
        if not password  or len(password) < 8: 
            return json.dumps({"succeed": False, "error": "密码不合法，最少8个字符"})
        
        count = db.session.query(ScmmsUser).filter(ScmmsUser.name == name).count()
        if(count > 0):
            return json.dumps({"succeed": False, "error": "指定的用户名已存在"})
        
        # 加密保存密码
        pwd = kyencrypt.encrypt(11, password)
        
        rec =  ScmmsUser(name = name,  password = pwd, role = role, createtime = createtime, enabled = enabled)
        
        db.session.add(rec)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("addUsersManager failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed":False, "error": "数据库操作失败"})

#删除用户数据接口
@app.route("/delUsersManager", methods = ["POST"])
def delUsersManager():
    try:
        args = json.loads(request.get_data())
        id = str(args.get("id"))

        if not id:
            return json.dumps({"succeed": False, "error": "用户名不合法"})

        queryres = db.session.query(ScmmsUser).filter(ScmmsUser.id == id).first()
        if not queryres:
            return json.dumps({"succeed":False, "error": "指定的用户名不存在"})
        
        db.session.query(ScmmsUser).filter(ScmmsUser.id == id).delete()
        db.session.commit()
        return json.dumps({"succeed":True, "error":""})
    except:
        wrapError("delUsersManager failed")
        wrapError(traceback.format_exc())

#编辑用户数据接口
@app.route("/editUsersManager", methods = ["POST"])
def editUserManager():
    try:
        print("editUsersManager")
        args = json.loads(request.get_data())
        
        id = args.get("id")
        enabled = args.get("enabled")
        password = args.get("password")
        role = args.get("role")
        createtime = datetime.datetime.now()
        #enabled = 1 
        
        print(len(password))
        print(password)
        if not password  or len(password) < 8: 
            return json.dumps({"succeed": False, "error": "密码不合法，最少8个字符"})
        
        queryres = db.session.query(ScmmsUser).filter(ScmmsUser.id == id).first()
        if not queryres:
            return json.dumps({"succeed":False, "error": "指定账户名不存在"})         

        pwd = kyencrypt.encrypt(11, password)

        queryres.enabled = enabled
        queryres.password = pwd
        queryres.role = role        
        db.session.commit()
        print("编辑用户成功")
        return json.dumps({"succeed": True, "error": " "})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed" : False, "error": "数据库操作失败"})      

#更新用户密码
@app.route("/updateUserPassword", methods=["POST"])
def updateUserPassword():
    try:
        args = json.loads(request.get_data())
        name = str(args.get("name"))
        role = str(args.get("role"))
        password = str(args.get("password"))

        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})

        queryres = db.session.query(ScmmsUser).filter(and_(ScmmsUser.name == name,ScmmsUser.role == role)).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定账户名不存在"})

        if len(password) < 8:
            return json.dumps({"succeed": False, "error": "密码太弱，至少为8位"})

        # 加密保存密码
        pwd = kyencrypt.encrypt(11, password)

        queryres.password = pwd
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})
  
@app.route("/getRsyslogs")
def getRsyslogs():
    # print("getRsyslogs")
    # print(request.args.get("jumppage"))
    # print(request.args.get("pagesize"))
    dbcontent = checkdb()
    setappconfig(dbcontent)
    dbconfig = {
        "host":dbcontent["ip"],
        "user":dbcontent["user"],
        "password":dbcontent["password"],
        "database":dbcontent["database"],
        "port":int(dbcontent["port"]),
        "charset":"utf8"
    }
    if not checkdbconnection(dbconfig):
        return json.dumps({"succeed": False, "error":"无法连接该数据库"})
    jumppage = int(request.args.get("jumppage"))
    pagesize = int(request.args.get("pagesize"))
    offset = (jumppage-1)*pagesize
    try:
        sqlchar = 'SET NAMES latin1;'
        db.session.execute(sqlchar,bind=db.get_engine(app,"syslog"))
        sql = 'select * from SystemEvents limit %d,%d' %(offset,pagesize)
        queryres = db.session.execute(sql,bind=db.get_engine(app, "syslog")).fetchall()
        sql1 = 'select distinct FromHost from SystemEvents'
        domainres = db.session.execute(sql1,bind=db.get_engine(app, "syslog")).fetchall()
        sql2 = 'select count(*) from SystemEvents'
        totalres = db.session.execute(sql2,bind=db.get_engine(app, "syslog")).fetchall()
        # sqlchar = 'SET NAMES utf8'
        # db.session.execute(sqlchar,bind=db.get_engine(app,"syslog"))
        totalnumber = totalres[0]
        totalrecords = int(totalnumber[0])
        result = {"log": [], "domain": []}
        if len(queryres) != 0:
            for item in queryres:
                record = []
                for x in item:
                    if is_jsonable(x):
                        #print(x)
                        pass
                    else:
                        x = str(x)
                        #对于不能序列化的字段，先将其转换为字符串再序列化
                        #print("it can not be serializable")
                        #print(x)
                    record.append(x)
                result["log"].append(record)
        #get domain lists
        #sql1 = 'select distinct FromHost from SystemEvents'
        #domainres = db.session.execute(sql1).fetchall()
        if len(domainres) != 0:
            for y in domainres:  
                # print(type(y[0]))  
                if is_jsonable(y[0]):
                    #print(y[0])
                    pass
                else:
                    y = str(y[0])
                    #对于不能序列化的字段，先将其转换为字符串再序列化
                    #print("it can not be serializable")
                    #print(y[0])
                result["domain"].append(y[0])
        result["totalrecords"] = totalrecords
        # print("the totalrecords is ")
        # print(totalrecords)
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getRsyslogs failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getRsyslogsByPage")
def getRsyslogsByPage():
    # print("getRsyslogsByPage")
    # print(request.args.get("jumppage"))
    # print(request.args.get("pagesize"))
    dbcontent = checkdb()
    setappconfig(dbcontent)
    jumppage = int(request.args.get("jumppage"))
    pagesize = int(request.args.get("pagesize"))
    begintime = request.args.get("begintime")
    endtime = request.args.get("endtime")
    domain = request.args.get("domain")
    priority = request.args.get("priority")
    search = request.args.get("search")
    timesearch = 0
    if len(begintime) != 0 and len(endtime) != 0 :
        timesearch = 1
    elif len(begintime) != 0 and len(endtime) == 0:
        timesearch = 2
    elif len(begintime) == 0 and len(endtime) != 0:
        timesearch = 3
    begintime = begintime + " 00:00:00"
    endtime = endtime + " 23:59:59"
    offset = (jumppage-1)*pagesize
    sql = ""
    priority = int(priority)
    if len(search) == 0:
        if domain.strip() == "all":
            if priority == -1:
                if timesearch == 0:
                    sql = 'select * from SystemEvents limit %d,%d' %(offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where ReceivedAt >= "%s" limit %d,%d' %(begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where ReceivedAt <= "%s" limit %d,%d' %(endtime,offset,pagesize)
                else:
                    sql =  'select * from SystemEvents where ReceivedAt between "%s" and "%s" limit %d,%d'  %(begintime,endtime,offset,pagesize)            
            else:
                if timesearch == 0:
                    sql = 'select * from SystemEvents where Priority = %d limit %d,%d' %(priority,offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where Priority = %d and ReceivedAt >= "%s" limit %d,%d' %(priority,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where Priority = %d and ReceivedAt <= "%s" limit %d,%d' %(priority,endtime,offset,pagesize)
                else:
                    sql = 'select * from SystemEvents where Priority = %d and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,begintime,endtime,offset,pagesize)
        else:
            if priority == -1:
                if timesearch == 0:
                    sql = 'select * from SystemEvents where FromHost = "%s" limit %d,%d' %(domain,offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where FromHost = "%s" and ReceivedAt >= "%s" limit %d,%d' %(domain,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where FromHost = "%s" and ReceivedAt <= "%s" limit %d,%d' %(domain,endtime,offset,pagesize)
                else:
                    sql = 'select * from SystemEvents where FromHost = "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(domain,begintime,endtime,offset,pagesize)
            else:
                if timesearch ==0:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" limit %d,%d' %(priority,domain,offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt >= "%s" limit %d,%d' %(priority,domain,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt <= "%s" limit %d,%d' %(priority,domain,endtime,offset,pagesize)
                else:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,domain,begintime,endtime,offset,pagesize)
    else:
        if domain.strip() == "all":
            if priority == -1:
                if timesearch == 0:
                    sql = 'select * from SystemEvents where Message REGEXP "%s" limit %d,%d' %(search,offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d' %(search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d' %(search,endtime,offset,pagesize)
                else:
                    sql = 'select * from SystemEvents where Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(search,begintime,endtime,offset,pagesize)
            else:
                if timesearch == 0:
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" limit %d,%d' %(priority,search,offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d' %(priority,search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d' %(priority,search,endtime,offset,pagesize)
                else:
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,search,begintime,endtime,offset,pagesize)
        else:
            if priority == -1:
                if timesearch == 0:
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" limit %d,%d' %(domain,search,offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d' %(domain,search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d' %(domain,search,endtime,offset,pagesize)
                else:
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(domain,search,begintime,endtime,offset,pagesize)
            else:
                if timesearch ==0:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" limit %d,%d' %(priority,domain,search,offset,pagesize)
                elif timesearch == 2:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d' %(priority,domain,search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d' %(priority,domain,search,endtime,offset,pagesize)
                else:
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,domain,search,begintime,endtime,offset,pagesize)
    try:
        # print(sql)
        sqlchar = 'SET NAMES latin1;'
        db.session.execute(sqlchar,bind=db.get_engine(app,"syslog"))
        queryres = db.session.execute(sql,bind=db.get_engine(app, "syslog")).fetchall()
        # sqlchar = 'SET NAMES utf8'
        # db.session.execute(sqlchar,bind=db.get_engine(app,"syslog"))
        result = {"log": []}
        if len(queryres) != 0:
            for item in queryres:
                record = []
                for x in item:
                    if is_jsonable(x):
                        #print(x)
                        pass
                    else:
                        x = str(x)
                        #print("it can not be serializable")
                        #print(x)
                    record.append(x)
                result["log"].append(record)
        #result["totalpage"] = totalpage
        #print("the totalpage is ")
        #print(totalpage)
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getRsyslogsByPage failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getRsyslogsByFilter")
def getRsyslogsByFilter():
    #global queryres
    # print("getRsyslogsByFilter")
    # print(request.args.get("jumppage"))
    # print(request.args.get("pagesize"))
    dbcontent = checkdb()
    setappconfig(dbcontent)
    jumppage = int(request.args.get("jumppage"))
    pagesize = int(request.args.get("pagesize"))
    begintime = request.args.get("begintime")
    endtime = request.args.get("endtime")
    domain = request.args.get("domain")
    priority = request.args.get("priority")
    search = request.args.get("search")
    timesearch = 0
    if len(begintime) != 0 and len(endtime) != 0:
        timesearch = 1
    elif len(begintime) != 0 and len(endtime) == 0:
        timesearch = 2 # 没有endtime，即begintime-latest data
    elif len(begintime) == 0 and len(endtime) != 0:
        timesearch = 3 # 没有begintime，即oldest data-endtime
    begintime = begintime + " 00:00:00"
    endtime = endtime + " 23:59:59"
    offset = (jumppage-1)*pagesize
    sql = ""
    sql2 = ""
    priority = int(priority)
    if len(search) == 0:
        if domain.strip() == "all":
            if priority == -1:
                if timesearch == 0:
                    sql2 = 'select count(*) from SystemEvents'
                    sql = 'select * from SystemEvents limit %d,%d' %(offset,pagesize)
                elif timesearch == 2: # 没有endtime，即begintime-latest data
                    sql2 = 'select count(*) from SystemEvents where ReceivedAt >= "%s"'  %(begintime)
                    sql = 'select * from SystemEvents where ReceivedAt >= "%s" limit %d,%d'  %(begintime,offset,pagesize)
                elif timesearch == 3: # 没有begintime，即oldest data-endtime
                    sql2 = 'select count(*) from SystemEvents where ReceivedAt <= "%s"'  %(endtime)
                    sql = 'select * from SystemEvents where ReceivedAt <= "%s" limit %d,%d'  %(endtime,offset,pagesize) 
                else:
                    sql2 = 'select count(*) from SystemEvents where ReceivedAt between "%s" and "%s"'  %(begintime,endtime)
                    sql = 'select * from SystemEvents where ReceivedAt between "%s" and "%s" limit %d,%d'  %(begintime,endtime,offset,pagesize)            
            else:
                if timesearch == 0:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d' %(priority)
                    sql = 'select * from SystemEvents where Priority = %d limit %d,%d' %(priority,offset,pagesize)
                elif timesearch == 2:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and ReceivedAt >= "%s"'  %(priority,begintime)
                    sql = 'select * from SystemEvents where Priority = %d and ReceivedAt >= "%s" limit %d,%d'  %(priority,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and ReceivedAt <= "%s"'  %(priority,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and ReceivedAt <= "%s" limit %d,%d'  %(priority,endtime,offset,pagesize) 
                else:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and ReceivedAt between "%s" and "%s"' %(priority,begintime,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,begintime,endtime,offset,pagesize)
        else:
            if priority == -1:
                if timesearch == 0:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s"' %(domain)
                    sql = 'select * from SystemEvents where FromHost = "%s" limit %d,%d' %(domain,offset,pagesize)
                elif timesearch == 2:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s" and ReceivedAt >= "%s"'  %(domain,begintime)
                    sql = 'select * from SystemEvents where FromHost = "%s" and ReceivedAt >= "%s" limit %d,%d'  %(domain,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s" and ReceivedAt <= "%s"'  %(domain,endtime)
                    sql = 'select * from SystemEvents where FromHost = "%s" and ReceivedAt <= "%s" limit %d,%d'  %(domain,endtime,offset,pagesize) 
                else:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s" and ReceivedAt between "%s" and "%s"' %(domain,begintime,endtime)
                    sql = 'select * from SystemEvents where FromHost = "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(domain,begintime,endtime,offset,pagesize)
            else:
                if timesearch ==0:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s"' %(priority,domain)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" limit %d,%d' %(priority,domain,offset,pagesize)
                elif timesearch == 2:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt >= "%s"'  %(priority,domain,begintime)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt >= "%s" limit %d,%d'  %(priority,domain,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt <= "%s"'  %(priority,domain,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt <= "%s" limit %d,%d'  %(priority,domain,endtime,offset,pagesize)
                else:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt between "%s" and "%s"' %(priority,domain,begintime,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,domain,begintime,endtime,offset,pagesize)
    else:
        if domain.strip() == "all":
            if priority == -1:
                if timesearch == 0:
                    sql2 = 'select count(*) from SystemEvents where Message REGEXP "%s"' %(search)
                    sql = 'select * from SystemEvents where Message REGEXP "%s" limit %d,%d' %(search,offset,pagesize)
                elif timesearch == 2:
                    sql2 = 'select count(*) from SystemEvents where Message REGEXP "%s" and ReceivedAt >= "%s"'  %(search,begintime)
                    sql = 'select * from SystemEvents where Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d'  %(search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql2 = 'select count(*) from SystemEvents where Message REGEXP "%s" and ReceivedAt <= "%s"'  %(search,endtime)
                    sql = 'select * from SystemEvents where Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d'  %(search,endtime,offset,pagesize)
                else:
                    sql2 = 'select count(*) from SystemEvents where Message REGEXP "%s" and ReceivedAt between "%s" and "%s"' %(search,begintime,endtime)
                    sql = 'select * from SystemEvents where Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(search,begintime,endtime,offset,pagesize)
            else:
                if timesearch == 0:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and Message REGEXP "%s"' %(priority,search)
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" limit %d,%d' %(priority,search,offset,pagesize)
                elif timesearch == 2:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt >= "%s"'  %(priority,search,begintime)
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d'  %(priority,search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt <= "%s"'  %(priority,search,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d'  %(priority,search,endtime,offset,pagesize)
                else:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt between "%s" and "%s"' %(priority,search,begintime,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,search,begintime,endtime,offset,pagesize)
        else:
            if priority == -1:
                if timesearch == 0:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s" and Message REGEXP "%s"' %(domain,search)
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" limit %d,%d' %(domain,search,offset,pagesize)
                elif timesearch == 2:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt >= "%s"'  %(domain,search,begintime)
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d'  %(domain,search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt <= "%s"'  %(domain,search,endtime)
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d'  %(domain,search,endtime,offset,pagesize)
                else:
                    sql2 = 'select count(*) from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt between "%s" and "%s"' %(domain,search,begintime,endtime)
                    sql = 'select * from SystemEvents where FromHost = "%s" and Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(domain,search,begintime,endtime,offset,pagesize)
            else:
                if timesearch ==0:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s"' %(priority,domain,search)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" limit %d,%d' %(priority,domain,search,offset,pagesize)
                elif timesearch == 2:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt >= "%s"'  %(priority,domain,search,begintime)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt >= "%s" limit %d,%d'  %(priority,domain,search,begintime,offset,pagesize)
                elif timesearch == 3:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt <= "%s"'  %(priority,domain,search,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt <= "%s" limit %d,%d'  %(priority,domain,search,endtime,offset,pagesize)
                else:
                    sql2 = 'select count(*) from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt between "%s" and "%s"' %(priority,domain,search,begintime,endtime)
                    sql = 'select * from SystemEvents where Priority = %d and FromHost = "%s" and Message REGEXP "%s" and ReceivedAt between "%s" and "%s" limit %d,%d' %(priority,domain,search,begintime,endtime,offset,pagesize)
    try:
        sqlchar = 'SET NAMES latin1;'
        db.session.execute(sqlchar,bind=db.get_engine(app,"syslog"))
        queryres = db.session.execute(sql,bind=db.get_engine(app, "syslog")).fetchall()
        totalres = db.session.execute(sql2,bind=db.get_engine(app, "syslog")).fetchall()
        # sqlchar = 'SET NAMES utf8'
        # db.session.execute(sqlchar,bind=db.get_engine(app,"syslog"))
        totalnumber = totalres[0]
        totalrecords = int(totalnumber[0])
        result = {"log": []}
        if len(queryres) != 0:
            for item in queryres:
                record = []
                for x in item:
                    if is_jsonable(x):
                        pass
                    else:
                        x = str(x)
                    record.append(x)
                result["log"].append(record)
        #get domain lists
        result["totalrecords"] = totalrecords
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getRsyslogsByFilter failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})
        
@app.route("/getdbconfig")
def getdbconfig():
    content = request.args.to_dict()
    # {'ip': '', 'port': '3306', 'database': '', 'user': '', 'password': '', 'keeptime': '3'}
    queryres = db.session.query(Config).filter(Config.configtype=="syslog").first()
    if queryres == None:
        return json.dumps({"succeed": True, "error": "", "content":content})
    dbcontent = json.loads(queryres.content)
    for i in content:
        content[i]=dbcontent[i]
    return json.dumps({"succeed": True, "error": "", "content":content})

    
@app.route("/setSyslogConfig", methods=["POST"])
def setSyslogConfig():
    # print("setSyslogConfig")
    f = kyfilelock.FLock("dlsyslog", "exclusive")
    with f:
        args = json.loads(request.get_data())
        ip = args.get("ip")
        if ip == "localhost" or ip == "127.0.0.1":
            return json.dumps({"succeed": False, "error": "服务器ip地址不能为localhost或127.0.0.1"})
        port = int(args.get("port"))
        database = args.get("database")
        user = args.get("user")
        password = args.get("password")
        keeptime = args.get("keeptime")

        dbconfig = {
            "host":ip,
            "user":user,
            "password":password,
            "database":database,
            "port":port,
            "charset":"utf8"
        }
        if not checkdbconnection(dbconfig):
            return json.dumps({"succeed": False, "error":"无法连接该数据库"})
        try:
            queryres = db.session.query(Config).filter(Config.configtype=='syslog').first()
            content = json.loads(queryres.content)
            dbconfig["ip"] = dbconfig["host"]
            dbconfig["keeptime"] = keeptime
            del dbconfig["host"]
            del dbconfig["charset"]
            flg = 0
            for i in dbconfig:
                if dbconfig[i] != content[i]:
                    flg = 1                
            if flg == 0:
                return json.dumps({"succeed": True, "error": ""})
            content["ip"] = ip
            content["port"] = port
            content["database"] = database
            content["user"] = user
            content["password"] = password
            content["keeptime"] = keeptime
            content["editCount"] = content["editCount"] + 1
            queryres.content = json.dumps(content)
            db.session.commit()
            return json.dumps({"succeed": True, "error": ""})

        except:
            wrapError("setSyslogConfig failed")
            wrapError(traceback.format_exc())
            return json.dumps({"succeed": False, "error": "数据库操作失败"})
# def setCharset(dbconfig):
#     grepcmd = "grep -rni 'init_connect=' /etc/my.cnf.d"
    
#     out, err, exc = sshutils.execRemoteCmd(dbconfig["ip"],dbconfig["port"],dbconfig["user"],grepcmd)
    

    

def checkdbconnection(dbconfig):
    res = False
    try:
        database = dbconfig["database"]
        dbconfig["database"] = None
        conn = pymysql.connect(**dbconfig) # 连接数据库 **用来解包字典
        cur = conn.cursor() # 游标
        # 判断数据库存不存在，若不存在，建库
        cur.execute("create database if not exists %s DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci;" %database)
        conn.commit()
        cur.close()
        conn.close()
        dbconfig["database"] = database
        conn = pymysql.connect(**dbconfig)
        cur = conn.cursor()
        # 判断表存在，若不存在建表
        cur.execute("select * from information_schema.tables where table_name='SystemEvents';")
        sevents = cur.fetchone()
        if sevents == None:
            cur.execute("CREATE TABLE IF NOT EXISTS SystemEvents(ID int unsigned not null auto_increment primary key,"\
                "CustomerID bigint,"\
                "ReceivedAt datetime NULL,"\
                "DeviceReportedTime datetime NULL,"\
                "Facility smallint NULL,"\
                "Priority smallint NULL,"\
                "FromHost varchar(60) NULL,"\
                "Message text,"\
                "NTSeverity int NULL,"\
                "Importance int NULL,"\
                "EventSource varchar(60),"\
                "EventUser varchar(60) NULL,"\
                "EventCategory int NULL,"\
                "EventID int NULL,"\
                "EventBinaryData text NULL,"\
                "MaxAvailable int NULL,"\
                "CurrUsage int NULL,"\
                "MinUsage int NULL,"\
                "MaxUsage int NULL,"\
                "InfoUnitID int NULL ,"\
                "SysLogTag varchar(60),"\
                "EventLogType varchar(60),"\
                "GenericFileName VarChar(60),"\
                "SystemID int NULL) DEFAULT CHARSET = utf8;")
        cur.execute("select * from information_schema.tables where table_name='SystemEventsProperties';")
        seventprop = cur.fetchone()
        if(seventprop == None):    
            cur.execute("CREATE TABLE IF NOT EXISTS SystemEventsProperties(ID int unsigned not null auto_increment primary key,"\
                "SystemEventID int NULL ,"\
                "ParamName varchar(255) NULL ,"\
                "ParamValue text NULL) DEFAULT CHARSET = utf8;")
        conn.commit()
        cur.close()
        conn.close()
        res = True
    except:
        wrapError("无法连接该配置服务器数据库")
        wrapError(traceback.format_exc())
    return res


@app.route("/deleteGroup", methods=["POST"])
def deleteGroup():
    args = json.loads(request.get_data())
    grpuuid = args.get("uuid")

    try:
        db.session.query(Asgroup).filter(Asgroup.groupid==grpuuid).delete()
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("deleteGroup failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

def ifAssertInGroup(asserts):
    # 检查资产是否已经被其它的分组占用
    allGroupAssert = []
    groups = db.session.query(Asgroup).filter().all()
    for x in groups:
        arr = x.groupasserts.split(";")
        for ele in arr:
            if not ele:
                continue
            if not ele in allGroupAssert:
                allGroupAssert.append(ele)

    for ele in asserts.split(";"):
        # 不校验资产是否存在了，费性能，如果指定了不存在的资产，展示的时候直接忽略掉就行，没影响
        if not ele:
            continue
        if ele in allGroupAssert:
            return True

    return False

def ifAssertInOtherGroup(asserts, expgroup):
    # 检查资产是否已经被其它的分组占用
    allGroupAssert = []
    groups = db.session.query(Asgroup).filter().all()
    for x in groups:
        arr = x.groupasserts.split(";")
        for ele in arr:
            if not ele:
                continue
            if not ele in allGroupAssert and not x.groupid == expgroup:
                allGroupAssert.append(ele)

    for ele in asserts.split(";"):
        # 不校验资产是否存在了，费性能，如果指定了不存在的资产，展示的时候直接忽略掉就行，没影响
        if not ele:
            continue
        if ele in allGroupAssert:
            return True

    return False

@app.route("/editGroup", methods=["POST"])
def editGroup():
    args = json.loads(request.get_data())
    grpuuid = args.get("uuid")
    name = args.get("name")
    desc = args.get("desc")
    asserts = args.get("asserts")

    try:
        queryres = db.session.query(Asgroup).filter(Asgroup.groupid==grpuuid).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定的分组ID不存在"})

        count = db.session.query(Asgroup).filter(and_(Asgroup.groupid!=grpuuid, Asgroup.groupname==name)).count()
        if count > 0:
            return json.dumps({"succeed": False, "error": "分组名称已经存在"})

        if ifAssertInOtherGroup(asserts, grpuuid):
            return json.dumps({"succeed": False, "error": "资产已经被其它分组占用:%s" %ele})

        queryres.groupname = name
        queryres.groupdesc = desc
        queryres.groupasserts = asserts
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("addGroup failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addGroup", methods=['POST'])
def addGroup():
    args = json.loads(request.get_data())
    name = args.get("name")
    desc = args.get("desc")
    asserts = args.get("asserts")

    try:
        if db.session.query(Asgroup).filter().count() >= 1000:
            return json.dumps({"succeed": False, "error": "分组数已达到上限1000"})

        count = db.session.query(Asgroup).filter(Asgroup.groupname==name).count()
        if count > 0:
            return json.dumps({"succeed": False, "error": "分组名称已经存在"})

        if ifAssertInGroup(asserts):
            return json.dumps({"succeed": False, "error": "资产已经被其它分组占用"})

        grpuuid = str(uuid.uuid4())
        record = Asgroup(groupid=grpuuid, groupname=name, groupdesc=desc, groupasserts=asserts)
        db.session.add(record)
        db.session.commit()

        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("addGroup failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getGroups")
def getGroups():
    try:
        queryres = db.session.query(Asgroup).filter().all()
        result = [i.to_json() for i in queryres]

        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getGroups failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getGroupsAndAsserts")
def getGroupsAndAsserts():
    try:
        grpqueryres = db.session.query(Asgroup).filter().all()
        grpresult = [i.to_json() for i in grpqueryres]

        assqueryres = db.session.query(Assert).filter(Assert.assertstate!="deleted").all()
        assresult = [i.to_json() for i in assqueryres]

        result = {
            "allgroups": grpresult,
            "allasserts": assresult
        }
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getGroupsAndAsserts failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getFindConfig")
def getFindConfig():
    try:
        queryres = db.session.query(Config).filter(Config.configtype=="find").first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "自动发现配置不存在"})

        findconfig = json.loads(queryres.content)
        return json.dumps({"succeed": True, "error": "", "data": findconfig})
    except:
        wrapError("get find config failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/setFindConfig", methods=["POST"])
def setFindConfig():
    try:
        args = json.loads(request.get_data())
        queryres = db.session.query(Config).filter(Config.configtype=="find").first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "自动发现配置不存在"})

        enabled = args.get("enabled")
        iprange = args.get("iprange")
        content = {"enabled": enabled, "iprange": iprange}
        queryres.content = json.dumps(content)
        db.session.commit()

        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("get find config failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/delAssert", methods=["POST"])
def delAssert():
    args = json.loads(request.get_data())
    assertid = args.get("id")

    try:
        queryres = db.session.query(Assert).filter(Assert.assertid==assertid).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定的记录不存在"})

        queryres.assertstate = "deleted"
        db.session.commit()
        return json.dumps({"succeed": True, "error": "", "data": {}})
    except:
        wrapError("delAssert failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addAssert", methods=["POST"])
def addAssert():
    args = json.loads(request.get_data())
    ip = args.get("ip")
    name = args.get("name")
    finger = args.get("finger")
    tag = args.get("tag")
    user = args.get("user")
    if not user == "root" and not user == "cockpit-client":
        return json.dumps({"succeed": False, "error": "不允许添加该用户"})
    try:
        count = db.session.query(Assert).filter().count()
        if count >= 1000 :
            return json.dumps({"succeed": False, "error": "资产数已达到上限1000"})
        f = kyfilelock.FLock("addAssert","exclusive")
        with f:
            db.session.commit()
            queryres = db.session.query(Assert).filter(Assert.assertid==finger).first()
            if queryres :
                # 该资产的指纹已经存在，则直接修改之前的记录里的
                queryres.assertip = ip
                queryres.assertname = name
                #queryres.finger = finger
                queryres.asserttag = tag
                queryres.assertuser = user
                queryres.assertstate = ""
            else:
                res = Assert(assertid=finger, asserttag=tag, assertname=name, 
                    assertip=ip, assertuser=user)
                db.session.add(res)

            db.session.commit()
        return json.dumps({"succeed": True, "error": "", "data": {}})
    except:
        wrapError("setAssert failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/setAssert", methods=["POST"])
def setAssert():
    args = json.loads(request.get_data())
    recordid = args.get("id")
    tag = args.get("tag")
    name = args.get("name")
    location = args.get("location")
    usage = args.get("usage")
    ip = args.get("ip")
    groupid = args.get("group")
    oldgroupid = args.get("oldgroup")

    try:
        record = db.session.query(Assert).filter(Assert.assertid==recordid).first()
        if not record:
            return json.dumps({"succeed": False, "error": "记录不存在"})

        record.asserttag = tag
        record.assertname = name
        record.assertlocation = location
        record.assertusage = usage
        record.assertip = ip

        if not groupid == oldgroupid:
            # 或 查询有两种方式
            #grpres = db.session.query(Asgroup).filter(or_(Asgroup.groupid==groupid,Asgroup.groupid==oldgroupid))
            grpres = db.session.query(Asgroup).filter((Asgroup.groupid==groupid) | (Asgroup.groupid==oldgroupid)).all()
            if not grpres:
                return json.dumps({"succeed": False, "error": "分组记录不存在"})

            for res in grpres:
                if res.groupid == oldgroupid:
                    res.groupasserts = res.groupasserts.replace(recordid, "")
                elif res.groupid == groupid:
                    res.groupasserts = res.groupasserts + ";" + recordid

        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("setAssert failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getAsserts")
def getAsserts():
    try:
        queryres = db.session.query(Assert).filter(Assert.assertstate!="deleted").all()
        result = [i.to_json() for i in queryres]
        #session.expire_all()
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getAsserts failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getAssertRules")
def getAssertRules():
    try:
        # 需要获取规则信息，也需要获取规则对应的策略信息，所以要查询策略，各个类型的策略
        userstras = []

        assertid = request.args.get("id")
        queryres = db.session.query(Rules).filter(Rules.applyto.contains(assertid)).all()
        rules = []
        for i in queryres:
            tmp = i.to_json()
            rules.append(tmp)

            if tmp["type"] == "user" and not tmp["straid"] in userstras:
                userstras.append(tmp["straid"])

        # 查询规则对应的策略名称
        userStraIdNameMap = {}
        querystra = db.session.query(Batchuser).filter(Batchuser.id.in_(userstras)).all()
        for i in querystra:
            userStraIdNameMap[i.id] = i.name

        for i in rules:
            if i["type"] == "user":
                i["strafrom"] = "账户密码"
                i["straname"] = userStraIdNameMap.get(i["straid"])
            elif i["type"] == "security":
                i["strafrom"] = "安全管理"
                i["straname"] = "默认策略"

        return json.dumps({"succeed": True, "error": "", "data": rules})
    except:
        wrapError("getAssertRules failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addUser", methods=["POST"])
def addUser():
    try:
        args = json.loads(request.get_data())
        name = args.get("name")
        fullname = args.get("fullname")
        pwd = args.get("pwd")
        lockuser = args.get("lockuser")
        isadmin = args.get("isadmin")
        deleteuser = args.get("deleteuser")

        if db.session.query(Batchuser).filter().count() >= 1000:
            return json.dumps({"succeed": False, "error": "账户策略数已达上限1000"})

        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})

        if (not fullname or len(fullname) < 1 or len(fullname) > 32):
            return json.dumps({"succeed": False, "error": "账户全名不合法"})

        if not deleteuser and (not pwd or len(pwd) < 8):
            return json.dumps({"succeed": False, "error": "密码不合法，最少8个字符"})

        count = db.session.query(Batchuser).filter(Batchuser.name==name).count()
        if (count > 0):
            return json.dumps({"succeed": False, "error": "指定的名称已经存在"})

        # 加密保存密码
        pwd = kyencrypt.encrypt(11, pwd)

        rec = Batchuser(name=name, fullname=fullname, password=pwd, lockuser=lockuser,
            isadmin=isadmin, deleteuser=deleteuser)
        db.session.add(rec)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("addUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/delUser", methods=["POST"])
def delUser():
    try:
        args = json.loads(request.get_data())
        name = str(args.get("name"))

        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})

        queryres = db.session.query(Batchuser).filter(Batchuser.name==name).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定的账户策略不存在"})

        # 评审意见，策略被引用时，不能直接删除策略
        rulecount = db.session.query(Rules).filter(and_(Rules.type=='user', Rules.straid==queryres.id)).count()
        if rulecount > 0:
            return json.dumps({"succeed": False, "error": "被 %d 条规则引用，请先删除规则"%rulecount })

        db.session.query(Batchuser).filter(Batchuser.name==name).delete()
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("delUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/editUserPassword", methods=["POST"])
def editUserPassword():
    try:
        args = json.loads(request.get_data())
        name = str(args.get("name"))
        pwd = str(args.get("pwd"))

        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})

        queryres = db.session.query(Batchuser).filter(Batchuser.name==name).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定账户名不存在"})

        if len(pwd) < 8:
            return json.dumps({"succeed": False, "error": "密码太弱，至少为8位"})

        # 加密保存密码
        pwd = kyencrypt.encrypt(11, pwd)

        queryres.password = pwd
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/editUserNormal", methods=["POST"])
def editUserNormal():
    try:
        args = json.loads(request.get_data())
        name = str(args.get("name"))
        fullname = str(args.get("fullname"))
        isadmin = args.get("isadmin")
        lockuser = args.get("lockuser")
        deleteuser = args.get("deleteuser")

        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})

        queryres = db.session.query(Batchuser).filter(Batchuser.name==name).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定账户名不存在"})

        if (not fullname or len(fullname) < 1 or len(fullname) > 32):
            return json.dumps({"succeed": False, "error": "用户全名不合法"})

        queryres.fullname = fullname
        queryres.isadmin = isadmin
        queryres.lockuser = lockuser
        queryres.deleteuser = deleteuser
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addUserKey", methods=["POST"])
def addUserKey():
    try:
        args = json.loads(request.get_data())
        name = str(args.get("name"))
        authkey = str(args.get("authkey"))

        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})

        if len(authkey.split(" ")) < 3:
            return json.dumps({"succeed": False, "error": "公钥内容不合法"})

        queryres = db.session.query(Batchuser).filter(Batchuser.name==name).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定账户名不存在"})

        if queryres.authkey.find(authkey) >= 0:
            return json.dumps({"succeed": False, "error": "该公钥已经添加"})

        tmparr = queryres.authkey.split(";")
        if len(tmparr) >= 1000:
            return json.dumps({"succeed": False, "error": "公钥已达上限数1000"})
        if len(queryres.authkey) > 1:
            queryres.authkey = queryres.authkey + ";" + authkey
        else:
            queryres.authkey = authkey
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/delUserKey", methods=["POST"])
def delUserKey():
    try:
        args = json.loads(request.get_data())
        name = str(args.get("name"))
        authkey = str(args.get("authkey"))

        if not name or len(name) < 1 or len(name) > 32:
            return json.dumps({"succeed": False, "error": "账户名不合法"})

        if len(authkey.split(" ")) < 3:
            return json.dumps({"succeed": False, "error": "公钥内容不合法"})

        queryres = db.session.query(Batchuser).filter(Batchuser.name==name).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定账户名不存在"})

        arr = queryres.authkey.split(";")
        exist = False
        for key in arr:
            if key == authkey:
                exist = True

        if not exist:
            return json.dumps({"succeed": False, "error": "指定的公钥不存在"})

        arr.remove(key)
        queryres.authkey = ";".join(arr)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getUsers")
def getUsers():
    try:
        queryres = db.session.query(Batchuser).filter().all()
        result = []
        for i in queryres:
            tmp = i.to_json()
            tmp["password"] = kyencrypt.decrypt(11, tmp["password"])
            result.append(tmp)

        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getUserPassByRuleid")
def getUserPassByRuleid():
    '''获取某条用户规则下发的所有资产账户密码'''
    try:
        ruleid = str(request.args.get("ruleid"))
        if not ruleid:
            return json.dumps({"succeed": False, "error": "参数错误"})

        queryres = db.session.query(Rules).filter(Rules.id==ruleid).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "规则不存在"})

        userstracontent = json.loads(queryres.stracontent)
        username = userstracontent["name"]
        asserts = queryres.applyto.split(";")
        newasserts = [ele for ele in asserts if len(ele)>0 ]

        result = []
        res = db.session.query(AssertUserPass).filter(and_(AssertUserPass.username==username, AssertUserPass.assertid.in_(newasserts))).all()
        for ele in res:
            elejson = ele.to_json()
            del elejson["id"]
            elejson["password"] = kyencrypt.decrypt(11, elejson["password"])
            result.append(elejson)

        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getUsers failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getUploadProgress")
def getUploadProgress():
    '''获取某条用户规则下发的所有资产账户密码'''
    try:
        filepath = str(request.args.get("filepath"))
        if not filepath:
            return json.dumps({"succeed": False, "error": "参数错误"})

        with open(filepath + ".progress", "r") as fd:
            line = fd.readline().replace("\r", "").replace("\n", "")
            return json.dumps({"succeed": True, "error": "", "data": line})
    except:
        wrapError("getUploadProgress failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "操作失败"})

@app.route("/checkSoftRepo", methods=["POST"])
def checkSoftRepo():
    args = json.loads(request.get_data())
    name = args.get("reponame")
    querycount = db.session.query(SoftRepo).filter(SoftRepo.name==name).count()
    if querycount:
        wrapError("添加软件源操作失败，同名软件源已经存在")
        return json.dumps({"succeed": False, "error": "添加软件源操作失败，同名软件源已经存在"})
    else:
        return json.dumps({"succeed": True, "error": ""})

@app.route("/addSoftRepo", methods=["POST"])
def addSoftRepo():
    # print("addSoftRepo")
    args = json.loads(request.get_data())
    jsonfilename = args.get("jsonfile")
    rpmfilename = args.get("rpmfile")

    #import pdb;pdb.set_trace()

    jsonfilepath = jsonfilename
    if not jsonfilepath.startswith("/tmp/"):
        jsonfilepath = "/tmp/" + jsonfilename

    jsonfile = open(jsonfilepath, "rb")
    jsoninfo = json.load(jsonfile)
    jsonfile.close()
    name = jsoninfo["name"]
    tag = jsoninfo["tag"]
    supportedarch = json.dumps(jsoninfo["supportedArch"])
    supportedrelease = json.dumps(jsoninfo["supportedOSRelease"])
    createtime = jsoninfo["createTime"]
    description = jsoninfo["description"]
    checksum = jsoninfo["checksum"]
    cveinfo = json.dumps(jsoninfo["cve"])
    cve = False
    if (len(cveinfo)):
        cve = True

    # 此处调换了顺序是因为报错返回前没有清除临时文件
    delete_json = "rm -f " + jsonfilepath
    out = subprocess.run(delete_json,shell=True)
    if out.returncode != 0:
        wrapError("清除临时文件失败:%s" %jsonfilepath )

    #查询数据库中是否有同名的软件源，如果有，直接返回失败结果
    # querycount = db.session.query(SoftRepo).filter(SoftRepo.name==name).count()
    # if querycount:
    #     wrapError("添加软件源操作失败，同名软件源已经存在")
    #     return json.dumps({"succeed": False, "error": "添加软件源操作失败，同名软件源已经存在"})

    rpmfilepath = rpmfilename
    if not rpmfilepath.startswith("/tmp/"):
        rpmfilepath = "/tmp/" + rpmfilename

    try:
        #根据md5值判断上传到服务器的文件是否和原文件一致        
        md5_command = "md5sum " + rpmfilepath
        ret1 = subprocess.run(md5_command,shell=True,stdout=subprocess.PIPE)
        out = str(ret1.stdout,encoding="utf-8").split(' ')[0]
        if out != checksum:
            wrapError("软件源md5值验证失败")
            return json.dumps({"succeed": False, "error": "软件源md5值验证失败"})

        unzip_command = "tar -xzf " + rpmfilepath + " -C " + "/var/www/html/repos/"
        ret2 = subprocess.run(unzip_command,shell=True)
        # print(ret2)
        if ret2.returncode != 0:
            wrapError("解压缩软件源压缩文件失败")
            return json.dumps({"succeed": False, "error": "添加软件源操作失败，请重试"})

        #解压缩成功后，删除切片文件和压缩文件
        delete_command = "rm -f '%s' '%s.progress' " %(rpmfilepath, rpmfilepath)
        res = subprocess.run(delete_command,shell=True)
        # print(res)
        if res.returncode != 0:
            wrapError("清除临时文件失败")

        #将软件源添加到相关目录后，需要保证文件目录可访问，并重启httpd服务
        restart_command = "chmod -R 755 " + "/var/www/html/repos/" + " && " + "systemctl restart httpd"
        res1 = subprocess.run(restart_command,shell=True)
        # print(res1)
        if res1.returncode != 0:
            return json.dumps({"succeed": False, "error": "重启软件源服务器失败，请通知管理员处理"})

        #获取软件源包含的软件列表
        pkglist_command = "cd " + "/var/www/html/repos/" + name + " && " + "ls -lR | grep -v ^d | awk '{print $9}' | tr -s '\n' | grep .rpm"
        # print(pkglist_command)
        res2 = subprocess.run(pkglist_command,shell=True,stdout=subprocess.PIPE)
        # print(res2)
        if res2.returncode != 0:
            wrapError("清除临时文件:%s失败"%name)
            return json.dumps({"succeed": False, "error": "清除临时文件失败, 未找到文件：%s"%name})

        #pkglist = str(res2.stdout,encoding="utf-8").replace("\n",";")
        pkglist = str(res2.stdout,encoding="utf-8")
        # print(pkglist)

        #如果以上步骤均执行成功，则软件源生成成功，需要将软件源信息写入数据库，repo文件信息保存到repoinfo字段中
        repofile = name + ".repo"
        repoinfo = "[" + name + "]" + ";" + "name=UniKylin-$releasever" + " - " + name + ";" + "gpgcheck=0" + ";" + "baseurl=http://" + httpserverip + ":" + httpserverport + "/repos/" + name + "/" +";" + "gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-kylin-release"

        softrepo = SoftRepo(name=name, tag=tag, supportedarch=supportedarch, supportedrelease=supportedrelease,
            buildtime=createtime, pkglist=pkglist, repofile=repofile, repoinfo=repoinfo, ip=httpserverip, iscve=cve, cveinfo=cveinfo, iseffective=True, isdelete=False)
        wrapInfo("added repo:%s" %(softrepo.name) )

        db.session.add(softrepo)
        db.session.commit()

        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("addSoftRepo failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "添加软件源操作失败"})

@app.route("/getSoftRepos")
def getSoftRepos():
    # print("getSoftRepos")
    try:
        queryres = db.session.query(SoftRepo).filter().all()
        result = [i.to_json() for i in queryres]
        # print(result)
        #getRepoStrategies()
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getSoftRepos failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getRepoStrategies")
def getRepoStrategies():
    # print("getRepoStrategies")
    try:
        queryres = db.session.query(RepoStrategy).filter().all()
        result = [i.to_json() for i in queryres]
        # print(result)
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getRepoStrategies failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getUpdateStrategies")
def getUpdateStrategies():
    # print("getUpdateStrategies")
    try:
        queryres = db.session.query(UpdateStrategy).filter().all()
        result = [i.to_json() for i in queryres]
        # print(result)
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getUpdateStrategies failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addRepoStrategies", methods=["POST"])
def addRepoStrategies():
    # print("addRepoStrategies")
    args = json.loads(request.get_data())
    name = args.get("name")
    softrepostr = args.get("softrepostr")
    softreponamestr = args.get("softreponamestr")
    # print(name)
    # print(softrepostr)
    # print(softreponamestr)
    try:
        repostrategy_add = RepoStrategy(name=name, repoidlist=softrepostr, reponamelist=softreponamestr, repofilelist="")
        db.session.add(repostrategy_add)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("addRepoStrategies failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addUpdateStrategies", methods=["POST"])
def addUpdateStrategies():
    # print("addUpdateStrategies")
    args = json.loads(request.get_data())
    name = args.get("name")
    pkglist = args.get("pkglist")
    # print(name)
    # print(pkglist)
    try:
        updatestrategy_add = UpdateStrategy(name=name, pkglist=pkglist)
        db.session.add(updatestrategy_add)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("addRepoStrategies failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/delUpdateStrategy", methods=["POST"])
def delUpdateStrategy():
    # print("delUpdateStrategy")
    try:
        args = json.loads(request.get_data())
        strategyid = args.get("id")
        # print(strategyid)
        ruletype = "update"

        queryres = db.session.query(UpdateStrategy).filter(UpdateStrategy.id==strategyid).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定的软件升级策略不存在"})

        #如果升级策略被升级规则引用，则无法直接删除
        queryrule = db.session.query(Rules).filter(Rules.straid==strategyid, Rules.type==ruletype).first()
        if queryrule:
            return json.dumps({"succeed": False, "error": "指定的软件升级策略被规则引用，请先删除相关规则"})

        db.session.delete(queryres)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("delUpdateStrategy failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/delRepoStrategy", methods=["POST"])
def delRepoStrategy():
    # print("delRepoStrategy")
    try:
        args = json.loads(request.get_data())
        strategyid = args.get("id")
        # print(strategyid)
        ruletype = "repo"

        queryres = db.session.query(RepoStrategy).filter(RepoStrategy.id==strategyid).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定的软件源策略不存在"})

        #如果升级策略被升级规则引用，则无法直接删除
        queryrule = db.session.query(Rules).filter(Rules.straid==strategyid, Rules.type==ruletype).first()
        if queryrule:
            return json.dumps({"succeed": False, "error": "指定的软件源策略被规则引用，请先删除相关规则"})

        db.session.delete(queryres)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("delRepoStrategy failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/delSoftRepo", methods=["POST"])
def delSoftRepo():
    # print("delSoftRepo")
    try:
        args = json.loads(request.get_data())
        softrepoid = args.get("id")
        # print(softrepoid)

        queryres = db.session.query(SoftRepo).filter(SoftRepo.id==softrepoid).first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "指定的软件源不存在"})
        # queryrepos = db.session.query(RepoStrategy.repoidlist).all()
        # print(queryrepos)
        #删除软件源之前需要先判断，如果软件源被软件源策略引用，需要先删除相关策略
        # for queryrepo in queryrepos:
        #     # print(queryrepo[0])
        #     if (str(softrepoid) in str(queryrepo[0]).split(";")):
        #         return json.dumps({"succeed": False, "error": "软件源被策略引用，请先删除相关策略"})
        querycontent = db.session.query(Rules.stracontent).all()
        for content in querycontent:
            acontent=json.loads(content[0])
            for i in acontent:
                if queryres.name == i:
                    return json.dumps({"succeed": False, "error": "软件源被规则引用，请先删除相关规则"})
    
        #需要先在软件源服务器上删除相关目录，删除成功后，重启httpd服务，再删除数据库相关条目
        reponame = queryres.name
        # print(reponame)
        delete_command = "rm -rf " + "/var/www/html/repos/" + reponame
        res = subprocess.run(delete_command,shell=True)
        # print(res)
        if res.returncode != 0:
            wrapError("清除软件源文件失败")
            # print("清除软件源文件失败")
            return json.dumps({"succeed": False, "error": "清除软件源文件失败"})
        restart_command = "systemctl restart httpd"
        res1 = subprocess.run(restart_command,shell=True)
        # print(res1)
        if res1.returncode != 0:
            wrapError("清除软件源文件失败")
            # print("清除软件源文件失败")
            return json.dumps({"succeed": False, "error": "重启软件源服务器失败"})
        db.session.delete(queryres)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("delSoftRepo failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getRules")
def getRules():
    try:
        ruletype = request.args.get("type")
        if not ruletype:
            return json.dumps({"succeed": False, "error": "必须指定规则类型"})

        # 涉及到和后台下发程序同时读写规则数据库表，要加独占锁
        def getRulesByType(ruletype):
            f = kyfilelock.FLock("%srules" %ruletype, "share")
            with f:
                db.session.commit()
                # if ruletype == "all":
                #     queryres = db.session.query(Rules).filter().all()
                # else:
                #     queryres = db.session.query(Rules).filter(Rules.type==ruletype).all()
                queryres = db.session.query(Rules).filter(Rules.type==ruletype).all()
                result = [i.to_json() for i in queryres]
                return result

        result = getRulesByType(ruletype)
        #if not ruletype == "all":
            #result = getRulesByType(ruletype)
        #else:
            #result = []
            #result = result + getRulesByType("user")
            #result = result + getRulesByType("security")
            #result = result + getRulesByType("update")
            #result = result + getRulesByType("repo")

        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getRules failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addRule", methods=["POST"])
def addRule():
    # print("addRule")
    try:
        
        args = json.loads(request.get_data())
        ruletype = args.get("type")
        name = str(args.get("name"))
        straid = args.get("straid")
        asserts = args.get("asserts")
        applyto = args.get("asserts")

        if not asserts or len(asserts) < 1:
                return json.dumps({"succeed": False, "error": "至少需要选择一个资产"})

        if not ruletype:
            return json.dumps({"succeed": False, "error": "必须指定规则类型"})
        # 涉及到和后台下发程序同时读写规则数据库表，要加独占锁
        f = kyfilelock.FLock("%srules" %ruletype, "exclusive")
        with f:
            if db.session.query(Rules).filter(Rules.type==ruletype).count() >= 1000:
                return json.dumps({"succeed": False, "error": "规则数已达上限1000"})

            if not name or len(name) < 1 or len(name) > 32:
                return json.dumps({"succeed": False, "error": "请输入合法名称，1-32字符"})

            if name[0] == " ":
                return json.dumps({"succeed": False, "error": "名称不能以空格开头"})

            if db.session.query(Rules).filter(and_(Rules.type==ruletype, Rules.name==name)).count() > 0:
                return json.dumps({"succeed": False, "error": "指定的规则名称已经存在"})

            if ruletype == "user":
                queryuser = db.session.query(Batchuser).filter(Batchuser.id==straid).first()
                if not queryuser:
                    return json.dumps({"succeed": False, "error": "指定的账户策略不存在"})

                stracontent = queryuser.to_json()
                del stracontent["id"]
                stracontent = json.dumps(stracontent)
                rec = Rules(type=ruletype, name=name, straid=straid, stracontent=stracontent, applyto=asserts)
                db.session.add(rec)
                db.session.commit()
                return json.dumps({"succeed": True, "error": ""})
            elif ruletype == "security":
                querysecurity = db.session.query(Config).filter(Config.configtype=="security").first()
                if not querysecurity:
                    return json.dumps({"succeed": False, "error": "指定的安全策略不存在"})

                straid = 0
                stracontent = querysecurity.content
                rec = Rules(type=ruletype, name=name, straid=straid, stracontent=stracontent, applyto=asserts)
                db.session.add(rec)
                db.session.commit()
                return json.dumps({"succeed": True, "error": ""})
            
            # elif ruletype == "repo": # 已弃用
                # queryrepo = db.session.query(RepoStrategy).filter(RepoStrategy.id==straid).first()
                # if not queryrepo:
                #     return json.dumps({"succeed": False, "error": "指定的软件源策略不存在"})
                # stracontent = queryrepo.name
                # # print(stracontent)

                # repoidlist = queryrepo.repoidlist
                # repos = repoidlist.split(";")
                # # print(repos)

                # editcount = 1
                # applycount = {}
                # #此处将资产列表以；分割
                # asserts = asserts.split(";")
                
                # for assertid in asserts:
                #     assert_completed = True
                #     assertip = db.session.query(Assert.assertip).filter(Assert.assertid==assertid).first()
                #     assertuser = db.session.query(Assert.assertuser).filter(Assert.assertid==assertid).first()
                #     #考虑ip为本机的情况,同样用paramiko远程连接
                #     # print("the assertip is")
                #     # print(assertip[0])
                #     # print("the assertuser is")
                #     # print(assertuser[0])
                #     assertapply = {"applycount":0,"result":""}
                #     ssh = paramiko.SSHClient()   #创建sshclient
                #     ssh.load_system_host_keys() 
                #     #创建连接，考虑连接不成功的情况
                #     try:
                #         ssh.connect(hostname=assertip[0], port=22, username="root")
                #     except:
                #         #打印信息，跳出本次循环
                #         msg = "cannot connect to " + assertip[0]
                #         print(msg)
                #         wrapError(msg)
                #         assert_completed = False
                #         assertapply["applycount"] = editcount
                #         assertapply["result"] = msg
                #         applycount[assertid] = assertapply
                #         continue
                #     arch_command = "arch"
                #     stdin, stdout, stderr = ssh.exec_command(arch_command)
                #     assertarch = stdout.readlines()
                #     err = stderr.readlines()
                #     # print(assertarch[0])
                    
                #     for repoid in repos:
                #         repofile = db.session.query(SoftRepo.repofile).filter(SoftRepo.id==repoid).first()
                #         repoinfo = db.session.query(SoftRepo.repoinfo).filter(SoftRepo.id==repoid).first()
                #         # print(repofile[0])
                #         # print(repoinfo[0])
                #         path = "/tmp/repos/"
                #         if not os.path.exists(path):
                #             os.mkdir(path)
                #         repofilepath = path + repofile[0]
                #         f=open(repofilepath, "w+", encoding="utf-8")
                #         contents = repoinfo[0].split(";")
                #         for content in contents:
                #             #判断是否为baseurl那一行，如果是，需要在行尾加架构
                #             if "baseurl" in content:
                #                 content += assertarch[0]
                #                 f.write(content)
                #             else:   
                #                 f.write(content)
                #                 f.write('\n')
                #         f.close()
                
                #         sftp = ssh.open_sftp()
                #         remotepath = "/etc/yum.repos.d/" + repofile[0]
                #         # 执行上传动作
                #         try:
                #             sftp.put(repofilepath, remotepath)
                #         except:
                #             msg = "upload" + repofile + " to" + assertip[0] + " failed"
                #             wrapError(msg)
                #             assert_completed = False
                #             assertapply["applycount"] = editcount
                #             assertapply["result"] = msg
                #             applycount[assertid] = assertapply
                #             break
                #     #执行所有远程操作后，关闭ssh连接
                #     ssh.close()
                #     #修改日志的editCount
                #     if assert_completed:    
                #         assertapply["applycount"] = editcount 
                #         assertapply["result"] = "succeed"
                #     applycount[assertid] = assertapply
                # applycount = json.dumps(applycount)
                # #此处将资产列表以；合并
                # #asserts = asserts.join(";")
                # rec = Rules(type=ruletype, name=name, straid=straid, stracontent=stracontent, applyto=applyto, editcount=editcount, applycount=applycount)
                # db.session.add(rec)
                # db.session.commit()
                # return json.dumps({"succeed": True, "error": ""})

            elif ruletype == "update":
                # 根据评审意见，此处添加规则只写入数据库，不做下发处理
                # 下发以及重新下发都由daemon处理
                # 此时editcount = 1和状态为processing为了触发后台
                stracontent = args.get("stracontent") # => {"repo1:rpm1","repo2:rpm2"}
                tmpstra = {} # => {repo1:[rpm1,rpm2],repo2:[rpm1,rpm2]}
                repo = ""
                for i in stracontent:
                    i = i.split(":")
                    repo = i[0]
                    if not tmpstra.get(repo):
                        tmpstra[repo] = []
                    tmpstra[repo].append(i[1])
                # print(tmpstra)
                stracontent=json.dumps(tmpstra)
                applycount={}
                applyto = applyto.split(";")
                for i in applyto:
                    applycount[i] = {"applycount":0, "result": ""}
                applycount = json.dumps(applycount)
                rec = Rules(type=ruletype, name=name, straid=straid, stracontent=stracontent , applyto=asserts, editcount=1, applycount=applycount, status="processing")                
                db.session.add(rec)
                db.session.commit()
                return json.dumps({"succeed": True, "error":""})
                # queryupdate = db.session.query(UpdateStrategy).filter(UpdateStrategy.id==straid).first()
                # if not queryupdate:
                #     return json.dumps({"succeed": False, "error": "指定的软件升级策略不存在"})
                # stracontent = queryupdate.name
                # # print(stracontent)
                # pkglist = queryupdate.pkglist
                # # print(pkglist)
                # editcount = 1
                # applycount = {}
                # #此处将资产列表以；分割
                # asserts = asserts.split(";")

                # for assertid in asserts:
                #     assert_completed = True
                #     assertip = db.session.query(Assert.assertip).filter(Assert.assertid==assertid).first()
                #     assertuser = db.session.query(Assert.assertuser).filter(Assert.assertid==assertid).first()
                #     #考虑ip为本机的情况,同样用paramiko远程连接
                #     # print("the assertip is")
                #     # print(assertip[0])
                #     # print("the assertuser is")
                #     # print(assertuser[0])
                #     assertapply = {"applycount":0,"result":""}
                #     ssh = paramiko.SSHClient()   #创建sshclient
                #     ssh.load_system_host_keys() 
                #     #创建连接，考虑连接不成功的情况
                #     try:
                #         ssh.connect(hostname=assertip[0], port=22, username="root")
                #     except:
                #         #打印信息，跳出本次循环
                #         msg = "cannot connect to " + assertip[0]
                #         print(msg)
                #         wrapError(msg)
                #         assert_completed = False
                #         assertapply["applycount"] = editcount
                #         assertapply["result"] = msg
                #         applycount[assertid] = assertapply
                #         continue
                #     update_command = "yum makecache && yum -y install"
                #     packages = pkglist.split(";")
                #     for package in packages:
                #         update_command += " "+ package
                #     # print(update_command)
                #     stdin, stdout, stderr = ssh.exec_command(update_command)
                #     out = stdout.readlines()
                #     err = stderr.readlines()
                #     print(err)
                #     print(out)
                #     ssh.close()    
                #     #如果执行成功，修改result和editcount字段
                #     if len(err) == 0:
                #         assertapply["applycount"] = editcount 
                #         assertapply["result"] = "succeed"
                #     else:
                #         assertapply["result"] = err
                #         assertapply["applycount"] = editcount
                #     applycount[assertid] = assertapply
                # applycount = json.dumps(applycount)
                # #此处将资产列表以；合并
                # #asserts = asserts.join(";")
                # rec = Rules(type=ruletype, name=name, straid=straid, stracontent=stracontent, applyto=applyto, editcount=editcount, applycount=applycount)
                # db.session.add(rec)
                # db.session.commit()
                # return json.dumps({"succeed": True, "error": ""})            
            
    except:
        wrapError("addRule failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/editRule", methods=["POST"])
def editRule():
    try:
        args = json.loads(request.get_data())
        id = args.get("id")
        # 规则类型不让改
        ruletype = args.get("type")
        name = str(args.get("name"))
        straid = str(args.get("straid"))
        asserts = args.get("asserts")

        if not ruletype:
            return json.dumps({"succeed": False, "error": "必须指定规则类型"})
        # 涉及到和后台下发程序同时读写规则数据库表，要加独占锁
        f = kyfilelock.FLock("%srules" %ruletype, "exclusive")
        with f:
            queryres = db.session.query(Rules).filter(Rules.id==id).first()
            if not queryres:
                return json.dumps({"succeed": False, "error": "指定的规则不存在"})

            if not name or len(name) < 1 or len(name) > 32:
                return json.dumps({"succeed": False, "error": "请输入合法名称，最多32字符"})

            if db.session.query(Rules).filter(and_(Rules.name==name, Rules.id!=id)).count() > 0:
                return json.dumps({"succeed": False, "error": "指定的规则名称已经存在"})

            if queryres.type == "user":
                querystra = db.session.query(Batchuser).filter(Batchuser.id==straid).first()
                if not querystra:
                    return json.dumps({"succeed": False, "error": "指定的账户策略名称不存在"})

                stracontent = querystra.to_json()
                del stracontent["id"]
                stracontent = json.dumps(stracontent)
            elif queryres.type == "security":
                querysecurity = db.session.query(Config).filter(Config.configtype=="security").first()
                if not querysecurity:
                    return json.dumps({"succeed": False, "error": "指定的安全策略不存在"})

                stracontent = querysecurity.content
                straid = 0

            if not asserts or len(asserts) < 1:
                return json.dumps({"succeed": False, "error": "至少需要选择一个资产"})

            queryres.name = name
            queryres.straid = straid
            # queryres.stracontent = stracontent
            # queryres.applyto = asserts
            queryres.stracontent = stracontent
            queryres.applyto = asserts
            queryres.applytime = datetime.datetime.now()

            # 每次编辑规则就将 editcount + 1 表示规则已经是新的规则
            queryres.editcount = queryres.editcount + 1
            queryres.status = "redownload"
            db.session.commit()
            return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("editRule failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/delRule", methods=["POST"])
def delRule():
    try:
        args = json.loads(request.get_data())
        ruleid = args.get("id")
        ruletype = args.get("type")

        if not ruletype:
            return json.dumps({"succeed": False, "error": "必须指定规则类型"})
        # 涉及到和后台下发程序同时读写规则数据库表，要加独占锁
        f = kyfilelock.FLock("%srules" %ruletype, "exclusive")

        # 评审意见，如果后台下发还没有完成，给前台提示，如果锁处于占用状态，说明后台还在下发
        ret = f.trylock()
        if not ret == "succeed":
            return json.dumps({"succeed": False, "error": "规则还在下发中，请稍后重试"})

        with f:
            queryrule = db.session.query(Rules).filter(Rules.id==ruleid).first()
            if not queryrule:
                return json.dumps({"succeed": False, "error": "指定的规则名称不存在"})

            # 评审意见,如果删除软件源规则，需要删除发送到资产的repo文件
            if queryrule.type == "update":
                # isdelete = True => 触发daemon删除检测机制
                # 注：如果isdelete==True的话，证明曾经试图删除过，此时无法编辑
                queryrule.isdelete = True
                db.session.commit()
                return json.dumps({"succeed": True, "error": ""})
            else :
                db.session.delete(queryrule)
                db.session.commit()
                return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("delRule failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/downloadRule", methods=["POST"])
def downloadRule():
    # print("downloadRule")
    try:
        args = json.loads(request.get_data())
        ruleid = args.get("id")
        ruletype = args.get("type")
        # print(ruleid)
        # print(ruletype)

        if not ruletype:
            return json.dumps({"succeed": False, "error": "必须指定规则类型"})
        # 涉及到和后台下发程序同时读写规则数据库表，要加独占锁
        f = kyfilelock.FLock("%srules" %ruletype, "exclusive")
        with f:
            db.session.commit()
            queryrule = db.session.query(Rules).filter(Rules.id==ruleid).first()
            if not queryrule:
                return json.dumps({"succeed": False, "error": "指定的规则不存在"})
            straid = queryrule.straid
            if queryrule.type == "user":
                db.session.commit()
                querystra = db.session.query(Batchuser).filter(Batchuser.id==queryrule.straid).first()
                if not querystra:
                    return json.dumps({"succeed": False, "error": "指定的账户策略名称不存在"})
                stracontent = querystra.to_json()
                del stracontent["id"]
                stracontent = json.dumps(stracontent)
            elif queryrule.type == "security":
                db.session.commit()
                querysecurity = db.session.query(Config).filter(Config.configtype=="security").first()
                if not querysecurity:
                    return json.dumps({"succeed": False, "error": "指定的安全策略不存在"})
                stracontent = querysecurity.content
                straid = 0
            # elif queryrule.type == "repo":
            #     queryrepo = db.session.query(RepoStrategy).filter(RepoStrategy.id==straid).first()
            #     if not queryrepo:
            #         return json.dumps({"succeed": False, "error": "指定的软件源策略不存在"})
                # neededit = 0
                # applycount = json.loads(queryrule.applycount)
                # for assid in applycount.keys():
                #     if applycount[assid]["result"] != "succeed":
                #         err = applycount[assid]["result"]
                #         res = False
                #         neededit += 1 
                # if neededit != 0:
                #     queryrule.editcount += 1
            elif queryrule.type == "update":
                queryrule.editcount += 1
                pass
                # queryupdate = db.session.query(UpdateStrategy).filter(UpdateStrategy.id==straid).first()
                # if not queryupdate:
                #     return json.dumps({"succeed": False, "error": "指定的软件升级策略不存在"})
                # neededit = 0
                # applycount = json.loads(queryrule.applycount)
                # for assid in applycount.keys():
                #     if applycount[assid]["result"] != "succeed":
                #         err = applycount[assid]["result"]
                #         res = False
                #         neededit += 1
                # if neededit != 0:
                #     queryrule.editcount += 1
            # 评审意见，重新下发时不清空上次下发的结果，如果上次下发成功且策略没变的不重新下发
            queryrule.status = "redownload"
            if queryrule.type == "user" or queryrule.type == "security":
                if not json.loads(stracontent) == json.loads(queryrule.stracontent):
                    # 如果有编辑过规则
                    queryrule.stracontent = stracontent
                    queryrule.editcount = queryrule.editcount + 1
                loadac = json.loads(queryrule.applycount)
                for i in loadac:
                    if not loadac[i].get("result") == "succeed":
                        queryrule.status = "redownload"
                        db.session.commit()
                        return json.dumps({"succeed": True, "error": ""})
                db.session.commit()
                return json.dumps({"succeed": True, "error": "该规则已下发成功，请勿重复下发"})  
            db.session.commit()
            return json.dumps({"succeed": True, "error": ""})
            # return json.dumps({"succeed": res, "error": "%s"%err})
    except:
        wrapError("downloadRule failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

def realGetSecurity():
    try:
        defcontent = {
            "loginFailedCnt": 3,
            "loginFailedLockTime": 300,
            "passwordComplex": 1,
            "disableUdisk": 0,
            "passwordExpired": 90,
            "selinuxConfig": "enforcing",
            "sessionTimeout": 90,
            "securityCommand": ""
        }
        queryres = db.session.query(Config).filter(Config.configtype=="security").first()
        if not queryres:
            newrec = Config(configtype="security", content=json.dumps(defcontent))
            db.session.add(newrec)
            db.session.commit()
            result = newrec.to_json()
        else:
            result = queryres.to_json()

        return result
    except:
        wrapError("getSecurity failed")
        wrapError(traceback.format_exc())
        return defcontent

@app.route("/getSecurity")
def getSecurity():
    result = realGetSecurity()
    return json.dumps({"succeed": True, "error": "", "data": result})

@app.route("/editSecurity", methods=["POST"])
def editSecurity():
    try:
        args = json.loads(request.data)
        failcnt = int(args.get("loginFailedCnt"))
        if failcnt < 1:
            return json.dumps({"succeed": False, "error": "失败次数至少为1"})

        locktime = int(args.get("loginFailedLockTime"))
        if locktime < 60:
            return json.dumps({"succeed": False, "error": "锁定时间至少为60S"})

        passcomplex = args.get("passwordComplex")
        if not passcomplex == 0 and not passcomplex == 1:
            return json.dumps({"succeed": False, "error": "密码复杂度参数必须为0或1"})

        disableudisk = args.get("disableUdisk")
        if not disableudisk == 0 and not disableudisk == 1:
            return json.dumps({"succeed": False, "error": "禁用U盘参数必须为0或1"})

        passexpire = int(args.get("passwordExpired"))
        if passexpire < 1:
            return json.dumps({"succeed": False, "error": "密码过期时间至少为1天"})

        selinuxcfg  = args.get("selinuxConfig")
        if not selinuxcfg in ["enforcing", "permissive", "disabled"]:
            return json.dumps({"succeed": False, "error": "selinux设置不合法"})

        sessiontimeout  = int(args.get("sessionTimeout"))
        if sessiontimeout < 60 and not sessiontimeout == 0:
            return json.dumps({"succeed": False, "error": "会话超时时间至少为60S或者为0表示关闭"})

        securityCommand = args.get("securityCommand")
        queryres = db.session.query(Config).filter(Config.configtype=="security").first()
        if not queryres:
            return json.dumps({"succeed": False, "error": "配置不存在"})

        content = {
            "loginFailedCnt": failcnt,
            "loginFailedLockTime": locktime,
            "passwordComplex": passcomplex,
            "disableUdisk": disableudisk,
            "passwordExpired": passexpire,
            "selinuxConfig": selinuxcfg,
            "sessionTimeout": sessiontimeout,
            "securityCommand": securityCommand
        }
        queryres.content = json.dumps(content)
        db.session.commit()
        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("editSecurity failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getRuleState")
def getRuleState():
    try:
        ruletype = request.args.get("type")
        if not ruletype:
            return json.dumps({"succeed": False, "error": "必须指定规则类型"})

        # fixme
        # 只读取的话，暂时不加锁；如果有问题再加
        def getRulesByType(ruletype):
            db.session.commit()
            queryres = db.session.query(Rules).filter(Rules.type==ruletype).all()
            result = [i.to_json() for i in queryres]
            return result
        result = getRulesByType(ruletype)
        if len(result) == 0:
            return json.dumps({"succeed": True, "error": "", "data": ""})
        if ruletype == "security" or ruletype == "user":
            for i in result:
                rapply = json.loads(i["applycount"]) if i["applycount"] else ""
                redit = i["editcount"]
                rstatus = i["status"] if i["status"] else ""
                flg = False
                if len(rstatus) != 0 or len(rapply) == 0:
                    i["status"] = "下发中"
                else:
                    for j in rapply:
                        if rapply[j]["applycount"] < redit:
                            i["status"] = "下发中"
                        if not rapply[j]["result"] == "succeed":
                            flg = True
                            i["status"] = "下发失败"
                    if not flg:
                        i["status"] = "下发成功"

        # print(result)

        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getRules failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getExpiredPass")
def getExpiredPass():
    try:
        db.session.commit()
        queryres = db.session.query(Assert).filter(Assert.assertpass=="expired",Assert.assertstate!="deleted").all()
        # 过滤掉服务器本身和cockpit-client无密码账户, 减少资源占用
        ret = []
        for auser in queryres:
            aip = auser.assertip
            auseraccount = auser.assertuser
            ausername = auser.assertname
            userip = auseraccount+"@"+aip
            ret.append({"asserts":ausername,"ip":userip})
        return json.dumps({"succeed": True, "data": ret})

    except:
        wrapError("getAssertPasswd failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

def clearAuditlog():
    configres = db.session.query(Config).filter(Config.configtype=="auditlog").first()
    keeptimebak = 4
    if configres != None:
        dbcontent = json.loads(configres.content)
        keeptimebak = (int)(dbcontent["keeptime"])

    if keeptimebak != 4:
        now = time.time()
        timestamp = 0
        if keeptimebak == 1:
            timestamp = 30 * 60 * 60 * 24
        elif keeptimebak == 2:
            timestamp = 2 * 30 * 60 * 60 * 24
        elif keeptimebak == 3:
            timestamp = 3 * 30 * 60 * 60 * 24
        endtime = now - timestamp
        limittime = datetime.datetime.fromtimestamp(endtime)
        f = kyfilelock.FLock("deleteAuditlog","exclusive")
        with f:
            db.session.query(AuditLog).filter(AuditLog.time < limittime).delete()
            db.session.commit()

    return json.dumps({"succeed": True, "error": ""})

@app.route("/getAuditlogs")
def getAuditlogs():
    jumppage = int(request.args.get("jumppage"))
    pagesize = int(request.args.get("pagesize"))
    offset = (jumppage-1)*pagesize
    try:
        clearAuditlog()

        queryres = db.session.query(AuditLog).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
        totalrecords = db.session.query(AuditLog).filter().count()
        userres = db.session.query(AuditLog.user).order_by("time").order_by("id").all()
        userlist = sorted(set(userres),key=userres.index)

        result = {"log": [], "user": []}
        if len(userlist) != 0:
            for y in userlist:  
                # print(type(y[0]))  
                if is_jsonable(y[0]):
                    #print(y[0])
                    pass
                else:
                    y = str(y[0])
                    #对于不能序列化的字段，先将其转换为字符串再序列化
                    #print("it can not be serializable")
                    #print(y[0])
                result["user"].append(y[0])

        result["log"] = [i.to_json() for i in queryres]
        result["totalrecords"] = totalrecords

        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getAuditlogs failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getAuditlogsByPage")
def getAuditlogsByPage():
    jumppage = int(request.args.get("jumppage"))
    pagesize = int(request.args.get("pagesize"))
    begintime = request.args.get("begintime")
    endtime = request.args.get("endtime")
    user = request.args.get("user")
    op = request.args.get("operator")

    timesearch = 0
    if len(begintime) != 0 and len(endtime) != 0:
        timesearch = 1
    elif len(begintime) != 0 and len(endtime) == 0:
        timesearch = 2 # 没有endtime，即begintime-latest data
    elif len(begintime) == 0 and len(endtime) != 0:
        timesearch = 3 # 没有begintime，即oldest data-endtime
    begintime = begintime + " 00:00:00"
    endtime = endtime + " 23:59:59"

    queryres = ""
    op = int(op)
    offset = (jumppage-1)*pagesize

    try:
        if user.strip() == "all":
            if op == -1:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:
                    queryres = db.session.query(AuditLog).filter(AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:
                    queryres = db.session.query(AuditLog).filter().order_by("time").order_by("id").limit(pagesize).offset(offset).all()
            else:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
        else:
            if op == -1:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
            else:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:   
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:   
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op).order_by("time").order_by("id").limit(pagesize).offset(offset).all()

        result = [i.to_json() for i in queryres]

        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getAuditlogsByPage failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/getAuditlogsByFilter")
def getAuditlogsByFilter():
    jumppage = int(request.args.get("jumppage"))
    pagesize = int(request.args.get("pagesize"))
    begintime = request.args.get("begintime")
    endtime = request.args.get("endtime")
    user = request.args.get("user")
    op = request.args.get("operator")

    offset = (jumppage-1)*pagesize

    timesearch = 0
    if len(begintime) != 0 and len(endtime) != 0:
        timesearch = 1
    elif len(begintime) != 0 and len(endtime) == 0:
        timesearch = 2 # 没有endtime，即begintime-latest data
    elif len(begintime) == 0 and len(endtime) != 0:
        timesearch = 3 # 没有begintime，即oldest data-endtime
    begintime = begintime + " 00:00:00"
    endtime = endtime + " 23:59:59"

    totalrecords = 0
    queryres = ""
    op = int(op)

    try:
        if user.strip() == "all":
            if op == -1:
                if timesearch == 1:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.time <= endtime, AuditLog.time >= begintime).count()
                elif timesearch == 2:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.time >= begintime).count()
                elif timesearch == 3:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.time <= endtime).count()
                else:
                    totalrecords = db.session.query(AuditLog).filter().count()
            else:
                if timesearch == 1:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time <= endtime, AuditLog.time >= begintime).count()
                elif timesearch == 2:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time >= begintime).count()
                elif timesearch == 3:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time <= endtime).count()
                else:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.operator==op).count()
        else:
            if op == -1:
                if timesearch == 1:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time <= endtime, AuditLog.time >= begintime).count()
                elif timesearch == 2:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time >= begintime).count()
                elif timesearch == 3:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time <= endtime).count()
                else:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user).order_by("time").count()
            else:
                if timesearch == 1:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time <= endtime, AuditLog.time >= begintime).count()
                elif timesearch == 2:
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time >= begintime).count()
                elif timesearch == 3:   
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time <= endtime).count()
                else:   
                    totalrecords = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op).order_by("time").count()

        #print("totalrecords:", totalrecords, "src offset:", offset)
        if totalrecords <= offset:
            if totalrecords == 0:
                offset = 0
            else :
                offset = ((totalrecords -1 ) // pagesize) * pagesize
        #print("offset:", offset)
    
        if user.strip() == "all":
            if op == -1:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:
                    queryres = db.session.query(AuditLog).filter(AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:
                    queryres = db.session.query(AuditLog).filter().order_by("time").order_by("id").limit(pagesize).offset(offset).all()
            else:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op, AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:
                    queryres = db.session.query(AuditLog).filter(AuditLog.operator==op).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
        else:
            if op == -1:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
            else:
                if timesearch == 1:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time <= endtime, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 2:
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time >= begintime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                elif timesearch == 3:   
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op, AuditLog.time <= endtime).order_by("time").order_by("id").limit(pagesize).offset(offset).all()
                else:   
                    queryres = db.session.query(AuditLog).filter(AuditLog.user==user, AuditLog.operator==op).order_by("time").order_by("id").limit(pagesize).offset(offset).all()

        result = {"log": []}
        result["log"] = [i.to_json() for i in queryres]
        result["totalrecords"] = totalrecords
        return json.dumps({"succeed": True, "error": "", "data": result})
    except:
        wrapError("getAuditlogsByFilter failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/addAuditlog", methods=['POST'])
def addAuditlog():
    args = json.loads(request.get_data())
    user = args.get("user")
    op = args.get("operator")
    obj = args.get("auditobject")
    cent = args.get("content")
    try:
        f = kyfilelock.FLock("addAuditlog","exclusive")
        with f:
            record = AuditLog(time=datetime.datetime.now(), user=user, operator=op, auditobject=obj, content=cent)
            db.session.add(record)
            db.session.commit()
        return json.dumps({"succeed": True, "error": ""})

    except:
        wrapError("addAuditlog failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

@app.route("/deleteAuditlog", methods=['POST'])
def deleteAuditlog():
    args = json.loads(request.get_data())
    id = args.get("id")
    try:
        f = kyfilelock.FLock("deleteAuditlog","exclusive")
        with f:
            db.session.query(AuditLog).filter(AuditLog.id==id).delete()
            db.session.commit()

        return json.dumps({"succeed": True, "error": ""})
    except:
        wrapError("deleteAuditlog failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

def checkAuditlogConfig():
    checking = db.session.query(Config).filter(Config.configtype=="auditlog").count()
    content = {}
    if checking == 0:
        # 检查Config表中有无'auditlog'配置,若没有则创建默认配置
        configtype = 'auditlog' 
        editCount = 1
        applyCount = {}
        content = {
            "keeptime": "4",
            "editCount": editCount,
            "applyCount": applyCount
        }
        content["applyCount"] = applyCount
        record = Config(configtype=configtype, content=json.dumps(content))
        db.session.add(record)
        db.session.commit()
    return content

@app.route("/getAuditlogConfig")
def getAuditlogConfig():
    content = request.args.to_dict()
    # {'keeptime': '4'}
    queryres = db.session.query(Config).filter(Config.configtype=="auditlog").first()
    if queryres == None:
        return json.dumps({"succeed": True, "error": "", "content":content})
    dbcontent = json.loads(queryres.content)
    for i in content:
        content[i]=dbcontent[i]
    return json.dumps({"succeed": True, "error": "", "content":content})

@app.route("/setAuditlogConfig", methods=["POST"])
def setAuditlogConfig():
    # print("setAuditlogConfig")
    f = kyfilelock.FLock("dlauditlog", "exclusive")
    with f:
        checkAuditlogConfig()
        args = json.loads(request.get_data())
        keeptime = args.get("keeptime")

        try:
            queryres = db.session.query(Config).filter(Config.configtype=='auditlog').first()
            content = json.loads(queryres.content)
            content["keeptime"] = keeptime
            content["editCount"] = content["editCount"] + 1
            queryres.content = json.dumps(content)
            db.session.commit()
            return json.dumps({"succeed": True, "error": ""})

        except:
            wrapError("setAuditlogConfig failed")
            wrapError(traceback.format_exc())
            return json.dumps({"succeed": False, "error": "数据库操作失败"})


if __name__ == "__main__":
    pass
    # flask 自带的服务器可以用于开发、调试，用于产品中的需要更专业的服务器软件 gunicorn
    # gunicorn -w 4 -b 127.0.0.1:9091 flaskserver:app
    #app.run(debug=True, host="0.0.0.0", port="9091")
