#!/usr/bin/python3
import os
import sys
import json
import traceback
import uuid
import datetime
import logging
import time
import paramiko
import string
import random
import copy
import subprocess
from dateutil.relativedelta import relativedelta
import resource
# 修改文件打开数为INFINITY
resource.setrlimit(
    resource.RLIMIT_CORE,
    (resource.RLIM_INFINITY, resource.RLIM_INFINITY))
from configparser import ConfigParser
from logging.handlers import RotatingFileHandler
from flask_sqlalchemy import SQLAlchemy

curdirname, curfilename = os.path.split(os.path.realpath(__file__))
LIB_DIR = "%s/../manager/lib" %curdirname
if not LIB_DIR in sys.path:
    sys.path.append(LIB_DIR)

from tables import Assert, Config, Asgroup, Batchuser, Rules, SoftRepo,RepoStrategy,UpdateStrategy,\
    AssertUserPass
from tables import app
import sshutils
import kyfilelock
import kyencrypt
import pymysql
pymysql.install_as_MySQLdb()
db = SQLAlchemy(app)

gMonitorLog = None
logging.basicConfig()
Rthandler = RotatingFileHandler('/var/log/scmms_daemon.log' ,
        maxBytes=2 * 1024 * 1024, backupCount=2)
formatter = logging.Formatter('%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s')
Rthandler.setFormatter(formatter)

gMonitorLog = logging.getLogger('gMonitorLog')
gMonitorLog.setLevel(logging.INFO)
gMonitorLog.addHandler(Rthandler)

def wrapInfo(msg):
    dt_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S: ")
    gMonitorLog.info(dt_prefix + msg)

def wrapError(msg):
    dt_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S: ")
    gMonitorLog.error(dt_prefix + msg)

# 下发用户规则到资产的脚本，上传到资产上面执行
# 不用使用 ' 在这个脚本里
userRuleDownloadShell = '''#!/bin/bash

username="{{username}}"
deleteuser="{{deleteuser}}"
passwdstr="{{passwdstr}}"
isadmin="{{isadmin}}"
lockuser="{{lockuser}}"
authkeys="{{authkeys}}"

if [[ $deleteuser -ne 0 ]]; then
    userdel -rf $username
    ret=$?
    if [[ $ret -eq 0 ]];then
        echo "userdel -rf $username succeed"
    elif [[ $ret -eq 6 ]];then
        echo "userdel -rf $username succeed, username donot exist"
    else
        echo "userdel -rf $username failed,ret:$ret"
        exit $ret
    fi

    groupdel $username
    ret=$?
    if [[ $ret -eq 0 ]];then
        echo "groupdel $username succeed"
    elif [[ $ret -eq 6 ]];then
        echo "groupdel $username succeed, groupname donot exist"
    else
        echo "groupdel $username failed,ret:$ret"
        exit $ret
    fi

    echo "deleteuser $username succeed"
    exit 0
fi

# create user
useradd -m $username
ret=$?
if [[ $ret -eq 0 ]];then
    echo "useradd -m $username succeed"
elif [[ $ret -eq 9 ]];then
    echo "useradd -m $username succeed, username exist"
else
    echo "useradd -m $username failed,ret:$ret"
    exit $?
fi

# set password
echo "$passwdstr" | passwd "$username" --stdin
if [[ $? -ne 0 ]];then
    echo "set password failed"
    exit 1
fi

# set admin rights
if [[ $isadmin -ne 0 ]];then
    usermod -g wheel $username
    if [[ $? -ne 0 ]];then
        echo "set admin rights failed"
        exit 1
    fi
else
    usermod -g $username $username
    if [[ $? -ne 0 ]];then
        echo "set not admin rights failed"
        exit 1
    fi
fi

# set lockuser
if [[ $lockuser -ne 0 ]];then
    passwd -lf $username
    if [[ $? -ne 0 ]];then
        echo "lockuser $username failed"
        exit 1
    fi
else
    passwd -uf $username
    if [[ $? -ne 0 ]];then
        echo "unlock $username failed"
        exit 1
    fi
fi

# set authkey
if [[ $authkeys != "" && $authkeys != ";" ]];then
    OLD_IFS="$IFS"
    IFS=";" authkeyarr=$authkeys
    
    for key in ${authkeyarr[@]}; do
        home=$(cat /etc/passwd | grep "^${username}:" | cut -d ":" -f 6)
        if [[ $home == "" ]];then
            echo "get homedir of $username failed"
            exit 1
        fi

        mkdir -p $home/.ssh && touch $home/.ssh/authorized_keys && sudo chmod -R 700 $home/.ssh
        cat $home/.ssh/authorized_keys | grep "$key" >/dev/null || echo "$key" >> $home/.ssh/authorized_keys
        sudo chown -R $username $home/ && sudo chmod 644 $home/.ssh/*
        if [[ $? -ne 0 ]];then
            echo "add authkey failed"
            exit 1
        fi

        echo "add authkey succeed: $key"
    done
    IFS="$OLD_IFS"
fi

echo "download userrulr of $username succeed"
exit 0

'''
def downloadUserRule(rule, userstra, assid):
    """
    rule: 规则数据库记录
    userstra: 策略内容
    applycount: 规则下发情况
    assid: 资产ID
    ret: succeed:下发成功  其它:下发失败
    """

    asserttmp = db.session.query(Assert).filter(Assert.assertid==assid).first()
    if not asserttmp:
        # 资产是不删除的，出现这种情况说明数据出错了
        wrapError("downloadUserRule failed,assert donot exist:" %assid)
        return "assert donot exist"

    if asserttmp.assertstate == "deleted":
        # 资产标记为删除的，不下发
        wrapError("downloadUserRule failed,assert has been deleted:" %assid)
        return "assert has been deleted"

    wrapInfo("begin downloading user rule,ip:%s ruldid:%s" %(asserttmp.assertip, rule.id) )

    shellTmp = copy.copy(userRuleDownloadShell)
    shellTmp = shellTmp.replace("{{username}}", userstra["name"])
    shellTmp = shellTmp.replace("{{deleteuser}}", str(userstra["deleteuser"]))

    # 设置密码
    randomstr = "".join(random.sample(string.digits, k=4))
    passwdstr = kyencrypt.decrypt(11, userstra["password"]) + randomstr
    shellTmp = shellTmp.replace("{{passwdstr}}", passwdstr)

    shellTmp = shellTmp.replace("{{isadmin}}", str(userstra["isadmin"]))
    shellTmp = shellTmp.replace("{{lockuser}}", str(userstra["lockuser"]))
    shellTmp = shellTmp.replace("{{authkeys}}", userstra["authkey"])

    cmd = "chmod 777 /tmp/userRuleDownloadShell; sudo echo '%s' > /tmp/userRuleDownloadShell && sudo bash /tmp/userRuleDownloadShell" %shellTmp
    res, err, exit_status = sshutils.execRemoteCmd(asserttmp.assertip, 22, asserttmp.assertuser, cmd, timeout=30)
    if not exit_status == 0:
        wrapInfo("download user rule:%s failed res:%s" %(userstra["name"], res))
        return "upload and exec download shell failed," + res

    decpassword = kyencrypt.encrypt(11, passwdstr)
    rec = db.session.query(AssertUserPass).filter(AssertUserPass.assertid==assid,AssertUserPass.username==userstra["name"]).first()
    if rec:
        rec.password = decpassword
    else:
        rec = AssertUserPass(assertid=assid, username=userstra["name"], password=decpassword)
        db.session.add(rec)
    db.session.commit()

    wrapInfo("download userrule of %s succeed,ip:%s" %(userstra["name"], asserttmp.assertip))
    return "succeed"

def downloadUserRules():
    queryres = []
    f = kyfilelock.FLock("userrules", "share")
    with f:
        queryres = db.session.query(Rules).filter(Rules.type=="user").all()
        count = len(queryres)
        if count <= 1:
            maxResLength = 60000
        else:
            maxResLength = 60000//count

    # 降低锁粒度，防止前台长时间等待
    for rule in queryres:
        f = kyfilelock.FLock("userrules", "exclusive")
        with f:
            #if len(rule.applycount) > 0 and not rule.status == "redownload":
                # 说明已经下发过，并且没有要求重新下发
            #    continue

            # 对应的策略
            userstra = json.loads(rule.stracontent)
            lastDownloadRes = {}
            if len(rule.applycount) > 0:
                lastDownloadRes = json.loads(rule.applycount)
            downloadResult = {}
            allasserts = rule.applyto.split(";")
            for assid in allasserts:
                if not assid or len(assid) < 1:
                    continue

                # 先保留上次下发的结果
                if lastDownloadRes.get(assid):
                    downloadResult[assid] = lastDownloadRes.get(assid)

                # 评审意见，如果规则上次下发成功且应用的是最新的策略不重新下发
                if lastDownloadRes.get(assid) and lastDownloadRes[assid]["result"] == "succeed" \
                    and lastDownloadRes[assid]["applycount"] >= rule.editcount:
                    #wrapInfo("donot redownload user rule again,assid:%s rule:%s" \
                    #    %(assid, rule.id) )
                    #rule.status = ""
                    continue

                # 如果上次已经下发过了，并且没有指定重新下发就不下发了
                if lastDownloadRes.get(assid) and not rule.status == "redownload":
                    continue

                # 单台资产的下发捕获所有异常，
                try:
                    ret = downloadUserRule(rule, userstra, assid)
                except Exception as ex:
                    wrapError(traceback.format_exc())
                    ret = traceback.format_exc()[:maxResLength]

                downloadResult[assid] = {
                    "applycount": rule.editcount,
                    "result": ret
                }

                rule.applycount = json.dumps(downloadResult)
                db.session.commit()

            rule.status = ""
            db.session.commit()


# 下发安全规则到资产的脚本，上传到资产上面执行
# 不用使用 ' 在这个脚本里
securityRuleDownloadShell = '''#!/bin/bash

failcnt="{{failcnt}}"
locktime="{{locktime}}"
passcomplex="{{passcomplex}}"
passexpired="{{passexpired}}"
selinuxtype="{{selinuxtype}}"
sessionto="{{sessionto}}"
disableudisk="{{disableudisk}}"

rpm -aq | grep security-center-dbus > /dev/null
if [[ $? -ne 0 ]];then
    echo "Cannot find package security-center-dbus"
    exit 1
fi

if [[ $failcnt -lt 0 || $locktime -lt 0 ]];then
    echo "downloadSecurityRule login fail lock time invalid"
    exit 1
fi

dbus-send --system --print-reply --dest=com.kscmms.security.center.qtdbus  / com.kscmms.security.center.qtdbus.sf.setLoginFailedOpr int32:$failcnt int32:$locktime
if [[ $? -ne 0 ]];then
    echo "downloadSecurityRule setLoginFailedOpr failed"
    exit 1
fi

if [[ $passcomplex -eq 0 ]];then
    passcomplex=false
else
    passcomplex=true
fi

dbus-send --system --print-reply --dest=com.kscmms.security.center.qtdbus  / com.kscmms.security.center.qtdbus.sf.setPasswordComplex boolean:$passcomplex
if [[ $? -ne 0 ]];then
    echo "downloadSecurityRule setPasswordComplex failed"
    exit 1
fi

if [[ $passexpired -lt 1 ]];then
    echo "downloadSecurityRule password expire invalid"
    exit 1
fi

dbus-send --system --print-reply --dest=com.kscmms.security.center.qtdbus  / com.kscmms.security.center.qtdbus.sf.setPasswordExpired int32:$passexpired
if [[ $? -ne 0 ]];then
    echo "downloadSecurityRule setPasswordExpired failed"
    exit 1
fi

# disable password expired of cockpit-client
cat /etc/passwd | grep "^cockpit-client:" >/dev/null && chage -M 99999 cockpit-client

if [[ $selinuxtype != "enforcing" && $selinuxtype != "permissive" && $selinuxtype != "disabled" ]];then
    echo "downloadSecurityRule selinux type invalid"
    exit 1
fi

dbus-send --system --print-reply --dest=com.kscmms.security.center.qtdbus  / com.kscmms.security.center.qtdbus.sf.setSelinuxConfig string:$selinuxtype
if [[ $? -ne 0 ]];then
    echo "downloadSecurityRule setSelinuxConfig failed"
    exit 1
fi

if [[ $selinuxtype == "enforcing" ]]; then
    sudo setenforce 1
else
    sudo setenforce 0
fi

if [[ $sessionto -lt 60 && $ssession -ne 0 ]];then
    echo "downloadSecurityRule ssession timeout invalid"
    exit 1
fi

dbus-send --system --print-reply --dest=com.kscmms.security.center.qtdbus  / com.kscmms.security.center.qtdbus.sf.setSessionTimeout int32:$sessionto
if [[ $? -ne 0 ]];then
    echo "downloadSecurityRule setSessionTimeout failed"
    exit 1
fi

if [[ $disableudisk -ne 0 ]];then
    rmmod uas
    rmmod usb_storage
    lsmod | grep "^usb_storage "
    if [[ $? -eq 0 ]];then
        echo "rmmod usb_storage failed"
        exit 1
    fi

    for i in $(find /lib/modules/ | grep usb-storage.ko | grep -v .bak$);do
        mv $i ${i}.bak;
    done
else
    for i in $(find /lib/modules/ | grep usb-storage.ko | grep .bak$);do
        mv $i ${i%.*};
    done
fi

echo "succeed"
exit 0
'''

def downloadSecurityRule(rule, securitystra, assid):
    """
    rule: 规则数据库记录
    securitystra: 策略内容
    applycount: 规则下发情况
    assid: 资产ID
    ret: succeed:下发成功  其它:下发失败
    """

    asserttmp = db.session.query(Assert).filter(Assert.assertid==assid).first()
    if not asserttmp:
        # 资产是不删除的，出现这种情况说明数据出错了
        wrapError("downloadSecurityRule failed,assert donot exist:" %assid)
        # -1 表示下发失败
        return "assert donot exist"

    if asserttmp.assertstate == "deleted":
        # 资产标记为删除的，不下发
        wrapError("downloadSecurityRule failed,assert has been deleted:" %assid)
        # -1 表示下发失败
        return "assert has been deleted"

    wrapInfo("begin downloading security rule,ip:%s" %asserttmp.assertip)

    shellTmp = copy.copy(securityRuleDownloadShell)
    shellTmp = shellTmp.replace("{{failcnt}}", str(securitystra.get("loginFailedCnt")))
    shellTmp = shellTmp.replace("{{locktime}}", str(securitystra.get("loginFailedLockTime")))
    shellTmp = shellTmp.replace("{{passcomplex}}", str(securitystra.get("passwordComplex")))
    shellTmp = shellTmp.replace("{{passexpired}}", str(securitystra.get("passwordExpired")))
    shellTmp = shellTmp.replace("{{selinuxtype}}", str(securitystra.get("selinuxConfig")))
    shellTmp = shellTmp.replace("{{sessionto}}", str(securitystra.get("sessionTimeout")))
    shellTmp = shellTmp.replace("{{disableudisk}}", str(securitystra.get("disableUdisk")))

    # 将安全策略配置的下发放到脚本里批量处理，不包括执行命令的部分，执行命令的部分有特殊字符，不方便处理
    cmd = "chmod 777 /tmp/securityBatchCmdShell; sudo chmod 777 /tmp/securityRuleDownloadShell; sudo echo '%s' > /tmp/securityRuleDownloadShell && sudo bash /tmp/securityRuleDownloadShell" %shellTmp
    res, err, exit_status = sshutils.execRemoteCmd(asserttmp.assertip, 22, asserttmp.assertuser, cmd, timeout=30)
    if not exit_status == 0:
        wrapInfo("download security rule failed res:%s err:%s exit_status:%d" %(res, err, exit_status))
        return "upload and exec download shell failed," + res

    # 将批量执行的命令放到一个脚本文件，上传
    localfile = "/tmp/securityBatchCmdShell"
    with open(localfile, "w") as fd:
        fd.write(str(securitystra.get("securityCommand")))
    sshutils.uploadFile(asserttmp.assertip, 22, asserttmp.assertuser, localfile)

    # 批量执行安全命令，超时时间 10S
    cmd = "sudo timeout 10 bash /tmp/securityBatchCmdShell"
    res, err, exit_status = sshutils.execRemoteCmd(asserttmp.assertip, 22, asserttmp.assertuser, cmd, timeout=30)
    if not exit_status == 0:
        wrapInfo("download security rule exec cmd failed res:%s err:%s exit_status:%d" %(res, err, exit_status))
        return "exec cmd failed"

    wrapInfo("download security rule succeed,ip:%s" %(asserttmp.assertip))
    return "succeed"

def downloadSecurityRules():

    queryres = []
    f = kyfilelock.FLock("securityrules", "share")
    with f:
        queryres = db.session.query(Rules).filter(Rules.type=="security").all()
        count = len(queryres)
        if count <= 1:
            maxResLength = 60000
        else:
            maxResLength = 60000//count

    for rule in queryres:
        f = kyfilelock.FLock("securityrules", "exclusive")
        with f:
            #if len(rule.applycount) > 0 and not rule.status == "redownload":
                # 已经有了下发结果，说明已经下发过
            #    continue

            # 对应的策略
            securitystra = json.loads(rule.stracontent)
            lastDownloadRes = {}
            if len(rule.applycount) > 0:
                lastDownloadRes = json.loads(rule.applycount)

            downloadResult = {}
            allasserts = rule.applyto.split(";")
            for assid in allasserts:
                if not assid or len(assid) < 1:
                    continue

                # 先保留上次下发的结果
                if lastDownloadRes.get(assid):
                    downloadResult[assid] = lastDownloadRes.get(assid)

                # 评审意见，重新下发时不清空上次下发的结果，如果上次下发成功且策略没变的不重新下发
                if lastDownloadRes.get(assid) and lastDownloadRes[assid]["result"] == "succeed" \
                    and lastDownloadRes[assid]["applycount"] >= rule.editcount:
                    # wrapInfo("donot redownload security rule again,assid:%s rule:%s" \
                    #    %(assid, rule.id) )
                    # rule.status = ""
                    continue

                # 如果上次已经下发过了，并且没有指定重新下发就不下发了
                if lastDownloadRes.get(assid) and not rule.status == "redownload":
                    continue

                try:
                    ret = downloadSecurityRule(rule, securitystra, assid)
                except Exception as ex:
                    wrapError(traceback.format_exc())
                    ret = traceback.format_exc()[:maxResLength]

                downloadResult[assid] = {
                    "applycount": rule.editcount,
                    "result": ret
                }

                rule.applycount = json.dumps(downloadResult)
                db.session.commit()

            rule.status = ""
            db.session.commit()

def delUpdateRules():
    f = kyfilelock.FLock("update", "exclusive")
    with f:
        db.session.commit()
        queryrule = db.session.query(Rules).filter(Rules.type=="update").all()
        for arule in queryrule:
            deletecount = {}
            if not arule.isdelete:
                continue
            # 获得repos列表
            stracontent = json.loads(arule.stracontent)
            repos = []
            for r in stracontent:
                repos.append(r)
            # 获得资产列表
            applyto = arule.applyto.split(";")
            for assertid in applyto:
                assertip = db.session.query(Assert.assertip).filter(Assert.assertid==assertid).first()
                assertusr = db.session.query(Assert.assertuser).filter(Assert.assertid==assertid).first()[0]
                ausr = assertusr if len(assertusr) != 0 else "root"
                if db.session.query(Assert.assertstate).filter(Assert.assertid==assertid).first()[0] == "deleted":
                    # 过滤掉已删除的id
                    continue

                assertdelete = {}
                ssh = paramiko.SSHClient()   #创建sshclient
                ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
                #创建连接，考虑连接不成功的情况
                try:
                    wrapInfo("begin del update rule,ip:%s" %assertip[0])
                    ssh.connect(hostname=assertip[0], port=22, username=ausr, timeout=30)
                except:
                    #打印信息，跳出本次循环
                    err = "正在删除%s的repo文件"%assertip[0]
                    msg = "cannot connect to " + assertip[0]
                    wrapError(err)
                    wrapError(msg)
                    assertdelete["error"] = msg
                    deletecount[assertip[0]] = assertdelete
                    arule.deletecount = json.dumps(deletecount)
                    db.session.commit()
                    continue
                for reponame in repos:
                    repofile = db.session.query(SoftRepo.repofile).filter(SoftRepo.name==reponame).first()
                    remotepath = "/etc/yum.repos.d/" + repofile[0]
                    # 执行删除操作
                    delete_command = "rm -f " + remotepath
                    if not ausr == "root":
                        delete_command = "sudo "+delete_command
                    stdin, stdout, stderr = ssh.exec_command(delete_command, timeout=30)
                    out = stdout.readlines()
                    err = stderr.readlines()
                    if len(err) != 0 :
                        msg = "删除" + assertip[0]+"repo文件时出错:\n"
                        wrapError(msg)
                        wrapError(err)
                        assertdelete["error"] = str(msg)+str(err)
                        deletecount[assertip[0]] = assertdelete
                        arule.deletecount = json.dumps(deletecount)
                        db.session.commit()
                        break
                #执行所有远程操作后，关闭ssh连接
                ssh.close()
            # arule.isdelete = False
            # 后台会一直尝试删除，直到资产被删或者重新连接上
            if len(deletecount) == 0:
                db.session.delete(arule)
            db.session.commit()

# def downloadRepoRules():
#     # 暂时弃用
#     f = kyfilelock.FLock("reporules", "exclusive")
#     with f:
#         queryrule = db.session.query(Rules).filter(Rules.type=="repo").all()
#         for rule in queryrule:
#             if not rule.status == "redownload":
#                 continue
#             applycount = json.loads(rule.applycount)
#             asserts = rule.applyto.split(";")
#             editcount = rule.editcount
#             # straid = rule.straid
#             stracontent = rule.stracontent
#             repos = db.session.query(RepoStrategy).filter(RepoStrategy.name==stracontent).first()
#             repoidlist = repos.repoidlist
#             rr = repoidlist.split(";")
#             for assertid in asserts:
#                 #判断资产上次规则下发是否成功，如果成功，直接跳过
#                 assertapply = applycount[assertid]
#                 if assertapply["result"] == "succeed":
#                     assertapply["applycount"] = editcount
#                     continue
#                 assert_completed = True
#                 assertip = db.session.query(Assert.assertip).filter(Assert.assertid==assertid).first()
#                 #考虑ip为本机的情况,同样用paramiko远程连接
#                 assertapply = {"applycount":0,"result":""}
#                 ssh = paramiko.SSHClient()   #创建sshclient
#                 ssh.load_system_host_keys() 
#                 #创建连接，考虑连接不成功的情况
#                 try:
#                     ssh.connect(hostname=assertip[0], port=22, username='root')
#                 except:
#                     #打印信息，跳出本次循环
#                     mesg = "donwloading reporule: "
#                     msg = "cannot connect to " + assertip[0]
#                     wrapError(mesg+msg)
#                     assert_completed = False
#                     assertapply["applycount"] = editcount
#                     assertapply["result"] = msg
#                     applycount[assertid] = assertapply
#                     continue
#                 arch_command = "arch"
#                 stdin, stdout, stderr = ssh.exec_command(arch_command)
#                 assertarch = stdout.readlines()
#                 err = stderr.readlines()
#                 for repoid in rr:
#                     repofile = db.session.query(SoftRepo.repofile).filter(SoftRepo.id==repoid).first()
#                     repoinfo = db.session.query(SoftRepo.repoinfo).filter(SoftRepo.id==repoid).first()
#                     path = "/tmp/repos/"
#                     if not os.path.exists(path):
#                         os.mkdir(path)
#                     repofilepath = path + repofile[0]
#                     f=open(repofilepath, "w+", encoding="utf-8")
#                     contents = repoinfo[0].split(";")
#                     for content in contents:
#                         #判断是否为baseurl那一行，如果是，需要在行尾加架构
#                         if "baseurl" in content:
#                             content += assertarch[0]
#                             f.write(content)
#                         else:   
#                             f.write(content)
#                             f.write('\n')
#                     f.close()
                
#                     sftp = ssh.open_sftp()
#                     remotepath = "/etc/yum.repos.d/" + repofile[0]
#                         # 执行上传动作
#                     try:
#                         sftp.put(repofilepath, remotepath)
#                     except:
#                         msg = "upload " + repofile + " to " + assertip[0] + " failed"
#                         wrapError(msg)
#                         assert_completed = False
#                         assertapply["applycount"] = editcount
#                         assertapply["result"] = msg
#                         applycount[assertid] = assertapply
#                         break
#                         #执行所有远程操作后，关闭ssh连接
#                 ssh.close()
#                         #修改日志的editCount
#                 if assert_completed:    
#                     assertapply["applycount"] = editcount 
#                     assertapply["result"] = "succeed"
#                 applycount[assertid] = assertapply
#             rule.status = ""
#             rule.applycount = json.dumps(applycount)
#             db.session.commit()

def donwloadUpdateRepos(assertid,repolist):
    assertip = db.session.query(Assert.assertip).filter(Assert.assertid==assertid).first()
    assertusr = db.session.query(Assert.assertuser).filter(Assert.assertid==assertid).first()[0]
    ausr = assertusr if len(assertusr) != 0 else "root" 
    #考虑ip为本机的情况,同样用paramiko远程连接
    ssh = paramiko.SSHClient()   #创建sshclient
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    #创建连接，考虑连接不成功的情况
    try:
        wrapInfo("begin download update rule,ip:%s" %assertip[0])
        ssh.connect(hostname=assertip[0], port=22, username=ausr, timeout=30)
    except:
        #打印信息，跳出本次循环
        mesg = "Donwloading repos: "
        msg = "Cannot connect to " + assertip[0]
        wrapError(mesg+msg)
        return {"succeed": False, "error":(mesg+msg)}
    arch_command = "arch"
    stdin, stdout, stderr = ssh.exec_command(arch_command, timeout=30)
    if stderr.readlines():
        msg = "Donwloading repos:\n Cannot connect to %s"%assertip[0]
        wrapError(msg)
        return {"succeed": False, "error":msg}
    assertarch = stdout.readlines()
    # err = stderr.readlines()
    #执行所有远程操作后，关闭ssh连接
    ssh.close()
    for reponame in repolist:
        repofile = db.session.query(SoftRepo.repofile).filter(SoftRepo.name==reponame).first()
        repoinfo = db.session.query(SoftRepo.repoinfo).filter(SoftRepo.name==reponame).first()
        path = "/etc/yum.repos.d/"
        repofilepath = path + repofile[0]
        cmdlist=[]
        cmdlist.append("rm -f %s"%repofilepath)
        cmdlist.append("touch %s"%repofilepath)
        cmdlist.append("chmod a+w %s"%repofilepath)
        contents = repoinfo[0].split(";")
        for c in contents:
            if "baseurl" in c:
                c += assertarch[0]
            cmdlist.append("printf \'%s\n\' >> %s"%(c,repofilepath))
        for i in cmdlist:
            out,err,exc = sshutils.execRemoteCmd(assertip[0],22,ausr,i)
            if len(err)>0:
                wrapError(err)
                return {"succeed": False, "error":err}
    return {"succeed": True, "error":""}

def downloadPkgs(assertid,repolist):
    res = {}
    assertip = db.session.query(Assert.assertip).filter(Assert.assertid==assertid).first()
    assertusr = db.session.query(Assert.assertuser).filter(Assert.assertid==assertid).first()[0]
    ausr = assertusr if len(assertusr) != 0 else "root"
    #考虑ip为本机的情况,同样用paramiko远程连接
    ssh = paramiko.SSHClient()   #创建sshclient
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    #创建连接，考虑连接不成功的情况
    try:
        wrapInfo("begin download pkgs,ip:%s" %assertip[0])
        ssh.connect(hostname=assertip[0], port=22, username=ausr, timeout=30)
    except:
        #打印信息，跳出本次循环
        mesg = "Applying repos: "
        msg = "Cannot connect to " + assertip[0]
        wrapError(mesg+msg)
        return {"succeed": False, "error":mesg+msg}
    ssh.close()
    update_command = "yum makecache && yum -y install"
    if not ausr == "root":
        update_command = "sudo yum makecache && sudo yum -y install"
    packages = []
    pkgs = []
    for i in repolist:
        packages += repolist[i]
    for i in packages:
        arpm = i.split(".")
        if arpm[-1] == "rpm":
            pkgs.append(i[:-4])
    for package in pkgs:
        update_command += " "+ package
    out, err, exc = sshutils.execRemoteCmd(assertip[0], 22, ausr, update_command)
    if exc == 0:
        res["succeed"] = True 
        res["error"] = ""
    else:
        res["succeed"] = False
        res["error"] = err
    return res

def downloadUpdateRules():
    f = kyfilelock.FLock("update", "exclusive")
    with f:
        queryrule = db.session.query(Rules).filter(Rules.type=="update").all()
        for rule in queryrule:
            if not rule.status == "redownload" and not rule.status == "processing":
                continue
            db.session.commit()
            editcount = rule.editcount
            applycount = json.loads(rule.applycount)
            applyto = rule.applyto.split(';')
            pkglist = json.loads(rule.stracontent) #rule.stracontent: {repo1:[rpm1,rpm2],repo2:[rpm1,rpm2]}
            all_status = "processing"
            maxResLength = 60000//len(applyto)
            for aassert in applyto:
                if applycount[aassert]["applycount"] < editcount:
                    applycount[aassert]["result"] = "processing"
                    rule.applycount = json.dumps(applycount)
                    db.session.commit()
                    repores = donwloadUpdateRepos(aassert,pkglist)
                    if not repores["succeed"]:
                        applycount[aassert]["result"] = repores["error"][:maxResLength]
                        applycount[aassert]["applycount"] = editcount
                        rule.applycount = json.dumps(applycount)
                        db.session.commit()
                        all_status = "failed"
                        continue
                    # print("资产：",aassert,"下发软件源完成！开始安装软件包")
                    pkgres = downloadPkgs(aassert,pkglist)
                    if not pkgres["succeed"]:
                        applycount[aassert]["result"] = pkgres["error"][:maxResLength]
                        applycount[aassert]["applycount"] = editcount
                        rule.applycount = json.dumps(applycount)
                        db.session.commit()
                        all_status = "failed"
                        continue
                    # print("资产%s已经完成下发和安装,录入数据库..." %aassert)
                    applycount[aassert]["result"] = "succeed"
                    applycount[aassert]["applycount"] = editcount
                    rule.applycount = json.dumps(applycount)
                    db.session.commit()
            if not all_status == "failed":
                rule.status = "succeed"
            else:
                rule.status = "failed"
            db.session.commit()

            # applycount = json.loads(rule.applycount)
            # asserts = rule.applyto.split(";")
            # editcount = rule.editcount
            # for assertid in asserts:
            #     #判断资产上次规则下发是否成功，如果成功，直接跳过
            #     assertapply = applycount[assertid]
            #     if assertapply["result"] == "succeed":
            #         assertapply["applycount"] = rule.editcount
            #         continue
            #     assert_completed = True
            #     assertip = db.session.query(Assert.assertip).filter(Assert.assertid==assertid).first()
            #     #考虑ip为本机的情况,同样用paramiko远程连接
            #     assertapply = {"applycount":0,"result":""}
            #     ssh = paramiko.SSHClient()   #创建sshclient
            #     ssh.load_system_host_keys() 
            #     #创建连接，考虑连接不成功的情况
            #     try:
            #         ssh.connect(hostname=assertip[0], port=22, username="root")
            #     except:
            #         #打印信息，跳出本次循环
            #         mesg = "donwloading updaterule: "
            #         msg = "cannot connect to " + assertip[0]
            #         wrapError(mesg+msg)
            #         assert_completed = False
            #         assertapply["applycount"] = editcount
            #         assertapply["result"] = msg
            #         applycount[assertid] = assertapply
            #         continue
            #     update_command = "yum makecache && yum -y install"
            #     packages = pkglist.split(";")
            #     for package in packages:
            #         update_command += " "+ package
            #     # print(update_command)
            #     stdin, stdout, stderr = ssh.exec_command(update_command)
            #     out = stdout.readlines()
            #     err = stderr.readlines()
            #     print(err)
            #     print(out)
            #     ssh.close()    
            #     #如果执行成功，修改result和editcount字段
            #     if len(err) == 0 and assert_completed == True:
            #         assertapply["applycount"] = editcount 
            #         assertapply["result"] = "succeed"
            #     else:
            #         assertapply["result"] = err
            #         assertapply["applycount"] = editcount
            #     applycount[assertid] = assertapply
            # rule.status = ""
            # rule.applycount = json.dumps(applycount)
            # db.session.commit()

def downloadsyslog():
    f = kyfilelock.FLock("dlsyslog", "exclusive")
    with f:
        queryres = db.session.query(Config).filter(Config.configtype=='syslog').first()
        if not queryres:
            return 
        content = json.loads(queryres.content)
        ip = content["ip"]
        port = content["port"]
        database = content["database"]
        user = content["user"]
        password = content["password"]
        editCount = content["editCount"]
        applyCount = content["applyCount"]
        # del applyCount["2"]
        # content["applyCount"] = applyCount
        # queryres.content = json.dumps(content)
        # db.session.commit()
        # return
        aport = 22 # fixme 此处port按要求 暂时绑定为22
        # getRemoteFile(ip,str(port),database,user,str(password)) # 在本地生成模板
        assertidlist = db.session.query(Assert.id).filter().all()
        localfile = "/tmp/cockpit/rsyslog.conf"
        remotepath = "/etc/rsyslog.d/"
        for assertid in assertidlist:
            if db.session.query(Assert.assertstate).filter(Assert.id==assertid[0]).first()[0] == "deleted":
                # 过滤已删除资产
                continue
            if str(assertid[0]) not in applyCount:
                # 如果有新添加的资产则加到syslog config里
                applyCount[str(assertid[0])] = 0
                if not setuprsyslog(assertid,aport):
                    continue
            if editCount > applyCount[str(assertid[0])]:
                # 当前资产需要更新日志服务器
                ass = db.session.query(Assert).filter(Assert.id==str(assertid[0])).first()
                atag = ass.asserttag.split(";")[0]
                if ass.assertip == "localhost":
                    # 如果设置服务端ip为日志服务器的话，找到已绑定服务器的ip
                    fi=open("/usr/share/cockpit/manager.database/httpd.json")
                    serverip = json.load(fi)
                    fi.close()
                    aip = serverip["httpserverip"]
                else:
                    aip = ass.assertip
                ausername = ass.assertuser if len(ass.assertuser) != 0 else "root"
                # 获取信息+修改资产备注(/etc/rsyslog.conf)
                if not checkglobalname(aip,aport,ausername,atag):
                    # 修改当前资产的/etc/rsyslog.conf失败
                    wrapError("修改%s的rsyslog配置文件失败"%aip)
                    continue
                if not echotemplate(aip,ip,aport,port,database,ausername,user,password,atag):
                    wrapError("上传rsyslog配置到%s失败" %aip)
                    continue
                # sshutils.uploadFile(aip, aport, ausername, localfile, remotepath)
                # if not ausername == "root":
                #     mvcmd = "mv "+remotepath+"rsyslog.conf /etc/rsyslog.d/"
                #     out, err, exc = sshutils.execRemoteCmd(aip, aport, ausername, mvcmd)
                #     if len(err)>0:
                #         wrapError("转移文件失败，%s配置日志服务器失败"%aip)
                restartcmd="systemctl restart rsyslog"
                if atag == "3.2":
                    restartcmd="service rsyslog restart"
                elif atag == "3.0":
                    restartcmd="ln -s /usr/lib64/rsyslog/ommysql.so /usr/lib64/rsyslog/ommysql"
                    print("restartcmd:", restartcmd)
                    out, err, exc = sshutils.execRemoteCmd(aip, aport, ausername, restartcmd, 120)
                    if len(err) > 0 or exc != 0:
                        wrapError("日志服务器下发：%s ln -s failed"%aip)
                    restartcmd="ps aux | grep log | wc -l"
                    print("restartcmd:", restartcmd)
                    out, err, exc = sshutils.execRemoteCmd(aip, aport, ausername, restartcmd, 120)
                    if len(err) > 0 or exc != 0:
                        wrapError("日志服务器下发：%s ps aux failed"%aip)
                    else:
                        if int(out) < 3:
                            restartcmd="rklogd -n & rsyslogd -n & syslogd -n &"
                            print("restartcmd:", restartcmd)
                            out, err, exc = sshutils.execRemoteCmd(aip, aport, ausername, restartcmd, 120)
                            if len(err) > 0 or exc != 0:
                                wrapError("日志服务器下发：重启%s的rsyslog服务失败"%aip)
                    
                    restartcmd="rm -f /var/run/*log*;/etc/init.d/rsyslog restart"

                print("restartcmd:", restartcmd)
                if atag == "3.0":
                    out, err, exc = sshutils.execRemoteCmd(aip, aport, ausername, restartcmd, 120)
                    if len(err) > 0 or exc != 0:
                        wrapError("日志服务器下发：重启%s的rsyslog服务失败"%aip)
                else:
                    out, err, exc = sshutils.execRemoteCmd(aip, aport, ausername, restartcmd)
                    if len(err) > 0 or exc != 0:
                        wrapError("日志服务器下发：重启%s的rsyslog服务失败"%aip)
                

                applyCount[str(assertid[0])] = editCount
        content["applyCount"] = applyCount
        queryres.content = json.dumps(content)
        db.session.commit()

def setuprsyslog(aid,aport):
    aip = db.session.query(Assert.assertip).filter(Assert.id==str(aid[0])).first()[0]
    assertusr = db.session.query(Assert.assertuser).filter(Assert.id==str(aid[0])).first()[0]
    ausr = assertusr if len(assertusr) != 0 else "root"
    grep_command = "rpm -aq | grep rsyslog-mysql"
    install_command = "yum install rsyslog-mysql -y"
    restart_command = "systemctl restart rsyslog"
    try:
        out,err,exc = sshutils.execRemoteCmd(aip, aport,ausr,grep_command)
    except:
        wrapError("无法连接到%s"%aip)
        return False
    if len(out) > 0:
        # 已经有rsyslog-mysql了
        return True
    else:
        try:
            err = ""
            rerr = ""
            out,err,exc = sshutils.execRemoteCmd(aip, aport,ausr,install_command)
            rout,rerr,rexc = sshutils.execRemoteCmd(aip, aport,ausr,restart_command)
            return True
        except:
            wrapError("安装rsyslog-mysql时无法连接到%s"%aip)
            wrapError(err)
            wrapError(rerr)
    return False

def checkglobalname(assip,assport,assusername,atag):
    local_host_name = ""
    if atag == "3.4":
        local_host_name = 'global(localHostname="' + assip + '")'
    elif atag == "3.3":
        local_host_name = '$Localhostname ' + assip
    grep_command = "grep -rni 'localHostname' /etc/rsyslog.conf"
    try:
        out, err, exc = sshutils.execRemoteCmd(assip, assport, assusername, grep_command)
    except:
        wrapError("日志服务器配置下发: 无法连接到%s"%assip)
        return False
    if len(err):
        wrapError(err)
        return False
    insert_command = ""
    if len(out):
        result = out.replace("\n", "")
        if result.find(local_host_name) == -1: 
            strlist= result.split(':')
            resnum = strlist[0] 
            insert_command = "sed -i '" + resnum + "c " + local_host_name + "' /etc/rsyslog.conf"
        else:
            strlist= result.split(':')
            resnum = strlist[0]
    else:
        if atag == "3.4":
            insert_command = "sed -i '/global(workDirectory/a\\" + local_host_name + "' /etc/rsyslog.conf"
        elif atag == "3.3":
            insert_command = "sed -i '/$WorkDirectory/a\\" + local_host_name + "' /etc/rsyslog.conf"
        elif atag == "3.0":
            insert_command = "grep -q '^$IncludeConfig' /etc/rsyslog.conf || sed -i '1i\$IncludeConfig \/etc\/rsyslog.d\/*.conf' /etc/rsyslog.conf"
    #判断insert_command是否为空，如不为空，则执行insert_command语句
    if len(insert_command) != 0:
        out, err, exc = sshutils.execRemoteCmd(assip, assport, assusername, insert_command)
    if len(err) != 0 :
        wrapError(err)
        return False
    return True
    
def echotemplate(ip,sip,port,sport,database,user,suser,password,atag):
    alist = []
    if atag == "3.2" or atag == "3.0":
        alist.append('\"\$template insertp1,\\"insert into SystemEvents (Message, Facility, FromHost, Priority, DeviceReportedTime, ReceivedAt, InfoUnitID, SysLogTag)'\
                " values "\
                '(\'%msg%\', %syslogfacility%, \'' + ip + '\', %syslogseverity%, \'%timereported:::date-mysql%\', \'%timegenerated:::date-mysql%\', %iut%, \'%syslogtag%\')\\", sql\"')
    else:
        alist.append('\"\$template insertp1,\\"insert into SystemEvents (Message, Facility, FromHost, Priority, DeviceReportedTime, ReceivedAt, InfoUnitID, SysLogTag)'\
                " values "\
                '(\'%msg%\', %syslogfacility%, \'%HOSTNAME%\', %syslogseverity%, \'%timereported:::date-mysql%\', \'%timegenerated:::date-mysql%\', %iut%, \'%syslogtag%\')\\", sql\"')
    alist.append("\"\$ModLoad ommysql\"")
    alist.append("\"\$ActionOmmysqlServerPort "+str(sport)+"\"")
    alist.append("\"*.*" + "\t" + ":ommysql:" + sip + "," + database + "," + suser + "," + str(password) + ";insertp1\"")
    out,err,exc=sshutils.execRemoteCmd(ip,port,user,"rm -f /etc/rsyslog.d/rsyslog.conf")
    if len(err) > 0:
        wrapError(err)
        return err
    out,err,exc=sshutils.execRemoteCmd(ip,port,user,"touch /etc/rsyslog.d/rsyslog.conf")
    if len(err) > 0:
        if atag == "3.0":
            out,err,exc=sshutils.execRemoteCmd(ip,port,user,"mkdir /etc/rsyslog.d")
            if len(err) > 0:
                wrapError(err)
                return err
            else:
                out,err,exc=sshutils.execRemoteCmd(ip,port,user,"touch /etc/rsyslog.d/rsyslog.conf")
                if len(err) > 0:
                    wrapError(err)
                    return err
        else:
            wrapError(err)
            return err

    out,err,exc=sshutils.execRemoteCmd(ip,port,user,"chmod a+w /etc/rsyslog.d/rsyslog.conf")
    if len(err) > 0:
        wrapError(err)
        return err
    for i in alist:
        cmd = "echo %s >> /etc/rsyslog.d/rsyslog.conf" % i
        out,err,exc=sshutils.execRemoteCmd(ip,port,user,cmd)
        if len(err):
            wrapError(err)
            return err

    return "succeed"

# def getRemoteFile(ip, port, database, user, password):
#     #生成rsyslog.conf配置文件，以便于远程传输到资产
#     template = '$template insertp1,"insert into SystemEvents (Message, Facility, FromHost, Priority, DeviceReportedTime, ReceivedAt, InfoUnitID, SysLogTag) values '
#     template = template + "('%msg%', %syslogfacility%, '%HOSTNAME%', %syslogseverity%, '%timereported:::date-mysql%', '%timegenerated:::date-mysql%', %iut%, '%syslogtag%')"
#     template = template + '",sql'
#     modload = "$ModLoad ommysql"
#     serverport = "$ActionOmmysqlServerPort " + port
#     transfer = "*.*" + "\t" + ":ommysql:" + ip + "," + database + "," + user + "," + password + ";insertp1"
#     path = "/tmp/cockpit"
#     if not os.path.exists(path):
#         os.mkdir(path)
#     f=open("/tmp/cockpit/rsyslog.conf", "w+", encoding="utf-8")
#     f.write(template)
#     f.write('\n')
#     f.write(modload)
#     f.write('\n')
#     f.write(serverport)
#     f.write('\n')
#     f.write(transfer)
#     f.write('\n')
#     f.close()

def deletesyslog():
    '''
    从日志服务器设置中查找keeptime
    定时删除日志文件
    '''
    f = kyfilelock.FLock("dlsyslog", "exclusive")
    with f:
        db.session.commit()
        syslogcount = db.session.query(Config).filter(Config.configtype=="syslog").count()
        if syslogcount <= 0:
            return
        queryconfig = db.session.query(Config).filter(Config.configtype=="syslog").first()
        content = json.loads(queryconfig.content)
        keeptime = int(content["keeptime"])
        app.config['SQLALCHEMY_BINDS'] = {'syslog': ("mysql://" + content["user"] + ":" + content["password"] + "@" + content["ip"] + ":"+str(content["port"])+"/" + content["database"])}
        app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
        app.config['SOLALCHEMY_ECHO'] = False
        app.config['SQLALCHEMY_POOL_RECYCLE'] = 3600
        dates = datetime.date.today()
        datestr = (dates-relativedelta(months=keeptime)).strftime("%Y-%m-%d")
        sql = 'DELETE FROM SystemEvents WHERE ReceivedAt < "%s 00:00:00"' %datestr
        db.session.execute(sql,bind=db.get_engine(app, "syslog"))

def checkpass():
    '''
    使用ssh发送ls指令定时检查资产密码过期
    '''
    db.session.commit()
    # 选取未删除的所有资产
    sshcmd = ''
    queryassert = db.session.query(Assert).filter(Assert.assertstate=="", Assert.assertip!="localhost", Assert.assertuser!="cockpit-client").all()
    # 过滤掉服务器本身和cockpit-client无密码账户, 减少资源占用
    for auser in queryassert:
        aip = auser.assertip
        auseraccount = auser.assertuser
        sshcmd = "ssh %s@%s \'ls\'"%(auseraccount,aip)
        try:
            res = subprocess.run(sshcmd,shell=True,stderr=subprocess.PIPE,encoding='utf-8',timeout=30)
        except:
            # timeout and other unexcepted problem 
            werr="Checking %s@%s password expiration, connection Timeout"%(auseraccount,aip)
            wrapError(werr)
            continue
        err = res.stderr
        if ('expired' in err) and ('password' in err):
            auser.assertpass="expired"
        elif ('No' in err) and ('route' in err) and ('host\n' in err):
            werr="Checking %s@%s password expiration, No route to host"%(auseraccount,aip)
            wrapError(werr)
            continue
        else:
            # reconnected keep assertpass as NULL
            auser.assertpass=""
        db.session.commit()

# ================================================================================
def main():
    wrapInfo("scmms_daemon start...")
    count = 0
    while True:
        try:
            db.session.commit()
            downloadUserRules()
            downloadSecurityRules()
            # downloadRepoRules()
            delUpdateRules()
            downloadUpdateRules()
            downloadsyslog()
            db.session.commit()
            if count == 9:
                deletesyslog()
                checkpass()
                count = 0
        except Exception as ex:
            wrapError(traceback.format_exc())
        time.sleep(20)
        count += 1

if __name__ == "__main__":
    main()
