"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = getNetworkManager;

var utils = _interopRequireWildcard(require("./utils"));

var service = _interopRequireWildcard(require("./service"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2013 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */
var _ = cockpit.gettext;
var C_ = cockpit.gettext;

function nm_debug() {
  if (window.debugging == "all" || window.debugging == "nm") console.debug.apply(console, arguments);
}

function connection_settings(c) {
  if (c && c.Settings && c.Settings.connection) {
    return c.Settings.connection;
  } else {
    // It is a programming error if we ever access a Connection
    // object that doesn't have it's settings yet, and we expect
    // each Connection object to have "connection" settings.
    console.warn("Incomplete 'Connection' object accessed", c); // HACK - phantomjs console.trace() prints nothing

    try {
      throw new Error();
    } catch (e) {
      console.log(e.stack);
    }

    return {};
  }
}
/* NetworkManagerModel
 *
 * The NetworkManager model maintains a mostly-read-only data
 * structure that represents the state of the NetworkManager service
 * on a given machine.
 *
 * The data structure consists of JavaScript values such as objects,
 * arrays, and strings that point at each other.  It might have
 * cycles.  In general, it follows the NetworkManager D-Bus API but
 * tries to hide annoyances such as endian issues.
 *
 * For example,
 *
 *    var manager = model.get_manager();
 *    manager.Devices[0].ActiveConnection.Ipv4Config.Addresses[0][0]
 *
 * is the first IPv4 address of the first device as a string.
 *
 * The model initializes itself asynchronously and emits the 'changed'
 * event whenever anything changes.  If you only access the data
 * structure from within the 'changed' event handler, you should
 * always see it in a complete state.
 *
 * In other words, any change in the data structure from one 'changed'
 * event to the next represents a real change in the state of
 * NetworkManager.
 *
 * When a new model is created, its main 'manager' object starts out
 * as 'null'.  The first 'changed' event signals that initialization
 * is complete and that the whole data structure is now stable and
 * reachable from the 'manager' object.
 *
 * Methods are invoked directly on the objects in the data structure.
 * For example,
 *
 *    manager.Devices[0].disconnect();
 *    manager.Devices[0].ActiveConnection.deactivate();
 *
 * TODO - document the details of the data structure.
 */

/* HACK
 *
 * NetworkManager doesn't implement the standard o.fd.DBus.Properties
 * interface.
 *
 * 1) NM does not emit the PropertiesChanged signal on the
 *    o.fd.DBus.Properties interface but rather on its own interfaces
 *    like o.fd.NetworkManager.Device.Wired.
 *
 * 2) NM does not always emit the PropertiesChanged signal on the
 *    interface whose properties have changed.  For example, when a
 *    property on o.fd.NM.Device changes, this might be notified by a
 *    PropertiesChanged signal on the o.fd.NM.Device.Wired interface
 *    for the same object path.
 *
 * https://bugzilla.gnome.org/show_bug.cgi?id=729826
 *
 * We cope with this here by merging all properties of all interfaces
 * for a given object path.  This is appropriate and nice for
 * NetworkManager, and we should probably keep it that way even if
 * NetworkManager would use a standard o.fd.DBus.Properties API.
 */


function NetworkManagerModel(inithost) {
  /*
   * The NetworkManager model doesn't need proxies in its DBus client.
   * It uses the 'raw' dbus events and methods and constructs its own data
   * structure.  This has the advantage of avoiding wasting
   * resources for maintaining the unused proxies, avoids some code
   * complexity, and allows to do the right thing with the
   * pecularities of the NetworkManager API.
   *
   * However, we do use a fake object manager since that allows us
   * to avoid a lot of 'GetAll' round trips during initialization
   * and helps with removing obsolete objects.
   */
  var self = this;
  /* HACK: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=808162 */

  var hacks = {};
  if (cockpit.manifests["network"] && cockpit.manifests["network"]["hacks"]) hacks = cockpit.manifests["network"]["hacks"];
  var options = {};

  if (inithost) {
    options["host"] = inithost;
  }

  if (hacks.with_networkmanager_needs_root) options["superuser"] = "try";
  var client = cockpit.dbus("org.freedesktop.NetworkManager", options);
  self.client = client;
  /* resolved once first stage of initialization is done */

  self.preinit = new Promise(function (resolve, reject) {
    client.call("/org/freedesktop/NetworkManager", "org.freedesktop.DBus.Properties", "Get", ["org.freedesktop.NetworkManager", "State"], {
      flags: ""
    }).fail(complain).done(function (reply, options) {
      if (options.flags) {
        if (options.flags.indexOf(">") !== -1) utils.set_byteorder("be");else if (options.flags.indexOf("<") !== -1) utils.set_byteorder("le");
        resolve();
      }
    });
  });
  /* Mostly generic D-Bus stuff.  */

  var objects = {};

  function complain() {
    console.warn.apply(console, arguments);
  }

  function conv_Object(type) {
    return function (path) {
      return get_object(path, type);
    };
  }

  function conv_Array(conv) {
    return function (elts) {
      return elts.map(conv);
    };
  }

  function priv(obj) {
    return obj[' priv'];
  }

  var outstanding_refreshes = 0;

  function push_refresh() {
    outstanding_refreshes += 1;
  }

  function pop_refresh() {
    outstanding_refreshes -= 1;
    if (outstanding_refreshes === 0) export_model();
  }

  function get_object(path, type) {
    if (path == "/") return null;

    function Constructor() {
      this[' priv'] = {};
      priv(this).type = type;
      priv(this).path = path;

      for (var p in type.props) {
        this[p] = type.props[p].def;
      }
    }

    if (!objects[path]) {
      Constructor.prototype = type.prototype;
      objects[path] = new Constructor();
      if (type.refresh) type.refresh(objects[path]);
      if (type.exporters && type.exporters[0]) type.exporters[0](objects[path]);
    }

    return objects[path];
  }

  function peek_object(path) {
    return objects[path] || null;
  }

  function drop_object(path) {
    var obj = objects[path];

    if (obj) {
      if (priv(obj).type.drop) priv(obj).type.drop(obj);
      delete objects[path];
      export_model();
    }
  }

  function set_object_properties(obj, props) {
    var p, decl, val;
    decl = priv(obj).type.props;

    for (p in decl) {
      val = props[decl[p].prop || p];

      if (val !== undefined) {
        if (decl[p].conv) val = decl[p].conv(val);

        if (val !== obj[p]) {
          obj[p] = val;
          if (decl[p].trigger) decl[p].trigger(obj);
        }
      }
    }
  }

  function remove_signatures(props_with_sigs) {
    var props = {};

    for (var p in props_with_sigs) {
      if (props_with_sigs.hasOwnProperty(p)) {
        props[p] = props_with_sigs[p].v;
      }
    }

    return props;
  }

  function objpath(obj) {
    if (obj && priv(obj).path) return priv(obj).path;else return "/";
  }

  function call_object_method(obj, iface, method) {
    var dfd = new $.Deferred();
    client.call(objpath(obj), iface, method, Array.prototype.slice.call(arguments, 3)).fail(function (ex) {
      dfd.reject(ex);
    }).done(function (reply) {
      dfd.resolve.apply(dfd, reply);
    });
    return dfd.promise();
  }

  var interface_types = {};
  var max_export_phases = 0;
  var export_pending;

  function set_object_types(all_types) {
    all_types.forEach(function (type) {
      if (type.exporters && type.exporters.length > max_export_phases) max_export_phases = type.exporters.length;
      type.interfaces.forEach(function (iface) {
        interface_types[iface] = type;
      });
    });
  }

  function signal_emitted(path, iface, signal, args) {
    var obj = peek_object(path);

    if (obj) {
      var type = priv(obj).type;

      if (signal == "PropertiesChanged") {
        push_refresh();
        set_object_properties(obj, remove_signatures(args[0]));
        pop_refresh();
      } else if (type.signals && type.signals[signal]) type.signals[signal](obj, args);
    }
  }

  function interface_properties(path, iface, props) {
    var type = interface_types[iface];
    if (type) set_object_properties(get_object(path, type), props);
  }

  function interface_removed(path, iface) {
    /* For NetworkManager we can make this assumption */
    drop_object(path);
  }

  var export_model_deferred = null;

  function export_model() {
    function doit() {
      var phase, path, obj, exp;

      for (phase = 0; phase < max_export_phases; phase++) {
        for (path in objects) {
          obj = objects[path];
          exp = priv(obj).type.exporters;
          if (exp && exp[phase]) exp[phase](obj);
        }
      }

      $(self).trigger('changed');

      if (export_model_deferred) {
        export_model_deferred.resolve();
        export_model_deferred = null;
      }
    }

    if (!export_pending) {
      export_pending = true;
      window.setTimeout(function () {
        export_pending = false;
        doit();
      }, 300);
    }
  }

  self.synchronize = function synchronize() {
    if (outstanding_refreshes === 0) {
      return cockpit.resolve();
    } else {
      if (!export_model_deferred) export_model_deferred = cockpit.defer();
      return export_model_deferred.promise();
    }
  };
  /**
   * Handle NM not running
   */


  var nm_service = service.proxy("NetworkManager");
  var nm_enabled = null;
  var nm_running = null;

  function update_nm_trouble() {
    nm_debug("update_nm_trouble; enabled", nm_enabled, "running", nm_running); // need to wait until we have both pieces of information

    if (nm_enabled === null || nm_running === null) return; // running

    if (nm_running) {
      $("#networking-nm-crashed").hide();
      $("#networking-nm-disabled").hide();
      $("#networking-graphs").show();
      $("#networking-interfaces").show(); // NM appearing will also trigger a device update, which hides it if necessary

      $("#networking-unmanaged-interfaces").show();
    } else {
      $("#networking-graphs").hide();
      $("#networking-interfaces").hide();
      $("#networking-unmanaged-interfaces").hide();

      if (nm_enabled) {
        $("#networking-nm-disabled").hide();
        $("#networking-nm-crashed").show();
      } else {
        $("#networking-nm-disabled").show();
        $("#networking-nm-crashed").hide();
      }
    }
  }

  nm_service.addEventListener('changed', function () {
    nm_enabled = nm_service.enabled;
    update_nm_trouble();
  }); // track NM going away or reappearing

  client.addEventListener("owner", function (event, owner) {
    nm_debug("NetworkManager owner changed:", JSON.stringify(owner));
    nm_running = owner !== null;
    update_nm_trouble();
  }); // Troubleshoot link and start button

  $("#networking-nm-crashed a").click(function () {
    cockpit.jump("/system/services#/NetworkManager.service", cockpit.transport.host);
  });
  $("#networking-nm-crashed button").click(nm_service.start); // Enable NM button

  $("#networking-nm-disabled button").click(function () {
    nm_service.enable();
    nm_service.start();
  });
  var subscription;
  var watch;
  self.preinit.then(function () {
    subscription = client.subscribe({}, signal_emitted);
    watch = client.watch({});
    $(client).on("notify", function (event, data) {
      $.each(data, function (path, ifaces) {
        $.each(ifaces, function (iface, props) {
          if (props) interface_properties(path, iface, props);else interface_removed(path, iface);
        });
      });
    });
  });

  self.close = function close() {
    subscription.remove();
    watch.remove();
    $(client).off("notify");
    client.close("unused");
  };
  /* NetworkManager specific data conversions and utility functions.
   */


  function ip4_address_from_nm(addr) {
    return [utils.ip4_to_text(addr[0]), utils.ip_prefix_to_text(addr[1]), utils.ip4_to_text(addr[2], true)];
  }

  function ip4_address_to_nm(addr) {
    return [utils.ip4_from_text(addr[0]), utils.ip4_prefix_from_text(addr[1]), utils.ip4_from_text(addr[2], true)];
  }

  function ip4_route_from_nm(addr) {
    return [utils.ip4_to_text(addr[0]), utils.ip_prefix_to_text(addr[1]), utils.ip4_to_text(addr[2], true), utils.ip_metric_to_text(addr[3])];
  }

  function ip4_route_to_nm(addr) {
    return [utils.ip4_from_text(addr[0]), utils.ip4_prefix_from_text(addr[1]), utils.ip4_from_text(addr[2], true), utils.ip_metric_from_text(addr[3])];
  }

  function ip6_address_from_nm(addr) {
    return [utils.ip6_to_text(addr[0]), utils.ip_prefix_to_text(addr[1]), utils.ip6_to_text(addr[2], true)];
  }

  function ip6_address_to_nm(addr) {
    return [utils.ip6_from_text(addr[0]), parseInt(addr[1], 10) || 64, utils.ip6_from_text(addr[2], true)];
  }

  function ip6_route_from_nm(addr) {
    return [utils.ip6_to_text(addr[0]), utils.ip_prefix_to_text(addr[1]), utils.ip6_to_text(addr[2], true), utils.ip_metric_to_text(addr[3])];
  }

  function ip6_route_to_nm(addr) {
    return [utils.ip6_from_text(addr[0]), utils.ip_prefix_from_text(addr[1]), utils.ip6_from_text(addr[2], true), utils.ip_metric_from_text(addr[3])];
  }

  function settings_from_nm(settings) {
    function get(first, second, def) {
      if (settings[first] && settings[first][second]) return settings[first][second].v;else return def;
    }

    function get_ip(first, addr_from_nm, route_from_nm, ip_to_text) {
      return {
        method: get(first, "method", "auto"),
        ignore_auto_dns: get(first, "ignore-auto-dns", false),
        ignore_auto_routes: get(first, "ignore-auto-routes", false),
        addresses: get(first, "addresses", []).map(addr_from_nm),
        dns: get(first, "dns", []).map(ip_to_text),
        dns_search: get(first, "dns-search", []),
        routes: get(first, "routes", []).map(route_from_nm)
      };
    }

    var result = {
      connection: {
        type: get("connection", "type"),
        uuid: get("connection", "uuid"),
        interface_name: get("connection", "interface-name"),
        timestamp: get("connection", "timestamp", 0),
        id: get("connection", "id", _("Unknown")),
        autoconnect: get("connection", "autoconnect", true),
        autoconnect_slaves: get("connection", "autoconnect-slaves", -1),
        slave_type: get("connection", "slave-type"),
        master: get("connection", "master")
      }
    };

    if (!settings.connection.master) {
      result.ipv4 = get_ip("ipv4", ip4_address_from_nm, ip4_route_from_nm, utils.ip4_to_text);
      result.ipv6 = get_ip("ipv6", ip6_address_from_nm, ip6_route_from_nm, utils.ip6_to_text);
    }

    if (settings["802-3-ethernet"]) {
      result.ethernet = {
        mtu: get("802-3-ethernet", "mtu"),
        assigned_mac_address: get("802-3-ethernet", "assigned-mac-address")
      };
    }

    if (settings.bond) {
      /* Options are documented as part of the Linux bonding driver.
         https://www.kernel.org/doc/Documentation/networking/bonding.txt
      */
      result.bond = {
        options: $.extend({}, get("bond", "options", {})),
        interface_name: get("bond", "interface-name")
      };
    }

    function JSON_parse_carefully(str) {
      try {
        return JSON.parse(str);
      } catch (e) {
        return null;
      }
    }

    if (settings.team) {
      result.team = {
        config: JSON_parse_carefully(get("team", "config", "{}")),
        interface_name: get("team", "interface-name")
      };
    }

    if (settings["team-port"] || result.connection.slave_type == "team") {
      result.team_port = {
        config: JSON_parse_carefully(get("team-port", "config", "{}"))
      };
    }

    if (settings.bridge) {
      result.bridge = {
        interface_name: get("bridge", "interface-name"),
        stp: get("bridge", "stp", true),
        priority: get("bridge", "priority", 32768),
        forward_delay: get("bridge", "forward-delay", 15),
        hello_time: get("bridge", "hello-time", 2),
        max_age: get("bridge", "max-age", 20),
        ageing_time: get("bridge", "ageing-time", 300)
      };
    }

    if (settings["bridge-port"] || result.connection.slave_type == "bridge") {
      result.bridge_port = {
        priority: get("bridge-port", "priority", 32),
        path_cost: get("bridge-port", "path-cost", 100),
        hairpin_mode: get("bridge-port", "hairpin-mode", false)
      };
    }

    if (settings.vlan) {
      result.vlan = {
        parent: get("vlan", "parent"),
        id: get("vlan", "id"),
        interface_name: get("vlan", "interface-name")
      };
    }

    return result;
  }

  function settings_to_nm(settings, orig) {
    var result = $.extend(true, {}, orig);

    function set(first, second, sig, val, def) {
      if (val === undefined) val = def;
      if (!result[first]) result[first] = {};
      if (val !== undefined) result[first][second] = cockpit.variant(sig, val);else delete result[first][second];
    }

    function set_ip(first, addrs_sig, addr_to_nm, routes_sig, route_to_nm, ips_sig, ip_from_text) {
      set(first, "method", 's', settings[first].method);
      set(first, "ignore-auto-dns", 'b', settings[first].ignore_auto_dns);
      set(first, "ignore-auto-routes", 'b', settings[first].ignore_auto_routes);
      var addresses = settings[first].addresses;
      if (addresses) set(first, "addresses", addrs_sig, addresses.map(addr_to_nm));
      var dns = settings[first].dns;
      if (dns) set(first, "dns", ips_sig, dns.map(ip_from_text));
      set(first, "dns-search", 'as', settings[first].dns_search);
      var routes = settings[first].routes;
      if (routes) set(first, "routes", routes_sig, routes.map(route_to_nm)); // Never pass "address-labels" back to NetworkManager.  It
      // is documented as "internal only", but needs to somehow
      // stay in sync with "addresses".  By not passing it back
      // we don't have to worry about that.
      //

      delete result[first]["address-labels"];
    }

    set("connection", "id", 's', settings.connection.id);
    set("connection", "autoconnect", 'b', settings.connection.autoconnect);
    set("connection", "autoconnect-slaves", 'i', settings.connection.autoconnect_slaves);
    set("connection", "uuid", 's', settings.connection.uuid);
    set("connection", "interface-name", 's', settings.connection.interface_name);
    set("connection", "type", 's', settings.connection.type);
    set("connection", "slave-type", 's', settings.connection.slave_type);
    set("connection", "master", 's', settings.connection.master);
    if (settings.ipv4) set_ip("ipv4", 'aau', ip4_address_to_nm, 'aau', ip4_route_to_nm, 'au', utils.ip4_from_text);else delete result.ipv4;
    if (settings.ipv6) set_ip("ipv6", 'a(ayuay)', ip6_address_to_nm, 'a(ayuayu)', ip6_route_to_nm, 'aay', utils.ip6_from_text);else delete result.ipv6;

    if (settings.bond) {
      set("bond", "options", 'a{ss}', settings.bond.options);
      set("bond", "interface-name", 's', settings.bond.interface_name);
    } else delete result.bond;

    if (settings.team) {
      set("team", "config", 's', JSON.stringify(settings.team.config));
      set("team", "interface-name", 's', settings.team.interface_name);
    } else delete result.team;

    if (settings.team_port) set("team-port", "config", 's', JSON.stringify(settings.team_port.config));else delete result["team-port"];

    if (settings.bridge) {
      set("bridge", "interface-name", 's', settings.bridge.interface_name);
      set("bridge", "stp", 'b', settings.bridge.stp);
      set("bridge", "priority", 'u', settings.bridge.priority);
      set("bridge", "forward-delay", 'u', settings.bridge.forward_delay);
      set("bridge", "hello-time", 'u', settings.bridge.hello_time);
      set("bridge", "max-age", 'u', settings.bridge.max_age);
      set("bridge", "ageing-time", 'u', settings.bridge.ageing_time);
    } else delete result.bridge;

    if (settings.bridge_port) {
      set("bridge-port", "priority", 'u', settings.bridge_port.priority);
      set("bridge-port", "path-cost", 'u', settings.bridge_port.path_cost);
      set("bridge-port", "hairpin-mode", 'b', settings.bridge_port.hairpin_mode);
    } else delete result["bridge-port"];

    if (settings.vlan) {
      set("vlan", "parent", 's', settings.vlan.parent);
      set("vlan", "id", 'u', settings.vlan.id);
      set("vlan", "interface-name", 's', settings.vlan.interface_name); // '1' is the default, but we need to set it explicitly anyway.

      set("vlan", "flags", 'u', 1);
    } else delete result.vlan;

    if (settings.ethernet) {
      set("802-3-ethernet", "mtu", 'u', settings.ethernet.mtu);
      set("802-3-ethernet", "assigned-mac-address", 's', settings.ethernet.assigned_mac_address); // Delete cloned-mac-address so that assigned-mac-address gets used.

      delete result["802-3-ethernet"]["cloned-mac-address"];
    } else delete result["802-3-ethernet"];

    return result;
  }

  function device_type_to_symbol(type) {
    // This returns a string that is suitable for the connection.type field of
    // Connection.Settings, except for "ethernet".
    switch (type) {
      case 0:
        return 'unknown';

      case 1:
        return 'ethernet';
      // 802-3-ethernet

      case 2:
        return '802-11-wireless';

      case 3:
        return 'unused1';

      case 4:
        return 'unused2';

      case 5:
        return 'bluetooth';

      case 6:
        return '802-11-olpc-mesh';

      case 7:
        return 'wimax';

      case 8:
        return 'modem';

      case 9:
        return 'infiniband';

      case 10:
        return 'bond';

      case 11:
        return 'vlan';

      case 12:
        return 'adsl';

      case 13:
        return 'bridge';

      case 14:
        return 'loopback';

      case 15:
        return 'team';

      case 16:
        return 'tun';

      case 17:
        return 'ip_tunnel';

      case 18:
        return 'macvlan';

      case 19:
        return 'vxlan';

      case 20:
        return 'veth';

      default:
        return '';
    }
  }

  function device_state_to_text(state) {
    switch (state) {
      // NM_DEVICE_STATE_UNKNOWN
      case 0:
        return "?";
      // NM_DEVICE_STATE_UNMANAGED

      case 10:
        return "";
      // NM_DEVICE_STATE_UNAVAILABLE

      case 20:
        return _("Not available");
      // NM_DEVICE_STATE_DISCONNECTED

      case 30:
        return _("Inactive");
      // NM_DEVICE_STATE_PREPARE

      case 40:
        return _("Preparing");
      // NM_DEVICE_STATE_CONFIG

      case 50:
        return _("Configuring");
      // NM_DEVICE_STATE_NEED_AUTH

      case 60:
        return _("Authenticating");
      // NM_DEVICE_STATE_IP_CONFIG

      case 70:
        return _("Configuring IP");
      // NM_DEVICE_STATE_IP_CHECK

      case 80:
        return _("Checking IP");
      // NM_DEVICE_STATE_SECONDARIES

      case 90:
        return _("Waiting");
      // NM_DEVICE_STATE_ACTIVATED

      case 100:
        return _("Active");
      // NM_DEVICE_STATE_DEACTIVATING

      case 110:
        return _("Deactivating");
      // NM_DEVICE_STATE_FAILED

      case 120:
        return _("Failed");

      default:
        return "";
    }
  }

  var connections_by_uuid = {};

  function set_settings(obj, settings) {
    if (obj.Settings && obj.Settings.connection && obj.Settings.connection.uuid) delete connections_by_uuid[obj.Settings.connection.uuid];
    obj.Settings = settings;
    if (settings && settings.connection && settings.connection.uuid) connections_by_uuid[settings.connection.uuid] = obj;
  }

  function refresh_settings(obj) {
    push_refresh();
    client.call(objpath(obj), "org.freedesktop.NetworkManager.Settings.Connection", "GetSettings").always(pop_refresh).fail(complain).done(function (reply) {
      var result = reply[0];

      if (result) {
        priv(obj).orig = result;
        set_settings(obj, settings_from_nm(result));
      }
    });
  }

  function refresh_udev(obj) {
    if (obj.Udi.indexOf("/sys/") !== 0) return;
    push_refresh();
    cockpit.spawn(["udevadm", "info", obj.Udi], {
      err: 'message'
    }).done(function (res) {
      var props = {};

      function snarf_prop(line, env, prop) {
        var prefix = "E: " + env + "=";

        if (line.indexOf(prefix) === 0) {
          props[prop] = line.substr(prefix.length);
        }
      }

      res.split('\n').forEach(function (line) {
        snarf_prop(line, "ID_MODEL_FROM_DATABASE", "IdModel");
        snarf_prop(line, "ID_VENDOR_FROM_DATABASE", "IdVendor");
      });
      set_object_properties(obj, props);
    }).fail(function (ex) {
      /* udevadm info exits with 4 when device doesn't exist */
      if (ex.exit_status !== 4) {
        console.warn(ex.message);
        console.warn(ex);
      }
    }).always(pop_refresh);
  }

  function handle_updated(obj) {
    refresh_settings(obj);
  }
  /* NetworkManager specific object types, used by the generic D-Bus
   * code and using the data conversion functions.
   */


  var type_Manager;
  var type_Ipv4Config = {
    interfaces: ["org.freedesktop.NetworkManager.IP4Config"],
    props: {
      Addresses: {
        conv: conv_Array(ip4_address_from_nm),
        def: []
      }
    }
  };
  var type_Ipv6Config = {
    interfaces: ["org.freedesktop.NetworkManager.IP6Config"],
    props: {
      Addresses: {
        conv: conv_Array(ip6_address_from_nm),
        def: []
      }
    }
  };
  var type_Connection = {
    interfaces: ["org.freedesktop.NetworkManager.Settings.Connection"],
    props: {
      Unsaved: {}
    },
    signals: {
      Updated: handle_updated
    },
    refresh: refresh_settings,
    drop: function drop(obj) {
      set_settings(obj, null);
    },
    prototype: {
      copy_settings: function copy_settings() {
        return $.extend(true, {}, this.Settings);
      },
      apply_settings: function apply_settings(settings) {
        var self = this;

        try {
          return call_object_method(self, "org.freedesktop.NetworkManager.Settings.Connection", "Update", settings_to_nm(settings, priv(self).orig)).done(function () {
            set_settings(self, settings);
          });
        } catch (e) {
          return cockpit.reject(e);
        }
      },
      activate: function activate(dev, specific_object) {
        return call_object_method(get_object("/org/freedesktop/NetworkManager", type_Manager), "org.freedesktop.NetworkManager", "ActivateConnection", objpath(this), objpath(dev), objpath(specific_object));
      },
      delete_: function delete_() {
        return call_object_method(this, "org.freedesktop.NetworkManager.Settings.Connection", "Delete");
      }
    },
    exporters: [function (obj) {
      obj.Masters = [];
      obj.Slaves = [];
      obj.Interfaces = [];
    }, null, null, // Needs: type_Interface.Connections
    //
    // Sets:  type_Connection.Slaves
    //        type_Connection.Masters
    //
    function (obj) {
      var master, iface; // Most of the time, a connection has zero or one masters,
      // but when a connection refers to its master by interface
      // name, we might end up with more than one master
      // connection so we just collect them all.
      //
      // TODO - Nail down how NM really handles this.

      function check_con(con) {
        var master_settings = connection_settings(con);
        var my_settings = connection_settings(obj);

        if (master_settings.type == my_settings.slave_type) {
          obj.Masters.push(con);
          con.Slaves.push(obj);
        }
      }

      var cs = connection_settings(obj);

      if (cs.slave_type) {
        master = connections_by_uuid[cs.master];

        if (master) {
          obj.Masters.push(master);
          master.Slaves.push(obj);
        } else {
          iface = peek_interface(cs.master);

          if (iface) {
            iface.Connections.forEach(check_con);
          }
        }
      }
    }]
  };
  var type_ActiveConnection = {
    interfaces: ["org.freedesktop.NetworkManager.Connection.Active"],
    props: {
      Connection: {
        conv: conv_Object(type_Connection)
      },
      Ip4Config: {
        conv: conv_Object(type_Ipv4Config)
      },
      Ip6Config: {
        conv: conv_Object(type_Ipv6Config)
      } // See below for "Master"

    },
    prototype: {
      deactivate: function deactivate() {
        return call_object_method(get_object("/org/freedesktop/NetworkManager", type_Manager), "org.freedesktop.NetworkManager", "DeactivateConnection", objpath(this));
      }
    }
  };
  var type_Device = {
    interfaces: ["org.freedesktop.NetworkManager.Device", "org.freedesktop.NetworkManager.Device.Wired", "org.freedesktop.NetworkManager.Device.Bond", "org.freedesktop.NetworkManager.Device.Team", "org.freedesktop.NetworkManager.Device.Bridge", "org.freedesktop.NetworkManager.Device.Vlan"],
    props: {
      DeviceType: {
        conv: device_type_to_symbol
      },
      Interface: {},
      StateText: {
        prop: "State",
        conv: device_state_to_text,
        def: _("Unknown")
      },
      State: {},
      HwAddress: {},
      AvailableConnections: {
        conv: conv_Array(conv_Object(type_Connection)),
        def: []
      },
      ActiveConnection: {
        conv: conv_Object(type_ActiveConnection)
      },
      Ip4Config: {
        conv: conv_Object(type_Ipv4Config)
      },
      Ip6Config: {
        conv: conv_Object(type_Ipv6Config)
      },
      Udi: {
        trigger: refresh_udev
      },
      IdVendor: {
        def: ""
      },
      IdModel: {
        def: ""
      },
      Driver: {
        def: ""
      },
      Carrier: {
        def: true
      },
      Speed: {},
      Managed: {
        def: false
      } // See below for "Slaves"

    },
    prototype: {
      activate: function activate(connection, specific_object) {
        return call_object_method(get_object("/org/freedesktop/NetworkManager", type_Manager), "org.freedesktop.NetworkManager", "ActivateConnection", objpath(connection), objpath(this), objpath(specific_object));
      },
      activate_with_settings: function activate_with_settings(settings, specific_object) {
        try {
          return call_object_method(get_object("/org/freedesktop/NetworkManager", type_Manager), "org.freedesktop.NetworkManager", "AddAndActivateConnection", settings_to_nm(settings), objpath(this), objpath(specific_object));
        } catch (e) {
          return cockpit.reject(e);
        }
      },
      disconnect: function disconnect() {
        return call_object_method(this, 'org.freedesktop.NetworkManager.Device', 'Disconnect');
      }
    }
  }; // The 'Interface' type does not correspond to any NetworkManager
  // object or interface.  We use it to represent a network device
  // that might or might not actually be known to the kernel, such
  // as the interface of a bond that is currently down.
  //
  // This is a HACK: NetworkManager should export Device nodes for
  // these.

  var type_Interface = {
    interfaces: [],
    exporters: [function (obj) {
      obj.Device = null;
      obj._NonDeviceConnections = [];
      obj.Connections = [];
      obj.MainConnection = null;
    }, null, // Needs: type_Interface.Device
    //        type_Interface._NonDeviceConnections
    //
    // Sets:  type_Connection.Interfaces
    //        type_Interface.Connections
    //        type_Interface.MainConnection
    function (obj) {
      if (!obj.Device && obj._NonDeviceConnections.length === 0) {
        drop_object(priv(obj).path);
        return;
      }

      function consider_for_main(con) {
        if (!obj.MainConnection || connection_settings(obj.MainConnection).timestamp < connection_settings(con).timestamp) {
          obj.MainConnection = con;
        }
      }

      obj.Connections = obj._NonDeviceConnections;

      if (obj.Device) {
        obj.Device.AvailableConnections.forEach(function (con) {
          if (obj.Connections.indexOf(con) == -1) obj.Connections.push(con);
        });
      }

      obj.Connections.forEach(function (con) {
        consider_for_main(con);
        con.Interfaces.push(obj);
      }); // Explicitly prefer the active connection.  The
      // active connection should have the most recent
      // timestamp, but only when the activation was
      // successful.  Also, there don't seem to be change
      // notifications when the timestamp changes.

      if (obj.Device && obj.Device.ActiveConnection && obj.Device.ActiveConnection.Connection) {
        obj.MainConnection = obj.Device.ActiveConnection.Connection;
      }
    }]
  };

  function get_interface(iface) {
    var obj = get_object(":interface:" + iface, type_Interface);
    obj.Name = iface;
    return obj;
  }

  function peek_interface(iface) {
    return peek_object(":interface:" + iface);
  }

  var type_Settings = {
    interfaces: ["org.freedesktop.NetworkManager.Settings"],
    props: {
      Connections: {
        conv: conv_Array(conv_Object(type_Connection)),
        def: []
      }
    },
    prototype: {
      add_connection: function add_connection(conf) {
        var dfd = $.Deferred();

        try {
          call_object_method(this, 'org.freedesktop.NetworkManager.Settings', 'AddConnection', settings_to_nm(conf, {})).done(function (path) {
            dfd.resolve(get_object(path, type_Connection));
          }).fail(function (error) {
            dfd.reject(error);
          });
        } catch (e) {
          dfd.reject(e);
        }

        return dfd.promise();
      }
    },
    exporters: [null, // Sets: type_Interface._NonDeviceConnections
    //
    function (obj) {
      if (obj.Connections) {
        obj.Connections.forEach(function (con) {
          function add_to_interface(name) {
            if (name) {
              var cons = get_interface(name)._NonDeviceConnections;

              if (cons.indexOf(con) == -1) cons.push(con);
            }
          }

          if (con.Settings) {
            if (con.Settings.connection) add_to_interface(con.Settings.connection.interface_name);
            if (con.Settings.bond) add_to_interface(con.Settings.bond.interface_name);
            if (con.Settings.team) add_to_interface(con.Settings.team.interface_name);
            if (con.Settings.bridge) add_to_interface(con.Settings.bridge.interface_name);
            if (con.Settings.vlan) add_to_interface(con.Settings.vlan.interface_name);
          }
        });
      }
    }]
  };
  type_Manager = {
    interfaces: ["org.freedesktop.NetworkManager"],
    props: {
      Version: {},
      Devices: {
        conv: conv_Array(conv_Object(type_Device)),
        def: []
      },
      ActiveConnections: {
        conv: conv_Array(conv_Object(type_ActiveConnection)),
        def: []
      }
    },
    prototype: {
      checkpoint_create: function checkpoint_create(devices, timeout) {
        var dfd = $.Deferred();
        call_object_method(this, 'org.freedesktop.NetworkManager', 'CheckpointCreate', devices.map(objpath), timeout, 0).done(function (path) {
          dfd.resolve(path);
        }).fail(function (error) {
          if (error.name != "org.freedesktop.DBus.Error.UnknownMethod") console.warn(error.message || error);
          dfd.resolve(null);
        });
        return dfd.promise();
      },
      checkpoint_destroy: function checkpoint_destroy(checkpoint) {
        if (checkpoint) {
          return call_object_method(this, 'org.freedesktop.NetworkManager', 'CheckpointDestroy', checkpoint);
        } else return $.when();
      },
      checkpoint_rollback: function checkpoint_rollback(checkpoint) {
        if (checkpoint) {
          return call_object_method(this, 'org.freedesktop.NetworkManager', 'CheckpointRollback', checkpoint);
        } else return $.when();
      }
    },
    exporters: [null, // Sets: type_Interface.Device
    //
    function (obj) {
      obj.Devices.forEach(function (dev) {
        if (dev.Interface) {
          var iface = get_interface(dev.Interface);
          iface.Device = dev;
        }
      });
    }]
  };
  /* Now create the cyclic declarations.
   */

  type_ActiveConnection.props.Master = {
    conv: conv_Object(type_Device)
  };
  type_Device.props.Slaves = {
    conv: conv_Array(conv_Object(type_Device)),
    def: []
  };
  /* Accessing the model.
   */

  self.list_interfaces = function list_interfaces() {
    var path, obj;
    var result = [];

    for (path in objects) {
      obj = objects[path];
      if (priv(obj).type === type_Interface) result.push(obj);
    }

    return result.sort(function (a, b) {
      return a.Name.localeCompare(b.Name);
    });
  };

  self.find_interface = peek_interface;

  self.get_manager = function () {
    return get_object("/org/freedesktop/NetworkManager", type_Manager);
  };

  self.get_settings = function () {
    return get_object("/org/freedesktop/NetworkManager/Settings", type_Settings);
  };

  function compare_versions(a, b) {
    function to_ints(str) {
      return str.split(".").map(function (s) {
        return s ? parseInt(s, 10) : 0;
      });
    }

    var a_ints = to_ints(a);
    var b_ints = to_ints(b);
    var len = Math.min(a_ints.length, b_ints.length);
    var i;

    for (i = 0; i < len; i++) {
      if (a_ints[i] == b_ints[i]) continue;
      return a_ints[i] - b_ints[i];
    }

    return a_ints.length - b_ints.length;
  }

  self.at_least_version = function at_least_version(version) {
    return compare_versions(self.get_manager().Version, version) >= 0;
  };
  /* Initialization.
   */


  set_object_types([type_Manager, type_Settings, type_Device, type_Ipv4Config, type_Ipv6Config, type_Connection, type_ActiveConnection]);
  get_object("/org/freedesktop/NetworkManager", type_Manager);
  get_object("/org/freedesktop/NetworkManager/Settings", type_Settings);
  return self;
}

function getNetworkManager(inithost) {
  var net = new NetworkManagerModel(inithost);
  return net;
}