"use strict";

var _jquery = _interopRequireDefault(require("jquery"));

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _mustache = _interopRequireDefault(require("mustache"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2013 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */
//import cockpit from 'cockpit';
var _ = cockpit.gettext;
var C_ = cockpit.gettext;
var g_asserts_bak = [];
var g_rules_bak = [];
var g_users_bak = [];
var g_repos_bak = [];
var g_updates_bak = [];
/* INITIALIZATION AND NAVIGATION
 *
 * The code above still uses the legacy 'Page' abstraction for both
 * pages and dialogs, and expects page.setup, page.enter, page.show,
 * and page.leave to be called at the right times.
 *
 * We cater to this with a little compatability shim consisting of
 * 'dialog_setup', 'page_show', and 'page_hide'.
 */

function show_error_dialog(title, message) {
  if (message) {
    (0, _jquery["default"])("#error-popup-title").text(title);
    (0, _jquery["default"])("#error-popup-message").text(message);
  } else {
    (0, _jquery["default"])("#error-popup-title").text(_("Error"));
    (0, _jquery["default"])("#error-popup-message").text(title);
  }

  (0, _jquery["default"])("#show-error-dialog").click();
}

function rules_init() {
  var account_empty_tmpl = (0, _jquery["default"])("#account-empty-tmpl").html();

  _mustache["default"].parse(account_empty_tmpl);

  var rules_list_tmpl = (0, _jquery["default"])("#rules-list-tmpl").html();

  _mustache["default"].parse(rules_list_tmpl); // 分页界面的相关变量放在这做成全局变量


  var g_vue_pagination_rules;
  var g_cur_page_rules = 1;
  var g_page_size_rules = 20;

  function processPagination(units) {
    // 分页展示
    if (!g_vue_pagination_rules) {
      g_vue_pagination_rules = new Vue({
        template: '<el-pagination @size-change="handleSizeChange" @current-change="handleCurrentChange" :current-page="current_page" :page-sizes="[20, 40, 60, 80]" :page-size="page_size" layout="total, sizes, prev, pager, next, jumper" :total="total"> </el-pagination>',
        el: "#rules-vue-app-div",
        methods: {
          handleSizeChange: function handleSizeChange(val) {
            g_page_size_rules = val;
            console.log("set g_page_size_rules to:", val);
            show_rules_page(g_rules_bak);
            registerRulesPage();
          },
          handleCurrentChange: function handleCurrentChange(val) {
            g_cur_page_rules = val;
            console.log("set g_cur_page_rules to:", val);
            show_rules_page(g_rules_bak);
            registerRulesPage();
          }
        },
        data: {
          current_page: g_cur_page_rules,
          page_size: g_page_size_rules,
          total: units.length
        }
      });
    } else {
      g_vue_pagination_rules.current_page = g_cur_page_rules;
      g_vue_pagination_rules.page_size = g_page_size_rules;
      g_vue_pagination_rules.total = units.length;
    }

    units = units.slice((g_cur_page_rules - 1) * g_page_size_rules, g_cur_page_rules * g_page_size_rules);
    return units;
  }

  function getAssertById(assertid) {
    for (var i = 0; i < g_asserts_bak.length; i++) {
      if (g_asserts_bak[i].assertid == assertid) {
        return g_asserts_bak[i];
      }
    }

    return false;
  }

  function getRuleById(id) {
    for (var i = 0; i < g_rules_bak.length; i++) {
      if (g_rules_bak[i].id == id) {
        return g_rules_bak[i];
      }
    }

    return false;
  }

  function show_rules_page(info) {
    var current_text_filter = (0, _jquery["default"])('#services-text-filter').val().toLowerCase();
    var header = {
      Name: "规则名称",
      Straname: "策略名称",
      Applyto: "应用到",
      Status: "规则状态",
      From: "来源于",
      Operate: "操作"
    };
    var units = [];

    function cmp_rule(a, b) {
      return info[a].id - info[b].id;
    }

    var sorted_rules_keys = Object.keys(info).sort(cmp_rule);
    sorted_rules_keys.forEach(function (id) {
      var unit = info[id];

      try {
        if (unit.type == "repo" || unit.type == "update") {
          unit.straname = unit.stracontent;
        } else {
          var userstra = _jquery["default"].parseJSON(unit.stracontent);

          unit.straname = userstra.name;

          if (unit.type == "security") {
            unit.straname = "默认策略";
          }
        }
      } catch (e) {
        console.log(e);
      }

      var applytotext = "";
      var assertsarr = unit.applyto.split(";");

      for (var i = 0; i < assertsarr.length; i++) {
        if (!assertsarr[i] || assertsarr[i].length < 1) {
          continue;
        }

        var asserttmp = getAssertById(assertsarr[i]);
        applytotext = applytotext + "资产：" + asserttmp.assertip + "-" + asserttmp.assertname + ";";
      }

      unit.applytotext = applytotext;

      if (unit.applycount.length < 1) {
        unit.status = "下发中";
      } else {
        try {
          var applyjson = _jquery["default"].parseJSON(unit.applycount);

          var keys = Object.keys(applyjson);

          for (var _i = 0; _i < keys.length; _i++) {
            if (applyjson[keys[_i]]["applycount"] < unit.editcount) {
              // 说明还是上次的下发结果
              unit.status = "尚未下发";
              break;
            }

            if (applyjson[keys[_i]]["result"] != "succeed") {
              // 只要有一个下发失败，就认为下发失败
              unit.status = "下发失败";
              break;
            }
          } // 如果全部都成功，才认为下发成功


          if (!unit.status || unit.status.length < 1) {
            unit.status = "下发成功";
          }
        } catch (e) {
          console.log(e);
        }
      }

      if (unit.type == "user") {
        unit.from = "账户密码";
      } else if (unit.type == "security") {
        unit.from = "安全管理";
      } else if (unit.type == "repo") {
        unit.from = "软件源";
      } else if (unit.type == "update") {
        unit.from = "软件升级";
      } else {
        unit.from = "";
      } // 匹配搜索字符串


      if (!current_text_filter) {
        units.push(unit);
        return;
      }

      if (unit.name && unit.name.indexOf(current_text_filter) != -1) {
        units.push(unit);
        return;
      }
    });

    if (units.length < 1) {
      var html = _mustache["default"].render(account_empty_tmpl, {
        empty_text: "没有匹配的规则"
      });

      return (0, _jquery["default"])("#accounts-list").html(html);
    }

    units = processPagination(units);

    var text = _mustache["default"].render(rules_list_tmpl, {
      table_head: header,
      units: units
    });

    (0, _jquery["default"])("#accounts-list").html(text);
  }

  var vue_choose_asserts;
  var vue_edit_choose_asserts; // vuedata 放在这里定义，新增，编辑，新增确认，编辑确认的时候都可以访问

  var vuedata;

  function getVuedataByAssertsAndGroups(info) {
    vuedata = [];
    var allGroupAsserts = [];
    var allasserts = info["allasserts"];

    function getAssertsById(id) {
      for (var i = 0; i < allasserts.length; i++) {
        if (allasserts[i].assertid == id) {
          return allasserts[i];
        }
      }

      return {};
    }

    for (var i = 0; i < info["allgroups"].length; i++) {
      var grouptmp = info["allgroups"][i];
      var unit = {
        id: grouptmp.groupid,
        label: "分组：" + grouptmp.groupname,
        children: []
      };
      var tmparr = grouptmp.groupasserts.split(";");

      for (var k = 0; k < tmparr.length; k++) {
        if (!tmparr[k] || tmparr[k].length < 1) {
          continue;
        }

        var asserttmp = getAssertsById(tmparr[k]);

        if (asserttmp.assertip == "localhost") {
          continue;
        }

        unit.children.push({
          id: tmparr[k],
          label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
        });

        if (allGroupAsserts.indexOf(tmparr[k]) < 0) {
          allGroupAsserts.push(tmparr[k]);
        }
      }

      vuedata.push(unit);
    }

    for (var _i2 = 0; _i2 < info["allasserts"].length; _i2++) {
      var _asserttmp = info["allasserts"][_i2];

      if (allGroupAsserts.indexOf(_asserttmp.assertid) >= 0) {
        continue;
      }

      if (_asserttmp.assertip == "localhost") {
        continue;
      }

      var _unit = {
        id: _asserttmp.assertid,
        label: "资产：" + _asserttmp.assertip + "-" + _asserttmp.assertname
      };
      vuedata.push(_unit);
    }

    return vuedata;
  }

  function registerRulesPage() {
    // 点击下发按钮触发后台下发
    (0, _jquery["default"])("button.download-rule-btn").unbind("click").click(function (e) {
      var ruleid = (0, _jquery["default"])(this).data("rule-id");
      var ruleunit = getRuleById(ruleid);
      var ruletype = ruleunit.type;
      console.log(ruleid);
      console.log(ruletype);
      var http = cockpit.http({
        "port": 9091
      });
      http.post("/downloadRule", {
        type: ruletype,
        id: ruleid
      }).fail(function (err) {
        console.log("downloadRule err:", err);
        show_error_dialog("错误", "通知后台下发失败");
      }).done(function (res) {
        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("downloadRule err:", res);
          return show_error_dialog("错误", "通知后台下发失败：" + info.error);
        }

        show_error_dialog("提示", "通知后台下发成功");
        setTimeout(function () {
          (0, _jquery["default"])("#error-popup-close").click();
          render_rules();
        }, 1500);
      });
    }); // 点击规则行上的更多按钮时，弹出菜单，需要在渲染完成后再绑定事件

    (0, _jquery["default"])(".user-rule-opr-more").unbind("click").on("click", function (e) {
      var top = (0, _jquery["default"])(this).offset().top;
      var left = (0, _jquery["default"])(this).offset().left;
      var top1 = (0, _jquery["default"])("html").scrollTop();
      top = top - top1;
      var width = parseInt((0, _jquery["default"])(this).css("width").replace("px", ""));
      var ruleid = (0, _jquery["default"])(this).data("rule-id");
      var ruleunit = getRuleById(ruleid);
      console.log("user-rule-opr-more");
      console.log(ruleunit.type); //对于软件源规则和软件升级规则，暂时不提供编辑功能

      if (ruleunit.type == "repo" || ruleunit.type == "update") {
        (0, _jquery["default"])("button.user-rule-edit").hide();
      } else {
        (0, _jquery["default"])("button.user-rule-edit").show();
      }

      (0, _jquery["default"])(".user-rule-popup-menu").css("top", top);
      (0, _jquery["default"])(".user-rule-popup-menu").css("left", left + width);

      if ((0, _jquery["default"])(".user-rule-popup-menu").css("display") == "none") {
        (0, _jquery["default"])(".user-rule-popup-menu").css("display", "inline-grid");
      } else {
        (0, _jquery["default"])(".user-rule-popup-menu").css("display", "none");
      }

      (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data", ruleid);
    });
    (0, _jquery["default"])(".user-rule-popup-btn").mouseover(function (e) {
      (0, _jquery["default"])(this).css("background-color", "#def3ff");
    }).mouseout(function (e) {
      (0, _jquery["default"])(this).css("background-color", "white");
    });
    (0, _jquery["default"])(window).unbind("click").on("click", function (e) {
      if (e.target.localName == "button" && (e.target.className.indexOf("user-rule-opr-more") >= 0 || e.target.className.indexOf("user-rule-del") >= 0 || e.target.className.indexOf("user-rule-status") >= 0 || e.target.className.indexOf("user-rule-edit") >= 0)) {
        return;
      }

      (0, _jquery["default"])(".user-rule-popup-menu").css("display", "none");
    });
    (0, _jquery["default"])("button.user-rule-status").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var ruleunit = getRuleById(ruleid);

      try {
        (0, _jquery["default"])("#rule-status-content").html("");

        var applycount = _jquery["default"].parseJSON(ruleunit.applycount);

        Object.keys(applycount).forEach(function (key) {
          // 渲染规则页面的时候已经获取过资产信息，这里直接用，不重新请求了
          var asserttmp = getAssertById(key);

          if (!asserttmp) {
            console.log("[user-rule-status]get assert failed,assertid:", applycount[key]);
            return;
          }

          var statustext = "";

          if (applycount[key]["applycount"] < ruleunit.editcount) {
            statustext = "尚未下发";
          } else {
            if (applycount[key]["result"] == "succeed") {
              statustext = "下发成功";
            } else {
              statustext = "下发失败";
            }
          }

          var tr = "<tr><td class='text-align-left'>" + asserttmp.assertip + "</td><td>" + asserttmp.assertname + "</td><td><span class=''>" + statustext + "</span></td></tr>";
          (0, _jquery["default"])("#rule-status-content").append((0, _jquery["default"])(tr));
        });
      } catch (e) {
        console.log(e);
      }
    });
    (0, _jquery["default"])("button.user-rule-edit").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var ruleunit = getRuleById(ruleid);
      (0, _jquery["default"])("#edit-rule-name").val(ruleunit.name);
      var http = cockpit.http({
        "port": 9091
      });
      http.get("/getUsers").fail(function (err) {
        console.log("getUsers err:", err);
        (0, _jquery["default"])("#edit-rule-msg").text("获取账号策略失败");
      }).done(function (res) {
        console.log("getUsers res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("getUsers err:", res);
          return (0, _jquery["default"])("#edit-rule-msg").text("获取账号策略失败：" + info.error);
        }

        info = info.data;
        g_users_bak = info;
        (0, _jquery["default"])("#edit-stra-name").html("");

        for (var i = 0; i < g_users_bak.length; i++) {
          var html = "";

          if (g_users_bak[i].id == ruleunit.straid) {
            html = '<option selected="selected" value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
          } else {
            html = '<option value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
          }

          (0, _jquery["default"])("#edit-stra-name").append((0, _jquery["default"])(html));
        } //$("#edit-stra-name").val(ruleunit.straname);
        // 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
        // 先获取资产和分组信息


        http.get("/getGroupsAndAsserts").fail(function (err) {
          console.log("getGroupsAndAsserts err:", err);
          (0, _jquery["default"])("#edit-rule-msg").text("获取资产和分组失败");
        }).done(function (res) {
          console.log("getGroupsAndAsserts res:", res);

          var info = _jquery["default"].parseJSON(res);

          if (!info.succeed) {
            console.log("getGroupsAndAsserts err:", res);
            return (0, _jquery["default"])("#edit-rule-msg").text("获取资产和分组失败：" + info.error);
          }

          info = info.data;
          vuedata = getVuedataByAssertsAndGroups(info);

          if (!vue_edit_choose_asserts) {
            vue_edit_choose_asserts = new Vue({
              template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
              el: "#vue-app-edit-div",
              methods: {
                getCheckedNodes: function getCheckedNodes() {
                  return this.$refs.tree.getCheckedNodes();
                },
                setCheckedKeys: function setCheckedKeys(data) {
                  this.$refs.tree.setCheckedKeys(data);
                }
              },
              data: {
                data: vuedata,
                defaultProps: {
                  children: 'children',
                  label: 'label'
                }
              },
              watch: {
                data: function data(newdata, olddata) {//
                }
              }
            });
          }

          vue_edit_choose_asserts.data = vuedata;
          var checkedAsserts = ruleunit.applyto.split(";");
          vue_edit_choose_asserts.setCheckedKeys(checkedAsserts);
        });
      });
    });
    (0, _jquery["default"])("#edit-rule-confirm").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var choosedAssertsAndGroups = vue_edit_choose_asserts.getCheckedNodes();
      var choosedAsserts = [];

      for (var i = 0; i < choosedAssertsAndGroups.length; i++) {
        if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
          choosedAsserts.push(choosedAssertsAndGroups[i].id);
        }
      }

      if (choosedAsserts.length < 1) {
        return (0, _jquery["default"])("#edit-rule-msg").text("请至少选择一个资产");
      }

      var assertsstr = choosedAsserts.join(";");
      var name = (0, _jquery["default"])("#edit-rule-name").val();

      if (!name || name.length < 1 || name.length > 32) {
        return (0, _jquery["default"])("#edit-rule-msg").text("请输入合法的名称，最大32字符");
      }

      var straid = (0, _jquery["default"])("#edit-stra-name option:selected").val();
      var http = cockpit.http({
        "port": 9091
      });
      http.post("/editRule", {
        id: ruleid,
        type: "user",
        name: name,
        straid: straid,
        asserts: assertsstr
      }).fail(function (err) {
        console.log("editRule err:", err);
        (0, _jquery["default"])("#edit-rule-msg").text("编辑规则失败");
      }).done(function (res) {
        console.log("editRule res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("editRule err:", res);
          return (0, _jquery["default"])("#edit-rule-msg").text("编辑规则失败：" + info.error);
        }

        (0, _jquery["default"])("#edit-rule-msg").text("编辑规则成功");
        setTimeout(function (e) {
          (0, _jquery["default"])("#edit-rule-close").click();
          render_rules();
        }, 1500);
      });
    });
    (0, _jquery["default"])("button.user-rule-del").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var ruleunit = getRuleById(ruleid);
      var ruletype = ruleunit.type;
      (0, _jquery["default"])("#confirm-del-user-rule-msg").text("确认要删除规则 " + ruleunit.name + " 吗？");
      (0, _jquery["default"])("button#confirm-del-user-rule-confirm").unbind("click").click(function (e) {
        (0, _jquery["default"])("#confirm-del-user-rule-msg").text("规则正在删除，请耐心等待");
        var http = cockpit.http({
          "port": 9091
        });
        http.post("/delRule", {
          type: ruletype,
          id: ruleid
        }).fail(function (err) {
          console.log("delRule err:", err);
          (0, _jquery["default"])("#confirm-del-user-rule-msg").text("删除规则失败");
        }).done(function (res) {
          console.log("getUsers res:", res);

          var info = _jquery["default"].parseJSON(res);

          if (!info.succeed) {
            console.log("getUsers err:", res);
            (0, _jquery["default"])("#confirm-del-user-rule-msg").text("删除规则失败：" + info.error);
            return;
          }

          (0, _jquery["default"])("#confirm-del-user-rule-msg").text("删除规则成功");
          setTimeout(function (e) {
            (0, _jquery["default"])("#confirm-del-user-rule-close").click();
            render_rules();
          }, 1500);
        });
      });
    });
  }

  function render_rules() {
    console.log("render_rules");
    (0, _jquery["default"])(".user-rule-popup-menu").css("display", "none");
    (0, _jquery["default"])("#loading-fallback").show();
    var http = cockpit.http({
      "port": 9091
    });
    http.get("/getRules", {
      "type": "all"
    }).fail(function (err) {
      console.log("getRules err:", err);
      (0, _jquery["default"])("#loading-fallback").hide();
    }).done(function (res) {
      console.log("getRules res:", res);

      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        console.log("getRules err:", res);
        return (0, _jquery["default"])("#loading-fallback").hide();
      }

      info = info.data;
      g_rules_bak = info; // 规则里面还要展示应用到的资产，需要获取资产详细信息

      http.get("/getAsserts").fail(function (err) {
        console.log("getAsserts err:", err);
        return (0, _jquery["default"])("#loading-fallback").hide();
      }).done(function (res) {
        console.log("getAsserts res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("getAsserts err:", res);
          (0, _jquery["default"])("#loading-fallback").hide();
          return;
        }

        info = info.data;
        g_asserts_bak = info;
        show_rules_page(g_rules_bak);
        registerRulesPage();
        (0, _jquery["default"])("#loading-fallback").hide();
      });
    });
  }

  (0, _jquery["default"])('#services-text-filter').change(function (e) {
    show_rules_page(g_rules_bak);
  });

  function changeAddRuleType() {
    var addRuleType = (0, _jquery["default"])("#add-rule-type option:selected").val();

    if (addRuleType == "user") {
      var http = cockpit.http({
        "port": 9091
      });
      http.get("/getUsers").fail(function (err) {
        console.log("getUsers err:", err);
        show_error_dialog("错误", "获取账号策略失败");
      }).done(function (res) {
        console.log("getUsers res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("getUsers err:", res);
          show_error_dialog("错误", "获取账号策略失败：" + info.error);
        }

        info = info.data;
        g_users_bak = info;
        (0, _jquery["default"])("#add-stra-name").html("");

        for (var i = 0; i < g_users_bak.length; i++) {
          var html = '<option value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
          (0, _jquery["default"])("#add-stra-name").append((0, _jquery["default"])(html));
        }
      });
    } else if (addRuleType == "repo") {
      var _http = cockpit.http({
        "port": 9091
      });

      _http.get("/getRepoStrategies").fail(function (err) {
        console.log("getRepoStrategies err:", err);
        show_error_dialog("错误", "获取软件源策略失败");
      }).done(function (res) {
        console.log("getRepoStrategies res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("getRepoStrategies err:", res);
          show_error_dialog("错误", "获取软件源策略失败：" + info.error);
        }

        info = info.data;
        g_repos_bak = info;
        (0, _jquery["default"])("#add-stra-name").html("");

        for (var i = 0; i < g_repos_bak.length; i++) {
          var html = '<option value="' + g_repos_bak[i].id + '">' + g_repos_bak[i].name + '</option>';
          (0, _jquery["default"])("#add-stra-name").append((0, _jquery["default"])(html));
        }
      });
    } else if (addRuleType == "update") {
      var _http2 = cockpit.http({
        "port": 9091
      });

      _http2.get("/getUpdateStrategies").fail(function (err) {
        console.log("getUpdateStrategies err:", err);
        show_error_dialog("错误", "获取升级策略失败");
      }).done(function (res) {
        console.log("getUpdateStrategies res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("getUpdateStrategies err:", res);
          show_error_dialog("错误", "获取升级策略失败：" + info.error);
        }

        info = info.data;
        g_updates_bak = info;
        (0, _jquery["default"])("#add-stra-name").html("");

        for (var i = 0; i < g_updates_bak.length; i++) {
          var html = '<option value="' + g_updates_bak[i].id + '">' + g_updates_bak[i].name + '</option>';
          (0, _jquery["default"])("#add-stra-name").append((0, _jquery["default"])(html));
        }
      });
    } else if (addRuleType == "security") {
      (0, _jquery["default"])("#add-stra-name").html("");
      var html = '<option value="default">默认策略</option>';
      (0, _jquery["default"])("#add-stra-name").append((0, _jquery["default"])(html));
    }
  }

  (0, _jquery["default"])("#add-rule-type").change(changeAddRuleType);
  (0, _jquery["default"])("#add-rule").unbind("click").click(function (e) {
    (0, _jquery["default"])("#add-stra-name").html("");
    (0, _jquery["default"])("#add-rule-name").val("");
    (0, _jquery["default"])("#add-rule-msg").text("");
    changeAddRuleType();
    var http = cockpit.http({
      "port": 9091
    }); // 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
    // 先获取资产和分组信息

    http.get("/getGroupsAndAsserts").fail(function (err) {
      console.log("getGroupsAndAsserts err:", err);
      (0, _jquery["default"])("#add-rule-msg").text("获取资产和分组失败");
    }).done(function (res) {
      console.log("getGroupsAndAsserts res:", res);

      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        console.log("getGroupsAndAsserts err:", res);
        return (0, _jquery["default"])("#add-rule-msg").text("获取资产和分组失败：" + info.error);
      }

      info = info.data;
      vuedata = getVuedataByAssertsAndGroups(info);

      if (!vue_choose_asserts) {
        vue_choose_asserts = new Vue({
          template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
          el: "#vue-app-div",
          methods: {
            getCheckedNodes: function getCheckedNodes() {
              return this.$refs.tree.getCheckedNodes();
            },
            setCheckedKeys: function setCheckedKeys(data) {
              this.$refs.tree.setCheckedKeys(data);
            }
          },
          data: {
            data: vuedata,
            defaultProps: {
              children: 'children',
              label: 'label'
            }
          },
          watch: {
            data: function data(newdata, olddata) {//
            }
          }
        });
      }

      vue_choose_asserts.data = vuedata;
    });
  }); // 确认添加规则时的操作

  (0, _jquery["default"])("#add-rule-confirm").unbind("click").click(function (e) {
    var choosedAssertsAndGroups = vue_choose_asserts.getCheckedNodes();
    var choosedAsserts = [];

    for (var i = 0; i < choosedAssertsAndGroups.length; i++) {
      if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
        choosedAsserts.push(choosedAssertsAndGroups[i].id);
      }
    }

    if (choosedAsserts.length < 1) {
      return (0, _jquery["default"])("#add-rule-msg").text("请至少选择一个资产");
    }

    var assertsstr = choosedAsserts.join(";");
    var name = (0, _jquery["default"])("#add-rule-name").val();

    if (!name || name.length < 1 || name.length > 32) {
      return (0, _jquery["default"])("#add-rule-msg").text("请输入合法的名称，最大32字符");
    }

    var straid = (0, _jquery["default"])("#add-stra-name option:selected").val();

    if (!straid || straid.length < 1) {
      return (0, _jquery["default"])("#add-rule-msg").text("请选择合法的策略");
    }

    var ruletype = (0, _jquery["default"])("#add-rule-type option:selected").val();
    var http = cockpit.http({
      "port": 9091
    });
    (0, _jquery["default"])("#add-rule-msg").text("规则正在添加，请耐心等待");
    http.post("/addRule", {
      type: ruletype,
      name: name,
      straid: straid,
      asserts: assertsstr
    }).fail(function (err) {
      console.log("addRule err:", err);
      (0, _jquery["default"])("#add-rule-msg").text("添加规则失败");
    }).done(function (res) {
      console.log("addRule res:", res);

      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        console.log("addRule err:", res);
        return (0, _jquery["default"])("#add-rule-msg").text("添加规则失败：" + info.error);
      }

      (0, _jquery["default"])("#add-rule-msg").text("添加规则成功");
      setTimeout(function (e) {
        (0, _jquery["default"])("#add-rule-close").click();
        render_rules();
      }, 1500);
    });
  });
  render_rules();
}

if (window.location.pathname.indexOf("manager/rules.html") > 0) {
  (0, _jquery["default"])(rules_init);
}