"use strict";

var _jquery = _interopRequireDefault(require("jquery"));

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _mustache = _interopRequireDefault(require("mustache"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2013 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */
//import cockpit from 'cockpit';
var _ = cockpit.gettext;
var C_ = cockpit.gettext;
var g_security_bak = [];
var g_rules_bak = [];
var g_asserts_bak = [];
var intv = 0; // 定时器全局变量

function show_error_dialog(title, message) {
  if (message) {
    (0, _jquery["default"])("#error-popup-title").text(title);
    (0, _jquery["default"])("#error-popup-message").text(message);
  } else {
    (0, _jquery["default"])("#error-popup-title").text(_("Error"));
    (0, _jquery["default"])("#error-popup-message").text(title);
  }

  (0, _jquery["default"])("#show-error-dialog").click();
}

function security_init() {
  var security_empty_tmpl = (0, _jquery["default"])("#security-empty-tmpl").html();

  _mustache["default"].parse(security_empty_tmpl);

  var rules_list_tmpl = (0, _jquery["default"])("#rules-list-tmpl").html();

  _mustache["default"].parse(rules_list_tmpl); // 分页界面的相关变量放在这做成全局变量


  var g_vue_pagination_security;
  var g_cur_page_security = 1;
  var g_page_size_security = 20;

  function processPagination(units) {
    // 分页展示
    if (!g_vue_pagination_security) {
      g_vue_pagination_security = new Vue({
        template: '<el-pagination @size-change="handleSizeChange" @current-change="handleCurrentChange" :current-page="current_page" :page-sizes="[20, 40, 60, 80]" :page-size="page_size" layout="total, sizes, prev, pager, next, jumper" :total="total"> </el-pagination>',
        el: "#security-vue-app-div",
        methods: {
          handleSizeChange: function handleSizeChange(val) {
            g_page_size_security = val;
            console.log("set g_page_size_security to:", val);
            show_rules_page();
            registerRulesPage();
          },
          handleCurrentChange: function handleCurrentChange(val) {
            g_cur_page_security = val;
            console.log("set g_cur_page_security to:", val);
            show_rules_page();
            registerRulesPage();
          }
        },
        data: {
          current_page: g_cur_page_security,
          page_size: g_page_size_security,
          total: units.length
        }
      });
    } else {
      g_vue_pagination_security.current_page = g_cur_page_security;
      g_vue_pagination_security.page_size = g_page_size_security;
      g_vue_pagination_security.total = units.length;
    }

    units = units.slice((g_cur_page_security - 1) * g_page_size_security, g_cur_page_security * g_page_size_security);
    return units;
  }

  function getAssertById(assertid) {
    for (var i = 0; i < g_asserts_bak.length; i++) {
      if (g_asserts_bak[i].assertid == assertid) {
        return g_asserts_bak[i];
      }
    }

    return false;
  }

  function getRuleById(id) {
    for (var i = 0; i < g_rules_bak.length; i++) {
      if (g_rules_bak[i].id == id) {
        return g_rules_bak[i];
      }
    }

    return false;
  }

  function editSecurity(e) {
    function disable_edit() {
      (0, _jquery["default"])("#security-fail-cnt").prop("disabled", true);
      (0, _jquery["default"])("#security-lock-time").prop("disabled", true);
      (0, _jquery["default"])("#security-pass-complex").prop("disabled", true);
      (0, _jquery["default"])("#security-disable-udisk").prop("disabled", true);
      (0, _jquery["default"])("#security-password-expire").prop("disabled", true);
      (0, _jquery["default"])("#selinux-type").prop("disabled", true);
      (0, _jquery["default"])("#security-session-timeout").prop("disabled", true);
      (0, _jquery["default"])("#security-command").prop("disabled", true);
    }

    function enable_edit() {
      (0, _jquery["default"])("#security-fail-cnt").prop("disabled", false);
      (0, _jquery["default"])("#security-lock-time").prop("disabled", false);
      (0, _jquery["default"])("#security-pass-complex").prop("disabled", false);
      (0, _jquery["default"])("#security-disable-udisk").prop("disabled", false);
      (0, _jquery["default"])("#security-password-expire").prop("disabled", false);
      (0, _jquery["default"])("#selinux-type").prop("disabled", false);
      (0, _jquery["default"])("#security-session-timeout").prop("disabled", false);
      (0, _jquery["default"])("#security-command").prop("disabled", false);
    } // 改变 checkbox 的 disabled 属性时也会触发 change 事件，调用 setup 的时候也会，这里过滤掉


    if ((0, _jquery["default"])('#security-fail-cnt').prop("disabled") || (0, _jquery["default"])('#security-lock-time').prop("disabled") || (0, _jquery["default"])('#security-pass-complex').prop("disabled") || (0, _jquery["default"])('#security-disable-udisk').prop("disabled") || (0, _jquery["default"])('#security-password-expire').prop("disabled") || (0, _jquery["default"])('#selinux-type').prop("disabled") || (0, _jquery["default"])('#security-session-timeout').prop("disabled") || (0, _jquery["default"])("#security-command").prop("disabled")) {
      return;
    }

    disable_edit();
    var failcnt = (0, _jquery["default"])("#security-fail-cnt").val();
    var locktime = (0, _jquery["default"])("#security-lock-time").val();
    var passcomplex = (0, _jquery["default"])("#security-pass-complex").prop("checked");

    if (passcomplex) {
      passcomplex = 1;
    } else {
      passcomplex = 0;
    }

    var disableudisk = (0, _jquery["default"])("#security-disable-udisk").prop("checked");

    if (disableudisk) {
      disableudisk = 1;
    } else {
      disableudisk = 0;
    }

    var passexpired = (0, _jquery["default"])("#security-password-expire").val();
    var selinuxcfg = (0, _jquery["default"])("#selinux-type").val();
    var sessiontimeout = (0, _jquery["default"])("#security-session-timeout").val();
    var security_command = (0, _jquery["default"])("#security-command").val();
    var http = cockpit.http({
      "port": 9091
    });
    http.post("/editSecurity", {
      "loginFailedCnt": failcnt,
      "loginFailedLockTime": locktime,
      "passwordComplex": passcomplex,
      "disableUdisk": disableudisk,
      "passwordExpired": passexpired,
      "selinuxConfig": selinuxcfg,
      "sessionTimeout": sessiontimeout,
      "securityCommand": security_command
    }).fail(function (err) {
      console.log("editSecurity err:", err);
      enable_edit();
      return show_error_dialog("错误", "设置安全策略失败");
    }).done(function (res) {
      console.log("editSecurity res:", res);

      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        enable_edit();
        show_security_page();
        return show_error_dialog("错误", "设置安全策略失败：" + info.error);
      }

      setTimeout(function () {
        render_security();
        enable_edit();
      }, function (e) {
        show_error_dialog("错误", "刷新数据失败");
      }, 1500);
    });
  }

  function show_security_page() {
    (0, _jquery["default"])("#security-fail-cnt").val(g_security_bak.loginFailedCnt);
    (0, _jquery["default"])("#security-lock-time").val(g_security_bak.loginFailedLockTime);

    if (g_security_bak.passwordComplex) {
      (0, _jquery["default"])("#security-pass-complex").prop("checked", true);
    } else {
      (0, _jquery["default"])("#security-pass-complex").prop("checked", false);
    }

    if (g_security_bak.disableUdisk) {
      (0, _jquery["default"])("#security-disable-udisk").prop("checked", true);
    } else {
      (0, _jquery["default"])("#security-disable-udisk").prop("checked", false);
    }

    (0, _jquery["default"])("#security-password-expire").val(g_security_bak.passwordExpired);
    (0, _jquery["default"])("#selinux-type").val(g_security_bak.selinuxConfig);
    (0, _jquery["default"])("#security-session-timeout").val(g_security_bak.sessionTimeout);
    (0, _jquery["default"])("#security-command").val(g_security_bak.securityCommand); // 编辑提交

    (0, _jquery["default"])("#security-fail-cnt").change(editSecurity);
    (0, _jquery["default"])("#security-lock-time").change(editSecurity);
    (0, _jquery["default"])("#security-pass-complex").change(editSecurity);
    (0, _jquery["default"])("#security-disable-udisk").change(editSecurity);
    (0, _jquery["default"])("#security-password-expire").change(editSecurity);
    (0, _jquery["default"])("#selinux-type").change(editSecurity);
    (0, _jquery["default"])("#security-session-timeout").change(editSecurity);
    (0, _jquery["default"])("#security-command").change(editSecurity);
  }

  function render_security() {
    // 如果是从账户详情页面跳回，需要先隐藏掉账户详情页面
    (0, _jquery["default"])("#rules").hide();
    (0, _jquery["default"])("#security").show();
    (0, _jquery["default"])("#loading-fallback").show();
    var http = cockpit.http({
      "port": 9091
    });
    http.get("/getSecurity").fail(function (err) {
      console.log("getSecurity err:", err);
      (0, _jquery["default"])("#loading-fallback").hide();
    }).done(function (res) {
      console.log("getSecurity res:", res);

      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        console.log("getSecurity err:", res);
        (0, _jquery["default"])("#loading-fallback").hide();
        return;
      }

      info = _jquery["default"].parseJSON(info.data.content);
      g_security_bak = info;
      show_security_page();
      (0, _jquery["default"])("#loading-fallback").hide();
    });
  }

  function registerRulesPage() {
    // 点击下发按钮触发后台下发
    (0, _jquery["default"])("button.download-rule-btn").unbind("click").click(function (e) {
      var ruleid = (0, _jquery["default"])(this).data("rule-id");
      var ruleunit = getRuleById(ruleid);
      var http = cockpit.http({
        "port": 9091
      });
      (0, _jquery["default"])("#download-rule-btn-msg").text("确认要重新下发规则 " + ruleunit.name + " 吗？");
      (0, _jquery["default"])("#download-rule-btn-confirm").unbind("click").click(function (e) {
        http.post("/downloadRule", {
          type: "security",
          id: ruleid
        }).fail(function (err) {
          console.log("downloadRule err:", err);
          (0, _jquery["default"])("#download-rule-btn-msg").text("通知后台下发失败");
        }).done(function (res) {
          var info = _jquery["default"].parseJSON(res);

          if (!info.succeed) {
            console.log("downloadRule err:", res);
            (0, _jquery["default"])("#download-rule-btn-msg").text("通知后台下发失败：" + info.error);
            return;
          }

          (0, _jquery["default"])("#download-rule-btn-msg").text("已通知后台下发");
          setTimeout(function (e) {
            (0, _jquery["default"])("#download-rule-btn-close").click();
            render_rules();
          }, 1500);
        });
      });
    }); // 点击规则行上的更多按钮时，弹出菜单，需要在渲染完成后再绑定事件

    (0, _jquery["default"])(".user-rule-opr-more").unbind("click").on("click", function (e) {
      var top = (0, _jquery["default"])(this).offset().top;
      var left = (0, _jquery["default"])(this).offset().left;
      var top1 = (0, _jquery["default"])("html").scrollTop();
      top = top - top1;
      var width = parseInt((0, _jquery["default"])(this).css("width").replace("px", ""));
      (0, _jquery["default"])(".user-rule-popup-menu").css("top", top);
      (0, _jquery["default"])(".user-rule-popup-menu").css("left", left + width);

      if ((0, _jquery["default"])(".user-rule-popup-menu").css("display") == "none") {
        (0, _jquery["default"])(".user-rule-popup-menu").css("display", "inline-grid");
      } else {
        (0, _jquery["default"])(".user-rule-popup-menu").css("display", "none");
      }

      var ruldid = (0, _jquery["default"])(this).data("rule-id");
      (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data", ruldid);
    });
    (0, _jquery["default"])(".user-rule-popup-btn").mouseover(function (e) {
      (0, _jquery["default"])(this).css("background-color", "#def3ff");
    }).mouseout(function (e) {
      (0, _jquery["default"])(this).css("background-color", "white");
    });
    (0, _jquery["default"])(window).unbind("click").on("click", function (e) {
      if (e.target.localName == "button" && (e.target.className.indexOf("user-rule-opr-more") >= 0 || e.target.className.indexOf("user-rule-del") >= 0 || e.target.className.indexOf("user-rule-status") >= 0 || e.target.className.indexOf("user-rule-edit") >= 0)) {
        return;
      }

      (0, _jquery["default"])(".user-rule-popup-menu").css("display", "none");
    });
    (0, _jquery["default"])("button.user-rule-status").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var ruleunit = getRuleById(ruleid);

      try {
        (0, _jquery["default"])("#rule-status-content").html("");

        var applycount = _jquery["default"].parseJSON(ruleunit.applycount);

        Object.keys(applycount).forEach(function (key) {
          // 渲染规则页面的时候已经获取过资产信息，这里直接用，不重新请求了
          var asserttmp = getAssertById(key);

          if (!asserttmp) {
            console.log("[user-rule-status]get assert failed,assertid:", applycount[key]);
            return;
          }

          var statustext = "";

          if (applycount[key]["applycount"] < ruleunit.editcount) {
            statustext = "尚未下发";
          } else {
            if (applycount[key]["result"] == "succeed") {
              statustext = "下发成功";
            } else {
              statustext = "下发失败";
            }
          }

          var tr = "<tr><td class='text-align-left'>" + asserttmp.assertip + "</td><td>" + asserttmp.assertname + "</td><td><span class=''>" + statustext + "</span>" + "</td><td><buttom data-toggle='modal' class='btn btn-default securityrules-status'" + " data-target='#securityrules-status-dialog' securityrules-assert-id='" + key + "'>查看详情</buttom>" + "</td></tr>";
          (0, _jquery["default"])("#rule-status-content").append((0, _jquery["default"])(tr));
        });
        (0, _jquery["default"])("buttom.securityrules-status").unbind("click").click(function (e) {
          var buttomid = (0, _jquery["default"])(this).attr("securityrules-assert-id");
          (0, _jquery["default"])("#securityrules-status-msg").text(applycount[buttomid]["result"]);
        });
      } catch (e) {
        console.log(e);
      }
    });
    (0, _jquery["default"])("button.user-rule-edit").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var ruleunit = getRuleById(ruleid);
      (0, _jquery["default"])("#edit-rule-name").val(ruleunit.name);
      var http = cockpit.http({
        "port": 9091
      });
      http.get("/getUsers").fail(function (err) {
        console.log("getUsers err:", err);
        (0, _jquery["default"])("#edit-rule-msg").text("获取账号策略失败");
      }).done(function (res) {
        console.log("getUsers res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("getUsers err:", res);
          return (0, _jquery["default"])("#edit-rule-msg").text("获取账号策略失败：" + info.error);
        }

        info = info.data;
        (0, _jquery["default"])("#edit-stra-name").html("");
        var html = '<option value="default">' + '默认策略' + '</option>';
        (0, _jquery["default"])("#edit-stra-name").append((0, _jquery["default"])(html)); //$("#edit-stra-name").val(ruleunit.straname);
        // 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
        // 先获取资产和分组信息

        http.get("/getGroupsAndAsserts").fail(function (err) {
          console.log("getGroupsAndAsserts err:", err);
          (0, _jquery["default"])("#edit-rule-msg").text("获取资产和分组失败");
        }).done(function (res) {
          // console.log("getGroupsAndAsserts res:", res);
          var info = _jquery["default"].parseJSON(res);

          if (!info.succeed) {
            console.log("getGroupsAndAsserts err:", res);
            return (0, _jquery["default"])("#edit-rule-msg").text("获取资产和分组失败：" + info.error);
          }

          info = info.data;
          vuedata = getVuedataByAssertsAndGroups(info);

          if (!vue_edit_choose_asserts) {
            vue_edit_choose_asserts = new Vue({
              template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
              el: "#vue-app-edit-div",
              methods: {
                getCheckedNodes: function getCheckedNodes() {
                  return this.$refs.tree.getCheckedNodes();
                },
                setCheckedKeys: function setCheckedKeys(data) {
                  this.$refs.tree.setCheckedKeys(data);
                }
              },
              data: {
                data: vuedata,
                defaultProps: {
                  children: 'children',
                  label: 'label'
                }
              },
              watch: {
                data: function data(newdata, olddata) {//
                }
              }
            });
          }

          vue_edit_choose_asserts.data = vuedata;
          var checkedAsserts = ruleunit.applyto.split(";");
          vue_edit_choose_asserts.setCheckedKeys(checkedAsserts);
        });
      });
    });
    (0, _jquery["default"])("#edit-rule-confirm").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var choosedAssertsAndGroups = vue_edit_choose_asserts.getCheckedNodes();
      var choosedAsserts = [];

      for (var i = 0; i < choosedAssertsAndGroups.length; i++) {
        if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
          choosedAsserts.push(choosedAssertsAndGroups[i].id);
        }
      }

      if (choosedAsserts.length < 1) {
        return (0, _jquery["default"])("#edit-rule-msg").text("请至少选择一个资产");
      }

      var assertsstr = choosedAsserts.join(";");
      var name = (0, _jquery["default"])("#edit-rule-name").val();

      if (!name || name.length < 1 || name.length > 32) {
        return (0, _jquery["default"])("#edit-rule-msg").text("请输入合法的名称，最大32字符");
      }

      var straid = (0, _jquery["default"])("#edit-stra-name option:selected").val();
      var http = cockpit.http({
        "port": 9091
      });
      http.post("/editRule", {
        id: ruleid,
        type: "security",
        name: name,
        straid: straid,
        asserts: assertsstr
      }).fail(function (err) {
        console.log("editRule err:", err);
        (0, _jquery["default"])("#edit-rule-msg").text("编辑规则失败");
      }).done(function (res) {
        console.log("editRule res:", res);

        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("editRule err:", res);
          return (0, _jquery["default"])("#edit-rule-msg").text("编辑规则失败：" + info.error);
        }

        (0, _jquery["default"])("#edit-rule-msg").text("编辑规则成功");
        setTimeout(function (e) {
          (0, _jquery["default"])("#edit-rule-close").click();
          render_rules();
        }, 1500);
      });
    });
    (0, _jquery["default"])("button.user-rule-del").unbind("click").click(function (e) {
      // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
      var ruleid = (0, _jquery["default"])("#user-rule-popup-menu-private-data").data("private-data");
      var ruleunit = getRuleById(ruleid);
      (0, _jquery["default"])("#confirm-del-user-rule-msg").text("确认要删除规则 " + ruleunit.name + " 吗？");
      (0, _jquery["default"])("button#confirm-del-user-rule-confirm").unbind("click").click(function (e) {
        var http = cockpit.http({
          "port": 9091
        });
        http.post("/delRule", {
          type: "security",
          id: ruleid
        }).fail(function (err) {
          console.log("delRule err:", err);
          (0, _jquery["default"])("#confirm-del-user-rule-msg").text("删除规则失败");
        }).done(function (res) {
          console.log("getUsers res:", res);

          var info = _jquery["default"].parseJSON(res);

          if (!info.succeed) {
            console.log("getUsers err:", res);
            (0, _jquery["default"])("#confirm-del-user-rule-msg").text("删除规则失败：" + info.error);
            return;
          }

          (0, _jquery["default"])("#confirm-del-user-rule-msg").text("删除规则成功");
          setTimeout(function (e) {
            (0, _jquery["default"])("#confirm-del-user-rule-close").click();
            render_rules();
          }, 1500);
        });
      });
    });
  }

  function show_rules_page() {
    if (g_rules_bak.length < 1) {
      var html = _mustache["default"].render(security_empty_tmpl, {
        empty_text: "尚未创建规则"
      });

      return (0, _jquery["default"])("#rules-list").html(html);
    }

    var header = {
      Name: "规则名称",
      Straname: "策略名称",
      Applyto: "应用到",
      Status: "规则状态",
      Operate: "操作"
    };
    var units = [];

    for (var i = 0; i < g_rules_bak.length; i++) {
      var unit = g_rules_bak[i];
      var applytotext = "";
      var assertsarr = unit.applyto.split(";");
      var deleted = [];

      for (var _i = 0; _i < assertsarr.length; _i++) {
        if (!assertsarr[_i] || assertsarr[_i].length < 1) {
          continue;
        }

        var asserttmp = getAssertById(assertsarr[_i]);

        if (!asserttmp.assertip) {
          // 过滤掉已删除的资产
          deleted.push(assertsarr[_i]);
          continue;
        }

        applytotext = applytotext + "资产：" + asserttmp.assertip + "-" + asserttmp.assertname + ";";
      }

      unit.applytotext = applytotext;

      try {
        unit.straname = "默认策略";
      } catch (e) {
        console.log(e);
      }

      if (unit.applycount.length < 1 || unit.status == "redownload") {
        unit.status = "下发中";
      } else {
        try {
          var applyjson = _jquery["default"].parseJSON(unit.applycount);

          var keys = Object.keys(applyjson);
          var flg = true;

          for (var _i2 = 0; _i2 < keys.length; _i2++) {
            if (applyjson[keys[_i2]]["applycount"] < unit.editcount) {
              // 说明还是上次的下发结果
              unit.status = "尚未下发";
              break;
            }

            if (applyjson[keys[_i2]]["result"] != "succeed") {
              // 判断失败的是否为删除的如果在删除的里面继续循环
              if (_jquery["default"].inArray(keys[_i2], deleted) == -1) {
                unit.status = "下发失败";
                flg = false;
                break;
              }
            }
          } // 如果全部都成功，才认为下发成功


          if (!unit.status || unit.status.length < 1 || flg) {
            unit.status = "下发成功";
          }
        } catch (e) {
          console.log(e);
        }
      }

      unit.ruleid = g_rules_bak[i].id;
      units.push(unit);
    } // 处理分页


    units = processPagination(units);

    var text = _mustache["default"].render(rules_list_tmpl, {
      table_head: header,
      units: units
    });

    (0, _jquery["default"])("#rules-list").html(text);
  }

  var vue_choose_asserts;
  var vue_edit_choose_asserts;
  var vuedata;

  function getVuedataByAssertsAndGroups(info) {
    vuedata = [];
    var allGroupAsserts = [];
    var allasserts = info["allasserts"];

    function getAssertsById(id) {
      for (var i = 0; i < allasserts.length; i++) {
        if (allasserts[i].assertid == id) {
          return allasserts[i];
        }
      }

      return {};
    }

    for (var i = 0; i < info["allgroups"].length; i++) {
      var grouptmp = info["allgroups"][i];
      var unit = {
        id: grouptmp.groupid,
        label: "分组：" + grouptmp.groupname,
        children: []
      };
      var tmparr = grouptmp.groupasserts.split(";");

      for (var k = 0; k < tmparr.length; k++) {
        if (!tmparr[k] || tmparr[k].length < 1) {
          continue;
        }

        var asserttmp = getAssertsById(tmparr[k]);

        if (asserttmp.assertip == "localhost") {
          continue;
        }

        if (!asserttmp.assertip) {
          // 过滤掉已经删除的
          continue;
        }

        unit.children.push({
          id: tmparr[k],
          label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
        });

        if (allGroupAsserts.indexOf(tmparr[k]) < 0) {
          allGroupAsserts.push(tmparr[k]);
        }
      }

      if (unit.children.length == 0) {
        continue;
      }

      vuedata.push(unit);
    }

    for (var _i3 = 0; _i3 < info["allasserts"].length; _i3++) {
      var _asserttmp = info["allasserts"][_i3];

      if (allGroupAsserts.indexOf(_asserttmp.assertid) >= 0) {
        continue;
      }

      if (_asserttmp.assertip == "localhost") {
        continue;
      }

      var _unit = {
        id: _asserttmp.assertid,
        label: "资产：" + _asserttmp.assertip + "-" + _asserttmp.assertname
      };
      vuedata.push(_unit);
    }

    return vuedata;
  }

  function render_rules() {
    (0, _jquery["default"])("#security").hide();
    (0, _jquery["default"])("#rules").show();
    (0, _jquery["default"])("#add-rule").show();
    (0, _jquery["default"])(".user-rule-popup-menu").css("display", "none");
    (0, _jquery["default"])("#loading-fallback").show();
    clearInterval(intv);
    var http = cockpit.http({
      "port": 9091
    });
    http.get("/getRules", {
      "type": "security"
    }).fail(function (err) {
      console.log("getRules of security err:", err);
      (0, _jquery["default"])("#loading-fallback").hide();
    }).done(function (res) {
      console.log("getRules of security res:", res);

      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        console.log("getRules of security err:", res);
        return (0, _jquery["default"])("#loading-fallback").hide();
      }

      info = info.data;
      g_rules_bak = info; // 规则里面还要展示应用到的资产，需要获取资产详细信息

      http.get("/getAsserts").fail(function (err) {
        console.log("getAsserts err:", err);
        return (0, _jquery["default"])("#loading-fallback").hide();
      }).done(function (res) {
        // console.log("getAsserts res:", res);
        var info = _jquery["default"].parseJSON(res);

        if (!info.succeed) {
          console.log("getAsserts err:", res);
          (0, _jquery["default"])("#loading-fallback").hide();
          return;
        }

        info = info.data;
        g_asserts_bak = info;
        show_rules_page();
        registerRulesPage();
        (0, _jquery["default"])("#loading-fallback").hide(); // 定时器获取规则状态

        var dlsucceed = /\u4e0b\u53d1\u4e2d/;
        intv = setInterval(function () {
          var isdone = true;

          for (var i = 0; i < g_rules_bak.length; i++) {
            if (dlsucceed.test(g_rules_bak[i].status)) {
              isdone = false;
              http.get("/getRuleState", {
                "type": "security"
              }).fail(function (err) {
                console.log("getRules of security err:", err);
              }).done(function (res) {
                var info = _jquery["default"].parseJSON(res);

                if (!info.succeed) {
                  console.log("getRules of security err:", res);
                }

                info = info.data;
                g_rules_bak = info ? info : g_rules_bak;
              });
            } else {
              for (var k in g_rules_bak) {
                var arule = g_rules_bak[k];
                (0, _jquery["default"])("#rule-" + arule["id"]).text(arule["status"]);
              }
            }
          }

          if (isdone) {
            clearInterval(intv);
          }
        }, 5000);
      });
    });
  }

  (0, _jquery["default"])("#add-rule").unbind("click").click(function (e) {
    (0, _jquery["default"])("#add-stra-name").html("");
    (0, _jquery["default"])("#add-rule-name").val("");
    var http = cockpit.http({
      "port": 9091
    });
    (0, _jquery["default"])("#add-stra-name").html("");
    var html = '<option value="default">' + '默认策略' + '</option>';
    (0, _jquery["default"])("#add-stra-name").append((0, _jquery["default"])(html)); // 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
    // 先获取资产和分组信息

    http.get("/getGroupsAndAsserts").fail(function (err) {
      console.log("getGroupsAndAsserts err:", err);
      (0, _jquery["default"])("#add-rule-msg").text("获取资产和分组失败");
    }).done(function (res) {
      // console.log("getGroupsAndAsserts res:", res);
      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        console.log("getGroupsAndAsserts err:", res);
        return (0, _jquery["default"])("#add-rule-msg").text("获取资产和分组失败：" + info.error);
      }

      info = info.data;
      vuedata = getVuedataByAssertsAndGroups(info);

      if (!vue_choose_asserts) {
        vue_choose_asserts = new Vue({
          template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
          el: "#vue-app-div",
          methods: {
            getCheckedNodes: function getCheckedNodes() {
              return this.$refs.tree.getCheckedNodes();
            },
            setCheckedKeys: function setCheckedKeys(data) {
              this.$refs.tree.setCheckedKeys(data);
            }
          },
          data: {
            data: vuedata,
            defaultProps: {
              children: 'children',
              label: 'label'
            }
          },
          watch: {
            data: function data(newdata, olddata) {//
            }
          }
        });
      }

      vue_choose_asserts.data = vuedata;
    });
  }); // 确认添加规则时的操作

  (0, _jquery["default"])("#add-rule-confirm").unbind("click").click(function (e) {
    var choosedAssertsAndGroups = vue_choose_asserts.getCheckedNodes();
    var choosedAsserts = [];

    for (var i = 0; i < choosedAssertsAndGroups.length; i++) {
      if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
        choosedAsserts.push(choosedAssertsAndGroups[i].id);
      }
    }

    if (choosedAsserts.length < 1) {
      return (0, _jquery["default"])("#add-rule-msg").text("请至少选择一个资产");
    }

    var assertsstr = choosedAsserts.join(";");
    var name = (0, _jquery["default"])("#add-rule-name").val();

    if (!name || name.length < 1 || name.length > 32) {
      return (0, _jquery["default"])("#add-rule-msg").text("请输入合法的名称，最大32字符");
    }

    var http = cockpit.http({
      "port": 9091
    });
    http.post("/addRule", {
      type: "security",
      name: name,
      asserts: assertsstr
    }).fail(function (err) {
      console.log("addRule of security err:", err);
      (0, _jquery["default"])("#add-rule-msg").text("添加规则失败");
    }).done(function (res) {
      console.log("addRule of security res:", res);

      var info = _jquery["default"].parseJSON(res);

      if (!info.succeed) {
        console.log("addRule of security err:", res);
        return (0, _jquery["default"])("#add-rule-msg").text("添加规则失败：" + info.error);
      }

      (0, _jquery["default"])("#add-rule-msg").text("添加规则成功");
      setTimeout(function (e) {
        (0, _jquery["default"])("#add-rule-close").click();
        render_rules();
      }, 1500);
    });
  });
  (0, _jquery["default"])('#security-filter li').on('click', function () {
    (0, _jquery["default"])('#security-filter li').removeClass('active').removeAttr('aria-current');
    (0, _jquery["default"])(this).addClass('active').attr('aria-current', true);
    var pattern = (0, _jquery["default"])(this).attr('data-pattern');

    if (pattern == "securityrules") {
      cockpit.location.go(["securityrules"]);
      (0, _jquery["default"])("#add-rule").show();
    } else if (pattern == "security") {
      cockpit.location.go([""]);
      (0, _jquery["default"])("#add-rule").hide();
    }
  }); // 根据 url 初始化渲染页面

  (0, _jquery["default"])("#security-filter li").removeClass('active').removeAttr('aria-current');

  function navigate() {
    if (window.location.href.indexOf("#/securityrules") > 0) {
      (0, _jquery["default"])("#securityrules-li").addClass('active').attr('aria-current', true);
      render_rules();
    } else {
      (0, _jquery["default"])("#security-li").addClass('active').attr('aria-current', true);
      render_security();
    }
  }

  (0, _jquery["default"])(cockpit).on("locationchanged", navigate);
  navigate();
}

if (window.location.pathname.indexOf("manager/security.html") > 0) {
  (0, _jquery["default"])(security_init);
}