import $ from "jquery";
import mustache from "mustache";

//import cockpit from "cockpit";
import moment from "moment";
import getNetworkManager from "./interfaces";
import machines from "./machines";
import * as credentials from "./credentials";
import * as ipfuncs from "./iputils";
import JQMeter from "./jqmeter";
import { writeAuditlog } from './write-auditlog';

function asserts_init() {
    console.log("asserts_init");

    /* Handles an href link to a server */
    $(document).on("click", "tr[class='unit-tr-row']", function(ev) {
      if (ev.target.localName === "button") {
        return ;
      }
        cockpit.jump("/", $(this).attr("data-address"));
        ev.preventDefault();
        return false;
    });

    var units_template = $("#services-units-tmpl").html();
    mustache.parse(units_template);
    var groups_template = $("#groups-units-tmpl").html();
    mustache.parse(groups_template);
    var empty_template = $("#service-empty-tmpl").html();
    mustache.parse(empty_template);
    var setting_template = $("#setting-tmpl").html();
    mustache.parse(setting_template);
    var asserts_detail_template = $("#service-template-tmpl").html();
    mustache.parse(asserts_detail_template);
    var groups_detail_template = $("#group-template-tmpl").html();
    mustache.parse(groups_detail_template);

    let intv=0;
    let g_asserts_bak = [];
    let g_groups_bak = [];
    let g_machines = machines.instance();
    let g_authkeys = credentials.keys_instance();
    $(g_authkeys).on("changed", update_available);
    // 自动发现资产的扫描计数
    let g_snip_finished = 0;
    // 自动发现资产时获取到的资产的信息
    let g_find_assert_info = {};

    // 分页界面的相关变量放在这做成全局变量
    let g_vue_pagination;
    let g_cur_page_asserts = 1;
    let g_cur_page_groups = 1;
    let g_page_size = 20;

    function processPagination(units) {
        // 分页展示
        if (!g_vue_pagination) {
            g_vue_pagination = new Vue({
                template: '<el-pagination @size-change="handleSizeChange" @current-change="handleCurrentChange" :current-page="current_page" :page-sizes="[20, 40, 60, 80]" :page-size="page_size" layout="total, sizes, prev, pager, next, jumper" :total="total"> </el-pagination>',
                el: "#vue-app-div",
                methods: {
                    handleSizeChange(val) {
                        g_page_size = val;
                        console.log("set g_page_size to:", val)

                        if (window.location.href.indexOf("#/groups") > 0) {
                            // group page
                            show_groups_page(g_groups_bak)
                            registerGroupsEvent();
                        } else {
                            // rules page
                            show_asserts_page(g_asserts_bak)
                            registerAssertsEvent()
                        }
                    },
                    handleCurrentChange(val) {
                        if (window.location.href.indexOf("#/groups") > 0) {
                            g_cur_page_groups = val;
                            console.log("set g_cur_page_groups to:", val)
                            show_groups_page(g_groups_bak)
                            registerGroupsEvent();
                        } else {
                            g_cur_page_asserts = val;
                            console.log("set g_cur_page_asserts to:", val)
                            show_asserts_page(g_asserts_bak)
                            registerAssertsEvent()
                        }
                    }
                },
                data: {
                    current_page: g_cur_page_asserts,
                    page_size: g_page_size,
                    total: units.length,
                }
            })
        } else {
            if (window.location.href.indexOf("#/groups") > 0) {
                g_vue_pagination.current_page = g_cur_page_groups;
            } else {
                g_vue_pagination.current_page = g_cur_page_asserts;
            }

            g_vue_pagination.page_size = g_page_size;
            g_vue_pagination.total = units.length;
        }

        if (window.location.href.indexOf("#/groups") > 0) {
            units = units.slice( (g_cur_page_groups-1) * g_page_size,
                g_cur_page_groups * g_page_size )
        } else {
            units = units.slice( (g_cur_page_asserts-1) * g_page_size,
            g_cur_page_asserts * g_page_size )
        }

        return units
    }

    function getUnitById(assertid) {
        let unit = [];
        for (let i=0;i<g_asserts_bak.length;i++) {
            if (g_asserts_bak[i].assertid == assertid) {
                unit = g_asserts_bak[i];
            }
        }
        if (!unit) {
            console.log("can not find unit,record id:", id);
        }
        return unit;
    }
    function updateAssertDesc(id, desc) {
        for (let i=0;i<g_asserts_bak.length;i++) {
            if (g_asserts_bak[i].uuid == id) {
                g_asserts_bak[i].desc = desc
            }
        }
    }

    function try_to_connect(address, options) {
        var dfd = $.Deferred();
        var conn_options = $.extend({ "payload": "echo",
                                      "host": address },
                                    options);

        var machine = g_machines.lookup(address);
        if (machine && machine.host_key && !machine.on_disk) {
            conn_options['temp-session'] = false; /* Compatiblity option */
            conn_options['session'] = 'shared';
            conn_options['host-key'] = machine.host_key;
        }
        var client = cockpit.channel(conn_options);
        client.send("x");
        $(client)
                .on("message", function() {
                    $(client).off();
                    client.close();
                    dfd.resolve();
                })
                .on("close", function(event, options) {
                    dfd.reject(options);
                });

        return dfd.promise();
    };
    function doAddAssert() {
        let ip = $("#add-server-ip").val();
        let name = $("#add-server-name").val();
        let user = $("#add-server-user").val();
        let type = $("#add-server-type").children("option:selected").val();
        let options = {
            "user": user
        };
        var reg = /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/
        if (!reg.test(ip)){
            return $("#add-server-msg").text(_("Please enter legitimate IP address. E.g.: 192.168.0.1"));
        }
        if (!name || name.length < 1 || name.length > 40) {
            return $("#add-server-msg").text(_("Please enter legitimate name, 1-40 characstics. English, Chinese, number or . - _ only."));
        }
        var rename = /[-\.\_a-zA-Z0-9\u4e00-\u9fa5]/

        for (var i = 0; i < name.length; i++){
            if (!rename.test(name[i])){
                return $("#add-server-msg").text(_("Please enter legitimate name, 1-40 characstics. English, Chinese, number or . - _ only."));
            }
        }

        if (g_authkeys.items.length < 1) {
            return $("#add-server-msg").text(_("Management Server does not have credentials. Please generate credential first."));
        }

        let authkeydata = "";
        for (var id in g_authkeys.items) {
            var key = g_authkeys.items[id];
            // 取第一个即可
            authkeydata = key.data;
            if (authkeydata.length > 0) {
                break;
            }
        }

        if (type === "0") {
            // 使用密码登录
            let password = $("#add-server-pass").val();
            options["password"] = password;
            options["session"] = 'shared'; //'private'
            options["temp-session"] = false;
        } else {
            // 使用证书登录
            let credit = $("#credit-identifier").children("option:selected").text();
            if (! credit) {
                return $("#add-server-msg").text(_("Please select a credential."));
            }
        }

        function realAddHost() {

            // 用户的主目录，需要执行命令去获取
            let home = "";

            function getAssertInfoBody(cmd, res) {
                console.log("exec cmd:", cmd, " res:", res);

                let arr = res.split("\n");
                if (arr.length < 2) {
                    return $("#add-server-msg").text(_("Get assert's version and fingerprints failed."));
                }
                let finger = arr[0].trim();
                if (finger.length != 32) {
                    return $("#add-server-msg").text(_("Get fingerprints failed."));
                }
                let tag = "";
                for (let i=1;i<arr.length;i++) {
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.4-") > 0) {
                        tag = "3.4";
                    }
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.3-") > 0) {
                        tag = "3.3";
                    }
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.2-") > 0) {
                        tag = "3.2";
                    }
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.0-") > 0) {
                        tag = "3.0";
                    }
                }
                if (tag.length < 1) {
                    return $("#add-server-msg").text(_("Get version tag information failed."));
                }

                if (tag == "3.0") {
                    let sudoers = "sed -i 's/^Defaults[ ]*requiretty/#&/g' /etc/sudoers";
                    cockpit.script(sudoers, {
                        host: ip
                    })
                    .then(res=>{
                        console.log(ip, "sed sudoers res:", res);
                    })
                    .catch(err=>{
                        console.log(ip, "sed sudoers err:", err);
                    })
                }

                let http = cockpit.http({
                        "port": 9091
                    });
                http.post("/addAssert", {
                    "ip": ip,
                    "name": name,
                    "finger": finger,
                    "tag": tag,
                    "user": user
                }).done(function(res) {
                    console.log("addAssert res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        let cent = "添加资产: " + ip + " 失败, 资产信息: name=" + name + ", tag=" + tag + ", user=" + user + ", 失败原因: " + info.error;
                        writeAuditlog(2, ip, cent);
                        return $("#add-server-msg").text(_("Add assert failed: ") + info.error);
                    }
                    $("#add-server-msg").text(_("Add assert succeed!"));
                    let cent = "添加资产: " + ip + " 成功, 资产信息: name=" + name + ", tag=" + tag + ", user=" + user;
                    writeAuditlog(2, ip, cent);
                    setTimeout(function(e) {
                        $("#add-server-close").click();
                        render_asserts_real();
                    }, 1000)
                }).fail(function(err) {
                    console.log(err);
                    let cent = "添加资产: " + ip + " 失败, 资产信息: name=" + name + ", tag=" + tag + ", user=" + user + ", 失败原因: " + err;
                    writeAuditlog(2, ip, cent);
                    $("#add-server-msg").text(_("Add assert failed"));
                })
            }

            function getAssertInfo() {
                console.log("[getAssertInfo]");
                // 还需要获取资产的标签和指纹信息
                let cmd = ["cat", "/etc/machine-id", "/etc/.kyinfo"];
                cockpit.spawn(cmd, {
                    host: ip
                })
                .then(res=>{
                    getAssertInfoBody(cmd, res);
                })
                .catch(err=>{
                    console.log(err);
                    let uuid = "dmidecode | grep UUID | sed 's/.*UUID://g' | sed 's/-//g' | sed 's/^[ ]*//g' > /etc/machine-id";
                    cockpit.script(uuid, {
                        host: ip
                    })
                    .then(res=>{
                        cockpit.spawn(cmd, {
                            host: ip
                        })
                        .then(res=>{
                            getAssertInfoBody(cmd, res);
                        })
                        .catch(err=>{
                            console.log(err);
                            return $("#add-server-msg").text(_("Get assert's version and fingerprints failed."));
                        })
                    })
                    .catch(err=>{
                        console.log(err);
                        return $("#add-server-msg").text(_("Get assert's version and fingerprints failed."));
                    })
                })
            }

            function writeAuthToFile(rawcontent) {
                authkeydata = authkeydata.replace("\n", "").replace("\r", "");
                let towrite = "";

                if (!rawcontent || rawcontent.length < 1) {
                    towrite = authkeydata;
                }else if (rawcontent.indexOf(authkeydata) >= 0) {
                    towrite = rawcontent;
                }else {
                    towrite = rawcontent + '\n' + authkeydata;
                }

                let authfile = home + "/.ssh/authorized_keys";
                let file = cockpit.file(authfile, {
                    "host": ip,
                });
                file.replace(towrite)
                .then(tag => {
                    console.log("上传证书到资产成功，tag:", tag);

                    getAssertInfo();
                })
                .catch(error => {
                    return $("#add-server-msg").text(_("Upload credential failed."));
                })
            }

            // 到此已经添加到资产列表，还需要将管理服务器的证书上传到资产
            function uploadAuth() {
                let cmd = "mkdir -p $HOME/.ssh && echo $HOME";
                cockpit.script(cmd, {
                    host: ip
                })
                .then(res=>{

                    home = res.replace("\n", "").replace("\r", "");
                    let authfile = home + "/.ssh/authorized_keys";
                    let file = cockpit.file(authfile, {
                        "host": ip,
                    });

                    file.read()
                    .then((content, tag)=>{
                        console.log("assert authfile:", authfile, " rawcontent:", content, " tag:", tag);
                        writeAuthToFile(content);
                    })
                    .catch(err=>{
                        console.log("assert authfile donot exist:", authfile);
                        writeAuthToFile("");
                    })
                })
                .catch(err=>{
                    console.log(err);
                    return $("#add-server-msg").text(_("Create directory failed while uploading credential."));
                })
            }

            if (g_machines.lookup(ip)) {
                g_machines.change(ip, options)
            }

            g_machines.add(user + "@" + ip, options)
            // 调用add接口后需要再次运行 try_to_connect 才能真正生效
            try_to_connect(ip, options)
            .fail(function(ex) {
                return $("#add-server-msg").text(_("Add assert to SCMMS failed."))
            })
            .done(function(res) {
                return uploadAuth();
            })

        }

        // 先尝试连接，看是否能够成功
        try_to_connect(ip, options)
            .fail(function(ex) {
                console.log(ex);

                if (ex.problem == "unknown-host") {

                    // 首次添加会出现这种情况，没有确认指纹，需要自动确认指纹
                    // 需要用 private 选项会列出资产的指纹信息
                    // options["session"] = 'private';
                    try_to_connect(ip, {"session": "private"})
                    .fail(function(ex) {

                        if (ex.problem !== "unknown-hostkey") {
                            // 使用 private 后报错类型是 unknown-hostkey
                            console.log(ex);
                            return $("#add-server-msg").text(_("Get fingerprints failed."));
                        }
                        let hostkey = ex["host-key"];
                        let hostfp = ex["host-fingerprint"];

                        console.log("auto add host footprint,fp:", hostfp, " key:", hostkey);

                        // cockpit 会把证书加到 /etc/ssh/ssh_known_hosts，无效，需要加到 ~/.ssh/known_hosts
                        /*g_machines.add_key(hostkey);
                        g_machines.change(ip, {
                            'host_key': hostkey
                        });*/

                        hostkey = hostkey.replace("\n", "").replace("\r", "");
                        let addauthcmd = ["bash", "/usr/share/cockpit/manager.cmd/addHostKey.sh", user, hostkey]

                        cockpit.spawn(addauthcmd).then(function(e) {
                            // 尝试连接成功，添加资产到服务器列表
                            realAddHost()
                        }).fail(function(e) {
                            console.log("exec cmd failed,cmd:", addauthcmd);
                            return $("#add-server-msg").text(_("Connect to assert failed. Add assert to knwon_hosts failed."));
                        })
                    })
                    .done(function(res){
                        // 异常情况，上次失败，这次不会成功
                    })
                    return
                }

                if (ex.problem == "no-host") {
                    return $("#add-server-msg").text(_("Connect to assert failed. Make sure ssh is opening."));
                }
                if (ex.problem == "access-denied" || ex.problem == "authentication-failed") {
                    return $("#add-server-msg").text(_("Connection refuesd. Make sure authentication is valid."));
                }
                return $("#add-server-msg").text(_("Add assert failed. Unknown error."));
            })
            .done(function(e) {
                realAddHost()
            })
            .always(function(e){});
    }

    function registerAssertsEvent() {
        $("#services-dropdown").change(function(e){
            show_asserts_page(g_asserts_bak);
            registerAssertsEvent();
        });
        $("#services-text-filter").change(function(e){
            show_asserts_page(g_asserts_bak);
            registerAssertsEvent();
        });
        // 资产详情
        /*$("tr.unit-tr-row").unbind("click").click(function(e) {
            let id = $(this).data("goto-unit");
            // assert detail dialog
        });*/
        $("#clear-all-filters").unbind("click").click(function(e) {
            $("#services-text-filter").val("");
            show_asserts_page(g_asserts_bak);
            registerAssertsEvent();
        })

        $("#add-server").unbind("click").click(function(e){
            $("#add-server-ip").val("");
            $("#add-server-name").val("");
            $("#add-server-user").val("root");
            // 目前绑定特权用户root, 不允许更改
            $("#add-server-user").attr("disabled",true);
            $("#add-server-pass").val("");
            $("#add-server-msg").text("");
        })

        // add server dialog
        function onTypeChange() {
            let value = $("#add-server-type").children("option:selected").val();
            if (value == "0") {
                $("#pass-area").show();
                $("#credit-area").hide();
            } else {
                update_available();
                $("#pass-area").hide();
                $("#credit-area").show();
            }
        }
        $("#add-server-type").change(function(e){
            onTypeChange();
        })
        onTypeChange();

        $("#add-server-confirm").unbind("click").click(function(e){
            $("#add-server-msg").text(_("Adding..."));
            doAddAssert();
        })

        // 点击规则行上的更多按钮时，弹出菜单，需要在渲染完成后再绑定事件
        $(".assert-opr-more").unbind("click").on("click", function(e){

            let top = $(this).offset().top;
            let left = $(this).offset().left;
            let top1 = $("html").scrollTop();
            top = top - top1;
            let width = parseInt($(this).css("width").replace("px", ""));

            $(".assert-popup-menu").css("top", top);

            //let bodywidth = parseInt($("body#services-page").css("width").replace("px", ""));
            //let menuwidth = parseInt($(".assert-popup-menu").css("width").replace("px", ""));
            // 如果向右展示超出了窗口宽度，就向左展示
            //if (left + width + menuwidth <= bodywidth) {
                $(".assert-popup-menu").css("left", left + width);
            //} else {
            //    $(".assert-popup-menu").css("left", left - menuwidth);
            //}

            if ($(".assert-popup-menu").css("display") == "none") {
                $(".assert-popup-menu").css("display", "inline-grid")
            } else {
                $(".assert-popup-menu").css("display", "none")
            }

            let assertid = $(this).data("assert-id")
            $("#assert-popup-menu-private-data").data("private-data", assertid)
        })
        $(".assert-popup-btn").mouseover(function(e){
            $(this).css("background-color", "#def3ff")
        })
        .mouseout(function(e){
            $(this).css("background-color", "white")
        })
        $(window).unbind("click").on("click", function(e){
            if ( e.target.localName == "button" &&
                (e.target.className.indexOf("assert-opr-more") >= 0 ||
                e.target.className.indexOf("assert-delete") >= 0 ||
                e.target.className.indexOf("assert-status") >= 0)) {
                return
            }
            $(".assert-popup-menu").css("display", "none")
        })
        $("button.assert-status").unbind("click").click(function(e){
            $("#assert-rules-msg").text("");
            $("#assert-rules-content").html("");

            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let assertid = $("#assert-popup-menu-private-data").data("private-data");
            let http = cockpit.http({
                    "port": 9091
                });
            http.get("/getAssertRules", {
                "id": assertid
            }).done(function(res) {
                console.log("getAssertRules res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    return $("#assert-rules-msg").text(_("Get rules info failed: ") + info.error);
                }
                info = info.data;
                if (info.length < 1) {
                    $("#assert-rules-msg").text(_("No rules on this assert"));
                    return
                }

                for (var i=0;i<info.length;i++) {
                    let statustext = ""
                    try {
                        let applycount = $.parseJSON(info[i].applycount);
                        if (!applycount[assertid]) {
                            // 没有下发记录，不成功也不失败，下发中
                            statustext = _("Downloading")
                        }
                        // 如果 applycount < editcount 则也是 尚未下发
                        else if (applycount[assertid]["applycount"] >= info[i].editcount && applycount[assertid]["result"] == "succeed") {
                            statustext = _("Succeed")
                        }
                        else if (applycount[assertid]["result"] == "succeed") {
                            statustext = _("Waiting")
                        }
                        else {
                            statustext = _("Failed ")
                        }
                    } catch(e) {
                        // json 格式异常，没有下发结果，表示是初始状态，为 下发中
                        statustext = _("Downloading")
                        console.log(e)
                    }

                    let tr = "<tr><td class='text-align-left'>" + info[i].name
                            + "</td><td>" + info[i].straname
                            + "</td><td>" + info[i].strafrom
                            + "</td><td><span class=''>"+statustext+"</span></td></tr>"
                    $("#assert-rules-content").append($(tr));
                }
            }).fail(function(err){
                console.log(err);
                $("#assert-rules-msg").text(_("Get rules info failed"));
            })
        })

        // delete assert
        $("button.assert-delete.assert-popup-btn").unbind("click").click(function(e) {

            let id = $("#assert-popup-menu-private-data").data("private-data");
            console.log("delete-assert-btn clicked, recore id:" + id);
            let unit = getUnitById(id);

            $("#confirm-operate-msg").text(_("Are you sure delete assert ") + unit.assertname + "?")
            $("#confirm-operate-confirm").unbind("click").click(function(e) {
                if (unit.assertip == "localhost") {
                    return $("#confirm-operate-msg").text(_("Localhost cannot delete"));
                }
                // 到此已经添加到资产列表，还需要添加到数据库
                let http = cockpit.http({
                        "port": 9091
                    });
                http.post("/delAssert", {
                    "id": id
                }).done(function(res) {
                    console.log("delAssert res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        let cent = "删除资产: " + unit.assertip + " 失败, 资产名称=" + unit.assertname + ", 失败原因: " + info.error;
                        writeAuditlog(2, unit.assertip, cent);
                        return $("#confirm-operate-msg").text("删除资产记录失败：" + info.error);
                    }

                    let machine = g_machines.lookup(unit.assertip);
                    if (machine)
                        g_machines.change(machine.key, { visible: false });
                    $("#confirm-operate-msg").text("删除资产成功");
                    let cent = "删除资产: " + unit.assertip + " 成功, 资产名称=" + unit.assertname;
                    writeAuditlog(2, unit.assertip, cent);
                    setTimeout(function(e) {
                        $("#confirm-operate-close").click();
                        render_asserts_real();
                    }, 1000)
                }).fail(function(err){
                    console.log(err);
                    $("#confirm-operate-msg").text("删除资产记录失败");
                    let cent = "删除资产: " + unit.assertip + " 失败, 资产名称=" + unit.assertname + ", 失败原因: " + err;
                    writeAuditlog(2, unit.assertip, cent);
                })
            })
        })

        // 资产自动发现
        $("#find-server").unbind("click").click(function(e) {
            //let iplist = ipfuncs.default.listIPByRange("1.2.3.4", "1.2.3.99");
            //iplist = ipfuncs.default.listIPByNetwork("1.2.3.3", 31);
            $("#find-msg").text("");

            let http = cockpit.http({
                "port": 9091
            });

            http.get("/getFindConfig").done(function(res) {
                //console.log("getFindConfig res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    $("#find-msg").text("获取配置失败，请稍后重试");
                    setTimeout(function(e) {
                        $("#find-close").click();
                    }, 2000)
                }

                if (info.data && info.data.enabled) {
                    $("#setting-check-input").attr("checked", true)
                } else {
                    $("#setting-check-input").attr("checked", false)
                }

                if (info.data && info.data.iprange) {
                    $("#find-ip-range").val(info.data.iprange)
                }

            }).fail(function(err) {
                console.log(err);
                $("#find-msg").text("获取配置失败，请稍后重试");
                setTimeout(function(e) {
                    $("#find-close").click();
                }, 2000)
            })
        })
        function updateProgress(raised, goal) {
            $("#jqmeter-container").html("");
            JQMeter.jQMeter_func('#jqmeter-container', {
                "goal": goal.toString(),
                "raised": raised.toString()
            });
        }
        function addScanResult(ip, state, allipcnt, hostdata) {
            //g_snip_finished = g_snip_finished + 1;

            if (state == "failed") {
                let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>----</td><td class=''>扫描失败</td></tr>"
                $("#find-result-content").append(tr);
                g_snip_finished = g_snip_finished + 1;
                return updateProgress(g_snip_finished, allipcnt);
            }

            function succeedConnectOpr() {

                // 扫描成功后的操作
                // 获取资产的名称、版本、指纹信息
                if (!hostdata.machineid || hostdata.machineid.length < 1 ||
                    !hostdata.hostname || hostdata.hostname.length < 1 ||
                    !hostdata.kyinfo || hostdata.kyinfo.length < 1) {
                    console.log("获取资产信息失败", hostdata);
                    let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>----</td><td class='add-find-result-err'>处理资产信息失败</td></tr>"
                    $("#find-result-content").append(tr);
                    g_snip_finished = g_snip_finished + 1;
                    return updateProgress(g_snip_finished, allipcnt);
                }

                let finger = hostdata.machineid[0].replace("\n", "").replace("\r", "");
                let hostname = hostdata.hostname[0].replace("\n", "").replace("\r", "");

                let tag = "";
                let arr = hostdata.kyinfo;
                for (let i=0;i<arr.length;i++) {
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.4-") > 0) {
                        tag = "3.4";
                    }
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.3-") > 0) {
                        tag = "3.3";
                    }
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.2-") > 0) {
                        tag = "3.2";
                    }
                    if (arr[i].startsWith("milestone") && arr[i].indexOf(" = 3.0-") > 0) {
                        tag = "3.0";
                    }
                }
                if (tag.length < 1) {
                    console.log(ip, "get assert version tag failed", err);
                    let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>----</td><td class='add-find-result-err'>获取资产标签失败</td></tr>"
                    $("#find-result-content").append(tr);
                    g_snip_finished = g_snip_finished + 1;
                    return updateProgress(g_snip_finished, allipcnt);
                }

                g_find_assert_info[ip] = {
                    "finger": finger,
                    "hostname": hostname,
                    "tag": tag
                }
                let assertadded = false;
                for(let i=0;i<g_asserts_bak.length;i++) {
                    if (finger == g_asserts_bak[i].assertid) {
                        assertadded = true;
                        break;
                    }
                }
                let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>"+hostname+"</td><td><span class='add-find-assert fa fa-plus-square-o' data-assert-ip='"+ip+"'></span></td></tr>"
                if (assertadded) {
                    tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>"+hostname+"</td><td><span class='added-find-assert' data-assert-ip='"+ip+"'>已添加</span></td></tr>"
                }
                $("#find-result-content").append(tr);
                g_snip_finished = g_snip_finished + 1;
                updateProgress(g_snip_finished, allipcnt);

                $("span.add-find-assert").unbind("click").click(function(e){
                    let spanele = $(this);
                    let assert_ip = $(this).data("assert-ip");
                    spanele.removeClass("fa");
                    spanele.removeClass("fa-plus-square-o");
                    spanele.removeClass("add-find-assert");
                    spanele.text("添加中");
                    if (g_machines.lookup(assert_ip)) {
                        g_machines.change(assert_ip, {"user": "cockpit-client"})
                    }

                    g_machines.add("cockpit-client@" + assert_ip, {"user": "cockpit-client"})
                    // 调用add接口后需要再次运行 try_to_connect 才能真正生效
                    try_to_connect(assert_ip, {"user": "cockpit-client"})
                    .fail(function(ex) {
                        console.log("add assert failed:", ex)
                        let machine = g_machines.lookup(assert_ip);
                        if (machine)
                            g_machines.change(machine.key, { visible: false });
                    })
                    .done(function(res){
                        let http = cockpit.http({
                            "port": 9091
                        });
                        http.post("/addAssert", {
                            "ip": assert_ip,
                            "name": g_find_assert_info[assert_ip]["hostname"],
                            "finger": g_find_assert_info[assert_ip]["finger"],
                            "tag": g_find_assert_info[assert_ip]["tag"],
                            "user": "cockpit-client"
                        }).done(function(res) {
                            //console.log("addAssert res:", res);

                            let info = $.parseJSON( res );
                            if (! info.succeed) {
                                let cent = "添加资产: " + assert_ip + " 失败, 资产名称=" + g_find_assert_info[assert_ip]["hostname"] + ", 失败原因: " + info.error;
                                writeAuditlog(2, assert_ip, cent);
                                return //console.log("add assert failed, return:", res)
                            }
                            spanele.attr("class", "added-find-assert").text("已添加");
                            let cent = "添加资产: " + assert_ip + " 成功, 资产名称=" + g_find_assert_info[assert_ip]["hostname"];
                            writeAuditlog(2, assert_ip, cent);

                        }).fail(function(err){
                            console.log("add assert failed, err:", err);
                            let machine = g_machines.lookup(assert_ip);
                            if (machine)
                                g_machines.change(machine.key, { visible: false });

                            let cent = "添加资产: " + assert_ip + " 失败, 资产名称=" + g_find_assert_info[assert_ip]["hostname"] + ", 失败原因: " + err;
                            writeAuditlog(2, assert_ip, cent);
                        })
                    })
                })
            }

            // succeed
            try_to_connect(ip, {
                "user": "cockpit-client"
                // 这时，证书已经上传，使用内置证书登录，不需要使用密码，option 不要提供任何参数
            })
            .fail(function(ex) {
                // unknown-host 表示没有添加指纹的情况
                if (ex.error != "unknown-host" && ex.error != "unknown-hostkey") {
                    console.log(ip, "connect to assert failed, err:", ex);
                    let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>----</td><td class='add-find-result-err'>连接失败</td></tr>"
                    $("#find-result-content").append(tr);
                    g_snip_finished = g_snip_finished + 1;
                    updateProgress(g_snip_finished, allipcnt);
                    return
                }

                // 首次添加会出现这种情况，没有确认指纹，需要自动确认指纹
                // 需要用 private 选项会列出资产的指纹信息
                try_to_connect(ip, {
                    "user": "cockpit-client",
                    "session": "private"
                })
                .fail(function(ex) {

                    if (ex.problem != "unknown-hostkey") {
                        // 使用 private 后报错类型是 unknown-hostkey
                        console.log(ip, "connect to assert failed, err:", ex);
                        let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>----</td><td class='add-find-result-err'>添加指纹时连接失败</td></tr>"
                        $("#find-result-content").append(tr);
                        g_snip_finished = g_snip_finished + 1;
                        updateProgress(g_snip_finished, allipcnt);
                        return
                    }
                    let hostkey = ex["host-key"];
                    let hostfp = ex["host-fingerprint"];

                    console.log("auto add host footprint when find,fp:", hostfp, " key:", hostkey);

                    hostkey = hostkey.replace("\n", "").replace("\r", "");
                    let addauthcmd = ["bash", "/usr/share/cockpit/manager.cmd/addHostKey.sh", "cockpit-client", hostkey]

                    cockpit.spawn(addauthcmd).then(function(e) {
                        succeedConnectOpr()
                    }).fail(function(ex) {
                        console.log(ip, "add host key failed, err:", ex);

                        let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>----</td><td class='add-find-result-err'>添加指纹失败</td></tr>"
                        $("#find-result-content").append(tr);
                        g_snip_finished = g_snip_finished + 1;
                        updateProgress(g_snip_finished, allipcnt);
                        return
                    })
                })
                .done(function(res){
                    // 异常情况，上次失败，这次不会成功
                    console.log(ip, "connect to assert succeed with private option, unnormal !!!");

                    let tr = "<tr><td class='text-align-left'>" +ip+ "</td><td>----</td><td class='add-find-result-err'>添加指纹时连接失败</td></tr>"
                    $("#find-result-content").append(tr);
                    g_snip_finished = g_snip_finished + 1;
                    updateProgress(g_snip_finished, allipcnt);
                    return
                })
            })
            .done(function(e) {
                succeedConnectOpr();
            })
        }

        function doFindWork(allips) {

            var goal = allips.length;
            var raised = 0;
            JQMeter.jQMeter_func('#jqmeter-container', {
                "goal": goal.toString(),
                "raised": raised.toString()
            });

            $("#show-find-result").click();
            setTimeout(function(e) {
                // 根据界面交互逻辑，扫描的逻辑做在前台，
                // 同时不影响前台响应，每100ms扫描一次
                if (g_authkeys.items.length < 1) {
                    console.log("get auth key failed, stop scan");
                    return ;
                }

                let authkeydata = "";
                for (var id in g_authkeys.items) {
                    var key = g_authkeys.items[id];
                    // 取第一个即可
                    authkeydata = key.data;
                    if (authkeydata.length > 0) {
                        break;
                    }
                }

                for (let i=0;i<allips.length;i++) {
                    //console.log("scan ip:", allips[i]);

                    let http = cockpit.http({
                        "address": allips[i],
                        "port": 9092
                    });

                    http.post("/uploadCredit", {
                        "key": "qazwsxedcrfv",
                        "auth": authkeydata
                    }).done(function(res) {

                        let info = $.parseJSON( res );
                        if (! info.succeed || !info.data) {
                            //console.log(allips[i], "uploadCredit failed, ", res);
                            let cent = "uploadCredit failed, reason: " + info.error;
                            writeAuditlog(2, allips[i], cent);
                            return addScanResult(allips[i], "failed", allips.length, {});
                        }

                        console.log(allips[i], "uploadCredit succeed");
                        let cent = "uploadCredit succeed";
                        writeAuditlog(2, allips[i], cent);
                        addScanResult(allips[i], "succeed", allips.length, info.data);

                    }).fail(function(err) {
                        //console.log(allips[i], "scan ip failed, host may not exist", err);
                        let cent = "uploadCredit failed, reason: " + err;
                        writeAuditlog(2, allips[i], cent);
                        addScanResult(allips[i], "failed", allips.length, {});
                    })
                }
            }, 100)

        }

        // 关闭发现窗口的时候，有可能已经加入了资产，刷新一下资产界面
        $("#find-close").unbind("click").click(function(e){
            render_asserts_real()
        })

        // 资产自动发现确认
        $("#find-confirm").unbind("click").click(function(e) {
            $("#find-msg").text("");

            let enabled = $("#setting-check-input").prop("checked")
            if (enabled) {
                enabled = 1;
            } else {
                enabled = 0;
            }

            let allips = [];
            let iprange = $("#find-ip-range").val();
            let iprangearr = iprange.split(";");
            for(let i=0;i<iprangearr.length;i++) {
                let tmp = iprangearr[i];
                if (tmp.indexOf("-") > 0) {
                    let tmparr = tmp.split("-");
                    if (tmparr.length < 2) {
                        return $("#find-msg").text("IP范围不合法：" + tmp);
                    }
                    let tmpiplist = ipfuncs.default.listIPByRange(tmparr[0], tmparr[1]);
                    if (tmpiplist.length < 1) {
                        return $("#find-msg").text("IP范围不合法：" + tmp);
                    }
                    allips.push.apply(allips, tmpiplist);
                }

                else if (tmp.indexOf("/") > 0) {
                    let tmparr = tmp.split("/");
                    if (tmparr.length < 2) {
                        return $("#find-msg").text("IP网段不合法：" + tmp);
                    }
                    let tmpiplist = ipfuncs.default.listIPByNetwork(tmparr[0],
                        parseInt(tmparr[1]))
                    if (tmpiplist.length < 1) {
                        return $("#find-msg").text("IP网段不合法：" + tmp);
                    }
                    allips.push.apply(allips, tmpiplist);
                }

                else if (ipfuncs.default.ip_valid(tmp)) {
                    allips.push(tmp);
                } else {
                    return $("#find-msg").text("IP不合法：" + tmp);
                }
            }

            allips = ipfuncs.default.getUniqArr(allips);
            if (allips.length < 1) {
                return $("#find-msg").text("IP范围为空");
            }
            if (allips.length > 1000) {
                return $("#find-msg").text("IP范围最大1000个");
            }

            let http = cockpit.http({
                    "port": 9091
                });
            http.post("/setFindConfig", {
                "enabled": enabled,
                "iprange": iprange
            }).done(function(res) {
                //console.log("setFindConfig res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    let cent = "提交发现资产配置失败, iprange=" + iprange + ", 失败原因: " + info.error;
                    writeAuditlog(2, iprange, cent);
                    return $("#find-msg").text("提交配置失败");
                }

                $("#find-msg").text("提交配置成功");
                let cent = "提交发现资产配置成功, iprange=" + iprange;
                writeAuditlog(2, iprange, cent);
                g_snip_finished = 0;
                $("#find-result-content").html("");
                doFindWork(allips);

            }).fail(function(err) {
                //console.log(err);
                let cent = "提交发现资产置失败, iprange=" + iprange + ", 失败原因: " + err;
                writeAuditlog(2, iprange, cent);
                return $("#find-msg").text("提交配置失败");
            })

        })

        // edit assert
        $("button.edit-assert-btn").unbind("click").click(function(e) {
            $("#edit-assert-msg").text("")
            let id = $(this).data("assert-id");
            console.log("edit-assert-btn clicked, recore id:" + id);
            let unit = getUnitById(id);
            let oldip = unit.assertip;
            let oldiparr = oldip.split("/");
            let network = getNetworkManager(oldiparr[0]);
            let oldipsuffix = ""
            if (oldiparr.length > 1) {
                oldipsuffix = oldiparr[1]
            }
            let oldtag = unit.asserttag;
            let showtag = oldtag;
            if (! showtag.endsWith(";")) {
                showtag = showtag + ";";
            }

            $("#edit-assert-ip").val(unit.assertip);
            $("#edit-assert-tag").val(showtag);
            $("#edit-assert-name").val(unit.assertname);
            $("#edit-assert-location").val(unit.assertlocation);
            $("#edit-assert-usage").val(unit.assertusage);

            // 处理资产所属的分组
            let assertGroupMap = {};
            for (let i=0;i<g_groups_bak.length;i++) {
                let assertsarr = g_groups_bak[i].groupasserts.split(";");
                for (let j=0;j<assertsarr.length;j++) {
                    if (! assertsarr[j]) {
                        continue;
                    }
                    assertGroupMap[assertsarr[j]] = g_groups_bak[i].groupid
                }
            }
            $("#assert-group").html("")
            let select_text=_("Not selected");
            let html = '<option value="notselected">'+select_text+'</option>';
            $("#assert-group").append($(html))
            for (let i=0;i<g_groups_bak.length;i++) {
                let html = '<option value="'+ g_groups_bak[i].groupid +'">'+ g_groups_bak[i].groupname +'</option>';
                $("#assert-group").append($(html))
            }
            $("#assert-group").val(assertGroupMap[id])

            $("#edit-assert-confirm").unbind("click").click(function(e) {
                $("#edit-assert-msg").text("应用中，请等待...");

                // 到此已经添加到资产列表，还需要添加到数据库
                let http = cockpit.http({
                        "port": 9091
                    });

                let ip = $("#edit-assert-ip").val();
                let tag = $("#edit-assert-tag").val();
                let name = $("#edit-assert-name").val();
                let loca = $("#edit-assert-location").val();
                let usage = $("#edit-assert-usage").val();
                let groupid = $("#assert-group option:selected").val();
                
                if (!name || name.length < 1 || name.length > 40) {
                    return $("#edit-assert-msg").text("请输入合法名称，1-40字符, 仅支持英文 中文 数字 . - _ ");
                }
                var rename = /[-\.\_a-zA-Z0-9\u4e00-\u9fa5]/
        
                for (var i = 0; i < name.length; i++){
                    if (!rename.test(name[i])){
                        return $("#edit-assert-msg").text("请输入合法名称，1-40字符, 仅支持英文 中文 数字 . - _ ");
                    }
                }

                if (tag != unit.asserttag) {
                    let tagarr = tag.split(";");
                    let versiontagexist = false;
                    for(let i=0;i<tagarr.length;i++) {
                        if (tagarr[i] == "3.3" || tagarr[i] == "3.4") {
                            versiontagexist = true;
                        }
                    }
                    if (! versiontagexist) {
                        return $("#edit-assert-msg").text("版本标签不允许删除");
                    }

                }

                if (loca.length > 40) {
                    return $("#edit-assert-msg").text("资产位置描述过长，请输入小于40个字符");
                }
                if (usage.length > 40){
                    return $("#edit-assert-msg").text("资产用途描述过长，请输入小于40个字符");
                }
                
                /*if (ip != oldip) {
                    // 修改IP
                    if (unit.assertip == "localhost") {
                        return $("#edit-assert-msg").text("管理服务器不允许修改IP");
                    }

                    return $("#edit-assert-msg").text("暂时不支持");

                    let newiparr = ip.split("/");
                    if (newiparr.length < 2) {
                        return $("#edit-assert-msg").text("IP请输入192.168.120.2/24的格式");
                    }
                    let nicipchanged = false;
                    let nics = network.list_interfaces()

                    nics.forEach(function (iface) {
                        if (!ipfuncs.default.isEmpty(iface.MainConnection)
                         && !ipfuncs.default.isEmpty(iface.MainConnection.Settings)
                         && !ipfuncs.default.isEmpty(iface.MainConnection.Settings.ipv4)
                         && !ipfuncs.default.isEmpty(iface.MainConnection.Settings.ipv4.addresses)) {
                            let addresses = iface.MainConnection.Settings.ipv4.addresses;
                            for(let i=0;i<addresses.length;i++) {

                                let nicip = iface.MainConnection.Settings.ipv4.addresses[i][0];
                                if( nicip == oldiparr[0]) {
                                    console.log("find nic to change:", iface.Name, " change to:", ip);
                                    nicipchanged = true;

                                    let settings = iface.MainConnection.Settings;
                                    settings.ipv4.addresses[i][0] = newiparr[0];
                                    settings.ipv4.addresses[i][1] = newiparr[1];
                                    iface.MainConnection.apply_settings(settings);

                                    // 修改了网卡的IP配置后，还要重启网卡生效
                                    // 重启网卡要先断开，会导致失联
                                    /*iface.MainConnection.activate(iface.Device).fail(function(err){
                                        console.log(err);
                                    });
                                    let cmd = "ifconfig " + iface.Name + " down;ifconfig " + iface.Name + " up &";
                                    console.log("exec cmd to change nic ip:", cmd);
                                    cockpit.spawn(cmd, {
                                        host: oldiparr[0]
                                    })
                                    .catch(err=>{
                                        console.log(err);
                                        console.log("重启网卡失败")
                                    })

                                    // 修改机器列表信息
                                    g_machines.add(newiparr[0], {"user": unit.assertuser}).fail(function(e){
                                        console.log("修改IP时添加机器信息失败");
                                    })
                                    break;
                                }
                            }
                         }
                    })
                    if (! nicipchanged) {
                        return $("#edit-assert-msg").text("修改网卡IP失败，没找到对应网卡");
                    }
                }*/

                if (tag.endsWith(";")){
                    tag=tag.substr(0,tag.length-1);
                }
                let postdata = {
                    "id": id,
                    "tag": tag,
                    "name": name,
                    "location": loca,
                    "usage": usage,
                    "ip": ip,
                    "group": groupid,
                    "oldgroup": assertGroupMap[id]
                }

                http.post("/setAssert", postdata).done(function(res) {
                    console.log("setAssert res:", res, " postdata:", postdata);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        let cent = "编辑资产记录: " + ip + " 失败, 编辑详情: name=" + name + ", tag=" + tag + ", groupid=" + groupid + ", 失败原因: " + info.error;
                        writeAuditlog(2, ip, cent);
                        return $("#edit-assert-msg").text("编辑资产记录失败：" + info.error);
                    }

                    $("#edit-assert-msg").text("编辑资产成功");
                    let cent = "编辑资产记录: " + ip + " 成功, 编辑详情: name=" + name + ", tag=" + tag + ", groupid=" + groupid;
                    writeAuditlog(2, ip, cent);
                    setTimeout(function(e) {
                        $("#edit-assert-close").click();
                        render_asserts_real();
                    }, 1000)
                }).fail(function(err) {
                    console.log(err);
                    $("#edit-assert-msg").text("编辑资产记录失败");
                    let cent = "编辑资产记录: " + ip + " 失败, 编辑详情: name=" + name + ", tag=" + tag + ", groupid=" + groupid + ", 失败原因: " + err;
                    writeAuditlog(2, ip, cent);
                })
            })
        })
    }

    function get_assert_state(ip) {
        let index = 0;
        for(index=0;index<g_machines.list.length;index++) {
            if (g_machines.list[index].address != ip) {
                continue;
            }
            if (g_machines.list[index].state == "connected") {
                return _("Online")
            }
            return _("Offline")
        }
        return ""
    }

    function show_asserts_page(info) {
        // 这个函数不申请数据，只是重新做下匹配
        let pattern = $('#services-filter li.active').attr('data-pattern');
        let current_text_filter = $('#services-text-filter').val().toLowerCase();
        let current_type_filter = parseInt($('#services-dropdown').val());

        let units = [];
        let header = {
            Id: _("Assert ID"),
            Name: _("Assert Name"),
            Loca: _("Assert Location"),
            Usage: _("Assert Usage"),
            Ip: _("Assert IP"),
            Tag: _("Assert Tag"),
            State: _("Assert Status"),
            Group: _("Assert Group"),
            Operate: _("Operation")
        };

        let assertGroupMap = {};
        // 处理资产所属的分组
        for (let i=0;i<g_groups_bak.length;i++) {
            let assertsarr = g_groups_bak[i].groupasserts.split(";");
            for (let j=0;j<assertsarr.length;j++) {
                if (! assertsarr[j]) {
                    continue;
                }
                assertGroupMap[assertsarr[j]] = g_groups_bak[i].groupname
            }
        }

        function cmp_assert(a, b) { return info[a].id > info[b].id }
        let sorted_assert_keys = Object.keys(info).sort(cmp_assert);
        sorted_assert_keys.forEach(function(id) {

            let unit = info[id];
            if (! unit.assertlocation) {
                unit.assertlocation = "----";
            }
            if (! unit.assertusage) {
                unit.assertusage = "----";
            }
            let state = get_assert_state(unit.assertip);
            if (state === "") {
                // invalid assert
                // return 
            }
            unit.group = assertGroupMap[unit.assertid];
            unit.state = state;
            unit.edit_text = _("Edit");
            unit.more_text = _("More");
            // 过滤掉状态类型
            let state_filter = {"0": "all", "1": _("Online"), "2": _("Offline")}[current_type_filter];
            if (current_type_filter != "0" && state_filter != unit.state) {
                return;
            }
            // 匹配搜索字符串
            if (! current_text_filter) {
                units.push(unit);
                return;
            }
            if (unit.assertname && unit.assertname.toLowerCase().indexOf(current_text_filter) != -1) {
                units.push(unit);
                return;
            }
            if (unit.assertlocation && unit.assertlocation.indexOf(current_text_filter) != -1) {
                units.push(unit);
                return;
            }
            if (unit.assertusage && unit.assertusage.indexOf(current_text_filter) != -1) {
                units.push(unit);
                return;
            }
            if (unit.assertip && unit.assertip.indexOf(current_text_filter) != -1) {
                units.push(unit);
                return;
            }
        });
        units = processPagination(units)

        var text = "";
        if (units.length > 0)
            text = mustache.render(units_template, {
                table_head: header,
                units: units
            });
        else
            text = mustache.render(empty_template);

        $("#services-list").html(text);
        $("#service-detail-list").hide();
        $("#services").show();

        $("#loading-fallback").hide();
    }

    function render_asserts_real() {
        console.log("machines:", g_machines.list);
        $("#loading-fallback").show();
        // 使用资产接口获取获取所有的资产信息
        clearInterval(intv);
        let http = cockpit.http({
                "port": 9091
            });
        http.get("/getAsserts").done(function(res) {
            //console.log("getAsserts res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getAsserts err:", res);
                $("#loading-fallback").hide();
                return ;
            }
            info = info.data;
            g_asserts_bak = info;

            // 资产需要展示所属的分组信息
            http.get("/getGroups")
            .fail(function(err){
                //console.log("getGroups err:", err);
                $("#loading-fallback").hide();
            })
            .done(function(res) {
                //console.log("get groups res:", res);
                let info = $.parseJSON( res );
                if (!info.succeed) {
                    return $("#loading-fallback").hide();
                }
                g_groups_bak = info.data;
                show_asserts_page(g_asserts_bak);
                registerAssertsEvent();
                $("#loading-fallback").hide();
                intv = setInterval(()=>{
                    $.each(g_machines.list,function(i,v){
                        let status = (v.state == "connected") ? _("Online") : _("Offline");
                        $("div#services-list tbody tr[data-address='"+v.address+"']").find("#unit-status").text(status);
                    });
                },2000);
            })
        }).fail(function(err){
            //console.log("getAsserts err:", err);
            $("#loading-fallback").hide();
        })
    }

    function update_available() {
        for (var id in g_authkeys.items) {
            var key = g_authkeys.items[id];
            $("#credit-identifier").html("");
            if (key.loaded) {
                $("#credit-identifier").append($("<option>").text(key.name));
            }
        }
    }

    function render_asserts() {
        console.log("render_asserts");
        $("#loading-fallback").show();

        $("#create-group").hide();
        $("#add-server").show();
        $("#find-server").show();
        $("#services-dropdown").show();

        $(g_machines).on("added.dashboard", function(e){
        });
        $(g_machines).on("removed.dashboard", function(e){
        });
        $(g_machines).on("updated.dashboard", function(e){
        });

        // 使用 dashboard 的接口获取到所有的机器信息，需要延时获取
        setTimeout(function() {
            render_asserts_real();
        }, 500)
    }


    function has_special_chars(str)
    {
        var arr = ["&", "\\", "/", "*", ">", "<", "@", "!"];
        for (var i = 0; i < arr.length; i++)
        {
            for (var j = 0; j < str.length; j++)
            {
                if (arr[i] == str.charAt(j))
                {
                    return true;
                }
            }
        }
    }

    function has_special_chars_name(str){
        var rename = /[-\.\_a-zA-Z0-9\u4e00-\u9fa5]/
        for (var i = 0; i < str.length; i++){
            if (!rename.test(str[i])){
                return true
            }
        }
        return false;
    }

    function getGroupById(id) {
        let unit = [];
        for (let i=0;i<g_groups_bak.length;i++) {
            if (g_groups_bak[i].groupid == id) {
                unit = g_groups_bak[i];
            }
        }
        if (!unit) {
            console.log("[g_groups_bak]can not find unit,uuid:", id);
        }
        return unit;
    }

    function registerEditGroupConfirmEvent() {
        $("#edit-group-confirm").unbind("click").click(function(e) {
            let uuid = $("#edit-group-id").val();
            let name = $("#edit-group-name").val();
            let desc = $("#edit-group-desc").val();
            if (! name || name.length < 1 || name.length > 40 || has_special_chars_name(name)) {
                $("#edit-group-msg").text("请输入合法名称，1-40字符, 仅支持英文 中文 数字 . - _ ");
                return;
            }
            if (! desc || desc.length < 1 || desc.length > 120) {
                $("#edit-group-msg").text("请输入合法描述，1-120字符，且不能有特殊字符");
                return;
            }

            let allasserts = "";
            $("#keepRenderingSort_to option").each(function(e){
                let assertid = $(this).val();
                allasserts = allasserts + assertid + ";";
            })
            let http = cockpit.http({
                    "port": 9091
                });
            http.post("/editGroup", {
                    "uuid": uuid,
                    "name": name,
                    "desc": desc,
                    "asserts": allasserts
                }).done(function(res) {
                console.log("editGroup res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    $("#edit-group-msg").text("编辑分组失败，" + info.error);
                    let cent = "编辑分组: " + uuid + " 失败, name: " + name + ", allasserts: " + allasserts + ", 失败原因: " + info.error;
                    writeAuditlog(2, uuid, cent);
                    return ;
                }

                $("#edit-group-msg").text("编辑分组成功");
                let cent = "编辑分组: " + uuid + " 成功, name: " + name + ", allasserts: " + allasserts;
                writeAuditlog(2, uuid, cent);
                setTimeout(function(e){
                    $("#edit-group-close").click();
                    render_groups();
                }, 1000)
            }).fail(function(err){
                console.log("editGroup err:", err);
                $("#edit-group-msg").text("编辑分组失败，请检查名称是否已经存在");
                let cent = "编辑分组: " + uuid + " 失败, name: " + name + ", allasserts: " + allasserts + ", 失败原因: " + err;
                writeAuditlog(2, uuid, cent);
            })
        })
    }

    function registerGroupsEvent() {
        function setSelectAssertUI(groupunit) {
            // 所有已分组的资产，元素是资产ID

            let allGroupAsserts = [];
            for(let i=0;i<g_groups_bak.length;i++) {
                let assertsarr = g_groups_bak[i].groupasserts.split(";")
                for (let j=0;j<assertsarr.length;j++) {
                    if (allGroupAsserts.indexOf(assertsarr[j]) < 0) {
                        allGroupAsserts.push(assertsarr[j])
                    }
                }
            }

            let keepRenderingSortHtml = "#keepRenderingSort";
            let keepRenderingSort_toHtml = "#keepRenderingSort_to"
            if (ipfuncs.default.isEmpty(groupunit)) {
                keepRenderingSortHtml = "#add_keepRenderingSort";
                keepRenderingSort_toHtml = "#add_keepRenderingSort_to"
            }

            $(keepRenderingSortHtml).html("");
            // 所有未分组的资产，元素是资产信息
            let noGroupAsserts = []
            for (let i=0;i<g_asserts_bak.length;i++) {
                if (allGroupAsserts.indexOf(g_asserts_bak[i].assertid) < 0) {
                    noGroupAsserts.push(g_asserts_bak[i]);
                    if (g_asserts_bak[i].assertip != "localhost") {
                        let html = "<option value='"+g_asserts_bak[i].assertid+"'>"
                        + g_asserts_bak[i].assertip 
                        + " "
                        + g_asserts_bak[i].assertname
                        + "</option>";
                        $(keepRenderingSortHtml).append($(html))
                    }
                }
            }

            $(keepRenderingSort_toHtml).html("");
            if (ipfuncs.default.isEmpty(groupunit)) {
                // 新增分组的时候还没有选择资产
                return;
            }
            // 该分组所有已经选择的资产
            let assertsarr = groupunit.groupasserts.split(";");
            for (let i=0;i<assertsarr.length;i++) {
                let assertunit = getUnitById(assertsarr[i]);
                if (ipfuncs.default.isEmpty(assertunit)) {
                    continue
                }
                let html = "<option value='"+assertsarr[i]+"'>"
                + assertunit.assertip 
                + " "
                + assertunit.assertname
                + "</option>";
                $(keepRenderingSort_toHtml).append($(html))
            }
        }
        $("#create-group").unbind("click").click(function(e) {
            $("#add-group-name").val("");
            $("#add-group-desc").val("");
            $("#add-group-msg").text("");

            let http = cockpit.http({
                    "port": 9091
                });
            http.get("/getAsserts").done(function(res) {
                console.log("getAsserts res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    $("#add-group-msg").text("获取资产信息失败，请稍后重试");
                    return setTimeout(function(e){
                        $("#add-group-close").click()
                    }, 1500)
                }
                g_asserts_bak = info.data;

                setSelectAssertUI([]);

            }).fail(function(err) {
                console.log("getGroups err:", err);
                $("#add-group-msg").text("获取资产信息失败，请稍后重试");
                return setTimeout(function(e) {
                    $("#add-group-close").click()
                }, 1500)
            })
        })
        $("#add_keepRenderingSort_rightAll").unbind("click").click(function(e) {
            $("#add_keepRenderingSort option").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#add_keepRenderingSort_to").append($(html))
            })
        })
        $("#add_keepRenderingSort_leftAll").unbind("click").click(function(e) {
            $("#add_keepRenderingSort_to option").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#add_keepRenderingSort").append($(html))
            })
        })

        $("#add_keepRenderingSort_rightSelected").unbind("click").click(function(e) {
            $("#add_keepRenderingSort option:selected").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#add_keepRenderingSort_to").append($(html))
            })
        })
        $("#add_keepRenderingSort_leftSelected").unbind("click").click(function(e) {
            $("#add_keepRenderingSort_to option:selected").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#add_keepRenderingSort").append($(html))
            })
        })
        $("#services-text-filter").change(function(e){
            show_groups_page(g_groups_bak);
            registerGroupsEvent();
        });
        $("#clear-all-filters").unbind("click").click(function(e) {
            $("#services-text-filter").val("");
            show_groups_page(g_groups_bak);
            registerGroupsEvent();
        })
        $("#add-group-confirm").unbind("click").click(function(e) {
            let name = $("#add-group-name").val();
            let desc = $("#add-group-desc").val();
            if (! name || name.length < 1 || name.length > 40 || has_special_chars_name(name)) {
                $("#add-group-msg").text("请输入合法名称，1-40字符, 仅支持英文 中文 数字 . - _ ");
                return;
            }
            if (! desc || desc.length < 1 || desc.length > 120) {
                $("#add-group-msg").text("请输入合法描述，1-120字符，且不能有特殊字符");
                return;
            }

            let allasserts = "";
            $("#add_keepRenderingSort_to option").each(function(e){
                let assertid = $(this).val();
                allasserts = allasserts + assertid + ";";
            })

            let http = cockpit.http({
                    "port": 9091
                });
            http.post("/addGroup", {
                    "name": name, 
                    "desc": desc,
                    "asserts": allasserts
                }).done(function(res) {
                console.log("addGroup res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("addGroup error, res:", res);
                    $("#add-group-msg").text("添加分组失败，", res.error);
                    let cent = "添加分组: " + name + " 失败, allasserts: " + allasserts + ", 失败原因: " + info.error;
                    writeAuditlog(2, name, cent);
                    return ;
                }

                console.log("addGroup res:", res);
                $("#add-group-msg").text("添加分组成功");
                let cent = "添加分组: " + name + " 成功, allasserts: " + allasserts;
                writeAuditlog(2, name, cent);
                setTimeout(function(e){
                    $("#add-group-close").click();
                    render_groups();
                }, 1000)
            }).fail(function(err){
                console.log("addGroup err:", err);
                $("#add-group-msg").text("添加分组失败，请检查名称是否已经存在");
                let cent = "添加分组: " + name + " 失败, allasserts: " + allasserts + ", 失败原因: " + err;
                writeAuditlog(2, name, cent);
            })
        })

        $("button.edit-group-btn").unbind("click").click(function(e) {

            let uuid = $(this).data("group-id");
            console.log("edit-group-btn clicked, group id:" + uuid);
            let unit = getGroupById(uuid);

            $("#edit-group-id").val(unit.groupid);
            $("#edit-group-name").val(unit.groupname);
            $("#edit-group-desc").val(unit.groupdesc);
            $("#edit-group-msg").text("");

            let http = cockpit.http({
                    "port": 9091
                });
            http.get("/getAsserts").done(function(res) {
                //console.log("getAsserts res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    $("#edit-group-msg").text("获取资产信息失败，请稍后重试");
                    return setTimeout(function(e){
                        $("#edit-group-close").click()
                    }, 1500)
                }
                g_asserts_bak = info.data;

                setSelectAssertUI(unit);

            }).fail(function(err) {
                console.log("getGroups err:", err);
                $("#edit-group-msg").text("获取资产信息失败，请稍后重试");
                return setTimeout(function(e) {
                    $("#edit-group-close").click()
                }, 1500)
            })
        })
        $("#keepRenderingSort_rightAll").unbind("click").click(function(e) {
            $("#keepRenderingSort option").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#keepRenderingSort_to").append($(html))
            })
        })
        $("#keepRenderingSort_leftAll").unbind("click").click(function(e) {
            $("#keepRenderingSort_to option").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#keepRenderingSort").append($(html))
            })
        })

        $("#keepRenderingSort_rightSelected").unbind("click").click(function(e) {
            $("#keepRenderingSort option:selected").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#keepRenderingSort_to").append($(html))
            })
        })
        $("#keepRenderingSort_leftSelected").unbind("click").click(function(e) {
            $("#keepRenderingSort_to option:selected").each(function () {
                let html = '<option value="' + $(this).val() + '">' + $(this).text() + '</option>';
                $(this).remove();
                $("#keepRenderingSort").append($(html))
            })
        })

        registerEditGroupConfirmEvent();
        $("button.delete-group-btn").unbind("click").click(function(e) {

            let uuid = $(this).data("group-id");
            console.log("delete-group-btn clicked, group id:" + uuid);
            let unit = getGroupById(uuid);

            $("#confirm-operate-private-data").data("private-data", uuid);
            $("#confirm-operate-msg").text("确认要删除分组 " + unit.groupname + " 吗？")

            // 阻止事件上传，不响应渲染分组详情的页面
            // e.stopPropagation();
        })
        $("#confirm-operate-confirm").unbind("click").click(function(e) {

            let uuid = $("#confirm-operate-private-data").data("private-data")
            let unit = getGroupById(uuid);

            let http = cockpit.http({
                "port": 9091
            })
            http.post("/deleteGroup", {"uuid": uuid}).done(function(res){
                console.log("deleteGroup res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("deleteGroup error, res:", res);
                    $("#confirm-operate-msg").text("删除分组失败");
                    let cent = "删除分组: " + unit.groupname + " 失败, 失败原因: " + info.error;
                    writeAuditlog(2, unit.groupname, cent);
                    return ;
                }

                $("#confirm-operate-msg").text("删除分组成功");
                let cent = "删除分组: " + unit.groupname + " 成功";
                writeAuditlog(2, unit.groupname, cent);
                setTimeout(function(e){
                    $("#confirm-operate-close").click();
                    render_groups();
                }, 1000)
            }).fail(function(err){
                console.log("deleteGroup err:", err);
                $("#confirm-operate-msg").text("删除分组失败");
                let cent = "删除分组: " + unit.groupname + " 失败, 失败原因: " + err;
                writeAuditlog(2, unit.groupname, cent);
                return ;
            })
        })
        // 不做分组详情了
        /* $("tr.group-unit-tr-row").unbind("click").click(function(e){
            // 如果是点击的是 编辑 和 删除按钮，则不进行处理
            if (e.target.localName === "button") {
                return ;
            }

            let uuid = $(this).data("goto-unit");
            render_group_detail(uuid);
            cockpit.location.go(["groups-" + uuid]);
        }) */
    }

    function show_groups_page(info) {
        // 这个函数不申请数据，只是重新做下匹配
        let current_text_filter = $('#services-text-filter').val().toLowerCase();

        let units = [];
        let header = {
            Name: _("Group Name"),
            Description: _("Group Description"),
            Assertcnt: _("Assert Number"),
            Operate: _("Operation")
        }

        function cmp_group(a, b) { return info[a].id - info[b].id }
        let sorted_group_keys = Object.keys(info).sort(cmp_group);
        sorted_group_keys.forEach(function(id) {
            let unit = info[id];
            // 获取资产数目
            let assertcnt = 0;
            let assertsarr = unit.groupasserts.split(";")
            unit.edit_text = _("Edit");
            unit.del_text = _("Delete");
            for (let i=0;i<assertsarr.length;i++) {
                if (assertsarr[i] && assertsarr[i].length > 0) {
                    let asserttmp = getUnitById(assertsarr[i]);
                    if (! asserttmp.hasOwnProperty("assertstate")){
                        continue
                    }
                    if ( asserttmp.assertstate == "deleted") {
                        continue
                    }
                    assertcnt = assertcnt + 1
                }
            }
            unit.assertcnt = assertcnt;
            // 匹配搜索字符串
            if (! current_text_filter) {
                units.push(unit);
                return;
            }
            if (unit.groupdesc && unit.groupdesc.toLowerCase().indexOf(current_text_filter) != -1) {
                units.push(unit);
                return;
            }
            if (unit.groupname && unit.groupname.indexOf(current_text_filter) != -1) {
                units.push(unit);
                return;
            }
        });

        // 分页展示
        units = processPagination(units)

        var text = "";
        if (units.length > 0)
            text = mustache.render(groups_template, {
                table_head: header,
                units: units
            });
        else
            text = mustache.render(empty_template);

        $("#services-list").html(text);
        $("#service-detail-list").hide();
        $("#services").show();

        $("#loading-fallback").hide();
    }

    function render_groups() {
        console.log("render_groups.");
        $("#loading-fallback").show();

        $("#create-group").show();
        $("#add-server").hide();
        $("#find-server").hide();
        $("#services-dropdown").hide();

        let http = cockpit.http({
                "port": 9091
            });
        // 获取资产数需要资产信息，需要知道资产是否已经被删除
        http.get("/getAsserts").done(function(res) {
            //console.log("getAsserts res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getAsserts err:", res);
                $("#loading-fallback").hide();
                return ;
            }
            g_asserts_bak = info.data;

            http.get("/getGroups").done(function(res) {
                //console.log("getGroups res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getGroups err:", res);
                    $("#loading-fallback").hide();
                    return ;
                }
                info = info.data;
                g_groups_bak = info;

                // 如果是分组详情页面，在这里拦截，转到分组详情页面，需要到这里因为要获取数据
                if (window.location.href.indexOf("#/groups-") > 0) {
                    let arr = window.location.href.split("#/groups-");
                    let uuid = arr[arr.length - 1];
                    render_group_detail(uuid);
                    return;
                }

                show_groups_page(info);
                registerGroupsEvent();
                $("#loading-fallback").hide();
            }).fail(function(err){
                console.log("getGroups err:", err);
                $("#loading-fallback").hide();
            })

        }).fail(function(err) {
            console.log("getAsserts err:", err);
            $("#loading-fallback").hide();
        })
    }

    $('#services-filter li').on('click', function() {
        $('#services-filter li')
            .removeClass('active')
            .removeAttr('aria-current');
        $(this)
            .addClass('active')
            .attr('aria-current', true);

        let pattern = $(this).attr('data-pattern');
        if (pattern == "asserts") {
            cockpit.location.go(["asserts"]);
            render_asserts();
        }
        else if (pattern == "groups") {
            cockpit.location.go(["groups"]);
            render_groups();
        }
    })

    // 根据 url 初始化渲染页面
    $("#services-filter li")
        .removeClass('active')
        .removeAttr('aria-current');

    if (window.location.href.indexOf("#/groups") > 0) {
        $("#groups-li").addClass('active')
            .attr('aria-current', true);
        render_groups();
    } else {
        $("#asserts-li").addClass('active')
            .attr('aria-current', true);
        render_asserts();
    }
}

/* These add themselves to jQuery so just including is enough */
cockpit.translate();
const _ = cockpit.gettext;
moment.locale(cockpit.language);

if (window.location.pathname.indexOf("manager/index.html") > 0) {
    asserts_init();
}
