/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2013 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */

import $ from 'jquery';
//import cockpit from 'cockpit';

import React from 'react';
import ReactDOM from 'react-dom';
import mustache from 'mustache';
import { writeAuditlog } from './write-auditlog';

const _ = cockpit.gettext;
const C_ = cockpit.gettext;

let g_asserts_bak = [];
let g_rules_bak = [];
let g_users_bak = [];
let g_repos_bak = [];
let g_updates_bak = [];

/* INITIALIZATION AND NAVIGATION
 *
 * The code above still uses the legacy 'Page' abstraction for both
 * pages and dialogs, and expects page.setup, page.enter, page.show,
 * and page.leave to be called at the right times.
 *
 * We cater to this with a little compatability shim consisting of
 * 'dialog_setup', 'page_show', and 'page_hide'.
 */

function show_error_dialog(title, message) {
    if (message) {
        $("#error-popup-title").text(title);
        $("#error-popup-message").text(message);
    } else {
        $("#error-popup-title").text(_("Error"));
        $("#error-popup-message").text(title);
    }

    $("#show-error-dialog").click();
}

function rules_init() {

    let account_empty_tmpl = $("#account-empty-tmpl").html();
    mustache.parse(account_empty_tmpl);

    let rules_list_tmpl = $("#rules-list-tmpl").html();
    mustache.parse(rules_list_tmpl);

    // 分页界面的相关变量放在这做成全局变量
    let g_vue_pagination_rules;
    let g_cur_page_rules = 1;
    let g_page_size_rules = 20;

    function processPagination(units) {
        // 分页展示
        if (!g_vue_pagination_rules) {
            g_vue_pagination_rules = new Vue({
                template: '<el-pagination @size-change="handleSizeChange" @current-change="handleCurrentChange" :current-page="current_page" :page-sizes="[20, 40, 60, 80]" :page-size="page_size" layout="total, sizes, prev, pager, next, jumper" :total="total"> </el-pagination>',
                el: "#rules-vue-app-div",
                methods: {
                    handleSizeChange(val) {
                        g_page_size_rules = val;
                        console.log("set g_page_size_rules to:", val)
                        show_rules_page(g_rules_bak)
                        registerRulesPage();
                    },
                    handleCurrentChange(val) {
                        g_cur_page_rules = val;
                        console.log("set g_cur_page_rules to:", val)
                        show_rules_page(g_rules_bak)
                        registerRulesPage();
                    }
                },
                data: {
                    current_page: g_cur_page_rules,
                    page_size: g_page_size_rules,
                    total: units.length,
                }
            })
        } else {
            g_vue_pagination_rules.current_page = g_cur_page_rules;
            g_vue_pagination_rules.page_size = g_page_size_rules;
            g_vue_pagination_rules.total = units.length;
        }

        units = units.slice( (g_cur_page_rules-1) * g_page_size_rules,
            g_cur_page_rules * g_page_size_rules )

        return units
    }

    function getAssertById(assertid) {
    	for (let i=0;i<g_asserts_bak.length;i++) {
    		if (g_asserts_bak[i].assertid == assertid) {
    			return g_asserts_bak[i]
    		}
    	}
    	return false;
    }

    function getRuleById(id) {
    	for (let i=0;i<g_rules_bak.length;i++) {
    		if (g_rules_bak[i].id == id) {
    			return g_rules_bak[i]
    		}
    	}
    	return false;
    }

    function show_rules_page(info) {
        let current_text_filter = $('#services-text-filter').val().toLowerCase();

    	let header = {
    		Name: "规则名称",
    		Straname: "策略名称",
    		Applyto: "应用到",
    		Status: "规则状态",
            From: "来源于",
    		Operate: "操作"
    	}

        let units = [];
        function cmp_rule(a, b) { return info[a].id - info[b].id }
        let sorted_rules_keys = Object.keys(info).sort(cmp_rule);
        sorted_rules_keys.forEach(function(id) {

            let unit = info[id];

            try {
                if (unit.type == "repo" || unit.type == "update"){
                    unit.straname = unit.stracontent;
                } 
                else {
                    let userstra = $.parseJSON(unit.stracontent);
                    unit.straname = userstra.name;
                    if (unit.type == "security") {
                        unit.straname = "默认策略"
                    }
                }
            } catch(e) {
                console.log(e)
            }

            let applytotext = "";
            let assertsarr = unit.applyto.split(";");
            for (let i=0;i<assertsarr.length;i++) {
                if (!assertsarr[i] || assertsarr[i].length < 1) {
                    continue
                }
                let asserttmp = getAssertById(assertsarr[i]);
                applytotext = applytotext + "资产：" + asserttmp.assertip + "-" + asserttmp.assertname + ";"
            }
            unit.applytotext = applytotext;

            if (unit.applycount.length < 1) {
                unit.status = "下发中"
            }
            else {
                try {
                    let applyjson = $.parseJSON(unit.applycount);
                    let keys = Object.keys(applyjson);
                    for (let i=0;i<keys.length;i++) {
                        if (applyjson[keys[i]]["applycount"] < unit.editcount) {
                            // 说明还是上次的下发结果
                            unit.status = "尚未下发"
                            break;
                        }
                        if (applyjson[keys[i]]["result"] != "succeed") {
                            // 只要有一个下发失败，就认为下发失败
                            unit.status = "下发失败"
                            break;
                        }
                    }
                    // 如果全部都成功，才认为下发成功
                    if (! unit.status || unit.status.length < 1) {
                        unit.status = "下发成功"
                    }
                } catch(e) {
                    console.log(e)
                }
            }

            if (unit.type == "user") {
                unit.from = "账户密码"
            }
            else if (unit.type == "security") {
                unit.from = "安全管理"
            }
            else if (unit.type == "repo"){
                unit.from = "软件源"
            }
            else if (unit.type == "update"){
                unit.from = "软件升级"
            }
            else {
                unit.from = ""
            }

            // 匹配搜索字符串
            if (! current_text_filter) {
                units.push(unit);
                return;
            }
            if (unit.name && unit.name.indexOf(current_text_filter) != -1) {
                units.push(unit);
                return;
            }
        });

        if (units.length < 1) {
            let html = mustache.render(account_empty_tmpl, {
                empty_text: "没有匹配的规则"
            })
            return $("#accounts-list").html(html);
        }

        units = processPagination(units);

    	let text = mustache.render(rules_list_tmpl, {
                table_head: header,
                units: units
            });
    	$("#accounts-list").html(text);
    }

    let vue_choose_asserts;
	let vue_edit_choose_asserts;
	// vuedata 放在这里定义，新增，编辑，新增确认，编辑确认的时候都可以访问
    let vuedata;
    function getVuedataByAssertsAndGroups(info) {
    	vuedata = [];
     	let allGroupAsserts = [];
        let allasserts = info["allasserts"];
        function getAssertsById(id) {
        	for (let i=0;i<allasserts.length;i++) {
        		if (allasserts[i].assertid == id) {
        			return allasserts[i]
        		}
        	}
        	return {}
        }

        for (let i=0;i<info["allgroups"].length;i++) {
        	let grouptmp = info["allgroups"][i];
        	let unit = {
        		id: grouptmp.groupid,
        		label: "分组：" + grouptmp.groupname,
        		children: []
        	}

    		let tmparr = grouptmp.groupasserts.split(";")
    		for (let k=0;k<tmparr.length;k++) {
    			if (!tmparr[k] || tmparr[k].length < 1) {
    				continue
    			}
    			let asserttmp = getAssertsById(tmparr[k]);
    			if (asserttmp.assertip == "localhost") {
    				continue
    			}
    			unit.children.push({
    				id: tmparr[k],
    				label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
    			})
    			if (allGroupAsserts.indexOf(tmparr[k]) < 0) {
    				allGroupAsserts.push(tmparr[k])
    			}
    		}

        	vuedata.push(unit);
        }

        for (let i=0;i<info["allasserts"].length;i++) {
        	let asserttmp = info["allasserts"][i];
        	if (allGroupAsserts.indexOf(asserttmp.assertid) >= 0) {
        		continue
        	}
        	if (asserttmp.assertip == "localhost") {
    				continue
    			}
        	let unit = {
        		id: asserttmp.assertid,
        		label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
        	}
        	vuedata.push(unit);
        }

        return vuedata;
    }

    function registerRulesPage() {
        // 点击下发按钮触发后台下发
        $("button.download-rule-btn").unbind("click").click(function(e){
            let ruleid = $(this).data("rule-id");
            let ruleunit = getRuleById(ruleid);
            let ruletype = ruleunit.type;
            console.log(ruleid);
            console.log(ruletype);
            let http = cockpit.http({
                "port": 9091
            });
            http.post("/downloadRule", {
                type: ruletype,
                id: ruleid
            })
            .fail(function(err){
                console.log("downloadRule err:", err);
                show_error_dialog("错误", "通知后台下发失败")
                let cent = "下发规则: " + ruleunit.name + " 失败, ruletype:" + ruletype + ", 失败原因: " + err;
                writeAuditlog(2, ruleunit.name, cent);
            })
            .done(function(res) {
                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("downloadRule err:", res);
                    let cent = "下发规则: " + ruleunit.name + " 失败, ruletype:" + ruletype + ", 失败原因: " + info.error;
                    writeAuditlog(2, ruleunit.name, cent);
                    return show_error_dialog("错误", "通知后台下发失败：" + info.error)
                }
                show_error_dialog("提示", "通知后台下发成功");
                let cent = "下发规则: " + ruleunit.name + " 成功, ruletype:" + ruletype;
                writeAuditlog(2, ruleunit.name, cent);
                setTimeout(function(){
                    $("#error-popup-close").click()
                    render_rules()
                }, 1500)
            })
        })
        // 点击规则行上的更多按钮时，弹出菜单，需要在渲染完成后再绑定事件
        $(".user-rule-opr-more").unbind("click").on("click", function(e){
            let top = $(this).offset().top;
            let left = $(this).offset().left;
            let top1 = $("html").scrollTop();
            top = top - top1;
            let width = parseInt($(this).css("width").replace("px", ""));

            let ruleid = $(this).data("rule-id");
            let ruleunit = getRuleById(ruleid);
            console.log("user-rule-opr-more");
            console.log(ruleunit.type);

            //对于软件源规则和软件升级规则，暂时不提供编辑功能
            if (ruleunit.type == "repo" || ruleunit.type == "update"){
                $("button.user-rule-edit").hide();
            } else {
                $("button.user-rule-edit").show();
            }

            $(".user-rule-popup-menu").css("top", top);
            $(".user-rule-popup-menu").css("left", left + width);

            if ($(".user-rule-popup-menu").css("display") == "none") {
                $(".user-rule-popup-menu").css("display", "inline-grid")
            } else {
                $(".user-rule-popup-menu").css("display", "none")
            }

            $("#user-rule-popup-menu-private-data").data("private-data", ruleid)
        })
        $(".user-rule-popup-btn").mouseover(function(e){
            $(this).css("background-color", "#def3ff")
        })
        .mouseout(function(e){
            $(this).css("background-color", "white")
        })
        $(window).unbind("click").on("click", function(e){
            if ( e.target.localName == "button" &&
                (e.target.className.indexOf("user-rule-opr-more") >= 0 ||
                e.target.className.indexOf("user-rule-del") >= 0 ||
                e.target.className.indexOf("user-rule-status") >= 0 ||
                e.target.className.indexOf("user-rule-edit") >= 0 )) {
                return
            }
            $(".user-rule-popup-menu").css("display", "none")
        })
        $("button.user-rule-status").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            try {
                $("#rule-status-content").html("");
                let applycount = $.parseJSON(ruleunit.applycount);
                Object.keys(applycount).forEach(function(key) {
                    // 渲染规则页面的时候已经获取过资产信息，这里直接用，不重新请求了
                    let asserttmp = getAssertById(key);
                    if (!asserttmp) {
                        console.log("[user-rule-status]get assert failed,assertid:", applycount[key]);
                        return
                    }
                    let statustext = ""
                    if (applycount[key]["applycount"] < ruleunit.editcount) {
                        statustext = "尚未下发"
                    } else {
                        if (applycount[key]["result"] == "succeed") {
                            statustext = "下发成功"
                        } else {
                            statustext = "下发失败"
                        }
                    }
                    let tr = "<tr><td class='text-align-left'>" + asserttmp.assertip
                            + "</td><td>" + asserttmp.assertname 
                            + "</td><td><span class=''>"+statustext+"</span></td></tr>"
                    $("#rule-status-content").append($(tr));
                })
            } catch(e) {
                console.log(e)
            }
        })
        $("button.user-rule-edit").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            $("#edit-rule-name").val(ruleunit.name);

            let http = cockpit.http({
                "port": 9091
            });
            http.get("/getUsers")
            .fail(function(err){
                console.log("getUsers err:", err);
                $("#edit-rule-msg").text("获取账号策略失败")
            })
            .done(function(res) {
                console.log("getUsers res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getUsers err:", res);
                    return $("#edit-rule-msg").text("获取账号策略失败：" + info.error)
                }
                info = info.data;
                g_users_bak = info;

                $("#edit-stra-name").html("");

                for (let i=0;i<g_users_bak.length;i++) {
                    let html = "";
                    if (g_users_bak[i].id == ruleunit.straid) {
                        html = '<option selected="selected" value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
                    } else {
                        html = '<option value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
                    }
                    $("#edit-stra-name").append($(html))
                }
                //$("#edit-stra-name").val(ruleunit.straname);

                // 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
                // 先获取资产和分组信息
                http.get("/getGroupsAndAsserts")
                .fail(function(err){
                    console.log("getGroupsAndAsserts err:", err);
                    $("#edit-rule-msg").text("获取资产和分组失败")
                })
                .done(function(res) {
                    console.log("getGroupsAndAsserts res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        console.log("getGroupsAndAsserts err:", res);
                        return $("#edit-rule-msg").text("获取资产和分组失败：" + info.error)
                    }
                    info = info.data;
                    vuedata = getVuedataByAssertsAndGroups(info);

                    if (!vue_edit_choose_asserts) {
                        vue_edit_choose_asserts = new Vue({
                            template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
                            el: "#vue-app-edit-div",
                            methods: {
                                getCheckedNodes() {
                                    return this.$refs.tree.getCheckedNodes();
                                },
                                setCheckedKeys(data) {
                                    this.$refs.tree.setCheckedKeys(data);
                                },
                            },
                            data: {
                                data: vuedata,
                                defaultProps: {
                                    children: 'children',
                                    label: 'label'
                                }
                            },
                            watch: {
                                data(newdata, olddata) {
                                    //
                                }
                            }
                        })
                    }
                    vue_edit_choose_asserts.data = vuedata;

                    let checkedAsserts = ruleunit.applyto.split(";");
                    vue_edit_choose_asserts.setCheckedKeys(checkedAsserts)
                })
            })
        })
        $("#edit-rule-confirm").unbind("click").click(function(e) {
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");

            let choosedAssertsAndGroups = vue_edit_choose_asserts.getCheckedNodes();
            let choosedAsserts = [];
            for (let i=0;i<choosedAssertsAndGroups.length;i++) {
                if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
                    choosedAsserts.push(choosedAssertsAndGroups[i].id)
                }
            }
            if (choosedAsserts.length < 1) {
                return $("#edit-rule-msg").text("请至少选择一个资产");
            }
            let assertsstr = choosedAsserts.join(";");

            let name = $("#edit-rule-name").val();
            if (!name || name.length < 1 || name.length > 32) {
                return $("#edit-rule-msg").text("请输入合法的名称，最大32字符");
            }
            let straid = $("#edit-stra-name option:selected").val();

            let http = cockpit.http({
                "port": 9091
            });
            http.post("/editRule", {
                id: ruleid,
                type: "user",
                name: name,
                straid: straid,
                asserts: assertsstr
            })
            .fail(function(err){
                console.log("editRule err:", err);
                $("#edit-rule-msg").text("编辑规则失败")
                let cent = "编辑规则: " + name + " 失败, ruletype=user, asserts=" + assertsstr + ", 失败原因: " + err;
                writeAuditlog(2, name, cent);
            })
            .done(function(res) {
                console.log("editRule res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("editRule err:", res);
                    let cent = "编辑规则: " + name + " 失败, ruletype=user, asserts=" + assertsstr + ", 失败原因: " + info.error;
                    writeAuditlog(2, name, cent);
                    return $("#edit-rule-msg").text("编辑规则失败：" + info.error)
                }
                $("#edit-rule-msg").text("编辑规则成功");
                let cent = "编辑规则: " + name + " 成功, ruletype=user, asserts=" + assertsstr;
                writeAuditlog(2, name, cent);
                setTimeout(function(e){
                    $("#edit-rule-close").click();
                    render_rules();
                }, 1500)
            })
        })
        $("button.user-rule-del").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            let ruletype = ruleunit.type;
            $("#confirm-del-user-rule-msg").text("确认要删除规则 " + ruleunit.name + " 吗？")

            $("button#confirm-del-user-rule-confirm").unbind("click").click(function(e) {
                $("#confirm-del-user-rule-msg").text("规则正在删除，请耐心等待");
                let http = cockpit.http({
                        "port": 9091
                    });
                http.post("/delRule", {
                    type: ruletype,
                    id: ruleid
                })
                .fail(function(err){
                    console.log("delRule err:", err);
                    $("#confirm-del-user-rule-msg").text("删除规则失败");
                    let cent = "删除规则: " + ruleunit.name + " 失败, ruletype: " + ruletype + ", 失败原因: " + err;
                    writeAuditlog(2, ruleunit.name, cent);
                })
                .done(function(res) {
                    console.log("getUsers res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        console.log("getUsers err:", res);
                        $("#confirm-del-user-rule-msg").text("删除规则失败：" + info.error);
                        let cent = "删除规则: " + ruleunit.name + " 失败, ruletype: " + ruletype + ", 失败原因: " + info.error;
                        writeAuditlog(2, ruleunit.name, cent);
                        return ;
                    }
                    $("#confirm-del-user-rule-msg").text("删除规则成功");
                    let cent = "删除规则: " + ruleunit.name + " 成功, ruletype: " + ruletype;
                    writeAuditlog(2, ruleunit.name, cent);
                    setTimeout(function(e){
                        $("#confirm-del-user-rule-close").click()
                        render_rules();
                    }, 1500)
                })
            })
        })
    }

    function render_rules() {
        console.log("render_rules");
    	$(".user-rule-popup-menu").css("display", "none")
    	$("#loading-fallback").show();

    	let http = cockpit.http({
                "port": 9091
            });
        http.get("/getRules", {"type": "all"})
        .fail(function(err) {
            console.log("getRules err:", err);
            $("#loading-fallback").hide();
        })
        .done(function(res) {
            console.log("getRules res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getRules err:", res);
                return $("#loading-fallback").hide();
            }
            info = info.data;
            g_rules_bak = info;

            // 规则里面还要展示应用到的资产，需要获取资产详细信息
            http.get("/getAsserts")
            .fail(function(err){
	            console.log("getAsserts err:", err);
	            return $("#loading-fallback").hide();
	        })
            .done(function(res) {
	            console.log("getAsserts res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
	                console.log("getAsserts err:", res);
	                $("#loading-fallback").hide();
	                return ;
	            }
	            info = info.data;
	            g_asserts_bak = info;

	            show_rules_page(g_rules_bak);
                registerRulesPage()
            	$("#loading-fallback").hide();
	        })
        })
    }

    $('#services-text-filter').change(function(e){
        show_rules_page(g_rules_bak);
    })

    function changeAddRuleType() {
        let addRuleType = $("#add-rule-type option:selected").val();
        if (addRuleType == "user") {
            let http = cockpit.http({
                "port": 9091
            });

            http.get("/getUsers")
            .fail(function(err) {
                console.log("getUsers err:", err);
                show_error_dialog("错误", "获取账号策略失败")
            })
            .done(function(res) {
                console.log("getUsers res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getUsers err:", res);
                    show_error_dialog("错误", "获取账号策略失败：" + info.error)
                }
                info = info.data;
                g_users_bak = info;

                $("#add-stra-name").html("");
                for (let i=0;i<g_users_bak.length;i++) {
                    let html = '<option value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
                    $("#add-stra-name").append($(html))
                }
            })
        }
        else if (addRuleType == "repo") {
            let http = cockpit.http({
                "port": 9091
            });

            http.get("/getRepoStrategies")
            .fail(function(err) {
                console.log("getRepoStrategies err:", err);
                show_error_dialog("错误", "获取软件源策略失败")
            })
            .done(function(res) {
                console.log("getRepoStrategies res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getRepoStrategies err:", res);
                    show_error_dialog("错误", "获取软件源策略失败：" + info.error)
                }
                info = info.data;
                g_repos_bak = info;

                $("#add-stra-name").html("");
                for (let i=0;i<g_repos_bak.length;i++) {
                    let html = '<option value="' + g_repos_bak[i].id + '">' + g_repos_bak[i].name + '</option>';
                    $("#add-stra-name").append($(html))
                }
            })
        }
        else if (addRuleType == "update") {
            let http = cockpit.http({
                "port": 9091
            });

            http.get("/getUpdateStrategies")
            .fail(function(err) {
                console.log("getUpdateStrategies err:", err);
                show_error_dialog("错误", "获取升级策略失败")
            })
            .done(function(res) {
                console.log("getUpdateStrategies res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getUpdateStrategies err:", res);
                    show_error_dialog("错误", "获取升级策略失败：" + info.error)
                }
                info = info.data;
                g_updates_bak = info;

                $("#add-stra-name").html("");
                for (let i=0;i<g_updates_bak.length;i++) {
                    let html = '<option value="' + g_updates_bak[i].id + '">' + g_updates_bak[i].name + '</option>';
                    $("#add-stra-name").append($(html))
                }
            })
        }
        else if (addRuleType == "security") {
            $("#add-stra-name").html("");
            let html = '<option value="default">默认策略</option>';
            $("#add-stra-name").append($(html))
        }
    }

    $("#add-rule-type").change(changeAddRuleType)

    $("#add-rule").unbind("click").click(function(e){
    	$("#add-stra-name").html("");
    	$("#add-rule-name").val("");
        $("#add-rule-msg").text("")

        changeAddRuleType();

        let http = cockpit.http({
                "port": 9091
            });
		// 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
		// 先获取资产和分组信息
		http.get("/getGroupsAndAsserts")
		.fail(function(err) {
            console.log("getGroupsAndAsserts err:", err);
            $("#add-rule-msg").text("获取资产和分组失败")
        })
        .done(function(res) {
            console.log("getGroupsAndAsserts res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getGroupsAndAsserts err:", res);
                return $("#add-rule-msg").text("获取资产和分组失败：" + info.error)
            }

            info = info.data;
            vuedata = getVuedataByAssertsAndGroups(info);

            if (!vue_choose_asserts) {
            	vue_choose_asserts = new Vue({
			    	template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
			    	el: "#vue-app-div",
			    	methods: {
				      	getCheckedNodes() {
				        	return this.$refs.tree.getCheckedNodes();
				      	},
				      	setCheckedKeys(data) {
							this.$refs.tree.setCheckedKeys(data);
						}
				    },
				    data: {
				    	data: vuedata,
				    	defaultProps: {
		          			children: 'children',
		          			label: 'label'
		        		}
		        	},
		        	watch: {
		        		data(newdata, olddata) {
		        			//
		        		}
		        	}
			    })
            }
            vue_choose_asserts.data = vuedata;
        })
    })

	// 确认添加规则时的操作
	$("#add-rule-confirm").unbind("click").click(function(e){
		let choosedAssertsAndGroups = vue_choose_asserts.getCheckedNodes();
		let choosedAsserts = [];
		for (let i=0;i<choosedAssertsAndGroups.length;i++) {
			if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
				choosedAsserts.push(choosedAssertsAndGroups[i].id)
			}
		}

		if (choosedAsserts.length < 1) {
			return $("#add-rule-msg").text("请至少选择一个资产");
		}
		let assertsstr = choosedAsserts.join(";");
		let name = $("#add-rule-name").val();
		if (!name || name.length < 1 || name.length > 32) {
			return $("#add-rule-msg").text("请输入合法的名称，最大32字符");
		}
		let straid = $("#add-stra-name option:selected").val();
		if (!straid || straid.length < 1) {
			return $("#add-rule-msg").text("请选择合法的策略");
		}
        let ruletype = $("#add-rule-type option:selected").val();

		let http = cockpit.http({
            "port": 9091
        });
        $("#add-rule-msg").text("规则正在添加，请耐心等待");
        http.post("/addRule", {
        	type: ruletype,
        	name: name,
        	straid: straid,
        	asserts: assertsstr
        })
        .fail(function(err){
            console.log("addRule err:", err);
            $("#add-rule-msg").text("添加规则失败")
            let cent = "添加规则: " + name + " 失败, ruletype=" + ruletype + ", asserts=" + assertsstr + ", 失败原因: " + err;
            writeAuditlog(2, name, cent);
        })
        .done(function(res) {
            console.log("addRule res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("addRule err:", res);
                let cent = "添加规则: " + name + " 失败, ruletype=" + ruletype + ", asserts=" + assertsstr + ", 失败原因: " + info.error;
                writeAuditlog(2, name, cent);
                return $("#add-rule-msg").text("添加规则失败：" + info.error)
            }
            $("#add-rule-msg").text("添加规则成功");
            let cent = "添加规则: " + name + " 成功, ruletype=" + ruletype + ", asserts="  + assertsstr;
            writeAuditlog(2, name, cent);
            setTimeout(function(e){
            	$("#add-rule-close").click();
            	render_rules();
            }, 1500)
        })
	})

    render_rules();
}

if (window.location.pathname.indexOf("manager/rules.html") > 0) {
    $(rules_init);
}
