/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2013 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */

import $ from 'jquery';
//import cockpit from 'cockpit';

import React from 'react';
import ReactDOM from 'react-dom';
import mustache from 'mustache';
import { writeAuditlog } from './write-auditlog';

const _ = cockpit.gettext;
const C_ = cockpit.gettext;

let g_security_bak = [];
let g_rules_bak = [];
let g_asserts_bak = [];
let intv=0; // 定时器全局变量

function show_error_dialog(title, message) {
    if (message) {
        $("#error-popup-title").text(title);
        $("#error-popup-message").text(message);
    } else {
        $("#error-popup-title").text(_("Error"));
        $("#error-popup-message").text(title);
    }

    $("#show-error-dialog").click();
}

function security_init() {

    let security_empty_tmpl = $("#security-empty-tmpl").html();
    mustache.parse(security_empty_tmpl);

    let rules_list_tmpl = $("#rules-list-tmpl").html();
    mustache.parse(rules_list_tmpl);

    // 分页界面的相关变量放在这做成全局变量
    let g_vue_pagination_security;
    let g_cur_page_security = 1;
    let g_page_size_security = 20;

    function processPagination(units) {
        // 分页展示
        if (!g_vue_pagination_security) {
            g_vue_pagination_security = new Vue({
                template: '<el-pagination @size-change="handleSizeChange" @current-change="handleCurrentChange" :current-page="current_page" :page-sizes="[20, 40, 60, 80]" :page-size="page_size" layout="total, sizes, prev, pager, next, jumper" :total="total"> </el-pagination>',
                el: "#security-vue-app-div",
                methods: {
                    handleSizeChange(val) {
                        g_page_size_security = val;
                        console.log("set g_page_size_security to:", val)
                        show_rules_page()
                        registerRulesPage();
                    },
                    handleCurrentChange(val) {
                        g_cur_page_security = val;
                        console.log("set g_cur_page_security to:", val)
                        show_rules_page()
                        registerRulesPage();
                    }
                },
                data: {
                    current_page: g_cur_page_security,
                    page_size: g_page_size_security,
                    total: units.length,
                }
            })
        } else {
            g_vue_pagination_security.current_page = g_cur_page_security;
            g_vue_pagination_security.page_size = g_page_size_security;
            g_vue_pagination_security.total = units.length;
        }

        units = units.slice( (g_cur_page_security-1) * g_page_size_security,
            g_cur_page_security * g_page_size_security )

        return units
    }

    function getAssertById(assertid) {
    	for (let i=0;i<g_asserts_bak.length;i++) {
    		if (g_asserts_bak[i].assertid == assertid) {
    			return g_asserts_bak[i]
    		}
    	}
    	return false;
    }

    function getRuleById(id) {
    	for (let i=0;i<g_rules_bak.length;i++) {
    		if (g_rules_bak[i].id == id) {
    			return g_rules_bak[i]
    		}
    	}
    	return false;
    }

    function editSecurity(e) {

        function disable_edit() {
            $("#security-fail-cnt").prop("disabled", true);
            $("#security-lock-time").prop("disabled", true);
            $("#security-pass-complex").prop("disabled", true);
            $("#security-disable-udisk").prop("disabled", true);
            $("#security-password-expire").prop("disabled", true);
            $("#selinux-type").prop("disabled", true);
            $("#security-session-timeout").prop("disabled", true);
            $("#security-command").prop("disabled", true);
        }
        function enable_edit() {
            $("#security-fail-cnt").prop("disabled", false);
            $("#security-lock-time").prop("disabled", false);
            $("#security-pass-complex").prop("disabled", false);
            $("#security-disable-udisk").prop("disabled", false);
            $("#security-password-expire").prop("disabled", false);
            $("#selinux-type").prop("disabled", false);
            $("#security-session-timeout").prop("disabled", false);
            $("#security-command").prop("disabled", false);
        }

        // 改变 checkbox 的 disabled 属性时也会触发 change 事件，调用 setup 的时候也会，这里过滤掉
        if ($('#security-fail-cnt').prop("disabled") ||
            $('#security-lock-time').prop("disabled") ||
            $('#security-pass-complex').prop("disabled") ||
            $('#security-disable-udisk').prop("disabled") ||
            $('#security-password-expire').prop("disabled") ||
            $('#selinux-type').prop("disabled") ||
            $('#security-session-timeout').prop("disabled") ||
            $("#security-command").prop("disabled")) {
            return ;
        }

        disable_edit();

        let failcnt = $("#security-fail-cnt").val();
        let locktime = $("#security-lock-time").val();
        
        let passcomplex = $("#security-pass-complex").prop("checked")
        if (passcomplex) {
            passcomplex = 1
        } else {
            passcomplex = 0
        }

        let disableudisk = $("#security-disable-udisk").prop("checked")
        if (disableudisk) {
            disableudisk = 1
        } else {
            disableudisk = 0
        }

        let passexpired = $("#security-password-expire").val();
        let selinuxcfg = $("#selinux-type").val();
        let sessiontimeout = $("#security-session-timeout").val();

        let security_command = $("#security-command").val();

        let http = cockpit.http({
            "port": 9091
        });
        http.post("/editSecurity", {
            "loginFailedCnt": failcnt,
            "loginFailedLockTime": locktime,
            "passwordComplex": passcomplex,
            "disableUdisk": disableudisk,
            "passwordExpired": passexpired,
            "selinuxConfig": selinuxcfg,
            "sessionTimeout": sessiontimeout,
            "securityCommand": security_command
        })
        .fail(function(err) {
            console.log("editSecurity err:", err);
            enable_edit();
            let cent = "设置安全策略失败, 失败原因: " + err;
            writeAuditlog(2, "安全策略", cent);
            return show_error_dialog("错误", "设置安全策略失败");
        })
        .done(function(res) {
            console.log("editSecurity res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                let cent = "设置安全策略失败, 失败原因: " + info.error;
                writeAuditlog(2, "安全策略", cent);
                enable_edit()
                show_security_page()
                return show_error_dialog("错误", "设置安全策略失败：" + info.error);
            }

            let cent = "设置安全策略成功";
            writeAuditlog(2, "安全策略", cent);
            setTimeout(function(){
                render_security()
                enable_edit()
            }, function(e){
                show_error_dialog("错误", "刷新数据失败");
            }, 1500)
        })
    }

    function show_security_page() {
        $("#security-fail-cnt").val(g_security_bak.loginFailedCnt);
        $("#security-lock-time").val(g_security_bak.loginFailedLockTime);

        if (g_security_bak.passwordComplex) {
            $("#security-pass-complex").prop("checked", true)
        } else {
            $("#security-pass-complex").prop("checked", false)
        }

        if (g_security_bak.disableUdisk) {
            $("#security-disable-udisk").prop("checked", true)
        } else {
            $("#security-disable-udisk").prop("checked", false)
        }

        $("#security-password-expire").val(g_security_bak.passwordExpired);
        $("#selinux-type").val(g_security_bak.selinuxConfig);
        $("#security-session-timeout").val(g_security_bak.sessionTimeout);
        $("#security-command").val(g_security_bak.securityCommand);
        // 编辑提交
        $("#security-fail-cnt").change(editSecurity);
        $("#security-lock-time").change(editSecurity);
        $("#security-pass-complex").change(editSecurity);
        $("#security-disable-udisk").change(editSecurity);
        $("#security-password-expire").change(editSecurity);
        $("#selinux-type").change(editSecurity);
        $("#security-session-timeout").change(editSecurity);
        $("#security-command").change(editSecurity);
    }

    function render_security() {
    	// 如果是从账户详情页面跳回，需要先隐藏掉账户详情页面
    	$("#rules").hide();
        $("#security").show();
    	$("#loading-fallback").show();

    	let http = cockpit.http({
                "port": 9091
            });
        http.get("/getSecurity")
        .fail(function(err){
            console.log("getSecurity err:", err);
            $("#loading-fallback").hide();
        })
        .done(function(res) {
            console.log("getSecurity res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getSecurity err:", res);
                $("#loading-fallback").hide();
                return ;
            }

            info = $.parseJSON(info.data.content);
            g_security_bak = info;

            show_security_page();
            $("#loading-fallback").hide();
        })
    }

    function registerRulesPage() {
        // 点击下发按钮触发后台下发
        $("button.download-rule-btn").unbind("click").click(function(e){
            let ruleid = $(this).data("rule-id");
            let ruleunit = getRuleById(ruleid);
            let http = cockpit.http({
                "port": 9091
            });
            $("#download-rule-btn-msg").text("确认要重新下发规则 " +  ruleunit.name + " 吗？")
            $("#download-rule-btn-confirm").unbind("click").click(function(e){
                http.post("/downloadRule", {
                    type: "security",
                    id: ruleid
                })
                .fail(function(err){
                    console.log("downloadRule err:", err);
                    $("#download-rule-btn-msg").text("通知后台下发失败")
                    let cent = "下发规则: " + ruleunit.name + " 失败, ruletype: security, 失败原因: " + err;
                    writeAuditlog(2, ruleunit.name, cent);
                })
                .done(function(res) {
                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        console.log("downloadRule err:", res);
                        $("#download-rule-btn-msg").text("通知后台下发失败：" + info.error);
                        let cent = "下发规则: " + ruleunit.name + " 失败, ruletype: security, 失败原因: " + info.error;
                        writeAuditlog(2, ruleunit.name, cent);
                        return ;
                    }
                    $("#download-rule-btn-msg").text("已通知后台下发");
                    let cent = "下发规则: " + ruleunit.name + " 成功, ruletype: security";
                    writeAuditlog(2, ruleunit.name, cent);
                    setTimeout(function(e){
                        $("#download-rule-btn-close").click()
                        render_rules();
                    }, 1500)
                })
            })
        })
        // 点击规则行上的更多按钮时，弹出菜单，需要在渲染完成后再绑定事件
        $(".user-rule-opr-more").unbind("click").on("click", function(e){
            let top = $(this).offset().top;
            let left = $(this).offset().left;
            let top1 = $("html").scrollTop();
            top = top - top1;
            let width = parseInt($(this).css("width").replace("px", ""));

            $(".user-rule-popup-menu").css("top", top);
            $(".user-rule-popup-menu").css("left", left + width);

            if ($(".user-rule-popup-menu").css("display") == "none") {
                $(".user-rule-popup-menu").css("display", "inline-grid")
            } else {
                $(".user-rule-popup-menu").css("display", "none")
            }

            let ruldid = $(this).data("rule-id")
            $("#user-rule-popup-menu-private-data").data("private-data", ruldid)
        })
        $(".user-rule-popup-btn").mouseover(function(e){
            $(this).css("background-color", "#def3ff")
        })
        .mouseout(function(e){
            $(this).css("background-color", "white")
        })
        $(window).unbind("click").on("click", function(e){
            if ( e.target.localName == "button" &&
                (e.target.className.indexOf("user-rule-opr-more") >= 0 ||
                e.target.className.indexOf("user-rule-del") >= 0 ||
                e.target.className.indexOf("user-rule-status") >= 0 ||
                e.target.className.indexOf("user-rule-edit") >= 0 )) {
                return
            }
            $(".user-rule-popup-menu").css("display", "none")
        })
        $("button.user-rule-status").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            try {
                $("#rule-status-content").html("");
                let applycount = $.parseJSON(ruleunit.applycount);
                Object.keys(applycount).forEach(function(key) {
                    // 渲染规则页面的时候已经获取过资产信息，这里直接用，不重新请求了
                    let asserttmp = getAssertById(key);
                    if (!asserttmp) {
                        console.log("[user-rule-status]get assert failed,assertid:", applycount[key]);
                        return
                    }
                    let statustext = ""
                    if (applycount[key]["applycount"] < ruleunit.editcount) {
                        statustext = "尚未下发"
                    } else {
                        if (applycount[key]["result"] == "succeed") {
                            statustext = "下发成功"
                        } else {
                            statustext = "下发失败"
                        }
                    }
                    let tr = "<tr><td class='text-align-left'>" + asserttmp.assertip
                            + "</td><td>" + asserttmp.assertname 
                            + "</td><td><span class=''>"+statustext+"</span>"
                            + "</td><td><buttom data-toggle='modal' class='btn btn-default securityrules-status'"
                            + " data-target='#securityrules-status-dialog' securityrules-assert-id='"+key
                            + "'>查看详情</buttom>"
                            + "</td></tr>"
                    $("#rule-status-content").append($(tr));
                })
                $("buttom.securityrules-status").unbind("click").click(function(e){
                    let buttomid = $(this).attr("securityrules-assert-id");
                    $("#securityrules-status-msg").text(applycount[buttomid]["result"]);
                })
            } catch(e) {
                console.log(e)
            }
        })
        $("button.user-rule-edit").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            $("#edit-rule-name").val(ruleunit.name);

            let http = cockpit.http({
                "port": 9091
            });
            http.get("/getUsers")
            .fail(function(err){
                console.log("getUsers err:", err);
                $("#edit-rule-msg").text("获取账号策略失败")
            })
            .done(function(res) {
                console.log("getUsers res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getUsers err:", res);
                    return $("#edit-rule-msg").text("获取账号策略失败：" + info.error)
                }
                info = info.data;

                $("#edit-stra-name").html("");
                let html = '<option value="default">' + '默认策略' + '</option>';
                $("#edit-stra-name").append($(html))

                //$("#edit-stra-name").val(ruleunit.straname);

                // 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
                // 先获取资产和分组信息
                http.get("/getGroupsAndAsserts")
                .fail(function(err){
                    console.log("getGroupsAndAsserts err:", err);
                    $("#edit-rule-msg").text("获取资产和分组失败")
                })
                .done(function(res) {
                    // console.log("getGroupsAndAsserts res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        console.log("getGroupsAndAsserts err:", res);
                        return $("#edit-rule-msg").text("获取资产和分组失败：" + info.error)
                    }
                    info = info.data;
                    vuedata = getVuedataByAssertsAndGroups(info);

                    if (!vue_edit_choose_asserts) {
                        vue_edit_choose_asserts = new Vue({
                            template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
                            el: "#vue-app-edit-div",
                            methods: {
                                getCheckedNodes() {
                                    return this.$refs.tree.getCheckedNodes();
                                },
                                setCheckedKeys(data) {
                                    this.$refs.tree.setCheckedKeys(data);
                                },
                            },
                            data: {
                                data: vuedata,
                                defaultProps: {
                                    children: 'children',
                                    label: 'label'
                                }
                            },
                            watch: {
                                data(newdata, olddata) {
                                    //
                                }
                            }
                        })
                    }
                    vue_edit_choose_asserts.data = vuedata;

                    let checkedAsserts = ruleunit.applyto.split(";");
                    vue_edit_choose_asserts.setCheckedKeys(checkedAsserts)
                })
            })
        })
        $("#edit-rule-confirm").unbind("click").click(function(e) {
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");

            let choosedAssertsAndGroups = vue_edit_choose_asserts.getCheckedNodes();
            let choosedAsserts = [];
            for (let i=0;i<choosedAssertsAndGroups.length;i++) {
                if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
                    choosedAsserts.push(choosedAssertsAndGroups[i].id)
                }
            }
            if (choosedAsserts.length < 1) {
                return $("#edit-rule-msg").text("请至少选择一个资产");
            }
            let assertsstr = choosedAsserts.join(";");

            let name = $("#edit-rule-name").val();
            if (!name || name.length < 1 || name.length > 32) {
                return $("#edit-rule-msg").text("请输入合法的名称，最大32字符");
            }
            let straid = $("#edit-stra-name option:selected").val();

            let http = cockpit.http({
                "port": 9091
            });
            http.post("/editRule", {
                id: ruleid,
                type: "security",
                name: name,
                straid: straid,
                asserts: assertsstr
            })
            .fail(function(err){
                console.log("editRule err:", err);
                $("#edit-rule-msg").text("编辑规则失败")
                let cent = "编辑安全管理规则: " + name + " 失败, asserts: " + assertsstr + ", 失败原因: " + err;
                writeAuditlog(2, name, cent);
            })
            .done(function(res) {
                console.log("editRule res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("editRule err:", res);
                    let cent = "编辑安全管理规则: " + name + " 失败, asserts: " + assertsstr + ", 失败原因: " + info.error;
                    writeAuditlog(2, name, cent);
                    return $("#edit-rule-msg").text("编辑规则失败：" + info.error)
                }
                $("#edit-rule-msg").text("编辑规则成功");
                let cent = "编辑安全管理规则: " + name + " 成功, asserts: " + assertsstr;
                writeAuditlog(2, name, cent);
                setTimeout(function(e){
                    $("#edit-rule-close").click();
                    render_rules();
                }, 1500)
            })
        })
        $("button.user-rule-del").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            $("#confirm-del-user-rule-msg").text("确认要删除规则 " + ruleunit.name + " 吗？")

            $("button#confirm-del-user-rule-confirm").unbind("click").click(function(e) {
                let http = cockpit.http({
                        "port": 9091
                    });
                http.post("/delRule", {
                    type: "security",
                    id: ruleid
                })
                .fail(function(err){
                    console.log("delRule err:", err);
                    $("#confirm-del-user-rule-msg").text("删除规则失败");
                    let cent = "删除安全管理规则: " + ruleunit.name + " 失败, 失败原因: " + err;
                    writeAuditlog(2, ruleunit.name, cent);
                })
                .done(function(res) {
                    console.log("getUsers res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        console.log("getUsers err:", res);
                        $("#confirm-del-user-rule-msg").text("删除规则失败：" + info.error);
                        let cent = "删除安全管理规则: " + ruleunit.name + " 失败, 失败原因: " + info.error;
                        writeAuditlog(2, ruleunit.name, cent);
                        return ;
                    }
                    $("#confirm-del-user-rule-msg").text("删除规则成功");
                    let cent = "删除安全管理规则: " + ruleunit.name + " 成功";
                    writeAuditlog(2, ruleunit.name, cent);
                    setTimeout(function(e){
                        $("#confirm-del-user-rule-close").click()
                        render_rules();
                    }, 1500)
                })
            })
        })
    }

    function show_rules_page() {
    	if (g_rules_bak.length < 1) {
    		let html = mustache.render(security_empty_tmpl, {
    			empty_text: "尚未创建规则"
    		})
    		return $("#rules-list").html(html);
    	}

    	let header = {
    		Name: "规则名称",
    		Straname: "策略名称",
    		Applyto: "应用到",
    		Status: "规则状态",
    		Operate: "操作"
    	}

    	let units = [];
        
    	for (let i=0;i<g_rules_bak.length;i++) {
    		let unit = g_rules_bak[i];
    		let applytotext = "";
            let assertsarr = unit.applyto.split(";");
            let deleted = [];
    		for (let i=0;i<assertsarr.length;i++) {
    			if (!assertsarr[i] || assertsarr[i].length < 1) {
    				continue
    			}
                let asserttmp = getAssertById(assertsarr[i]);
                if (!asserttmp.assertip){
                    // 过滤掉已删除的资产
                    deleted.push(assertsarr[i]);
                    continue
                }
    			applytotext = applytotext + "资产：" + asserttmp.assertip + "-" + asserttmp.assertname + ";"
    		}
    		unit.applytotext = applytotext;
    		try {
    			unit.straname = "默认策略";
    		} catch(e) {
    			console.log(e)
    		}

    		if (unit.applycount.length < 1 || unit.status == "redownload") {
    			unit.status = "下发中"
    		}
    		else {
    			try {
    				let applyjson = $.parseJSON(unit.applycount);
                    let keys = Object.keys(applyjson);
                    let flg = true;
	    			for (let i=0;i<keys.length;i++) {
	    				if (applyjson[keys[i]]["applycount"] < unit.editcount) {
	    					// 说明还是上次的下发结果
	    					unit.status = "尚未下发"
	    					break;
	    				}
	    				if (applyjson[keys[i]]["result"] != "succeed") {
                            // 判断失败的是否为删除的如果在删除的里面继续循环
                            if ($.inArray(keys[i], deleted) == -1){
                                unit.status = "下发失败"
                                flg = false;
                                break;
                            }
	    				}
	    			}
	    			// 如果全部都成功，才认为下发成功
	    			if (! unit.status || unit.status.length < 1 || flg) {
	    				unit.status = "下发成功"
	    			}
    			} catch(e) {
    				console.log(e)
    			}
    		}
            unit.ruleid=g_rules_bak[i].id;
    		units.push(unit);
    	}

        // 处理分页
        units = processPagination(units);

    	let text = mustache.render(rules_list_tmpl, {
                table_head: header,
                units: units
            });
    	$("#rules-list").html(text);
    }

    let vue_choose_asserts;
	let vue_edit_choose_asserts;
    let vuedata;
    function getVuedataByAssertsAndGroups(info) {
    	vuedata = [];
     	let allGroupAsserts = [];
        let allasserts = info["allasserts"];
        function getAssertsById(id) {
        	for (let i=0;i<allasserts.length;i++) {
        		if (allasserts[i].assertid == id) {
        			return allasserts[i]
        		}
        	}
        	return {}
        }

        for (let i=0;i<info["allgroups"].length;i++) {
        	let grouptmp = info["allgroups"][i];
        	let unit = {
        		id: grouptmp.groupid,
        		label: "分组：" + grouptmp.groupname,
        		children: []
        	}

    		let tmparr = grouptmp.groupasserts.split(";")
    		for (let k=0;k<tmparr.length;k++) {
    			if (!tmparr[k] || tmparr[k].length < 1) {
    				continue
    			}
    			let asserttmp = getAssertsById(tmparr[k]);
    			if (asserttmp.assertip == "localhost") {
    				continue
                }
                if (!asserttmp.assertip){
                    // 过滤掉已经删除的
                    continue
                }
    			unit.children.push({
    				id: tmparr[k],
    				label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
    			})
    			if (allGroupAsserts.indexOf(tmparr[k]) < 0) {
    				allGroupAsserts.push(tmparr[k])
    			}
    		}
            if (unit.children.length == 0){
                continue;
            }
        	vuedata.push(unit);
        }

        for (let i=0;i<info["allasserts"].length;i++) {
        	let asserttmp = info["allasserts"][i];
        	if (allGroupAsserts.indexOf(asserttmp.assertid) >= 0) {
        		continue
        	}
        	if (asserttmp.assertip == "localhost") {
    				continue
    			}
        	let unit = {
        		id: asserttmp.assertid,
        		label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
        	}
        	vuedata.push(unit);
        }

        return vuedata;
    }

    function render_rules() {
        $("#security").hide();
        $("#rules").show();
    	$("#add-rule").show();
    	$(".user-rule-popup-menu").css("display", "none")

    	$("#loading-fallback").show();
        clearInterval(intv)
    	let http = cockpit.http({
                "port": 9091
            });
        http.get("/getRules", {"type": "security"})
        .fail(function(err) {
            console.log("getRules of security err:", err);
            $("#loading-fallback").hide();
        })
        .done(function(res) {
            console.log("getRules of security res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getRules of security err:", res);
                return $("#loading-fallback").hide();
            }
            info = info.data;
            g_rules_bak = info;

            // 规则里面还要展示应用到的资产，需要获取资产详细信息
            http.get("/getAsserts")
            .fail(function(err){
	            console.log("getAsserts err:", err);
	            return $("#loading-fallback").hide();
	        })
            .done(function(res) {
	            // console.log("getAsserts res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
	                console.log("getAsserts err:", res);
	                $("#loading-fallback").hide();
	                return ;
	            }
	            info = info.data;
	            g_asserts_bak = info;

                show_rules_page();
                registerRulesPage();
            	$("#loading-fallback").hide();
                // 定时器获取规则状态
                var dlsucceed = /\u4e0b\u53d1\u4e2d/
                intv = setInterval(()=>{
                    let isdone = true
                    for (var i=0; i<g_rules_bak.length; i++){
                        if (dlsucceed.test(g_rules_bak[i].status)){   
                            isdone = false              
                            http.get("/getRuleState", {"type": "security"})
                            .fail(function(err) {
                                console.log("getRules of security err:", err);
                            })
                            .done(function(res) {
                                let info = $.parseJSON( res );
                                if (! info.succeed) {
                                    console.log("getRules of security err:", res);
                                }
                                info = info.data;
                                g_rules_bak = info ? info : g_rules_bak;
                            })
                        }else{
                            for (var k in g_rules_bak){
                                var arule = g_rules_bak[k];
                                $("#rule-"+arule["id"]).text(arule["status"]);
                            }
                        }
                    }
                    if (isdone){
                        clearInterval(intv);
                    }
                },5000);
	        })
        })
    }

    $("#add-rule").unbind("click").click(function(e){
    	$("#add-stra-name").html("");
    	$("#add-rule-name").val("");
    	let http = cockpit.http({
            "port": 9091
        });

        $("#add-stra-name").html("");
		let html = '<option value="default">' + '默认策略' + '</option>';
    	$("#add-stra-name").append($(html))

		// 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
		// 先获取资产和分组信息
		http.get("/getGroupsAndAsserts")
		.fail(function(err){
            console.log("getGroupsAndAsserts err:", err);
            $("#add-rule-msg").text("获取资产和分组失败")
        })
        .done(function(res) {
            // console.log("getGroupsAndAsserts res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getGroupsAndAsserts err:", res);
                return $("#add-rule-msg").text("获取资产和分组失败：" + info.error)
            }

            info = info.data;
            vuedata = getVuedataByAssertsAndGroups(info);

            if (!vue_choose_asserts) {
            	vue_choose_asserts = new Vue({
			    	template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
			    	el: "#vue-app-div",
			    	methods: {
				      	getCheckedNodes() {
				        	return this.$refs.tree.getCheckedNodes();
				      	},
				      	setCheckedKeys(data) {
							this.$refs.tree.setCheckedKeys(data);
						}
				    },
				    data: {
				    	data: vuedata,
				    	defaultProps: {
		          			children: 'children',
		          			label: 'label'
		        		}
		        	},
		        	watch: {
		        		data(newdata, olddata) {
		        			//
		        		}
		        	}
			    })
            }
            vue_choose_asserts.data = vuedata;
        })
    })
	// 确认添加规则时的操作
	$("#add-rule-confirm").unbind("click").click(function(e){
		let choosedAssertsAndGroups = vue_choose_asserts.getCheckedNodes();
		let choosedAsserts = [];
		for (let i=0;i<choosedAssertsAndGroups.length;i++) {
			if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
				choosedAsserts.push(choosedAssertsAndGroups[i].id)
			}
		}

		if (choosedAsserts.length < 1) {
			return $("#add-rule-msg").text("请至少选择一个资产");
		}
		let assertsstr = choosedAsserts.join(";");
		let name = $("#add-rule-name").val();
		if (!name || name.length < 1 || name.length > 32) {
			return $("#add-rule-msg").text("请输入合法的名称，最大32字符");
		}

		let http = cockpit.http({
            "port": 9091
        });
        http.post("/addRule", {
        	type: "security",
        	name: name,
        	asserts: assertsstr
        })
        .fail(function(err){
            console.log("addRule of security err:", err);
            $("#add-rule-msg").text("添加规则失败")
            let cent = "添加安全管理规则: " + name  +  " 失败, asserts: " + assertsstr + ", 失败原因: " + err;
            writeAuditlog(2, name, cent);
        })
        .done(function(res) {
            console.log("addRule of security res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("addRule of security err:", res);
                let cent = "添加安全管理规则: " + name  +  " 失败, asserts: " + assertsstr + ", 失败原因: " + info.error;
                writeAuditlog(2, name, cent);
                return $("#add-rule-msg").text("添加规则失败：" + info.error)
            }
            $("#add-rule-msg").text("添加规则成功");
            let cent = "添加安全管理规则: " + name  +  " 成功, asserts: " + assertsstr;
            writeAuditlog(2, name, cent);
            setTimeout(function(e){
            	$("#add-rule-close").click();
            	render_rules();
            }, 1500)
        })
	})

    $('#security-filter li').on('click', function() {
        $('#security-filter li')
            .removeClass('active')
            .removeAttr('aria-current');
        $(this)
            .addClass('active')
            .attr('aria-current', true);

        let pattern = $(this).attr('data-pattern');
        if (pattern == "securityrules") {
            cockpit.location.go(["securityrules"]);
            $("#add-rule").show();
        }
        else if (pattern == "security") {
            cockpit.location.go([""]);
            $("#add-rule").hide();
        }
    })

    // 根据 url 初始化渲染页面
    $("#security-filter li")
        .removeClass('active')
        .removeAttr('aria-current');

    function navigate() {
        if (window.location.href.indexOf("#/securityrules") > 0) {
            $("#securityrules-li").addClass('active')
                .attr('aria-current', true);
			render_rules();
	    } else {
            $("#security-li").addClass('active')
                .attr('aria-current', true);
	        render_security();
	    }
    }

    $(cockpit).on("locationchanged", navigate);
    navigate();
}

if (window.location.pathname.indexOf("manager/security.html") > 0) {
    $(security_init);
}
