/*
 * This file is part of Cockpit.
 *
 * Copyright (C) 2013 Red Hat, Inc.
 *
 * Cockpit is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * Cockpit is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
 */

import $ from 'jquery';
//import cockpit from 'cockpit';

import React from 'react';
import ReactDOM from 'react-dom';
import mustache from 'mustache';
import * as authorized_keys from './authorized-keys.js';

import 'patterns';
import 'bootstrap-datepicker/dist/js/bootstrap-datepicker';

import { writeAuditlog } from './write-auditlog';

//import ElementUI from 'element-ui' //新添加
//import Vue from 'vue'

//Vue.use(ElementUI)

const _ = cockpit.gettext;
const C_ = cockpit.gettext;

let g_asserts_bak = [];
let g_users_bak = [];
let g_rules_bak = [];
let intv = 0;

function password_quality(password) {
    var dfd = cockpit.defer();

    cockpit.spawn('/usr/bin/pwscore', { "err": "message" })
            .input(password)
            .done(function(content) {
                var quality = parseInt(content, 10);
                if (quality === 0) {
                    dfd.reject(new Error(_("Password is too weak")));
                } else if (quality <= 33) {
                    dfd.resolve("weak");
                } else if (quality <= 66) {
                    dfd.resolve("okay");
                } else if (quality <= 99) {
                    dfd.resolve("good");
                } else {
                    dfd.resolve("excellent");
                }
            })
            .fail(function(ex) {
                dfd.reject(new Error(ex.message || _("Password is not acceptable")));
            });

    return dfd.promise();
}

function is_valid_char_username(c) {
        return (c >= 'a' && c <= 'z') ||
            (c >= 'A' && c <= 'Z') ||
            (c >= '0' && c <= '9') ||
            c == '.' || c == '_' || c == '-';
    }

function check_username() {
    var dfd = cockpit.defer();
    var username = $('#accounts-create-user-name').val();

    for (var i = 0; i < username.length; i++) {
        if (!is_valid_char_username(username[i])) {
            dfd.reject(new Error(
                _("The user name can only consist of letters from a-z, digits, dots, dashes and underscores.")
            ));
            return dfd.promise();
        }
    }

    dfd.resolve();
    return dfd.promise();
}

PageAccounts.prototype = {
    _init: function() {
        this.id = "accounts";
        this.user_list_tmpl = $("#users-list-tmpl").html();
        mustache.parse(this.user_list_tmpl);
    },

    show: function() {
    	let self = this;

    	// 从 g_users_bak 获取用户数据
    	self.hide();
    	let header = {
    		Name: "账户名",
    		Fullname: "账户全名",
    		Ifdelete: "策略类型",
    		Operate: "操作"
    	}

    	let units = [];
    	for (let i=0;i<g_users_bak.length;i++) {
    		let unit = g_users_bak[i];
    		if (unit.deleteuser) {
    			unit["deleteusertext"] = "删除账户"
    		} else {
    			unit["deleteusertext"] = "创建账户"
    		}
    		units.push(unit);
    	}

        // 分页展示
        units = g_processPagination(units)

    	let text = mustache.render(self.user_list_tmpl, {
                table_head: header,
                units: units
            });
    	$("#accounts-list").html(text);

    },

    hide: function() {
    	$("#accounts-list").html("");
    }
};

function PageAccounts() {
    this._init();
}

PageAccountsCreate.prototype = {
    _init: function() {
        let self = this;
    },

    getRandomPass: function(length, hasNum, hasChar, hasSymbol, caseSense, lowerCase) {
	    var m = "";
	    if (hasNum == "0" && hasChar == "0" && hasSymbol == "0") return m;
        var achar = false;
        var asymbol = false;
	    for (var i = length; i >= 0; i--) {
	        var num = Math.floor((Math.random() * 89) + 33); // 33 - 122
	        if (
	            (
	                (hasNum == "0") && ((num >= 48) && (num <= 57))
	            ) || (
	                (hasChar == "0") && ((
	                    (num >= 65) && (num <= 90)
	                ) || (
	                    (num >= 97) && (num <= 122)
	                ))
	            ) || (
	                (hasSymbol == "0") && ((
	                    (num >= 33) && (num <= 47) // ! " #  $ % & ' ( ) * + , - . /
	                ) || (
	                    (num >= 58) && (num <= 64) // : ; < = > ?
	                ) || (
	                    (num >= 91) && (num <= 96) // [ \ ] ^ _
	                ))
                    //  || (
	                //     (num >= 123) && (num <= 127) // { | } ~ del
	                // ))
	            ) || (num == 34) || (num == 36) || (num == 39) || (num == 92) // " $ ' \
	        ) {
	            i++;
	            continue;
	        }
            if ((num >= 65 && num <=90) || (num >= 97 && num <=122)){
                achar = true;
            }
            if ((num >= 33) && (num <= 47) || (num >= 58) && (num <= 64) || (num >= 91) && (num <= 96)){
                asymbol = true;
            }

	        m += String.fromCharCode(num);
	    }
        if(caseSense == "0"){
            m = (lowerCase == "0")?m.toUpperCase():m.toLowerCase();
        }
        // 检测如果不含符号或者不含英文字母，需要修改为至少含有一个字母或者符号的密码
        if (!achar){
            m.replace(m[m.length-1],"a");
        }
        if (!asymbol){
            m.replace(m[m.length-2],"!");
        }
        return m;
	},

	setRandomPass: function() {
		let self = this;

		let randomPass = self.getRandomPass(16, 1, 1, 1, 1, 0);
		$("#accounts-create-pw1").val(randomPass);
		$("#accounts-create-pw2").val(randomPass);
		self.validate();
	},

    setup: function(render_users) {
        let self = this;
        self.render_users = render_users;

        $('#accounts-create-dialog .check-passwords').on('keydown change', $.proxy(this, "validate"));
        $('#accounts-create-create').unbind("click").on('click', $.proxy(this, "create"));
        $('#add-user-password-button').unbind("click").on('click', $.proxy(this, "setRandomPass"));
        $('#check-user-password-button').unbind("click").on('click', function(e){
        	$("#password-text").text($("#accounts-create-pw1").val());
        });
    },

    validate: function() {
    	let self = this;

        var ex;
        var fails = [];
        var pw = $('#accounts-create-pw1').val();
        if ($('#accounts-create-pw2').val() != pw) {
            ex = new Error(_("The passwords do not match"));
            ex.target = "#accounts-create-pw2";
            fails.push(ex);
        }
        if (!$('#accounts-create-user-name').val()) {
            ex = new Error(_("No user name specified"));
            ex.target = '#accounts-create-user-name';
            fails.push(ex);
        }
        if (!$('#accounts-create-full-name').val()) {
            ex = new Error(_("No full name specified"));
            ex.target = '#accounts-create-full-name';
            fails.push(ex);
        }

        /* The first check is immediately complete */
        var dfd = cockpit.defer();
        if (fails.length)
            dfd.reject(fails);
        else
            dfd.resolve();

        var promise_password = password_quality(pw)
                .fail(function(ex) {
                    ex.target = "#accounts-create-pw2";
                })
                .always(function(arg) {
                    var strength = this.state() == "resolved" ? arg : "weak";
                    var meter = $("#accounts-create-password-meter")
                            .removeClass("weak okay good excellent");
                    if (pw)
                        meter.addClass(strength);
                    var message = $("#accounts-create-password-meter-message");
                    if (strength == "excellent") {
                        message.text(_("Excellent password"));
                    } else {
                        message.text("");
                    }
                });

        var promise_username = check_username()
                .fail(function(ex) {
                    ex.target = "#accounts-create-user-name";
                });

        // Can't use Promise.all() here, because this promise is passed to
        // dialog(), which expects a promise with a progress() method (see
        // pkg/lib/patterns.js)
        // eslint-disable-next-line cockpit/no-cockpit-all
        return cockpit.all(dfd.promise(), promise_password, promise_username);
    },

    create: function() {
    	let self = this;

    	let name = $('#accounts-create-user-name').val();
    	let fullname = $('#accounts-create-full-name').val();
    	let pw1 = $("#accounts-create-pw1").val();
    	let pw2 = $("#accounts-create-pw2").val();
    	if (pw1 !== pw2) {
    		return $("#accounts-create-password-meter-message").text("密码不一致");
    	}
    	let passCheckTip = $("#accounts-create-password-meter");
    	if (passCheckTip[0].className.indexOf("weak") >= 0) {
    		return $("#accounts-create-password-meter-message").text("密码强度太低");
    	}

    	let lockuser = $('#accounts-create-locked').prop('checked');
    	lockuser = lockuser ? 1 : 0;
    	let isadmin = $('#accounts-create-isadmin').prop('checked');
    	isadmin = isadmin ? 1 : 0;
    	let deleteuser = $('#accounts-create-delete').prop('checked');
    	deleteuser = deleteuser ? 1 : 0;

    	if (!name || name.length < 1 || name.length > 32) {
    		return $("#create-user-msg").text("用户名不能为空且最大32字符");
    	}
	let allnum = true;
        for (var i = 0; i < name.length; i++) {
            if (!/[0-9]/.test(name[i])){
		allnum = false;
	    }
	    if (!is_valid_char_username(name[i])){
                return $("#create-user-msg").text("用户名含有非法字符");
            }
        }
	if (allnum){
	    return $("#create-user-msg").text("用户名不能为全数字");
	}

    	if (!fullname || fullname.length < 1 || fullname.length > 32) {
    		return $("#create-user-msg").text("用户全名不能为空且最大32字符");
    	}
        for (var i = 0; i < fullname.length; i++) {
            if (!is_valid_char_username(fullname[i])){
                return $("#create-user-msg").text("用户全名含有非法字符");
            }
        }

    	let http = cockpit.http({
                "port": 9091
            });
        http.post("/addUser", {
            "name": name,
            "fullname": fullname,
            "pwd": pw1,
            "lockuser": lockuser,
            "isadmin": isadmin,
            "deleteuser": deleteuser
        })
        .fail(function(err) {
            console.log("adduser err:", err);
            $("#create-user-msg").text("添加账户失败");
            let cent = "添加账户: " + name + " 失败, 账户全名: " + fullname + ", 失败原因: " + err;
            writeAuditlog(2, name, cent);
        })
        .done(function(res) {
            console.log("addUser res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                let cent = "添加账户: " + name + " 失败, 账户全名: " + fullname + ", 失败原因: " + info.error;
                writeAuditlog(2, name, cent);
                return $("#create-user-msg").text("添加账户失败：" + info.error);
            }

            $("#create-user-msg").text("添加账户成功");
            setTimeout(function(e) {
                $("#accounts-create-close").click();
                self.render_users();
            }, 1500)

            let cent = "添加账户: " + name + " 成功, 账户全名: " + fullname;
            writeAuditlog(2, name, cent);
        })
    },


};

function PageAccountsCreate() {
    this._init();
}

PageAccount.prototype = {
    _init: function(user) {
        this.id = "account-page";
        this.keys_template = $("#authorized-keys-tmpl").html();
        mustache.parse(this.keys_template);
        this.user = user;
    },

    setusername: function(user) {
    	this.user = user;
    },

    getUserInfo: function(user) {
    	for (let i=0;i<g_users_bak.length;i++) {
    		if (g_users_bak[i].name == user) {
    			return g_users_bak[i];
    		}
    	}

    	return false;
    },

    show: function() {
        let self = this;

        let exist = false;
        for (let i=0;i<g_users_bak.length;i++) {
        	if (g_users_bak[i].name == self.user) {
        		exist = true;
        	}
        }

        if (!exist) {
        	$("#account").hide();
        	$("#account-failure").show();
        } else {
        	$("#account").show();
        	$("#account-failure").hide();
        }
    },

    hide: function() {
    	$("#account").hide();
        $("#account-failure").hide();
    },

    validate: function() {
        var ex;
        var pw = $('#account-set-password-pw1').val();
        if ($('#account-set-password-pw2').val() != pw) {
            ex = new Error(_("The passwords do not match"));
            ex.target = "#account-set-password-pw2";
        }

        var dfd = cockpit.defer();
        if (ex)
            dfd.reject(ex);
        else
            dfd.resolve();

        var promise = password_quality(pw)
                .fail(function(ex) {
                    ex.target = "#account-set-password-pw2";
                })
                .always(function(arg) {
                    var strength = (this.state() == "resolved") ? arg : "weak";
                    var meter = $("#account-set-password-meter")
                            .removeClass("weak okay good excellent");
                    if (pw)
                        meter.addClass(strength);
                    var message = $("#account-set-password-meter-message");
                    if (strength == "excellent") {
                        message.text(_("Excellent password"));
                    } else {
                        message.text("");
                    }
                });

        // Can't use Promise.all() here, because this promise is passed to
        // dialog(), which expects a promise with a progress() method (see
        // pkg/lib/patterns.js)
        // eslint-disable-next-line cockpit/no-cockpit-all
        return cockpit.all(dfd.promise(), promise);
    },

    // 编辑成功后刷新数据
    refreshUserInfo: function(callback, callback_err) {
    	let http = cockpit.http({
            "port": 9091
        });
        http.get("/getUsers")
        .fail(function(err){
            console.log("getUsers err:", err);
            $("#loading-fallback").hide();
            callback_err();
        })
        .done(function(res) {
            // console.log("getUsers res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getUsers err:", res);
                $("#loading-fallback").hide();
                callback_err();
                return ;
            }
            info = info.data;
            g_users_bak = info;

            $("#loading-fallback").hide();
            callback()
        })
    },

    setup: function() {
    	let self = this;
    	let userinfo = self.getUserInfo(self.user);
    	if (!userinfo) {
    		return;
    	}

    	$("ol.breadcrumb li.active").text(userinfo.fullname);
    	$("span#account-title").text(userinfo.fullname);
    	$("#account-full-name").val(userinfo.fullname);
    	$("#account-user-name").val(userinfo.name);
    	$("#account-role-checkbox-10").prop("checked", userinfo.isadmin);
    	$("#account-locked").prop("checked", userinfo.lockuser);
    	$("#account-ifdelete").prop("checked", userinfo.deleteuser);

        $('#account .breadcrumb a').on("click", function() {
            cockpit.location.go('/');
        });
        $('#account-full-name').on('change', $.proxy(this, "edit_user"));
        $("#account-role-checkbox-10").on('change', $.proxy(this, "edit_user"));
        $("#account-locked").on('change', $.proxy(this, "edit_user"));
        $("#account-ifdelete").on('change', $.proxy(this, "edit_user"));

        // 证书界面
        let keystrarr = userinfo.authkey.split(";");
        let keys = [];
    	for (let i=0;i<keystrarr.length;i++) {
    		let keyarr = keystrarr[i].split(" ");
    		if (keyarr.length < 3) {
    			continue
    		}

    		let type = keyarr[0];
    		let content = keyarr[1];
    		let comment = keyarr[2];
    		let unit = {
    			"valid": true,
    			"fp": type + ":" + comment,
    			"comment": comment,
    			"raw": keystrarr[i],
                "allkeycontent": keystrarr[i]
    		}
    		keys.push(unit);
    	}
    	let keys_html = "";
        if (keys.length < 1) {
        	keys_html = mustache.render(this.keys_template, {
	            "empty": true
	        })
        } else {
        	keys_html = mustache.render(this.keys_template, {
	            "keys": keys
	        })
        }
        $('#account-authorized-keys-list').html(keys_html);

        // 设置密码
        $('#account-set-password-dialog .check-passwords').on('keydown change', $.proxy(this, "validate"));
        $("#account-set-password-apply").unbind("click").click(function(e) {
        	let pwcheck = $("#account-set-password-meter")[0];
        	if (pwcheck.className.indexOf("weak") >= 0) {
        		return $("#account-set-password-meter-message").text("密码太弱，请重新设置");
        	}
        	let pw1 = $("#account-set-password-pw1").val();
        	let pw2 = $("#account-set-password-pw2").val();
        	if (pw1 != pw2) {
        		return $("#account-set-password-meter-message").text("密码不一致，请重新设置");
        	}
        	let http = cockpit.http({
                "port": 9091
            });
	        http.post("/editUserPassword", {
	            "name": self.user,
	            "pwd": pw1
	        })
	        .fail(function(err) {
                console.log("editUserPassword err:", err);
                $("#account-set-password-meter-message").text("设置账户密码失败");
                let cent = "设置账户: " + self.user + " 密码失败, 失败原因: " + err;
                writeAuditlog(2, "账户", cent);
	        })
	        .done(function(res) {
	            console.log("editUserPassword res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
                    let cent = "设置账户: " + self.user + " 密码失败, 失败原因: "+ info.error;
                    writeAuditlog(2, "账户", cent);
                    return $("#account-set-password-meter-message").text("设置账户密码失败：" + info.error);
	            }

	            self.refreshUserInfo(function(){
	            	$("#account-set-password-meter-message").text("设置账户密码成功");
                    setTimeout(function() {
		                $("#account-set-password-close").click();
		            }, 1500)
	            }, function(e){
	            	$("#account-set-password-meter-message").text("刷新数据失败");
	            })

                let cent = "设置账户: " + self.user + " 密码成功";
                writeAuditlog(2, "账户", cent);
	        })
        })
        // 查看密码
        $("#password-show-button").unbind("click").click(function(e) {
        	let userinfotmp = self.getUserInfo(self.user);
        	$("#password-text").text(userinfotmp.password);
        })
        // 点击添加按钮的时候，要把文本框内的内容情况，可能上次添加会有残留
        $("#authorized-key-add").unbind("click").click(function(e){
            $("#authorized-keys-text").val("");
        })
        // 管理公钥
        $("#add-authorized-key").unbind("click").click(function(e) {
        	$("#add-authorized-key-message").text("");

        	let authkey = $("#authorized-keys-text").val();
        	if (!authkey || authkey.length < 1) {
        		return $("#add-authorized-key-message").text("请输入公钥内容")
        	}

        	let authkeyarr = authkey.split(" ");
        	if (authkeyarr.length < 3) {
        		return $("#add-authorized-key-message").text("请输入合法的公钥内容")
        	}

        	let http = cockpit.http({
                "port": 9091
            });
	        http.post("/addUserKey", {
	            "name": self.user,
	            "authkey": authkey
	        })
	        .fail(function(err) {
                console.log("addUserKey err:", err);
                $("#add-authorized-key-message").text("添加公钥失败");
                let cent = "添加账户: " + self.user + " 公钥失败, authkey=" + authkey + ", 失败原因: " + err;
                writeAuditlog(2, self.user, cent);
	        })
	        .done(function(res) {
	            console.log("addUserKey res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
                    return $("#add-authorized-key-message").text("添加公钥失败：" + info.error);
                    let cent = "添加账户: " + self.user + " 公钥失败, authkey=" + authkey + ", 失败原因: " + info.error;
                    writeAuditlog(2, self.user, cent);
	            }

	            self.refreshUserInfo(function(){
                    let cent = "添加账户: " + self.user + " 公钥成功, authkey=" + authkey;
                    writeAuditlog(2, self.user, cent);
                    $("#add-authorized-key-message").text("添加公钥成功");
                    self.setup();
	            	setTimeout(function(e) {
	                	$("#cancel-authorized-key").click();
	            		}, 1500)
	            	}, function(e){
	            		$("#add-authorized-key-message").text("刷新数据失败");
	            })
	        })
        })
		$("button.account-remove-key").unbind("click").click(function(e){
			let raw = $(this).data("raw");
			let http = cockpit.http({
                "port": 9091
            });
	        http.post("/delUserKey", {
	            "name": self.user,
	            "authkey": raw
	        })
	        .fail(function(err) {
                console.log("delUserKey err:", err);
                show_error_dialog("错误", "删除公钥失败");
                let cent = "删除账户: " + self.user + " 公钥失败, authkey=" + raw + ", 失败原因: " + err;
                writeAuditlog(2, self.user, cent);
	        })
	        .done(function(res) {
	            console.log("delUserKey res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
                    let cent = "删除账户: " + self.user + " 公钥失败, authkey=" + raw + ", 失败原因: " + info.error;
                    writeAuditlog(2, self.user, cent);
                    return show_error_dialog("错误", "删除公钥失败：" + info.error);
	            }
                let cent = "删除账户: " + self.user + " 公钥成功, authkey=" + raw;
                writeAuditlog(2, self.user, cent);
                self.refreshUserInfo(function(){
	            	self.setup();
	            	}, function(e){
	            		show_error_dialog("错误", "刷新数据失败");
	            })
	        })
		})
    },

    edit_user: function(e) {
    	let self = this;

    	function disable_edit() {
    		$('#account-locked').prop("disabled", true);
    		$('#account-role-checkbox-10').prop("disabled", true);
    		$('#account-ifdelete').prop("disabled", true);
    	}
    	function enable_edit() {
    		$('#account-locked').prop("disabled", false);
    		$('#account-role-checkbox-10').prop("disabled", false);
    		$('#account-ifdelete').prop("disabled", false);
    	}

    	// 改变 checkbox 的 disabled 属性时也会触发 change 事件，调用 setup 的时候也会，这里过滤掉
    	if ($('#account-locked').prop("disabled") ||
    		$('#account-role-checkbox-10').prop("disabled") ||
    		$('#account-ifdelete').prop("disabled")) {
    		return ;
    	}

    	disable_edit();
    	let user = $("#account-user-name").text();
    	let newfullname = $("#account-full-name").val();
    	if (!newfullname || newfullname.length < 1 || newfullname.length > 32) {
    		return enable_edit();
    	}

    	let lockuser = $('#account-locked').prop('checked');
    	lockuser = lockuser ? 1 : 0;
    	let isadmin = $('#account-role-checkbox-10').prop('checked');
    	isadmin = isadmin ? 1 : 0;
    	let deleteuser = $('#account-ifdelete').prop('checked');
    	deleteuser = deleteuser ? 1 : 0;

    	let http = cockpit.http({
            "port": 9091
        });
        http.post("/editUserNormal", {
            "name": self.user,
            "fullname": newfullname,
            "lockuser": lockuser,
            "isadmin": isadmin,
            "deleteuser": deleteuser
        })
        .fail(function(err) {
            console.log("editUserNormal err:", err);
            enable_edit();
            let cent = "设置账户: " + self.user + " 策略失败, 策略全名=" + newfullname + ", 失败原因: " + err;
            writeAuditlog(2, self.user, cent);
            return show_error_dialog("错误", "设置账户策略失败");
        })
        .done(function(res) {
            console.log("editUserPassword res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                enable_edit()
                let cent = "设置账户: " + self.user + " 策略失败, 策略全名=" + newfullname + ", 失败原因: " + info.error;
                writeAuditlog(2, self.user, cent);
                return show_error_dialog("错误", "设置账户策略失败：" + info.error);
            }

            let cent = "设置账户: " + self.user + " 策略成功, 策略全名=" + newfullname;
            writeAuditlog(2, self.user, cent);
            self.refreshUserInfo(function(){
	            self.setup();
            	enable_edit();
            }, function(e){
            	show_error_dialog("错误", "刷新数据失败");
            })
        })
    }

    // 
};

function PageAccount(user) {
    this._init(user);
}

function AccountExpiration() {
    $("#account-expiration").on("show.bs.modal", function(ev) {
        var account_id = $(ev.relatedTarget).data("account-id");
        $("#account-expiration").data("account-id", account_id);

        /* Fill in initial dialog values */
        var expire_date = $(ev.relatedTarget).data("expire-date");
        $(expire_date ? '#account-expiration-expires' : "#account-expiration-never").prop('checked', true);
        $('#account-expiration-input').val(expire_date ? expire_date.toISOString().substr(0, 10) : "");
        $('#account-expiration-input').prop('disabled', !expire_date);

        /* TRANSLATORS: This is split up and therefore cannot use ngettext plurals */
        var parts = _("Lock account on $0").split("$0");
        $("#account-expiration-before").text(parts[0]);
        $("#account-expiration-after").text(parts[1]);
    });

    $('#account-expiration .btn-primary').on('click', function() {
        var date, value;
        var ex;
        var promise = null;

        /* Parse the dialog data and validate */
        if ($('#account-expiration-expires').prop('checked')) {
            value = $('#account-expiration-input').val();
            if (!value) {
                ex = new Error(_("Please specify an expiration date"));
                ex.target = "#account-expiration-input";
            } else {
                date = new Date(value + "T12:00:00Z");
                if (isNaN(date.getTime()) || date.getTime() < 0) {
                    ex = new Error(_("Invalid expiration date"));
                    ex.target = "#account-expiration-input";
                }
            }
            if (ex)
                promise = cockpit.reject(ex);
        }

        if ($('#account-expiration-never').prop('checked')) {
            date = null;
            promise = null;
        }

        /* Actually perform the action if valid */
        var prog = ["/usr/sbin/usermod", "-e"];
        var account_id = $("#account-expiration").data("account-id");
        if (!promise) {
            if (date)
                prog.push(date.toISOString().substr(0, 10));
            else
                prog.push("");
            prog.push(account_id);
            promise = cockpit.spawn(prog, { "superuser" : true, "err": "message" });
        }

        $("#account-expiration").dialog("promise", promise);
    });

    $('#account-expiration input').on('change', function() {
        $('#account-expiration-input').prop('disabled', $("#account-expiration-never").prop('checked'));
    });

    $('#account-expiration-input').datepicker({
        autoclose: true,
        todayHighlight: true,
        format: 'yyyy-mm-dd'
    });

    $('#account-expiration-input').on("show.bs.modal", function(ev) {
        ev.stopPropagation();
    });
}

function PasswordExpiration() {
    var never = 99999;

    $("#password-expiration").on("show.bs.modal", function(ev) {
        var account_id = $(ev.relatedTarget).data("account-id");
        $("#password-expiration").data("account-id", account_id);

        /* Fill in initial dialog values */
        var expire_days = parseInt($(ev.relatedTarget).data("expire-days"), 10);
        if (isNaN(expire_days) || expire_days < 0 || expire_days >= never)
            expire_days = null;
        $(expire_days ? '#password-expiration-expires' : "#password-expiration-never").prop('checked', true);
        $('#password-expiration-input').val(expire_days || "");
        $('#password-expiration-input').prop('disabled', !expire_days);

        /* TRANSLATORS: This is split up and therefore cannot use ngettext plurals */
        var parts = _("Require password change every $0 days").split("$0");
        $("#password-expiration-before").text(parts[0]);
        $("#password-expiration-after").text(parts[1]);
    });

    $('#password-expiration .btn-primary').on('click', function() {
        var days, ex;
        var promise = null;

        if ($('#password-expiration-expires').prop('checked'))
            days = parseInt($('#password-expiration-input')
                    .val()
                    .trim(), 10);
        if ($('#password-expiration-never').prop('checked'))
            days = never;

        if (isNaN(days) || days < 0) {
            ex = new Error(_("Invalid number of days"));
            ex.target = "#password-expiration-input";
            promise = cockpit.reject(ex);
        }

        var account_id = $("#password-expiration").data("account-id");

        if (!promise) {
            promise = cockpit.spawn([ "/usr/bin/passwd", "-x", String(days), account_id ],
                                    { "superuser": true, "err": "message" });
        }

        $("#password-expiration").dialog("promise", promise);
    });

    $('#password-expiration input').on('change', function() {
        $('#password-expiration-input').prop('disabled', $("#password-expiration-never").prop('checked'));
    });
}

function PasswordReset() {
    $("#password-reset").on("show.bs.modal", function(ev) {
        var account_id = $(ev.relatedTarget).data("account-id");
        $("#password-reset").data("account-id", account_id);

        var msg = cockpit.format(_("The account '$0' will be forced to change their password on next login"),
                                 account_id);
        $("#password-reset .modal-body p").text(msg);
    });

    $("#password-reset .btn-primary").on("click", function() {
        var account_id = $("#password-reset").data("account-id");
        var promise = cockpit.spawn(["/usr/bin/passwd", "-e", account_id],
                                    { "superuser" : true, "err": "message" });
        $("#password-reset").dialog("promise", promise);
    });
}

/* INITIALIZATION AND NAVIGATION
 *
 * The code above still uses the legacy 'Page' abstraction for both
 * pages and dialogs, and expects page.setup, page.enter, page.show,
 * and page.leave to be called at the right times.
 *
 * We cater to this with a little compatability shim consisting of
 * 'dialog_setup', 'page_show', and 'page_hide'.
 */

function show_error_dialog(title, message) {
    if (message) {
        $("#error-popup-title").text(title);
        $("#error-popup-message").text(message);
    } else {
        $("#error-popup-title").text(_("Error"));
        $("#error-popup-message").text(title);
    }

    $("#show-error-dialog").click();
    //$("#accounts-create").click();
}

// 账户页面在 user_init 函数之外，需要一个全局变量才能访问到
let g_processPagination;

function user_init() {

    let account_empty_tmpl = $("#account-empty-tmpl").html();
    mustache.parse(account_empty_tmpl);

    let rules_list_tmpl = $("#rules-list-tmpl").html();
    mustache.parse(rules_list_tmpl);

    let overview_page = new PageAccounts();
    let create_page = new PageAccountsCreate();
    create_page.setup(render_users);
    let account_page = new PageAccount();

    // 分页界面的相关变量放在这做成全局变量
    let g_vue_pagination_user;
    let g_cur_page_user = 1;
    let g_cur_page_user_rules = 1;
    let g_page_size_user = 20;

    function processPagination(units) {
        // 分页展示
        if (!g_vue_pagination_user) {
            g_vue_pagination_user = new Vue({
                template: '<el-pagination @size-change="handleSizeChange" @current-change="handleCurrentChange" :current-page="current_page" :page-sizes="[20, 40, 60, 80]" :page-size="page_size" layout="total, sizes, prev, pager, next, jumper" :total="total"> </el-pagination>',
                el: "#user-vue-app-div",
                methods: {
                    handleSizeChange(val) {
                        g_page_size_user = val;
                        console.log("set g_page_size_user to:", val)

                        if (window.location.href.indexOf("#/usersrules") < 0) {
                            // group page
                            overview_page.show()
                        } else {
                            // rules page
                            show_rules_page()
                            registerRulesEvents()
                        }
                    },
                    handleCurrentChange(val) {
                        if (window.location.href.indexOf("#/usersrules") < 0) {
                            g_cur_page_user = val;
                            console.log("set g_cur_page_user to:", val)
                            overview_page.show()
                        } else {
                            g_cur_page_user_rules = val;
                            console.log("set g_cur_page_user_rules to:", val)
                            show_rules_page()
                            registerRulesEvents()
                        }
                    }
                },
                data: {
                    current_page: g_cur_page_user,
                    page_size: g_page_size_user,
                    total: units.length,
                }
            })
        } else {
            if (window.location.href.indexOf("#/usersrules") < 0) {
                g_vue_pagination_user.current_page = g_cur_page_user;
            } else {
                g_vue_pagination_user.current_page = g_cur_page_user_rules;
            }

            g_vue_pagination_user.page_size = g_page_size_user;
            g_vue_pagination_user.total = units.length;
        }

        let newunits;
        if (window.location.href.indexOf("#/usersrules") < 0) {
            newunits = units.slice( (g_cur_page_user-1) * g_page_size_user,
                g_cur_page_user * g_page_size_user )
        } else {
            newunits = units.slice( (g_cur_page_user_rules-1) * g_page_size_user,
            g_cur_page_user_rules * g_page_size_user )
        }

        return newunits
    }

    g_processPagination = processPagination;

    function getAssertById(assertid) {
    	for (let i=0;i<g_asserts_bak.length;i++) {
    		if (g_asserts_bak[i].assertid == assertid) {
    			return g_asserts_bak[i]
    		}
    	}
    	return false;
    }

    function getRuleById(id) {
    	for (let i=0;i<g_rules_bak.length;i++) {
    		if (g_rules_bak[i].id == id) {
    			return g_rules_bak[i]
    		}
    	}
    	return false;
    }

    function render_user_detail() {
    	let path = window.location.href;
    	let username = path.split("userdetail-")[1];

    	account_page.setusername(username);

    	// 如果还没有获取用户信息，先获取用户信息
    	if (g_users_bak.length < 1) {
    		let http = cockpit.http({
                "port": 9091
            });
	        http.get("/getUsers")
	        .fail(function(err){
	            console.log("getUsers err:", err);
	            $("#loading-fallback").hide();
	        })
	        .done(function(res) {
	            // console.log("getUsers res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
	                console.log("getUsers err:", res);
	                $("#loading-fallback").hide();
	                return ;
	            }
	            info = info.data;
	            g_users_bak = info;

	            $("#content-header").hide();
		    	$("#loading-fallback").hide();

		    	overview_page.hide();
		    	account_page.setup();
		    	account_page.show();
	        })
    	} else {
    		$("#content-header").hide();
	    	$("#loading-fallback").hide();

	    	overview_page.hide();
	    	account_page.setup();
	    	account_page.show();
    	}
    }

    function show_users_page() {
    	if (g_users_bak.length < 1) {
    		let html = mustache.render(account_empty_tmpl, {
    			empty_text: "尚未创建账户"
    		})
    		return $("#accounts-list").html(html);
    	}

    	$("#content-header").show();

    	account_page.hide();
    	overview_page.show();

    	$("button.delete-user-btn").unbind("click").click(function(e) {
    		let username = $(this).data("user-name");
    		$("#confirm-del-user-private-data").data("private-data", username);
        	$("#confirm-del-user-msg").text("确认要删除账户策略 " + username + " 吗?");
        })
        $("#confirm-del-user-confirm").unbind("click").click(function(e){
        	let name = $("#confirm-del-user-private-data").data("private-data");
        	let http = cockpit.http({
                "port": 9091
            });
	        http.post("/delUser", {
	            "name": name
	        })
	        .fail(function(err) {
                console.log("delUser err:", err);
                $("#confirm-del-user-msg").text("删除账户策略失败");
                let cent = "删除账户策略: " + name + " 失败，失败原因：" + err;
                writeAuditlog(2, name, cent);
	        })
	        .done(function(res) {
	            console.log("delUser res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
                    let cent = "删除账户策略: " + name + " 失败，失败原因：" + info.error;
                    writeAuditlog(2, name, cent);
                    return $("#confirm-del-user-msg").text("删除账户策略失败：" + info.error);
	            }

	            $("#confirm-del-user-msg").text("删除账户策略成功");
	            setTimeout(function(e) {
	                $("#confirm-del-user-close").click();
	                render_users();
	            }, 1500)
                let cent = "删除账户策略: " + name + " 成功";
                writeAuditlog(2, name, cent);
	        })
        })
        $("tr.unit-tr-user-row").unbind("click").click(function(e) {
        	if (e.target.localName === "button") {
        		return
        	}

        	let user = $(this).data("goto-unit");
        	cockpit.location.go(["userdetail-" + user])
        })
    }

    function render_users() {
    	// 如果是从账户详情页面跳回，需要先隐藏掉账户详情页面
    	account_page.hide();
    	$("#add-rule").hide();
    	$("#accounts-create").show();

    	$("#loading-fallback").show();
    	let http = cockpit.http({
                "port": 9091
            });
        http.get("/getUsers")
        .fail(function(err){
            console.log("getUsers err:", err);
            $("#loading-fallback").hide();
        })
        .done(function(res) {
            // console.log("getUsers res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getUsers err:", res);
                $("#loading-fallback").hide();
                return ;
            }
            info = info.data;
            g_users_bak = info;

            show_users_page();
            $("#loading-fallback").hide();
        })
    }

    function show_rules_page() {
    	if (g_rules_bak.length < 1) {
    		let html = mustache.render(account_empty_tmpl, {
    			empty_text: "尚未创建规则"
    		})
    		return $("#accounts-list").html(html);
    	}

    	account_page.hide();
    	overview_page.hide();

    	let header = {
    		Name: "规则名称",
    		Straname: "策略名称",
    		Applyto: "应用到",
    		Status: "规则状态",
    		Operate: "操作"
    	}

    	let units = [];
    	for (let i=0;i<g_rules_bak.length;i++) {
    		let unit = g_rules_bak[i];
    		let applytotext = "";
            let assertsarr = unit.applyto.split(";");
            let deleted = [];
    		for (let i=0;i<assertsarr.length;i++) {
    			if (!assertsarr[i] || assertsarr[i].length < 1) {
    				continue
    			}
                let asserttmp = getAssertById(assertsarr[i]);
                if (!asserttmp.assertip){
                    // 过滤掉已删除的资产 添加到已删除列表
                    deleted.push(assertsarr[i]);
                    continue
                }
    			applytotext = applytotext + "资产：" + asserttmp.assertip + "-" + asserttmp.assertname + ";"
    		}
    		unit.applytotext = applytotext;
    		try {
    			let userstra = $.parseJSON(unit.stracontent);
    			unit.straname = userstra.name;
    		} catch(e) {
    			console.log(e)
    		}

    		if (unit.applycount.length < 1 || unit.status == "redownload") {
    			unit.status = "下发中"
    		}
    		else {
    			try {
    				let applyjson = $.parseJSON(unit.applycount);
                    let keys = Object.keys(applyjson);
                    let flg = true;
	    			for (let i=0;i<keys.length;i++) {
	    				if (applyjson[keys[i]]["applycount"] < unit.editcount) {
	    					// 说明还是上次的下发结果
	    					unit.status = "尚未下发"
	    					break;
	    				}
	    				if (applyjson[keys[i]]["result"] != "succeed") {
                            // 判断失败的是否为删除的如果在删除的里面继续循环
                            if ($.inArray(keys[i], deleted) == -1){
                                unit.status = "下发失败"
                                flg = false;
                                break;
                            }
	    				}
                    }
                    // 如果全部都成功，才认为下发成功
	    			if (! unit.status || unit.status.length < 1 || flg) {
	    				unit.status = "下发成功"
	    			}
    			} catch(e) {
    				console.log(e)
    			}
    		}
            unit.ruleid=g_rules_bak[i].id;
    		units.push(unit);
    	}

        // 分页展示
        units = processPagination(units)

    	let text = mustache.render(rules_list_tmpl, {
                table_head: header,
                units: units
            });
    	$("#accounts-list").html(text);
    }

    let vue_choose_asserts;
	let vue_edit_choose_asserts;
	// vuedata 放在这里定义，新增，编辑，新增确认，编辑确认的时候都可以访问
    let vuedata;
    function getVuedataByAssertsAndGroups(info) {
    	vuedata = [];
     	let allGroupAsserts = [];
        let allasserts = info["allasserts"];
        function getAssertsById(id) {
        	for (let i=0;i<allasserts.length;i++) {
        		if (allasserts[i].assertid == id) {
        			return allasserts[i]
        		}
        	}
        	return {}
        }

        for (let i=0;i<info["allgroups"].length;i++) {
        	let grouptmp = info["allgroups"][i];
        	let unit = {
        		id: grouptmp.groupid,
        		label: "分组：" + grouptmp.groupname,
        		children: []
        	}

    		let tmparr = grouptmp.groupasserts.split(";")
    		for (let k=0;k<tmparr.length;k++) {
    			if (!tmparr[k] || tmparr[k].length < 1) {
    				continue
    			}
    			let asserttmp = getAssertsById(tmparr[k]);
    			if (asserttmp.assertip == "localhost") {
    				continue
                }
                if (!asserttmp.assertip){
                    // 过滤掉已经删除的
                    continue
                }
    			unit.children.push({
    				id: tmparr[k],
    				label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
    			})
    			if (allGroupAsserts.indexOf(tmparr[k]) < 0) {
    				allGroupAsserts.push(tmparr[k])
    			}
    		}
            if (unit.children.length == 0){
                continue;
            }
        	vuedata.push(unit);
        }

        for (let i=0;i<info["allasserts"].length;i++) {
        	let asserttmp = info["allasserts"][i];
        	if (allGroupAsserts.indexOf(asserttmp.assertid) >= 0) {
        		continue
        	}
        	if (asserttmp.assertip == "localhost") {
    				continue
    			}
        	let unit = {
        		id: asserttmp.assertid,
        		label: "资产：" + asserttmp.assertip + "-" + asserttmp.assertname
        	}
        	vuedata.push(unit);
        }

        return vuedata;
    }

    function registerRulesEvents() {
        // 点击下发按钮触发后台下发
        $("button.download-rule-btn").unbind("click").click(function(e){
            let ruleid = $(this).data("rule-id");
            let ruleunit = getRuleById(ruleid);
            let http = cockpit.http({
                "port": 9091
            });
            http.post("/downloadRule", {
                "type": "user",
                id: ruleid
            })
            .fail(function(err){
                console.log("downloadRule err:", err);
                show_error_dialog("错误", "通知后台下发失败")
                let cent = "下发规则: " + ruleunit.name + " 失败, ruletype: user, 失败原因: " + err;
                writeAuditlog(2, ruleunit.name, cent);
            })
            .done(function(res) {
                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("downloadRule err:", res);
                    let cent = "下发规则: " + ruleunit.name + " 失败, ruletype: user, 失败原因: " + info.error;
                    writeAuditlog(2, ruleunit.name, cent);
                    return show_error_dialog("错误", "通知后台下发失败：" + info.error)
                }
                if (info.error){
                    return show_error_dialog("提示", info.error)
                }
                show_error_dialog("提示", "通知后台下发成功");
                let cent = "下发规则: " + ruleunit.name + " 成功, ruletype: user";
                writeAuditlog(2, ruleunit.name, cent);
                setTimeout(render_rules, 1500)
            })
        })
        // 点击规则行上的更多按钮时，弹出菜单，需要在渲染完成后再绑定事件
        $(".user-rule-opr-more").unbind("click").on("click", function(e){
            let top = $(this).offset().top;
            let left = $(this).offset().left;
            let top1 = $("html").scrollTop();
            top = top - top1;
            let width = parseInt($(this).css("width").replace("px", ""));
            
            $(".user-rule-popup-menu").css("top", top);
            $(".user-rule-popup-menu").css("left", left + width);

            if ($(".user-rule-popup-menu").css("display") == "none") {
                $(".user-rule-popup-menu").css("display", "inline-grid")
            } else {
                $(".user-rule-popup-menu").css("display", "none")
            }

            let ruldid = $(this).data("rule-id")
            $("#user-rule-popup-menu-private-data").data("private-data", ruldid)
        })
        $(".user-rule-popup-btn").mouseover(function(e){
            $(this).css("background-color", "#def3ff")
        })
        .mouseout(function(e){
            $(this).css("background-color", "white")
        })
        $(window).unbind("click").on("click", function(e){
            if ( e.target.localName == "button" &&
                (e.target.className.indexOf("user-rule-opr-more") >= 0 ||
                e.target.className.indexOf("user-rule-del") >= 0 ||
                e.target.className.indexOf("user-rule-status") >= 0 ||
                e.target.className.indexOf("user-rule-edit") >= 0 )) {
                return
            }
            $(".user-rule-popup-menu").css("display", "none")
        })
        $("button.user-rule-status").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            try {
                $("#rule-status-content").html("");
                let applycount = $.parseJSON(ruleunit.applycount);
                Object.keys(applycount).forEach(function(key) {
                    // 渲染规则页面的时候已经获取过资产信息，这里直接用，不重新请求了
                    let asserttmp = getAssertById(key);
                    if (!asserttmp) {
                        console.log("[user-rule-status]get assert failed,assertid:", applycount[key]);
                        return
                    }
                    let statustext = ""
                    if (applycount[key]["applycount"] < ruleunit.editcount) {
                        statustext = "尚未下发"
                    } else {
                        if (applycount[key]["result"] == "succeed") {
                            statustext = "下发成功"
                        } else {
                            statustext = "下发失败"
                        }
                    }
                    let tr = "<tr><td class='text-align-left'>" + asserttmp.assertip
                            + "</td><td>" + asserttmp.assertname 
                            + "</td><td><span class=''>"+statustext+"</span>"
                            + "</td><td><buttom data-toggle='modal' class='btn btn-default userrule-status'"
                            + " data-target='#userrule-status-dialog' userrule-assert-id='"+key
                            + "'>查看详情</buttom>"
                            + "</td></tr>"
                    $("#rule-status-content").append($(tr));
                })
                $("buttom.userrule-status").unbind("click").click(function(e){
                    let buttomid = $(this).attr("userrule-assert-id");
                    $("#userrule-status-msg").text(applycount[buttomid]["result"]);
                })
            } catch(e) {
                console.log(e)
            }
        })
        $("button.user-rule-passwd").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            $("#rule-passwd-msg").text("")
            $("#rule-passwd-content").html("");
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let http = cockpit.http({
                "port": 9091
            });
            http.get("/getUserPassByRuleid", {"ruleid": ruleid})
            .fail(function(err){
                console.log("getUserPassByRuleid err:", err);
                $("#rule-passwd-msg").text("获取资产密码失败")
            })
            .done(function(res) {
                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getUserPassByRuleid err:", res);
                    return $("#rule-passwd-msg").text("获取资产密码失败：" + info.msg)
                }
                info = info.data;
                for(let i=0;i<info.length;i++) {
                    let assertid = info[i]["assertid"];
                    let asserttmp = getAssertById(assertid);
                    let tr = "<tr><td class='text-align-left'>" + asserttmp.assertip
                            + "</td><td>" + info[i]["username"]
                            +"</td><td class='show-password-area' data-id='"+assertid+"' data-passwd='"+info[i]["password"]+"'>" + "******"
                            + "</td><td>"
                            +"<button class='btn btn-default show-pass-btn' data-id='"+assertid+"'>显示</button>"
                            +"<button class='btn btn-default hide-pass-btn' data-id='"+assertid+"' style='margin-left:3px'>隐藏</button>"
                            +"</td></tr>"
                    $("#rule-passwd-content").append($(tr));
                }
                $("button.show-pass-btn").unbind("click").click(function(e){
                    let assertid = $(this).data("id");
                    let ele = $("td.show-password-area[data-id="+assertid+"]");
                    ele.text(ele.data("passwd"));
                })
                $("button.hide-pass-btn").unbind("click").click(function(e){
                    let assertid = $(this).data("id");
                    let ele = $("td.show-password-area[data-id="+assertid+"]");
                    ele.text("******")
                })
            })
        })
        $("button.user-rule-edit").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            $("#edit-rule-name").val(ruleunit.name);

            let http = cockpit.http({
                "port": 9091
            });
            http.get("/getUsers")
            .fail(function(err){
                console.log("getUsers err:", err);
                $("#edit-rule-msg").text("获取账号策略失败")
            })
            .done(function(res) {
                // console.log("getUsers res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("getUsers err:", res);
                    return $("#edit-rule-msg").text("获取账号策略失败：" + info.error)
                }
                info = info.data;
                g_users_bak = info;

                $("#edit-stra-name").html("");

                for (let i=0;i<g_users_bak.length;i++) {
                    let html = "";
                    if (g_users_bak[i].id == ruleunit.straid) {
                        html = '<option selected="selected" value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
                    } else {
                        html = '<option value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
                    }
                    $("#edit-stra-name").append($(html))
                }
                //$("#edit-stra-name").val(ruleunit.straname);

                // 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
                // 先获取资产和分组信息
                http.get("/getGroupsAndAsserts")
                .fail(function(err){
                    console.log("getGroupsAndAsserts err:", err);
                    $("#edit-rule-msg").text("获取资产和分组失败")
                })
                .done(function(res) {
                    // console.log("getGroupsAndAsserts res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        console.log("getGroupsAndAsserts err:", res);
                        return $("#edit-rule-msg").text("获取资产和分组失败：" + info.error)
                    }
                    info = info.data;
                    vuedata = getVuedataByAssertsAndGroups(info);

                    if (!vue_edit_choose_asserts) {
                        vue_edit_choose_asserts = new Vue({
                            template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
                            el: "#vue-app-edit-div",
                            methods: {
                                getCheckedNodes() {
                                    return this.$refs.tree.getCheckedNodes();
                                },
                                setCheckedKeys(data) {
                                    this.$refs.tree.setCheckedKeys(data);
                                },
                            },
                            data: {
                                data: vuedata,
                                defaultProps: {
                                    children: 'children',
                                    label: 'label'
                                }
                            },
                            watch: {
                                data(newdata, olddata) {
                                    //
                                }
                            }
                        })
                    }
                    vue_edit_choose_asserts.data = vuedata;

                    let checkedAsserts = ruleunit.applyto.split(";");
                    vue_edit_choose_asserts.setCheckedKeys(checkedAsserts)
                })
            })
        })
        $("#edit-rule-confirm").unbind("click").click(function(e) {
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");

            let choosedAssertsAndGroups = vue_edit_choose_asserts.getCheckedNodes();
            let choosedAsserts = [];
            for (let i=0;i<choosedAssertsAndGroups.length;i++) {
                if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
                    choosedAsserts.push(choosedAssertsAndGroups[i].id)
                }
            }
            if (choosedAsserts.length < 1) {
                return $("#edit-rule-msg").text("请至少选择一个资产");
            }
            let assertsstr = choosedAsserts.join(";");

            let name = $("#edit-rule-name").val();
            if (!name || name.length < 1 || name.length > 32) {
                return $("#edit-rule-msg").text("请输入合法的名称，最大32字符");
            }
            let straid = $("#edit-stra-name option:selected").val();

            let http = cockpit.http({
                "port": 9091
            });
            http.post("/editRule", {
                id: ruleid,
                type: "user",
                name: name,
                straid: straid,
                asserts: assertsstr
            })
            .fail(function(err){
                console.log("editRule err:", err);
                $("#edit-rule-msg").text("编辑规则失败")
                let cent = "编辑账户: " + name + " 规则失败, 失败原因: " + err;
                writeAuditlog(2, name, cent);
            })
            .done(function(res) {
                console.log("editRule res:", res);

                let info = $.parseJSON( res );
                if (! info.succeed) {
                    console.log("editRule err:", res);
                    let cent = "编辑账户: " + name + " 规则失败, 失败原因: " + info.error;
                    writeAuditlog(2, name, cent);
                    return $("#edit-rule-msg").text("编辑规则失败：" + info.error)
                }
                $("#edit-rule-msg").text("编辑规则成功");
                setTimeout(function(e){
                    $("#edit-rule-close").click();
                    render_rules();
                }, 1500)

                let cent = "编辑账户: " + name + " 规则成功";
                writeAuditlog(2, name, cent);
            })
        })
        $("button.user-rule-del").unbind("click").click(function(e){
            // 弹出框只有一个，直接用弹出框上的 ruleid 就是当前操作的 ruleid
            let ruleid = $("#user-rule-popup-menu-private-data").data("private-data");
            let ruleunit = getRuleById(ruleid);
            $("#confirm-del-user-rule-msg").text("确认要删除规则 " + ruleunit.name + " 吗？")

            $("button#confirm-del-user-rule-confirm").unbind("click").click(function(e) {
                let http = cockpit.http({
                        "port": 9091
                    });
                http.post("/delRule", {
                    type: "user",
                    id: ruleid
                })
                .fail(function(err){
                    console.log("delRule err:", err);
                    $("#confirm-del-user-rule-msg").text("删除规则失败");
                    let cent = "删除规则: " + ruleunit.name + " 失败, 失败原因: " + err;
                    writeAuditlog(2, ruleunit.name, cent);
                })
                .done(function(res) {
                    // console.log("getUsers res:", res);

                    let info = $.parseJSON( res );
                    if (! info.succeed) {
                        console.log("getUsers err:", res);
                        $("#confirm-del-user-rule-msg").text("删除规则失败：" + info.error);
                        let cent = "删除规则: " + ruleunit.name + " 失败, 失败原因: " + info.error;
                        writeAuditlog(2, ruleunit.name, cent);
                        return ;
                    }
                    $("#confirm-del-user-rule-msg").text("删除规则成功");
                    setTimeout(function(e){
                        $("#confirm-del-user-rule-close").click()
                        render_rules();
                    }, 1500)

                    let cent = "删除规则: " + ruleunit.name + " 成功";
                    writeAuditlog(2, ruleunit.name, cent);
                })
            })
        })
    }

    function render_rules() {
    	$("#content-header").show();
    	$("#accounts-create").hide();
    	$("#add-rule").show();
    	$(".user-rule-popup-menu").css("display", "none")

    	$("#loading-fallback").show();
        clearInterval(intv)
    	let http = cockpit.http({
                "port": 9091
            });
        http.get("/getRules", {"type": "user"})
        .fail(function(err){
            console.log("getRules err:", err);
            $("#loading-fallback").hide();
        })
        .done(function(res) {
            // console.log("getRules res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getRules err:", res);
                return $("#loading-fallback").hide();
            }
            info = info.data;
            g_rules_bak = info;

            // 规则里面还要展示应用到的资产，需要获取资产详细信息
            http.get("/getAsserts")
            .fail(function(err){
	            console.log("getAsserts err:", err);
	            return $("#loading-fallback").hide();
	        })
            .done(function(res) {
	            // console.log("getAsserts res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
	                console.log("getAsserts err:", res);
	                $("#loading-fallback").hide();
	                return ;
	            }
	            info = info.data;
	            g_asserts_bak = info;

	            show_rules_page();
            	$("#loading-fallback").hide();

            	registerRulesEvents()
                // 定时器获取规则状态
                var dlsucceed = /\u4e0b\u53d1\u4e2d/
                intv = setInterval(()=>{
                    let isdone = true
                    for (var i=0; i<g_rules_bak.length; i++){
                        if (dlsucceed.test(g_rules_bak[i].status)){   
                            isdone = false              
                            http.get("/getRuleState", {"type": "user"})
                            .fail(function(err) {
                                console.log("getRules of user err:", err);
                            })
                            .done(function(res) {
                                let info = $.parseJSON( res );
                                if (! info.succeed) {
                                    console.log("getRules of user err:", res);
                                }
                                info = info.data;
                                g_rules_bak = info ? info : g_rules_bak;
                            })
                        }else{
                            for (var k in g_rules_bak){
                                var arule = g_rules_bak[k];
                                $("#user-rule-"+arule["id"]).text(arule["status"]);
                            }
                        }
                    }
                    if (isdone){
                        clearInterval(intv);
                    }
                },5000);
	        })
        })
    }

    $("#add-rule").unbind("click").click(function(e){
    	$("#add-stra-name").html("");
    	$("#add-rule-name").val("");
        $("#add-rule-msg").text("");
    	let http = cockpit.http({
            "port": 9091
        });
        http.get("/getUsers")
        .fail(function(err){
            console.log("getUsers err:", err);
            $("#add-rule-msg").text("获取账号策略失败")
        })
        .done(function(res) {
            // console.log("getUsers res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("getUsers err:", res);
                return $("#add-rule-msg").text("获取账号策略失败：" + info.error)
            }
            info = info.data;
            g_users_bak = info;

            $("#add-stra-name").html("");
            for (let i=0;i<g_users_bak.length;i++) {
    			let html = '<option value="' + g_users_bak[i].id + '">' + g_users_bak[i].name + '</option>';
            	$("#add-stra-name").append($(html))
    		}

    		// 新增规则的选择资产和分组的控件用的是 element-ui 的 tree 控件，用 vue 来渲染
    		// 先获取资产和分组信息
    		http.get("/getGroupsAndAsserts")
    		.fail(function(err){
	            console.log("getGroupsAndAsserts err:", err);
	            $("#add-rule-msg").text("获取资产和分组失败")
	        })
	        .done(function(res) {
	            // console.log("getGroupsAndAsserts res:", res);

	            let info = $.parseJSON( res );
	            if (! info.succeed) {
	                console.log("getGroupsAndAsserts err:", res);
	                return $("#add-rule-msg").text("获取资产和分组失败：" + info.error)
	            }

	            info = info.data;
	            vuedata = getVuedataByAssertsAndGroups(info);

	            if (!vue_choose_asserts) {
	            	vue_choose_asserts = new Vue({
				    	template: '<el-tree :data="data" show-checkbox default-expand-all node-key="id" ref="tree" highlight-current :props="defaultProps"></el-tree>',
				    	el: "#vue-app-div",
				    	methods: {
					      	getCheckedNodes() {
					        	return this.$refs.tree.getCheckedNodes();
					      	},
					      	setCheckedKeys(data) {
								this.$refs.tree.setCheckedKeys(data);
							}
					    },
					    data: {
					    	data: vuedata,
					    	defaultProps: {
			          			children: 'children',
			          			label: 'label'
			        		}
			        	},
			        	watch: {
			        		data(newdata, olddata) {
			        			//
			        		}
			        	}
				    })
	            }
	            vue_choose_asserts.data = vuedata;
	        })
        })
    })
	// 确认添加规则时的操作
	$("#add-rule-confirm").unbind("click").click(function(e){
		let choosedAssertsAndGroups = vue_choose_asserts.getCheckedNodes();
		let choosedAsserts = [];
		for (let i=0;i<choosedAssertsAndGroups.length;i++) {
			if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
				choosedAsserts.push(choosedAssertsAndGroups[i].id)
			}
		}
		let choosedGroups = [];
		// 检查是否某个分组的所有资产都选择上了，如果都选择上了，则认为应用到分组内的所有资产，包括以后添加的?
		// 确定是否选择了分组不太确定，如果第一次建立规则时没选全分组内的资产，后来删除了分组内没选的资产，
		// 下次编辑的时候就会变成选择了分组内的全部资产了，还是只按照资产下发，如果改变了分组状态，
		// 需要重新编辑规则，并且下发
		for (let i=0;i<choosedAssertsAndGroups.length;i++) {
			if (choosedAssertsAndGroups[i].label.indexOf("资产") == 0) {
				continue
			}
		}
		if (choosedAsserts.length < 1) {
			return $("#add-rule-msg").text("请至少选择一个资产");
		}
		let assertsstr = choosedAsserts.join(";");
		let name = $("#add-rule-name").val();
		if (!name || name.length < 1 || name.length > 32) {
			return $("#add-rule-msg").text("请输入合法的名称，最大32字符");
		}
		let straid = $("#add-stra-name option:selected").val();
		if (!straid || straid.length < 1) {
			return $("#add-rule-msg").text("请选择合法的策略");
		}

		let http = cockpit.http({
            "port": 9091
        });
        http.post("/addRule", {
        	type: "user",
        	name: name,
        	straid: straid,
        	asserts: assertsstr
        })
        .fail(function(err){
            console.log("addRule err:", err);
            $("#add-rule-msg").text("添加规则失败")
            let cent = "添加规则: " + name + " 失败, 失败原因: " + err;
            writeAuditlog(2, name, cent);
        })
        .done(function(res) {
            console.log("addRule res:", res);

            let info = $.parseJSON( res );
            if (! info.succeed) {
                console.log("addRule err:", res);
                let cent = "添加规则: " + name + " 失败, 失败原因: " + info.error;
                writeAuditlog(2, name, cent);
                return $("#add-rule-msg").text("添加规则失败：" + info.error)
            }
            $("#add-rule-msg").text("添加规则成功");
            setTimeout(function(e){
            	$("#add-rule-close").click();
            	render_rules();
            }, 1500)
            let cent = "添加规则: " + name + " 成功";
            writeAuditlog(2, name, cent);
        })
	})

	$('#accounts-filter li').on('click', function() {
        $('#accounts-filter li')
            .removeClass('active')
            .removeAttr('aria-current');
        $(this)
            .addClass('active')
            .attr('aria-current', true);

        let pattern = $(this).attr('data-pattern');
        if (pattern == "usersrules") {
            cockpit.location.go(["usersrules"]);
            $("#add-account").hide();
            $("#add-rule").show();
            //render_rules();
        }
        else if (pattern == "users") {
            cockpit.location.go([""]);
            $("#add-account").show();
            $("#add-rule").hide();
            //render_users()
        }
    })

    $("#accounts-create").unbind("click").click(function(){
        $("input#accounts-create-user-name").val("");
        $("input#accounts-create-full-name").val("");
        $("input#accounts-create-pw1").val("");
        $("input#accounts-create-pw2").val("");
        $("#accounts-create-password-meter").removeClass("weak okay good excellent");
        $("#accounts-create-password-meter-message").text("");
        $("input#accounts-create-locked").prop("checked",false);
        $("input#accounts-create-isadmin").prop("checked",false);
        $("input#accounts-create-delete").prop("checked",false);
        $("#create-user-msg").text("");
    })

	// 根据 url 初始化渲染页面
    $("#accounts-filter li")
        .removeClass('active')
        .removeAttr('aria-current');

    function navigate() {
        if (window.location.href.indexOf("#/usersrules") > 0) {
	        $("#usersrules-li").addClass('active')
	            .attr('aria-current', true);
	        $("#add-account").hide();
			$("#add-rule").show();
			render_rules();
	    } else {
	        $("#users-li").addClass('active')
	            .attr('aria-current', true);
			$("#add-account").show();
			$("#add-rule").hide();
			if (window.location.href.indexOf("#/userdetail-") > 0) {
				render_user_detail()
			} else {
				render_users()
			}
	    }
    }

    $(cockpit).on("locationchanged", navigate);
    navigate();
}

if (window.location.pathname.indexOf("manager/user.html") > 0) {
    $(user_init);
}
