# -*- coding:utf-8 -*-
__author__ = "tangjie"

from flask import Flask
from flask_sqlalchemy import SQLAlchemy

import sys
import uuid
import json
import datetime
import kyencrypt

app=Flask("app")
app.config['SQLALCHEMY_DATABASE_URI'] = "mysql+pymysql://localuser:localuser@127.0.0.1/cockpit"
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.config['SOLALCHEMY_ECHO'] = False
app.config['SQLALCHEMY_POOL_RECYCLE'] = 3600

db = SQLAlchemy(app)

class Assert(db.Model): #继承生成的orm基类
    __tablename__ = "assert" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键
    assertid = db.Column(db.String(40)) # uuid length is 36
    assertname = db.Column(db.String(40)) # max length is 40
    assertlocation = db.Column(db.String(40)) # max length is 40
    assertusage = db.Column(db.String(40)) # max length is 40
    assertip = db.Column(db.String(40)) # max length is 40
    asserttag = db.Column(db.Text())
    assertstate = db.Column(db.String(10), default="")
    assertuser = db.Column(db.String(32), default="")
    assertpass = db.Column(db.String(10),default="")
    def to_json(self):
        return {
            "id": self.id,
            "assertid": self.assertid,
            "assertname": self.assertname,
            "assertlocation": self.assertlocation,
            "assertusage": self.assertusage,
            "assertip": self.assertip,
            "asserttag": self.asserttag,
            "assertstate": self.assertstate,
            "assertuser": self.assertuser,
            "assertpass": self.assertpass
        }

class Asgroup(db.Model): #继承生成的orm基类
    __tablename__ = "asgroup" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键
    groupid = db.Column(db.String(40)) # uuid length is 36
    groupname = db.Column(db.String(40)) # max length is 40
    groupdesc = db.Column(db.Text(), default="") # max length is 40
    groupasserts = db.Column(db.Text(), default="")
    createtime = db.Column(db.DateTime, default=datetime.datetime.now())

    def to_json(self):
        return {
            "id": self.id,
            "groupid": self.groupid,
            "groupname": self.groupname,
            "groupdesc": self.groupdesc,
            "groupasserts": self.groupasserts,
            "createtime": str(self.createtime)
        }

class Config(db.Model): #继承生成的orm基类
    __tablename__ = "config" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键
    configtype = db.Column(db.String(40), default="") # max length is 40
    content = db.Column(db.Text()) # max length is 40

    def to_json(self):
        return {
            "id": self.id,
            "configtype": self.configtype,
            "content": self.content
        }

class Batchuser(db.Model): #继承生成的orm基类
    __tablename__ = "batchuser" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键
    name = db.Column(db.String(40), default="") # max length is 40
    fullname = db.Column(db.String(40), default="") # max length is 40
    password = db.Column(db.Text(), default="") #
    lockuser = db.Column(db.Integer(), default=0)
    isadmin = db.Column(db.Integer(), default=0)
    deleteuser = db.Column(db.Integer(), default=0)
    authkey = db.Column(db.Text(), default="") #

    def to_json(self):
        return {
            "id": self.id,
            "name": self.name,
            "fullname": self.fullname,
            "password": self.password,
            "lockuser": self.lockuser,
            "isadmin": self.isadmin,
            "deleteuser": self.deleteuser,
            "authkey": self.authkey
        }

class AssertUserPass(db.Model): # 保存各台资产的各个用户的密码
    __tablename__ = "assertuserpass"

    id = db.Column(db.Integer, primary_key=True) #设置主键
    assertid = db.Column(db.String(40), default="") # max length is 40
    username = db.Column(db.String(40), default="") # max length is 40
    password = db.Column(db.Text(), default="") #

    def to_json(self):
        return {
            "id": self.id,
            "assertid": self.assertid,
            "username": self.username,
            "password": self.password
        }

#定义软件源表结构
class SoftRepo(db.Model):#继承生成的orm基类
    __tablename__ = "softrepo" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键
    name = db.Column(db.String(60), unique=True) #名称，要保证唯一性，因为软件源要在服务器上通过httpd发布
    tag = db.Column(db.String(10)) #版本标签，比如3.2、3.3、3.4
    supportedarch = db.Column(db.String(60)) #支持的架构，比如x86_64、aarch64
    supportedrelease = db.Column(db.String(60)) #支持的版本，比如3.2-10、3.3-6、3.3-6-pg
    buildtime = db.Column(db.String(60)) #生成时间,为方便转换，采用字符串表示
    pkglist = db.Column(db.Text()) #包列表，目前设为空值，考虑到不同架构，软件包可能不同
    repofile = db.Column(db.String(255)) #repo文件路径
    repoinfo = db.Column(db.Text()) #保存repo文件信息，包含repo文件名、name、baseurl等信息
    ip = db.Column(db.String(20)) #软件源绑定的服务器IP地址
    iscve = db.Column(db.Boolean, default=False) #是否包含cve漏洞修复
    cveinfo = db.Column(db.Text()) # 具体包含的cve漏洞信息，比如修复了哪些cve漏洞，需要有cve漏洞编号，从上传的rpm中收集软件源绑定的服务器IP地址
    iseffective = db.Column(db.Boolean, default=True) #软件源是否是有效的
    isdelete = db.Column(db.Boolean, default=False) #是否删除软件源

    def to_json(self):
        return{
            "id": self.id,
            "name": self.name,
            "tag": self.tag,
            "buildtime": self.buildtime,
            "pkglist": self.pkglist,
            "repofile": self.repofile,
            "repoinfo": self.repoinfo,
            "ip": self.ip,
            "iscve": self.iscve,
            "cveinfo": self.cveinfo,
            "iseffective": self.iseffective,
            "isdelete": self.isdelete
        }

#定义软件源策略表结构
class RepoStrategy(db.Model):
    __tablename__ = "repostrategy" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键
    name = db.Column(db.String(60)) #名称
    repoidlist = db.Column(db.Text()) #策略包含的软件源ID列表
    reponamelist = db.Column(db.Text()) #策略包含的软件源名称列表
    repofilelist = db.Column(db.Text()) #策略包含的软件源repo文件路径列表
    isdelete = db.Column(db.Boolean, default=False) #是否删除软件源策略

    def to_json(self):
        return{
            "id": self.id,
            "name" :self.name,
            "repoidlist": self.repoidlist,
            "reponamelist": self.reponamelist,
            "repofilelist": self.repofilelist,
            "isdelete": self.isdelete
        }

#定义软件升级策略表
class UpdateStrategy(db.Model):
    __tablename__ = "updatestrategy"  

    id = db.Column(db.Integer, primary_key=True) #设置主键
    name = db.Column(db.String(60))  #设置名称
    pkglist = db.Column(db.Text()) #升级软件包列表

    def to_json(self):
        return{
            "id": self.id,
            "name": self.name,
            "pkglist": self.pkglist
        }

class Rules(db.Model): #继承生成的orm基类
    __tablename__ = "rules" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键
    name = db.Column(db.String(40), default="") # max length is 40
    type = db.Column(db.String(40), default="") # max length is 40

    straid =db.Column(db.Integer, default=0)
    # 需要保存策略的内容，只下发该状态下的策略，不然在下发过程中编辑了策略，
    # 可能导致不同资产上下发不一致
    stracontent = db.Column(db.Text(), default="")

    applyto = db.Column(db.Text(), default="") #

    editcount = db.Column(db.Integer(), default=0)
    applycount = db.Column(db.Text(), default="") #
    # status 目前在用户规则和安全规则下发中用作重新下发标志 redownload
    status = db.Column(db.String(10), default="") # max length is 40
    createtime = db.Column(db.DateTime, default=datetime.datetime.now())
    applytime = db.Column(db.DateTime, default=datetime.datetime.now())
    isdelete = db.Column(db.Boolean, default=False) #是否删除规则
    deletecount = db.Column(db.Text(), default="") #规则删除情况

    def to_json(self):
        return {
            "id": self.id,
            "name": self.name,
            "type": self.type,
            "straid": self.straid,
            "stracontent": self.stracontent,
            "applyto": self.applyto,
            "editcount": self.editcount,
            "applycount": self.applycount,
            "status": self.status,
            "createtime": str(self.createtime),
            "applytime": str(self.applytime),
            "isdelete": self.isdelete,
            "deletecount": self.deletecount
        }

#创建SCMMS 用户表
class ScmmsUser(db.Model): #继承生成的orm基类
    __tablename__ = "scmmsuser" #表名
    id = db.Column(db.Integer, primary_key=True) #设置主键
    name = db.Column(db.String(40), default="") # 用户名
    role = db.Column(db.String(40), default="") # 角色
    password = db.Column(db.String(80), default="") # 密码
    createtime = db.Column(db.DateTime, default=datetime.datetime.now()) #创建时间
    enabled = db.Column(db.Boolean, default=True) #是否启用改用户 
    def to_json(self):
        return {
            "id": self.id,
            "name": self.name,
            "role": self.role,
            "password": self.password,
            "createtime": str(self.createtime),
            "enabled": self.enabled
        }

class AuditLog(db.Model): #继承生成的orm基类
    __tablename__ = "auditlog" #表名

    id = db.Column(db.Integer, primary_key=True) #设置主键

    time = db.Column(db.DateTime, default=datetime.datetime.now())
    user = db.Column(db.String(60), default="") 
    operator =db.Column(db.Integer, default=0)
    auditobject = db.Column(db.String(40), default="") 
    content = db.Column(db.String(256), default="") 

    def to_json(self):
        return {
            "id": self.id,
            "time": str(self.time),
            "user": self.user,
            "operator": self.operator,
            "auditobject": self.auditobject,
            "content": self.content
        }

if __name__ == '__main__':
    #创建表结构
    #父类Base调用所有继承他的子类来创建表结构
    # db.drop_all()
    db.create_all()
    querycnt = db.session.query(Assert).filter(Assert.assertip=="localhost").count()

    if querycnt < 1:
        with open("/etc/.kyinfo", "r") as fd:
            for line in fd.readlines():
                if line.startswith("milestone") and line.find(" = 3.4-") > 0:
                    tag = "3.4"
                if line.startswith("milestone") and line.find(" = 3.3-") > 0:
                    tag = "3.3"

        if not tag or not len(tag) == 3:
            print("get system version info failed")
            sys.exit(1)

        with open("/etc/machine-id", "r") as fd:
            finger = fd.readline().strip()

        if not finger or not len(finger) == 32:
            print("get system finger failed")
            sys.exit(1)

        res = Assert(assertid=finger, asserttag=tag, assertname="localhost", assertip="localhost")
        db.session.add(res)
        db.session.commit()

    mypassword = "kylinos.2020"
    pwd = kyencrypt.encrypt(11, mypassword)
    querycnt = db.session.query(ScmmsUser).filter(ScmmsUser.name == "sysadm").count()
    if querycnt < 1:
        rec = ScmmsUser(name = "sysadm", password = pwd, role = "sysadm", createtime = datetime.datetime.now(), enabled = 1)
        db.session.add(rec)
        db.session.commit()

    querycnt = db.session.query(ScmmsUser).filter(ScmmsUser.name == "secadm").count()
    if querycnt < 1:
        rec = ScmmsUser(name = "secadm", password = pwd, role = "secadm", createtime = datetime.datetime.now(), enabled = 1)
        db.session.add(rec)
        db.session.commit()

    querycnt = db.session.query(ScmmsUser).filter(ScmmsUser.name == "audadm").count()
    if querycnt < 1:
        rec = ScmmsUser(name = "audadm", password = pwd, role = "audadm", createtime = datetime.datetime.now(), enabled = 1)
        db.session.add(rec)
        db.session.commit()

    querycnt = db.session.query(Config).filter(Config.configtype=="find").count()
    if querycnt < 1:
        content = {"enabled": 0, "iprange": ""}
        res = Config(configtype="find", content=json.dumps(content))
        db.session.add(res)
        db.session.commit()
